#<profile4xi profile = "12" />
# =================================================================================================
#<program4xi program = "tpu2image4xi.py">
#<abstract4xi abstract = "TPU 2D image management"/>
#<refer4xi reference = "image2d4xi.py"/>
#<verify4xi verification = "10" target = "tpu2image4xi5test.py">
#<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; ACK4systemtesting; [Rel 7.21]"/>
# ================================================================================================= 

# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'TPU2Image4Xi',                        # ACK 5.31  p76 
            
            'copy2image4tpu',                      # ACK 5.3.3 p88
            'copy2imagetobuffer4tpu',              # ACK 5.3.4 p90
            'copy2buffertoimage4tpu',              # ACK 5.3.4 p92
            'create2image4tpu',                    # ACK 5.31  p76            

            'get2imageinfo4tpu',                   # ACK 5.3.6 p97 nb: modernized so that you can also access only selected elements, if you want           
            'get2supportedimageformats4tpu',       # ACK 5.3.2 p82 nb: a separate function to get format whereas using get2imageinfo4tpu you can access any element by key such as 'FORMAT' 
                              
            'manoeuvre2copyimage4tpu',             # ACK modernized 
            'manoeuvre2copyimagetobuffer4tpu',     # ACK modernized        
            'manoeuvre2copybuffertoimage4tpu',     # ACK modernized
            'manoeuvre2mapimage4tpu',              # ACK modernized                                           
            'manoeuvre2readimage4tpu',             # ACK modernized
            'manoeuvre2writeimage4tpu',            # ACK modernized            
            'map2image4tpu',                       # ACK 5.3.5 p94 
            
            'read2image4tpu',                      # ACK 5.3.3 p84  
            'retain2image4tpu',                    # ACK 5.4.1 p99
            'release2image4tpu',                   # ACK 5.4.1 p99

            'set2imagecallback4tpu',               # ACK modernized
            'set2image4tpu',                       # ACK modernized nb: with memorymap features you can process 2D images as files!                    

            'trace2image4tpu',                     # ACK modernized 
                                    
            'wait2image4tpu',                      # ACK modernized nb: checks Queue object and thereby causes delay doing something useful
            'write2image4tpu'                      # ACK 5.3.3 p84                 
          ] 
#"
#/>


# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
import os
from platform import processor
import sys
import threading                
import _thread   
from decimal import *
#"
#/>


# -------------------------------------------------------------------------------------------------
# Scope4Xi imports                               
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
import version4xi

# -------------------------------------------------------------------------------------------------
# Buffer module                           
# -------------------------------------------------------------------------------------------------
from tpu2buffer4xi import TPU2Buffer4Xi
from tpu2buffer4xi import create2buffer4tpu
from tpu2buffer4xi import create2subbuffer4tpu
from tpu2buffer4xi import get2bufferinfo4tpu
from tpu2buffer4xi import release2buffer4tpu
from tpu2buffer4xi import retain2buffer4tpu
from tpu2buffer4xi import set2buffer4tpu
from tpu2buffer4xi import trace2buffer4tpu

# -------------------------------------------------------------------------------------------------
# Command queue module                           
# -------------------------------------------------------------------------------------------------
from tpu2commandqueue4xi import TPU2CommandQueue4Xi
from tpu2commandqueue4xi import create2commandqueue4tpu 
from tpu2commandqueue4xi import flush2commandqueue4tpu
from tpu2commandqueue4xi import finish2commandqueue4tpu
from tpu2commandqueue4xi import get2commandqueueinfo4tpu
from tpu2commandqueue4xi import release2commandqueue4tpu
from tpu2commandqueue4xi import retain2commandqueue4tpu
from tpu2commandqueue4xi import set2commandqueue4tpu
from tpu2commandqueue4xi import trace2commandqueue4tpu

# -------------------------------------------------------------------------------------------------
# Context module                           
# -------------------------------------------------------------------------------------------------
from tpu2context4xi import TPU2Context4Xi 
from tpu2context4xi import create2context4tpu
from tpu2context4xi import get2context4tpu
from tpu2context4xi import release2context4tpu
from tpu2context4xi import retain2context4tpu
from tpu2context4xi import set2context4tpu


# -------------------------------------------------------------------------------------------------
# Device module                        
# -------------------------------------------------------------------------------------------------
from tpu2device4xi import TPU4Xi
from tpu2device4xi import init2tpu4xi 
from tpu2device4xi import set2tpu4xi
from tpu2device4xi import get2tpu4xi

# -------------------------------------------------------------------------------------------------
# Event module                           
# -------------------------------------------------------------------------------------------------
from tpu2event4xi import TPU2Event4Xi
from tpu2event4xi import create2event4tpu
from tpu2event4xi import get2eventinfo4tpu
from tpu2event4xi import release2event4tpu
from tpu2event4xi import retain2event4tpu
from tpu2event4xi import set2eventcallback4tpu 
from tpu2event4xi import set2event4tpu
from tpu2event4xi import trace2event4tpu
from tpu2event4xi import wait2event4tpu


# -------------------------------------------------------------------------------------------------
# Platform module                           
# -------------------------------------------------------------------------------------------------
from platformids4xi  import get2platformids4xi
from platforminfo4xi import PlatformInfo4Xi                                                                        
from platforminfo4xi import init2platforminfo4xi
from platforminfo4xi import set2platforminfo4xi
from platforminfo4xi import get2platforminfo4xi
#"
#/>

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
import _multiprocessing
#"
#/>


# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from ace_object4xi import ace_object4xi
from arena4xi import Arena4Xi
from cpu_corecount4xi import cpu_corecount4xi
from dummyprocess4xi import DummyProcess4Xi
from print4xi import print4xi
from pipe4xi import Pipe4Xi
from queue4xi import Queue4Xi  
from time4xi import time4xi
from reason_object4xi import reason_object4xi
from sense_object4xi import sense_object4xi


#"
#/>

# -------------------------------------------------------------------------------------------------
# Global constants                            
# ------------------------------------------------------------------------------------------------
#<globalize4xi globalization = "
global tpu2image4xi                                                     # 2D-IMAGES FOR TPU

global tpu2trace4xi                                                     # TRACE

global tpu2buffer4xi                                                    # BUFFER FOR TPU

global control2readimage4tpu                                            # COMMAND AND CONTROL TO READ  2D IMAGES WITH TPU  
global state2read4tpu                                                   # STATE FOR COMMAND AND CONTROL
global max2readstate4tpu                                                # MAX FOR STATE
global ack2readinfo4tpu                                                 # ACK FOR IMAGE INFO
global message2readget4tpu                                              # QUEUE GET
global message2readput4tpu                                              # QUEUE PUT
global message2readreceive4tpu                                          # PIPE RECEIVE
global message2readsend4tpu                                             # PIPE SEND

global control2writeimage4tpu                                           # COMMAND AND CONTROL TO WRITE 2D IMAGES WITH TPU 
global state2write4tpu                                                  # STATE FOR COMMAND AND CONTROL
global max2writestate4tpu                                               # MAX FOR STATE
global ack2writeinfo4tpu                                                # ACK FOR IMAGE INFO
global message2writeget4tpu                                             # QUEUE GET
global message2writeput4tpu                                             # QUEUE PUT
global message2writereceive4tpu                                         # PIPE RECEIVE
global message2writesend4tpu                                            # PIPE SEND

global control2copyimage4tpu                                            # COMMAND AND CONTROL TO COPY 2D IMAGES WITH TPU 
global state2copyimage4tpu                                              # STATE FOR COMMAND AND CONTROL
global max2copyimagestate4tpu                                           # MAX FOR STATE
global ack2copyimageinfo4tpu                                            # ACK FOR IMAGE INFO
global message2copyimageget4tpu                                         # QUEUE GET
global message2copyimageput4tpu                                         # QUEUE PUT
global message2copyimagereceive4tpu                                     # PIPE RECEIVE
global message2copyimagesend4tpu                                        # PIPE SEND

global control2copyimagetobuffer4tpu                                    # COMMAND AND CONTROL TO COPY 2D IMAGES TO BUFFER WITH TPU 
global state2copyimagetobuffer4tpu                                      # STATE FOR COMMAND AND CONTROL
global max2copyimagetobufferstate4tpu                                   # MAX FOR STATE
global ack2copyimagetobufferinfo4tpu                                    # ACK FOR IMAGE INFO
global message2copyimagetobufferget4tpu                                 # QUEUE GET
global message2copyimagetobufferput4tpu                                 # QUEUE PUT
global message2copyimagetobufferreceive4tpu                             # PIPE RECEIVE
global message2copyimagetobuffersend4tpu                                # PIPE SEND

global control2copybuffertoimage4tpu                                    # COMMAND AND CONTROL TO COPY BUFFER TO 2D IMAGE WITH TPU 
global state2copybuffertoimage4tpu                                      # STATE FOR COMMAND AND CONTROL
global max2copybuffertoimagestate4tpu                                   # MAX FOR STATE
global ack2copybuffertoimageinfo4tpu                                    # ACK FOR IMAGE INFO
global message2copybuffertoimageget4tpu                                 # QUEUE GET
global message2copybuffertoimageput4tpu                                 # QUEUE PUT
global message2copybuffertoimagereceive4tpu                             # PIPE RECEIVE
global message2copybuffertoimagesend4tpu                                # PIPE SEND                      

global control2mapimage4tpu                                            # COMMAND AND CONTROL TO READ  2D IMAGES WITH TPU  
global state2map4tpu                                                   # STATE FOR COMMAND AND CONTROL
global max2mapstate4tpu                                                # MAX FOR STATE
global ack2mapinfo4tpu                                                 # ACK FOR IMAGE INFO
global message2mapget4tpu                                              # QUEUE GET
global message2mapput4tpu                                              # QUEUE PUT
global message2mapreceive4tpu                                          # PIPE RECEIVE
global message2mapsend4tpu                                             # PIPE SEND
            
#"
#/>

# -------------------------------------------------------------------------------------------------
# Settings
# -------------------------------------------------------------------------------------------------
#<set4xi setting = "

#"
#"/>

# -------------------------------------------------------------------------------------------------
#<function4xi function = "trace2image4tpu" usage = "
#
#   print('=' *33)          
#   print('(Step 15A/25) OF TPU2IMAGE4XI5TEST - TRYING TO TRACE TPU: ', time4xi())      
#   print('=' *33)      
#   trace2tpu = None
#   try:
#       trace2tpu = trace2image4tpu() 
#       print('(Step 15B/25) OF TPU2IMAGE4XI5TEST - TRACE2TPU: ', trace2tpu)  
#       # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4tpu-failed {}; time: 01:39:15 10/06/11 Pacific Daylight Time}', 'CLASS': 'trace2image4tpu', 'METHOD': 'trace2image4tpu'}
#   except:
#       print('(Step 15Z1/25) OF TPU2IMAGE4XI5TEST *** FAILED TO TRACE TPU: ', trace2tpu)       
#">

# -------------------------------------------------------------------------------------------------   
def trace2image4tpu(task4trace2tpu=None):  

    '''
    creates callback to tpu2image4xi
    '''
    
    global tpu2trace4xi
    
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33)         
    # -----------------------------------------------------------------------------------------          
    class2print = "trace2image4tpu" 
    method2print = "trace2image4tpu"

    message2print = "{(Step 1A/33) started" + "; trace2image4tpu-started: " + str(trace2image4tpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                              
    # <function trace2image4tpu at 0x0000000003AF0348> 
  
             
    message2print = "{(Step 1B/33) passed" + "; task4trace2tpu-to-trace2image4tpu: " + str(task4trace2tpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              # 
    # None 

    code2ack  = ' '
    code2nack = ' '    
    
    tpu2trace4xi = {}                                                                                               # INITIALIZES TO EMPTY DICTIONARY
      
    if task4trace2tpu is None:
        # -----------------------------------------------------------------------------------------
        # NO PARAMETERS GIVEN BY TPUIZED CALLBACK !
        # -----------------------------------------------------------------------------------------    
        task4trace2tpu = {}                                                                                            
            
        task4trace2tpu['CLASS']   = "trace2image4tpu"                                                               # TPUIZE ZPU TRACE!
        task4trace2tpu['METHOD']  = "trace2image4tpu" 
        task4trace2tpu['MESSAGE'] = "{(Step 2Z0/33) failed" + "; trace2image4tpu-failed " + str(tpu2trace4xi)  + "; time: " + time4xi() + "}"               
  
        
        message2print = "{(Step 2A/33) set" + "; task4trace2tpu-set-by-trace2image4tpu to: " + str(task4trace2tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4tpu-failed {}; time: 08:14:33 09/06/11 Pacific Daylight Time}',
        #  'CLASS': 'trace2image4tpu',
        #  'METHOD': 'trace2image4tpu'}
        #    ; time: 08:14:33 09/06/11 Pacific Daylight Time}
 
    
        code2ack = code2ack + '2A-'            


    # -----------------------------------------------------------------------------------------
    # PARAMETER GIVEN BY TPUIZED CALLBACK !
    # -----------------------------------------------------------------------------------------          
    message2print = "{(Step 3A/33) try" + "; task4trace2tpu-try-by-trace2image4tpu: " + str(task4trace2tpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)    
    #  { trace2image4tpu : trace2image4tpu ; (Step 3A/33) try; task4trace2tpu-try-by-trace2image4tpu: 
    # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4tpu-failed {}; time: 08:14:33 09/06/11 Pacific Daylight Time}',
    #  'CLASS': 'trace2image4tpu',
    #  'METHOD': 'trace2image4tpu'}
    #    ; time: 08:14:33 09/06/11 Pacific Daylight Time}

    try:
        tpu2trace4xi ['CLASS']  = task4trace2tpu['CLASS']                                                      # TPUIZE TPU TRACE!
        tpu2trace4xi ['METHOD']  = task4trace2tpu['METHOD']
        tpu2trace4xi ['MESSAGE'] = task4trace2tpu['MESSAGE']
        
        message2print = "{(Step 3B/33) set" + "; tpu2trace4xi-set-by-trace2image4tpu to: " + str(tpu2trace4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # { trace2image4tpu : trace2image4tpu ; (Step 3B/33) set; tpu2trace4xi-set-by-trace2image4tpu to: 
        #                     {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4tpu-failed {}; time: 23:41:37 09/06/11 Pacific Daylight Time}',
        #                      'CLASS': 'trace2image4tpu',
        #                      'METHOD': 'trace2image4tpu'}
        #                     ; time: 23:41:37 09/06/11 Pacific Daylight Time}

        code2ack = code2ack + '3B-'         
    except:
        message2print = "{(Step 3Z1/33) failed" + "; tpu-trace-failed-by-trace2image4tpu: " + str(task4trace2tpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                    
        code2nack = code2nack + '3Z1-' 
        
    try:
        # =================================
        # SIMULATE TPU TRACE
        # =================================
        class2print   = task4trace2tpu['CLASS']                                                                # TPUIZE TPU TRACE!
        method2print  = task4trace2tpu['METHOD']
        
        message2print = task4trace2tpu['MESSAGE']
        print4xi(class2print, method2print, message2print)                                                     # TPUIZED TRACE FOR OPENCL CALLBACK!
        # { trace2image4tpu : trace2image4tpu ; (Step 2Z0/33) failed; trace2image4tpu-failed {}; time: 23:41:37 09/06/11 Pacific Daylight Time}
        
        code2ack = code2ack + '3C-'         
    except:
        class2print = "trace2image4tpu"                                                                        # SET TRACE BACK TO THIS FUNCTION!
        method2print = "trace2image4tpu"   
        message2print = "{(Step 3Z2/33) failed" + "; simulation-of-tpu-trace-failed-by-trace2image4tpu: " + str(task4trace2tpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                    
        code2nack = code2nack + '3Z2-' 
        
    try:
        # =================================
        # NORMAL TRACE
        # =================================                     
        class2print = "trace2image4tpu"                                                                        # SET TRACE BACK TO THIS FUNCTION!
        method2print = "trace2image4tpu"                
        message2print = "{(Step 3D/33) performed" + "; tpu-trace-performed-by-trace2image4tpu: " + str(task4trace2tpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # { trace2image4tpu : trace2image4tpu ; (Step 3D/33) performed; tpu-trace-performed-by-trace2image4tpu: 
        # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4tpu-failed {}; time: 23:41:37 09/06/11 Pacific Daylight Time}', 
        # 'CLASS': 'trace2image4tpu', 'METHOD': 'trace2image4tpu'}; time: 23:41:37 09/06/11 Pacific Daylight Time}

        code2ack = code2ack + '3D-' 
    except:
        message2print = "{(Step 3Z3/33) failed" + "; tpu-trace-failed-by-trace2image4tpu: " + str(task4trace2tpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                    
        code2nack = code2nack + '3Z3-' 

    # -----------------------------------------------------------------------------------------
    # (Step 33/3)           
    # -----------------------------------------------------------------------------------------
    message2print = "{(Step 33/33) returning" + "; tpu2trace4xi-returning-by-trace2image4tpu: " + str(tpu2trace4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4tpu-failed {}; time: 23:41:37 09/06/11 Pacific Daylight Time}', 
    #  'CLASS': 'trace2image4tpu', 
    #  'METHOD': 'trace2image4tpu'}; time: 23:41:37 09/06/11 Pacific Daylight Time}

    
    return tpu2trace4xi
            
#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "set2imagecallback4tpu" usage = "
#
#   print('(Step 14A/700) OF TPU2IMAGE4XI5TEST - TRYING TO SET 2D IMAGE CALLBACK OF TPU DEVICE ', time4xi())  
#   try:                               
#       response4setcallback2image = set2imagecallback4tpu()                                                                                                 
#       print('(Step 14B/700) OF TPU2IMAGE4XI5TEST - SET 2D IMAGE CALLBACK OF TPU DEVICE TO: ', response4setcallback2image)   
#       # <function trace2image4tpu at 0x0000000004E2CAC8>
#   except:
#       print('(Step 14Z1/700) OF TPU2IMAGE4XI5TEST *** FAILED TO SET 2D IMAGE CALLBACK OF TPU DEVICE ', time4xi())  
# 
#">

# ------------------------------------------------------------------------------------------------- 
 
def set2imagecallback4tpu(task4set2imagecallback=None):

    '''
    registers callback function to tpu2image4xi
    '''
    global tpu2image4xi
    
    class2print = "set2imagecallback4tpu" 
    method2print = "set2imagecallback4tpu"
    
    message2print = "{(Step 1A/70) started" + "; set2imagecallback4tpu-started: " + str(set2imagecallback4tpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # <function set2imagecallback4tpu at 0x000000000390B748>
    
    try:
    
        tpu2image4xi ['CALLBACK'] = trace2image4tpu
        
        message2print = "{(Step 1B/70) set callback2tpu" + "; imagecallback4tpu-set-by-set2imagecallback4tpu: " + str(tpu2image4xi ['CALLBACK']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <function trace2image4tpu at 0x0000000003A9E348> 
        
        message2print = "{(Step 1C/70) returning" + "; callback-of-tpu2image4xi-returning-by-set2imagecallback4tpu: " + str(tpu2image4xi ['CALLBACK']) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print) 
        # <function trace2image4tpu at 0x0000000003864A48> 
        
        return tpu2image4xi ['CALLBACK']      
        
    except:
        message2print = "{(Step 1Z1/70) failed" + "; callback2image-failed-by-set2imagecallback4tpu: " + str(callback2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         

        message2print = "{(Step 1Z2/70) returning" + "; trace2image4tpu-returning-by-set2imagecallback4tpu: " + str(trace2image4tpu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)      
            
        return trace2image4tpu
            
#</function4xi> 




# -------------------------------------------------------------------------------------------------
#<function4xi function = "create2image4tpu" usage = "
#   #1     
#   print('=' *33)          
#   print('(Step 2A/700) OF TPU2IMAGE4XI5TEST - TRYING TO CREATE 2D IMAGE4TPU ', time4xi())      
#   print('=' *33)        
#   # 
#   try:
#       response4create2Dimage = create2image4tpu()                                                                 
#       print('(Step 2B/700) OF TPU2IMAGE4XI5TEST - CREATED 2D IMAGE4TPU ', time4xi())
#       keys2response = response4create2Dimage.keys()
#       for key2response in sorted(keys2response):
#           print(key2response, ' : ', response4create2Dimage[key2response])
#           # 
#   except:
#       print('(Step 2Z1/700) OF TPU2IMAGE4XI5TEST *** FAILED TO CREATE 2D IMAGE4TPU *** ', time4xi())
#   #2
#   print('(Step 123A/700) OF TPU2IMAGE4XI5TEST - TRYING TO CREATE 2D IMAGE for TPU ', time4xi()) 
#   # 
#   try:
#       xi2tpu = TPU2Image4Xi()                            
#       response4create2Dimage = xi2tpu.create2image()                                  
#       keys2response = response4create2Dimage.keys()
#       print('(Step 122B/700) OF TPU2IMAGE4XI5TEST - CREATED 2D IMAGE ', time4xi())  
#       for key2response in sorted(keys2response):
#           print(key2response, ' : ', response4create2Dimage[key2response])
#           # 
#   except:
#       print('(Step 123Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO CREATE 2D IMAGE for TPU ', time4xi())    
#">

# -------------------------------------------------------------------------------------------------   
def create2image4tpu(task4imagecreate2tpu=None):

    '''
    creates 2D image for TPU
    '''
    
    global tpu2image4xi                                                                                              
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "create2image4tpu" 
    method2print = "create2image4tpu"

    message2print = "{(Step 1A/70) started" + "; create2image4tpu-started: " + str(create2image4tpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # <function create2image4tpu at 0x00000000038D97C8> 
  
             
    message2print = "{(Step 1B/70) passed" + "; task4imagecreate2tpu-to-create2image4tpu: " + str(task4imagecreate2tpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None 

    code2ack  = ' '
    code2nack = ' '    
    
    tpu2image4xi = {}                                                                                               # INITIALIZES TO EMPTY DICTIONARY
               
    try:             
        if task4imagecreate2tpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR CREATION - BUILD TASK TO TPU! 
            # -----------------------------------------------------------------------------------------    
            
            task4imagecreate2tpu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1C/70) initialized" + "; task4imagecreate2tpu-created-by-create2image4tpu to: " + str(task4imagecreate2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {}
            
            function2callback4tpu = trace2image4tpu                                                                 # CALLBACK FUNCTION FOR TPU
            
            message2print = "{(Step 1D/70) initialized" + "; function2callback4tpu-created-by-create2image4tpu to: " + str(function2callback4tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # <function trace2image4tpu at 0x00000000038D9348>
            
            code2ack  = code2ack + '1D-'
           
            args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "create2image4tpu"                                                          # MODULE OF TPUIZED OPENCL MODULE
            args2callback['METHOD']  = "create2image4tpu"                                                          # FUNCTION OF TPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; create2image4tpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR TPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1E/70) initialized" + "; args2callback-created-by-create2image4tpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {'MESSAGE': '{(Step 1Z1/33) failed; create2image4tpu-failed ; time: 00:54:15 09/06/11 Pacific Daylight Time}',
            #  'CLASS': 'create2image4tpu',
            #  'METHOD': 'create2image4tpu'}; time: 00:54:15 09/06/11 Pacific Daylight Time}

            code2ack  = code2ack + '1E-'
            
            # ---------------------------------------------------------------------------------------------
            # (1) CALLBACKS - EXTRA                      
            # ---------------------------------------------------------------------------------------------              
            task4imagecreate2tpu['DATA2CALLBACK'] = args2callback                                                   # USER DATA PASSED TO TPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagecreate2tpu['FUNCTION2CALLBACK'] = function2callback4tpu                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            code2ack  = code2ack + '2C-'
            message2print = "{(Step 1F/70) initialized" + "; task4imagecreate2tpu-created-by-create2image4tpu to: " + str(task4imagecreate2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # {'FUNCTION2CALLBACK': <function trace2image4tpu at 0x00000000038D9348>,
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4tpu-failed ; time: 00:54:15 09/06/11 Pacific Daylight Time}',
            #  'CLASS': 'create2image4tpu',
            #  'METHOD': 'create2image4tpu'}}; time: 00:54:15 09/06/11 Pacific Daylight Time}
            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - EXTRA                      
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-create2image4tpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # {}
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-create2image4tpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 00:54:15 09/06/11 Pacific Daylight Time}
                 
                code2ack  = code2ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-create2image4tpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '2Z-'
                 
            task4imagecreate2tpu['PLATFORM'] = platform2xi                                                          # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - REQUIRED                         
            # ---------------------------------------------------------------------------------------------    
            context2tpu = {}
            message2print = "{(Step 3A/70) trying" + "; context2tpu-trying-by-create2image4tpu: " + str(context2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2tpu = get2context4tpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context2tpu-got-by-create2image4tpu: " + str(context2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'COUNT2REFERENCE': 0,
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4tpu-failed ; time: 00:54:15 09/06/11 Pacific Daylight Time}',
                #  'CLASS': 'create2context4tpu',
                #  'METHOD': 'create2context4tpu'},
                #  'FUNCTION2CALLBACK': <function trace2context4tpu at 0x00000000037D0148>,
                #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
                #  'INFO2PRIVATE': None,
                #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1},
                #  'NACK2RETURN': ' ',
                #  'SIZE2PRIVATE': 0, 'PROPERTIES': None}; time: 00:54:15 09/06/11 Pacific Daylight Time}

                
                code2ack  = code2ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context2tpu-failed-by-create2image4tpu: " + str(context2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '3Z-'
                
            task4imagecreate2tpu ['CONTEXT'] = context2tpu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - EXTRA                            
            # ---------------------------------------------------------------------------------------------    
            device2tpu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device2tpu-trying-by-create2image4tpu: " + str(device2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2tpu = get2tpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device2tpu-got-by-create2image4tpu: " + str(device2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'RESOLUTION2TIMER4PROFILING': 318,
                #  'ID2PLATFORM': '0000000001509568',
                #  'PROFILING2QUEUE': 'ACK',
                #  'EXECUTE2KERNELS4OPENCL': 'ACK',
                #  'NATIVE2WIDTHLONG4VECTOR': 2,
                #  'MAX2UNITS4COMPUTE': 4,
                #  'SIZE2MEMORY4GLOBAL': 3221225472,
                #  'NATIVE2WIDTHFLOAT4VECTOR': 4,
                #  'MAX2SIZE4WORKGROUP': 1024,
                #  'PREFERRED2WIDTHCHAR4VECTOR': 16,
                #  'SIZE2CACHE': 65536,
                #  'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK',
                #  'TYPE4CACHE': 'Read/Write',
                #  'PREFERRED2WIDTHINT4VECTOR': 4,
                #  'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1,
                #  'NATIVE2WIDTHINT4VECTOR': 4,
                #  'SIZE2BUFFER4CONSTANT': 65536,
                #  'EXECUTE2FUNCTION4NATIVE': 'ACK',
                #  'PREFERRED2WIDTHDOUBLE4VECTOR': 0,
                #  'MAX2NUMBER4CONSTANTARGS': 8,
                #  'VERSION2DRIVER': '2.0',
                #  'EXTENSIONS': 'cl_amd_fp64 cl_khr_global_int32_base_atomics cl_khr_global_int32_extended_atomics cl_khr_local_int32_base_atomics 
              
                #  'AVAILABLE2COMPILER': 'ACK',
                #  'MAX2DIMENSIONS4WORKITEMS': 3,
                #  'SUPPORT2CORRECTION4ERROR': 0,
                #  'BITS2ADDRESS': 64,
                #  'AVAILABLE': 'ACK',
                #  'SIZE2MEMORY4LOCAL': 32768,
                #  'VENDOR': 'AuthenticAMD',
                #  'SUPPORT2IMAGE': 'NACK',
                #  'MAX2SIZE4KERNELARGUMENT': 4096,
                #  'ACK2RETURN': ' 1C-1D-1E-1F-1D-2A-
                #  'OUT-OF-ORDER2QUEUE': 'NACK',
                #  'ALIGNMENT2ADDRESS4BASE': 1024,
                #  'PROFILE': 'FULL_PROFILE',
                #  'TYPE2MEMORY4LOCAL': 'Global',
                #  'PREFERRED2WIDTHFLOAT4VECTOR': 4,
                #  'PREFERRED2WIDTHLONG4VECTOR': 2,
                #  'MAX2FREQUENCY4CLOCK': 3210,
                #  'PREFERRED2WIDTHSHORT4VECTOR': 8,
                #  'ID': 4033,
                #  'MAX2WORKITEM2': 1024,
                #  'NATIVE2WIDTHDOUBLE4VECTOR': 0,
                #  'ROUND2ZERO': 'ACK',
                #  'NAME': 'AMD64 Family 16 Model 4 Stepping 2, AuthenticAMD',
                #  'MAX2WORKITEM1': 1024,
                #  'MAX2WORKITEM0': 1024,
                #  'SIZE2LINE4CACHE': 64,
                #  'MAX2ALLOCATION4MEMORY': 1073741824,
                #  'NATIVE2WIDTHCHAR4VECTOR': 16,
                #  'QUIET2NANS': 'ACK',
                #  'NATIVE2WIDTHSHORT4VECTOR': 8,
                #  'ROUND2NEAREST4EVEN': 'ACK',
                #  'VERSION': 'OpenCL 1.1 ATI-Stream-v2.3 (451)',
                #  'NACK2RETURN': ' ',
                #  'MEMORY2UNIFIED4HOSTANDDEVICE': 1,
                #  'ENDIANESS2DEVICE': 'Little',
                #  'MINIMUM2ALIGNMENT4DATATYPE': 128,
                #  'ROUND2+VE3INFINITY': 'ACK',
                #  'TYPE': 'CL_DEVICE_TYPE_TPU',
                #  'DENORMS2SINGLEPRECISION4FLOATINGPOINTCAPABILITY': 'ACK'}; time: 00:54:15 09/06/11 Pacific Daylight Time}

                
                code2ack  = code2ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device2tpu-failed-by-create2image4tpu: " + str(device2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '4Z-'
                
            task4imagecreate2tpu ['DEVICE'] = device2tpu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) ERROR CODE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            task4imagecreate2tpu ['CODE2ERROR'] = 0             
            code2ack  = code2ack + '5B-'              

            # ---------------------------------------------------------------------------------------------
            # (6) QUEUE - EXTRA                          
            # ---------------------------------------------------------------------------------------------         
            queue2tpu = None
            
            message2print = "{(Step 6A/70) trying" + "; queue2tpu-trying-by-create2image4tpu: " + str(queue2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2tpu = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 6B/70) got" + "; queue2tpu-got-by-create2image4tpu: " + str(queue2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # <queue4xi.Queue4Xi object at 0x0000000003991B70> 
                
                code2ack  = code2ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; queue2tpu-failed-by-create2image4tpu: " + str(queue2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '6Z-'
                
            task4imagecreate2tpu ['QUEUE'] = queue2tpu

            
            # ---------------------------------------------------------------------------------------------
            # (7) PIPE - EXTRA                              
            # ---------------------------------------------------------------------------------------------         
            pipe2tpu = None
           
            message2print = "{(Step 7A/70) trying" + "; pipe2tpu-trying-by-create2image4tpu: " + str(pipe2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2tpu = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 7B/70) got" + "; pipe2tpu-got-by-create2image4tpu: " + str(pipe2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # (<read-write PipeConnection, handle 388>, <read-write PipeConnection, handle 392>)
                
                code2ack  = code2ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; pipe2tpu-failed-by-create2image4tpu: " + str(pipe2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '7Z-'
                
            task4imagecreate2tpu ['PIPE'] = pipe2tpu 
            
            # ---------------------------------------------------------------------------------------------
            # (8) COMMAND QUEUE INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2tpu = {}
           
            message2print = "{(Step 8A/70) trying" + "; commandqueueinfo2tpu-trying-by-create2image4tpu: " + str(commandqueueinfo2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2tpu = get2commandqueueinfo4tpu()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 8B/70) got" + "; commandqueueinfo2tpu-got-by-create2image4tpu: " + str(commandqueueinfo2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0,
                #  'COUNT2REFERENCE': 0,
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4tpu-failed ; time: 05:04:51 09/06/11 Pacific Daylight Time}',
                #  'CLASS': 'create2commandqueue4tpu',
                #  'METHOD': 'create2commandqueue4tpu'},
                #  'FUNCTION2CALLBACK': <function trace2commandqueue4tpu at 0x000000000372C848>,
                #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-8A-9B-10B-9B-10B-11B-12B-13B-14B-15B-16B-17B-18B-',
                #  'INFO2PRIVATE': None,
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003A7A6D8>,
                #  'NACK2RETURN': ' ',
                #  'CONTEXT': {'COUNT2REFERENCE': 0 ... 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4tpu-failed ; time: 05:04:51 09/06/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2context4tpu', 
                #  'METHOD': 'create2context4tpu'}, 
                #  'FUNCTION2CALLBACK': <function trace2context4tpu at 0x00000000036E0148>, 
                #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 
                #  'INFO2PRIVATE': None, 
                #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
                #  'NACK2RETURN': ' ', 
                #  'SIZE2PRIVATE': 0, 
                #  'PROPERTIES': None}, 
                #  'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318, ...
                
                code2ack  = code2ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; commandqueueinfo2tpu-failed-by-create2image4tpu: " + str(commandqueueinfo2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '8Z-'
        
            task4imagecreate2tpu ['COMMANDQUEUE'] = commandqueueinfo2tpu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (9) BUFFER INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2tpu = {}
           
            message2print = "{(Step 9A/70) trying" + "; bufferinfo2tpu-trying-by-create2image4tpu: " + str(bufferinfo2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2tpu = get2bufferinfo4tpu()                                                                   # SYSTEM CREATES BUFFER FOR TPU, IF NECESSARY
                message2print = "{(Step 9B/70) got" + "; bufferinfo2tpu-got-by-create2image4tpu: " + str(bufferinfo2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0,
                #  'COUNT2REFERENCE': 0,
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4tpu-failed ; time: 05:04:51 09/06/11 Pacific Daylight Time}',
                #                    'CLASS': 'create2buffer4tpu',
                #                    'METHOD': 'create2buffer4tpu'},
                #  'FUNCTION2CALLBACK': <function trace2buffer4tpu at 0x0000000003B2B348>,
                #  'POINTER2BUFFER': <mmap.mmap object at 0x0000000003ADD2F0>,
                #  'FLAGS': 0,
                #  'CONTEXT': {'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4tpu-failed ; time: 05:04:51 09/06/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2context4tpu', 
                #  'METHOD': 'create2context4tpu'}, 
                #  'FUNCTION2CALLBACK': <function trace2context4tpu at 0x00000000036E0148>, 
                #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 
                #  'INFO2PRIVATE': None, 
                #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
                #  'NACK2RETURN': ' ', 
                #  'SIZE2PRIVATE': 0, 
                #  'PROPERTIES': None}, 
                #  'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318, ...
                
                
                code2ack  = code2ack + '9B-' 
            except:
                message2print = "{(Step 9Z/70) failed" + "; bufferinfo2tpu-failed-by-create2image4tpu: " + str(bufferinfo2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '9Z-'
        
            task4imagecreate2tpu ['BUFFER'] = bufferinfo2tpu 
            
            # ---------------------------------------------------------------------------------------------
            # (10) EVENT INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
            eventinfo2tpu = {}
           
            message2print = "{(Step 10A/70) trying" + "; eventinfo2tpu-trying-by-create2image4tpu: " + str(eventinfo2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2tpu = get2eventinfo4tpu()                                                                     # SYSTEM CREATES EVENT IF NECESSAY   
                message2print = "{(Step 10B/70) got" + "; eventinfo2tpu-got-by-create2image4tpu: " + str(eventinfo2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0,
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4tpu-failed ; time: 05:10:51 09/06/11 Pacific Daylight Time}',
                #                    'CLASS': 'create2event4tpu',
                #                    'METHOD': 'create2event4tpu'},
                #  'FUNCTION2CALLBACK': <function trace2event4tpu at 0x0000000003747D48>,
                #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-6B-7B-8B-9B-10B-11B-12B-14B-15B-16B-',
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003A02898>,
                #  'NACK2RETURN': ' ',
                #  'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4tpu-failed ; time: 05:10:51 09/06/11 Pacific Daylight Time}', 
                #                                                      'CLASS': 'create2context4tpu', 'METHOD': 'create2context4tpu'}, 
                #  'FUNCTION2CALLBACK': <function trace2context4tpu at 0x00000000036F0148>,
                #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
                #  'INFO2PRIVATE': None,
                #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
                #  'NACK2RETURN': ' ',
                #  'SIZE2PRIVATE': 0,
                #  'PROPERTIES': None},
                #  'COMMANDQUEUE': {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': ...
                
                
                code2ack  = code2ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; eventinfo2tpu-failed-by-create2image4tpu: " + str(eventinfo2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '10Z-'
        
            task4imagecreate2tpu ['EVENT'] = eventinfo2tpu             
            
            # ---------------------------------------------------------------------------------------------
            # (11) FLAGS - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            flags2tpu = 0
           
            message2print = "{(Step 11A/70) trying" + "; flags2tpu-trying-by-create2image4tpu: " + str(flags2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                flags2tpu = 0                                                              
                message2print = "{(Step 11B/70) set" + "; flags2tpu-set-by-create2image4tpu to: " + str(flags2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; flags2tpu-failed-by-create2image4tpu: " + str(flags2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '11Z-'
        
            task4imagecreate2tpu ['FLAGS'] = flags2tpu             
            
            # ---------------------------------------------------------------------------------------------
            # (14) WIDTH - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            width2tpu = 0

            quantity2core = 1                                                                      # ORIGIN 
            quantity2skeleton = 1                                                                  # SKELETON WITH DISTANCE 1
            quantity2layers = 33                                                                   # LAYERS 10, 100 ... 900, 1000 ... 3300
            quantity2frame = 1                                                                     # FRAME
            quantity2extra = 4                                                                     # CONTROLS FOR GUI ETC...
            width2layer = 10                                                                       # SCALED TO 10 EACH LAYER            
            
            message2print = "{(Step 14A/70) trying" + "; width2tpu-trying-by-create2image4tpu: " + str(width2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                width2tpu = int(width2layer * (quantity2core + quantity2skeleton + quantity2layers + quantity2frame + quantity2extra))
                width2tpu = 2 * width2tpu                                                          # POS and NEG X  2*400 = 800 PIXELS
                                                                                                                                         
                message2print = "{(Step 14B/70) set" + "; width2tpu-set-by-create2image4tpu to: " + str(width2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 800 
                code2ack  = code2ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; width2tpu-failed-by-create2image4tpu: " + str(width2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '14Z-'
        
            task4imagecreate2tpu ['WIDTH'] = width2tpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (15) HEIGHT - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            height2tpu = 0
            quantity2core = 1                                                                      # ORIGIN 
            quantity2skeleton = 1                                                                  # SKELETON WITH DISTANCE 1
            quantity2layers = 33                                                                   # LAYERS 10, 100 ... 900, 1000 ... 3300
            quantity2frame = 1                                                                     # FRAME
            quantity2extra = 4                                                                     # CONTROLS FOR GUI
            
            quantity2total = quantity2core + quantity2skeleton + quantity2layers + quantity2frame + quantity2extra
            height2layer = 10                                                                      # SCALED TO 10 
                       
            message2print = "{(Step 15A/70) trying" + "; height2tpu-trying-by-create2image4tpu: " + str(height2tpu)  + "; time: " + time4xi() + "}"  
            print4xi(class2print, method2print, message2print) 
                        
            try:
                height2tpu = height2layer * quantity2total
                
                height2tpu = 2 * height2tpu                                                        # POS and NEG Y   2*400 = 800 PIXELS
                                                                                                                                         
                message2print = "{(Step 15B2/70) set" + "; height2tpu-set-by-create2image4tpu to: " + str(height2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 800 
                code2ack  = code2ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; height2tpu-failed-by-create2image4tpu: " + str(height2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '15Z-'
             
                        
            task4imagecreate2tpu ['HEIGHT'] = height2tpu 
            
            # ---------------------------------------------------------------------------------------------
            # (16) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.


            rowpitch2tpu = 0
            size2element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 1                                                                                  # ONE DIRECTIONS IN 2D
            
            message2print = "{(Step 16A/70) trying" + "; rowpitch2tpu-trying-by-create2image4tpu: " + str(rowpitch2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2tpu = int(width2tpu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 16B/70) set" + "; rowpitch2tpu-set-by-create2image4tpu to: " + str(rowpitch2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 800                                                                                             
                 
                code2ack  = code2ack + '16B-' 
            except:
                message2print = "{(Step 16Z/70) failed" + "; rowpitch2tpu-failed-by-create2image4tpu: " + str(rowpitch2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '16Z-'
        
            task4imagecreate2tpu ['ROWPITCH'] = rowpitch2tpu     
            
            # ---------------------------------------------------------------------------------------------
            # (17) POINTER - REQUIRED                           
            # ---------------------------------------------------------------------------------------------             
            pointer2tpu = None
            size2tpu = int(width2tpu * height2tpu)                                                                  # CALCULATED MEMORY
            if size2tpu == 0:
                size2tpu = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 17A/70) trying" + "; pointer2tpu-trying-by-create2image4tpu: " + str(pointer2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena2tpu = Arena4Xi(size2tpu)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 17B1/70) got" + "; arena2tpu-got-by-create2image4tpu: " + str(arena2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # <arena4xi.Arena4Xi object at 0x00000000041C0D68>
                
                size2arena = arena2tpu.size                                                                         # 
                              
                message2print = "{(Step 17B2/70) got" + "; size2arena-got-by-create2buffer4tpu: " + str(size2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 640000 
                
                buffer2arena = arena2tpu.buffer
                
                message2print = "{(Step 17B3/70) got" + "; buffer2arena-got-by-create2buffer4tpu: " + str(buffer2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # <mmap.mmap object at 0x0000000003AAF298> 
                
                code2ack  = code2ack + '17B-' 
                
                if size2arena < size2tpu:
                    size2tpu = size2arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 17C/70) set" + "; size2tpu-set-by-create2buffer4tpu to: " + str(size2tpu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code2ack  = code2ack + '17C-'
                else:
                    pass
                    
                pointer2tpu = buffer2arena
                message2print = "{(Step 17D/70) set" + "; pointer2tpu-set-by-create2buffer4tpu to: " + str(pointer2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # <mmap.mmap object at 0x0000000003AAF298>
                
                code2ack  = code2ack + '17D-'       
            except:
                message2print = "{(Step 17Z/70) failed" + "; pointer2tpu-failed-by-create2image4tpu: " + str(pointer2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '17Z-'
        
            task4imagecreate2tpu ['POINTER'] = pointer2tpu                            
            task4imagecreate2tpu ['SIZE']    = size2tpu                                                             # EXTRA
            
            
            # ---------------------------------------------------------------------------------------------
            # (18) FORMAT - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            format2tpu = {}
           
            message2print = "{(Step 18A/70) trying" + "; format2tpu-trying-by-create2image4tpu: " + str(format2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                format2tpu['ORDER'] = 'CL_RGBA'                                                              
                message2print = "{(Step 18B/70) set" + "; format2tpu-set-by-create2image4tpu to: " + str(format2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'ORDER': 'CL_RGBA'}
                 
                code2ack  = code2ack + '18B-'
                
                format2tpu['TYPE'] = 'CL_UNORM_INT8'                                                              
                message2print = "{(Step 18C/70) set" + "; format2tpu-set-by-create2image4tpu to: " + str(format2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'}
                
                code2ack  = code2ack + '18C-'                
                 
            except:
                message2print = "{(Step 18Z/70) failed" + "; format2tpu-failed-by-create2image4tpu: " + str(format2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '18Z-'
        
            task4imagecreate2tpu ['FORMAT'] = format2tpu                  
            
            # ---------------------------------------------------------------------------------------------
            # (19) DEPTH - MUST BE 0                               
            # ---------------------------------------------------------------------------------------------             
            depth2tpu = 0
            message2print = "{(Step 19A/70) set" + "; depth2tpu-set-by-create2image4tpu to: " + str(depth2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #  
            code2ack  = code2ack + '19A-' 
            task4imagecreate2tpu ['DEPTH'] = depth2tpu
                                                                                                                                       
        else:
            pass                                                                                                    # PARAMETER DATA RECEIVED                       
    except: 
        message2print = "{(Step 1Z1/70) yielded" + "; task4imagecreate2tpu-yielded-by-create2image4tpu: " + str(task4imagecreate2tpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '1Z1-' 

    # ---------------------------------------------------------------------------------------------
    # TASK EITHER GENERATED BY THIS PROGRAM (NONE GIVEN) OR GIVEN BY USER    
    # ---------------------------------------------------------------------------------------------
    keys2task = task4imagecreate2tpu.keys()        
    message2print = "{(Step 19A/70) using" + "; keys2task-using-by-create2image4tpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    # ['BUFFER',
    #  'CODE2ERROR',
    #  'COMMANDQUEUE',
    #  'CONTEXT',
    #  'DATA2CALLBACK',
    #  'DEVICE',
    #  'EVENT',
    #  'FLAGS',
    #  'FORMAT',
    #  'FUNCTION2CALLBACK',
    #  'HEIGHT',
    #  'PIPE',
    #  'PLATFORM',
    #  'POINTER',
    #  'QUEUE',
    #  'ROWPITCH',
    #  'SIZE',
    #  'WIDTH']; time: 00:53:55 10/05/11 Pacific Daylight Time}
  
    
    # ---------------------------------------------------------------------------------------------
    # (Step 20A/70) - CREATE 2D IMAGE FOR TPU! 
    # ---------------------------------------------------------------------------------------------    
    tpu2image4xi = {}                                                                                                                                                                               
    message2print = "{(Step 20A/70) initialized" + "; tpu2image4xi-initialized-by-create2image4tpu to: " + str(tpu2image4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # {} 
    
    message2print = "{(Step 21A0/70) try" + "; data2callback-try-by-create2image4tpu: {}" + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)      
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 21A/70) - DATA2CALLBACK - EXTRA 
        # -----------------------------------------------------------------------------------------               
        tpu2image4xi['DATA2CALLBACK'] = task4imagecreate2tpu['DATA2CALLBACK']
        
        message2print = "{(Step 21A1/70) initialized" + "; data2callback-created-by-create2image4tpu to: " + str(tpu2image4xi['DATA2CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'MESSAGE': '{(Step 1Z1/33) failed; create2image4tpu-failed ; time: 05:56:13 09/06/11 Pacific Daylight Time}',
        #  'CLASS': 'create2image4tpu',
        #  'METHOD': 'create2image4tpu'}; time: 05:56:13 09/06/11 Pacific Daylight Time}

        
        code2ack  = code2ack + '21A1-'
    except: 
        message2print = "{(Step 21Z1/70) yielded" + "; data2callback-tpu2image4xi-yielded-by-create2image4tpu: " + str(tpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '21Z1-'
    
    message2print = "{(Step 22B0/70) try" + "; function2callback-try-by-create2image4tpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22B/70) - FUNCTION2CALLBACK - EXTRA
        # -----------------------------------------------------------------------------------------               
        tpu2image4xi['FUNCTION2CALLBACK'] = task4imagecreate2tpu['FUNCTION2CALLBACK']
        
        message2print = "{(Step 22B1/70) initialized" + "; function2callback-created-by-create2image4tpu to: " + str(tpu2image4xi['FUNCTION2CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # <function trace2image4tpu at 0x0000000003B04348>  
        code2ack  = code2ack + '22B1-'
    except: 
        message2print = "{(Step 22Z2/70) yielded" + "; function2callback-tpu2image4xi-yielded-by-create2image4tpu: " + str(tpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '22Z2-' 
    
    message2print = "{(Step 22C0/70) try" + "; platform-try-by-create2image4tpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22C/70) - PLATFORM - EXTRA
        # -----------------------------------------------------------------------------------------               
        tpu2image4xi['PLATFORM'] = task4imagecreate2tpu['PLATFORM']
        
        message2print = "{(Step 22C1/70) initialized" + "; platform-created-by-create2image4tpu to: " + str(tpu2image4xi['PLATFORM'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 05:56:13 09/06/11 Pacific Daylight Time}
  
        code2ack  = code2ack + '22C1-'
    except: 
        message2print = "{(Step 22Z3/70) yielded" + "; platform-tpu2image4xi-yielded-by-create2image4tpu: " + str(tpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '22Z3-' 
        
        
      
    message2print = "{(Step 23A/70) try" + "; context2tpu-try-by-create2image4tpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)       
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 23/70) - CONTEXT - REQUIRED
        # -----------------------------------------------------------------------------------------               
        tpu2image4xi['CONTEXT'] = task4imagecreate2tpu['CONTEXT']
        
        message2print = "{(Step 23B/70) initialized" + "; context-created-by-create2image4tpu to: " + str(tpu2image4xi['CONTEXT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'COUNT2REFERENCE': 0,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4tpu-failed ; time: 05:56:12 09/06/11 Pacific Daylight Time}', 
        #  'CLASS': 'create2context4tpu',
        #  'METHOD': 'create2context4tpu'},
        #  'FUNCTION2CALLBACK': <function trace2context4tpu at 0x00000000036B0148>,
        #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
        #  'INFO2PRIVATE': None,
        #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
        #  'NACK2RETURN': ' ',
        #  'SIZE2PRIVATE': 0,
        #  'PROPERTIES': None}; time: 05:56:13 09/06/11 Pacific Daylight Time}

        
        code2ack  = code2ack + '23B-'
    except: 
        message2print = "{(Step 23Z1/70) yielded" + "; context-tpu2image4xi-yielded-by-create2image4tpu: " + str(tpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
    
        code2nack  = code2nack + '23Z1-' 
        
    message2print = "{(Step 24A/70) try" + "; device-try-by-create2image4tpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 24/70) - DEVICE - EXTRA
        # -----------------------------------------------------------------------------------------               
        tpu2image4xi['DEVICE'] = task4imagecreate2tpu['DEVICE']
        
        message2print = "{(Step 24B/70) initialized" + "; device-created-by-create2image4tpu to: " + str(tpu2image4xi['DEVICE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'RESOLUTION2TIMER4PROFILING': 318,
        #  'ID2PLATFORM': '0000000001509568',
        #  'PROFILING2QUEUE': 'ACK',
        #  'EXECUTE2KERNELS4OPENCL': 'ACK',
        #  'NATIVE2WIDTHLONG4VECTOR': 2,
        #  'MAX2UNITS4COMPUTE': 4,
        #  'SIZE2MEMORY4GLOBAL': 3221225472,
        #  'NATIVE2WIDTHFLOAT4VECTOR': 4,
        #  'MAX2SIZE4WORKGROUP': 1024,
        #  'PREFERRED2WIDTHCHAR4VECTOR': 16,
        #  'SIZE2CACHE': 65536,
        #  'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK',
        #  'TYPE4CACHE': 'Read/Write',
        #  'PREFERRED2WIDTHINT4VECTOR': 4,
        #  'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1,
        #  'NATIVE2WIDTHINT4VECTOR': 4,
        #  'SIZE2BUFFER4CONSTANT': 65536,
        #  'EXECUTE2FUNCTION4NATIVE': 'ACK',
        #  'PREFERRED2WIDTHDOUBLE4VECTOR': 0,
        #  'MAX2NUMBER4CONSTANTARGS': 8,
        #  'VERSION2DRIVER': '2.0', 'EXTENSIONS': 'cl_amd_fp64 ...
        
        
        code2ack  = code2ack + '24B-'
    except: 
        message2print = "{(Step 24Z1/70) yielded" + "; device-tpu2image4xi-yielded-by-create2image4tpu: " + str(tpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '24Z1-'
                 
    message2print = "{(Step 25A/70) try" + "; code2error-try-by-create2image4tpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 25/70) - ERROR CODE - REQUIRED
        # -----------------------------------------------------------------------------------------               
        tpu2image4xi['CODE2ERROR'] = task4imagecreate2tpu['CODE2ERROR']
        
        message2print = "{(Step 25B/70) initialized" + "; code2error-created-by-create2image4tpu to: " + str(tpu2image4xi['CODE2ERROR'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0 
        code2ack  = code2ack + '25B-'
    except: 
        message2print = "{(Step 25Z1/70) yielded" + "; code2error-tpu2image4xi-yielded-by-create2image4tpu: " + str(tpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '25Z1-'                
                
          
    message2print = "{(Step 26A/70) try" + "; queue-try-by-create2image4tpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 26/70) - QUEUE - EXTRA
        # -----------------------------------------------------------------------------------------               
        tpu2image4xi['QUEUE'] = task4imagecreate2tpu['QUEUE']
        
        message2print = "{(Step 26B/70) initialized" + "; queue-created-by-create2image4tpu to: " + str(tpu2image4xi['QUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <queue4xi.Queue4Xi object at 0x0000000003ED99E8>  
        code2ack  = code2ack + '26B-'
    except: 
        message2print = "{(Step 26Z1/70) yielded" + "; queue-tpu2image4xi-yielded-by-create2image4tpu: " + str(tpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '26Z1-' 
                                
    message2print = "{(Step 27A/70) try" + "; pipe-try-by-create2image4tpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                     
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 27/70) - PIPE - EXTRA
        # -----------------------------------------------------------------------------------------               
        tpu2image4xi['PIPE'] = task4imagecreate2tpu['PIPE']
        
        message2print = "{(Step 27B/70) initialized" + "; pipe-created-by-create2image4tpu to: " + str(tpu2image4xi['PIPE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # (<read-write PipeConnection, handle 376>, <read-write PipeConnection, handle 380>)  
        
        code2ack  = code2ack + '27B-'
    except: 
        message2print = "{(Step 27Z1/70) yielded" + "; pipe-tpu2image4xi-yielded-by-create2image4tpu: " + str(tpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '27Z1-'   

    message2print = "{(Step 28A/70) try" + "; commandqueue-try-by-create2image4tpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (28) COMMAND QUEUE - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        tpu2image4xi['COMMANDQUEUE']  = task4imagecreate2tpu['COMMANDQUEUE']  
 
        message2print = "{(Step 28B/70) initialized" + "; commandqueue-created-by-create2image4tpu to: " + str(tpu2image4xi['COMMANDQUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0,
        #  'COUNT2REFERENCE': 0,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4tpu-failed ; time: 07:13:59 09/06/11 Pacific Daylight Time}',
        #  'CLASS': 'create2commandqueue4tpu',
        #  'METHOD': 'create2commandqueue4tpu'},
        #  'FUNCTION2CALLBACK': <function trace2commandqueue4tpu at 0x00000000037BC848>,
        #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-8A-9B-10B-9B-10B-11B-12B-13B-14B-15B-16B-17B-18B-',
        #  'INFO2PRIVATE': None,
        #  'QUEUE': <queue4xi.Queue4Xi object at 0x000000000432B3C8>,
        #  'NACK2RETURN': ' ',
        #  'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
        
        code2ack  = code2ack + '28B-'
    except: 
        message2print = "{(Step 28Z1/70) yielded" + "; commandqueue-yielded-by-create2image4tpu: " + str(tpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
              
        code2nack  = code2nack + '28Z1-'    

    message2print = "{(Step 29A/70) try" + "; bufferinfo-try-by-create2image4tpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (29) BUFFER INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        tpu2image4xi['BUFFER']  = task4imagecreate2tpu['BUFFER']  
 
        message2print = "{(Step 29B/70) initialized" + "; bufferinfo-created-by-create2image4tpu to: " + str(tpu2image4xi['BUFFER'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0,
        #  'COUNT2REFERENCE': 0,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4tpu-failed ; time: 05:56:12 09/06/11 Pacific Daylight Time}', 
        #  'CLASS': 'create2buffer4tpu',
        #  'METHOD': 'create2buffer4tpu'},
        #  'FUNCTION2CALLBACK': <function trace2buffer4tpu at 0x0000000003AFC348>,
        #  'POINTER2BUFFER': <mmap.mmap object at 0x0000000003AAF2F0>,
        #  'FLAGS': 0,
        #  'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4tpu-failed ...
        
        code2ack  = code2ack + '29B-'
    except: 
        message2print = "{(Step 29Z1/70) yielded" + "; bufferinfo-yielded-by-create2image4tpu: " + str(tpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '29Z1-'            

    message2print = "{(Step 30A/70) try" + "; eventinfo-try-by-create2image4tpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (30) EVENT INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        tpu2image4xi['EVENT']  = task4imagecreate2tpu['EVENT']  
 
        message2print = "{(Step 30B/70) initialized" + "; eventinfo-created-by-create2image4tpu to: " + str(tpu2image4xi['EVENT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4tpu-failed ; time: 05:56:12 09/06/11 Pacific Daylight Time}',
        #                     'CLASS': 'create2event4tpu',
        #                     'METHOD': 'create2event4tpu'},
        #  'FUNCTION2CALLBACK': <function trace2event4tpu at 0x0000000003707D48>,
        #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-6B-7B-8B-9B-10B-11B-12B-14B-15B-16B-',
        #  'QUEUE': <queue4xi.Queue4Xi object at 0x00000000039C3860>,
        #  'NACK2RETURN': ' ',
        #  'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
        
        code2ack  = code2ack + '30B-'
    except: 
        message2print = "{(Step 30Z1/70) yielded" + "; eventinfo-yielded-by-create2image4tpu: " + str(tpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '30Z1-'            
                

    message2print = "{(Step 31A/70) try" + "; flags-try-by-create2image4tpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (31) FLAGS - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        tpu2image4xi['FLAGS']  = task4imagecreate2tpu['FLAGS']  
 
        message2print = "{(Step 31B/70) initialized" + "; flags-created-by-create2image4tpu to: " + str(tpu2image4xi['FLAGS'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 0
        code2ack  = code2ack + '31B-'
    except: 
        message2print = "{(Step 31Z1/70) yielded" + "; flags-yielded-by-create2image4tpu: " + str(tpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '31Z1-'     

    message2print = "{(Step 34A/70) try" + "; width-try-by-create2image4tpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (34) WIDTH - REQUIRED                              
        # ---------------------------------------------------------------------------------------------      
        tpu2image4xi['WIDTH']  = task4imagecreate2tpu['WIDTH']  
 
        message2print = "{(Step 34H/70) initialized" + "; width-created-by-create2image4tpu to: " + str(tpu2image4xi['WIDTH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 800 
        code2ack  = code2ack + '34H-'
    except: 
        message2print = "{(Step 34Z1/70) yielded" + "; width-yielded-by-create2image4tpu: " + str(tpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '34Z1-'   


    message2print = "{(Step 35A/70) try" + "; height-try-by-create2image4tpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (35) HEIGHT - REQUIRED                              
        # ---------------------------------------------------------------------------------------------      
        tpu2image4xi['HEIGHT']  = task4imagecreate2tpu['HEIGHT']  
 
        message2print = "{(Step 35H/70) initialized" + "; height-created-by-create2image4tpu to: " + str(tpu2image4xi['HEIGHT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 800 
        code2ack  = code2ack + '35H-'
    except: 
        message2print = "{(Step 35Z1/70) yielded" + "; height-yielded-by-create2image4tpu: " + str(tpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '35Z1-'   
 

    message2print = "{(Step 36A/70) try" + "; rowpitch-try-by-create2image4tpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (36) ROWPITCH - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        tpu2image4xi['ROWPITCH']  = task4imagecreate2tpu['ROWPITCH']  
 
        message2print = "{(Step 36B/70) initialized" + "; rowpitch-created-by-create2image4tpu to: " + str(tpu2image4xi['ROWPITCH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 800
        code2ack  = code2ack + '36B-'
    except: 
        message2print = "{(Step 36Z1/70) yielded" + "; rowpitch-yielded-by-create2image4tpu: " + str(tpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '36Z1-'

    message2print = "{(Step 37A/70) try" + "; pointer-try-by-create2image4tpu: None"  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (37) POINTER - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        tpu2image4xi['POINTER']  = task4imagecreate2tpu['POINTER']  
 
        message2print = "{(Step 37B/70) initialized" + "; pointer-created-by-create2image4tpu to: " + str(tpu2image4xi['POINTER'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <mmap.mmap object at 0x0000000003C050E0>
         
        code2ack  = code2ack + '37B-'
    except: 
        message2print = "{(Step 37Z1/70) yielded" + "; pointer-yielded-by-create2image4tpu: " + str(tpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '37Z1-'

    message2print = "{(Step 38A/70) try" + "; format-try-by-create2image4tpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (38) FORMAT - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        tpu2image4xi['FORMAT']  = task4imagecreate2tpu['FORMAT']  
 
        message2print = "{(Step 38B/70) initialized" + "; format-created-by-create2image4tpu to: " + str(tpu2image4xi['FORMAT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'} 
        code2ack  = code2ack + '38B-'
    except: 
        message2print = "{(Step 38Z1/70) yielded" + "; format-yielded-by-create2image4tpu: " + str(tpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '38Z1-'

    message2print = "{(Step 39A/70) try" + "; depth-try-by-create2image4tpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (39) DEPTH - MUST BE 0                              
        # ---------------------------------------------------------------------------------------------             
                        
        tpu2image4xi['DEPTH']  = task4imagecreate2tpu['DEPTH']  
 
        message2print = "{(Step 39B/70) initialized" + "; depth-created-by-create2image4tpu to: " + str(tpu2image4xi['DEPTH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        #  
        code2ack  = code2ack + '39B-'
    except: 
        message2print = "{(Step 39Z1/70) yielded" + "; depth-yielded-by-create2image4tpu: " + str(tpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '39Z1-'
                                                                                                               
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    tpu2image4xi ['ACK2RETURN']  = code2ack
    tpu2image4xi ['NACK2RETURN'] = code2nack  
    
    message2print = "{(Step 70/70) returning" + "; tpu2image4xi-returning-by-create2image4tpu: " + str(tpu2image4xi) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    # 'DEPTH'
    # 'WIDTH'
    # 'HEIGHT'
    return tpu2image4xi
            
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "set2image4tpu" usage = "
#    
#   print('(Step 4B/33) OF TPU2IMAGE4XI5TEST - TRYING TO GET PLATFORMIDS4XI', time4xi())
#   platform2xi = {}
#   try:
#       platform2xi = get2platformids4xi()
#       print('(Step 4B/33) of CONTEXT4TPU5TEST - GOT PLATFORMIDS4XI', platform2xi) 
#   except:
#       print('(Step 4Z/33) of CONTEXT4TPU5TEST *** FAILED TO GET PLATFORMIDS4XI', time4xi())     
#   
#   print('(Step 2A/33) OF DEVICE4TPU5TEST - TRYING TO INITIALIZE DEVICE4TPU ', time4xi())   
#   print('=' *33)        
#   try:
#       device2tpu = init2tpu4xi()                                                                                
#       print('(Step 2B/33) OF DEVICE4TPU5TEST - INITIALIZED DEVICE4TPU TO: ', device2tpu)        
#   except:
#       print('(Step 2Z/33) OF DEVICE4TPU5TEST *** FAILED TO INITIALIZE DEVICE4TPU', time4xi()) 
#   print('(Step 3A/33) OF TPU2IMAGE4XI5TEST - TRYING TO CREATE CONTEXT4TPU ', time4xi())    
#   print('=' *33)        
#   try:
#       context2tpu = create2context4tpu()                                                                               
#       print('(Step 3B/33) OF TPU2IMAGE4XI5TEST - CREATED CONTEXT4TPU: ', context2tpu)                                  
#   except:
#       print('(Step 3Z/33) OF TPU2IMAGE4XI5TEST *** FAILED TO CREATE CONTEXT4TPU', time4xi()) 
#          
#   task4set2image = {}
#   task4set2image  ['CONTEXT'] = context2tpu  
# 
#   ... other data for set operation ...
#    
#        
#   response4set2Dimage = {} 
#   print('(Step 11A/700) OF TPU2IMAGE4XI5TEST - TRYING TO SET 2D IMAGE4TPU ', time4xi())   
#   try:                               
#       response4set2Dimage = set2image4tpu(task4set2image)                                                                                                         
#                                                                                     
#       print('(Step 11B/700) OF TPU2IMAGE4XI5TEST - SET 2D IMAGE4TPU ', time4xi())
#       keys2response = response4set2Dimage.keys()
#       for key2response in sorted(keys2response):
#           print(key2response, ' : ', response4set2Dimage[key2response])
#           # 
#   except:
#       print('(Step 11Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO SET 2D IMAGE4TPU *** ', time4xi()) 
#
#">

# -------------------------------------------------------------------------------------------------   
def set2image4tpu(task4imageset2tpu=None):

    '''
    sets 2D images for TPU
    '''
    global tpu2image4xi  
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "set2image4tpu"
    method2print = "set2image4tpu"

    message2print = "{(Step 1A/33) started" + "; set2image4tpu-started: " + str(set2image4tpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function set2image4tpu at 0x0000000003A62848> 
    
    message2print = "{(Step 1B/33) passed" + "; task4imageset2tpu-to-set2image4tpu: " + str(task4imageset2tpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # {'CODE2ERROR': 0,
    #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003D4A710>,
    #  'CONTEXT': {'COUNT2REFERENCE': 0,
    #              'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4tpu-failed ; time: 07:13:59 09/06/11 Pacific Daylight Time}',
    #              'CLASS': 'create2context4tpu',
    #              'METHOD': 'create2context4tpu'},
    #              'FUNCTION2CALLBACK': <function trace2context4tpu at 0x0000000003770148>,
    #              'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
    #              'INFO2PRIVATE': None,
    #              'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
    #              'NACK2RETURN': ' ',
    #              'SIZE2PRIVATE': 0,
    #              'PROPERTIES': None},
    #  'COMMANDQUEUE': {'CODE2ERROR': 0,
    #              'COUNT2REFERENCE': 0,
    #              'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4tpu-failed ; time: 07:13:59 09/06/11 Pacific Daylight Time}',
    #              'CLASS': 'create2commandqueue4tpu',
    #              'METHOD': 'create2commandqueue4tpu'},
    #  'FUNCTION2CALLBACK': <function trace2commandqueue4tpu at 0x00000000037BC848>,
    #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-8A-9B-10B-9B-10B-11B-12B-13B-14B-15B-16B-17B-18B-',
    #  'INFO2PRIVATE': None,
    #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003EDA780>,
    #  'NACK2RETURN': ' ',
    #  'CONTEXT': {'COUNT2REFERENCE': 0,
    #              'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4tpu-failed ; time: 07:13:59 09/06/11 Pacific Daylight Time}',
    #              'CLASS': 'create2context4tpu',
    #              'METHOD': 'create2context4tpu'},
    #              'FUNCTION2CALLBACK': <function trace2context4tpu at 0x0000000003770148>,
    #              'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
    #              'INFO2PRIVATE': None,
    #              'PLATFORM': {'list2platforms': ['DragonXi'],
    #                           'ack2return': ' 2C-2E-2F-2G-',
    #                           'nack2return': ' ', 'available2platform': 1, 'max2entries': 1},
    #              'NACK2RETURN': ' ',
    #              'SIZE2PRIVATE': 0,
    #              'PROPERTIES': None},
    #              'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318,
    #                          'ID2PLATFORM': '0000000001509568',
    #  'PROPERTIES': None,
    #  'PIPE': (<read-write PipeConnection, handle 552>, <read-write PipeConnection, handle 556>)},
    #  'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK', 'EXECUTE2KERNELS4OPENCL': 'ACK',
    #              'NATIVE2WIDTHLONG4VECTOR': 2, 'MAX2UNITS4COMPUTE': 4, 'SIZE2MEMORY4GLOBAL': 3221225472, 'NATIVE2WIDTHFLOAT4VECTOR': 4, 'MAX2SIZE4WORKGROUP': 1024, 'PREFERRED2WIDTHCHAR4VECTOR': 16, 'SIZE2CACHE': 65536, 
    #              'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK', 'TYPE4CACHE': 'Read/Write', 'PREFERRED2WIDTHINT4VECTOR': 4, 'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1, 'NATIVE2WIDTHINT4VECTOR': 4, 
    #              'SIZE2BUFFER4CONSTANT': 65536, 'EXECUTE2FUNCTION4NATIVE': 'ACK', 'PREFERRED2WIDTHDOUBLE4VECTOR': 0, 'MAX2NUMBER4CONSTANTARGS': 8, 'VERSION2DRIVER': '2.0', 
            
    code2ack  = ' '
    code2nack = ' '
    # -----------------------------------------------------------------------------------------
    # CREATE IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if tpu2image4xi is None:
            tpu2image4xi = create2image4tpu()                                                                              # INITIALIZE IMAGE FOR TPU 
            message2print = "{(Step 1C/70) set" + "; tpu2image4xi-set-by-set2image4tpu to: " + str(tpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4tpu-skipped-by-set2image4tpu: " + str(create2image4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # <function create2image4tpu at 0x0000000003A627C8>;                                  
    except:
        try:
            tpu2image4xi = create2image4tpu()
            message2print = "{(Step 1E/70) set" + "; tpu2image4xi-set-by-set2image4tpu to: " + str(tpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            tpu2image4xi = {}
            message2print = "{(Step 1Z/70) yielded" + "; tpu2image4xi-yielded-by-set2image4tpu: " + str(tpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z-' 
                
    # ---------------------------------------------------------------------------------------------
    # SET            
    # ---------------------------------------------------------------------------------------------     
    message2print = "{(Step 22A0/70) try" + "; data2callback-try-by-set2image4tpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    #     
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 21A/70) - DATA2CALLBACK - EXTRA
        # -----------------------------------------------------------------------------------------               
        tpu2image4xi['DATA2CALLBACK'] = task4imageset2tpu['DATA2CALLBACK']
                                        
        message2print = "{(Step 21A1/70) set" + "; data2callback-set-by-set2image4tpu to: " + str(tpu2image4xi['DATA2CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        
        code2ack  = code2ack + '21A1-'
    except: 
        message2print = "{(Step 21Z1/70) yielded" + "; data2callback-tpu2image4xi-yielded-by-set2image4tpu: " + str(tpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        code2nack  = code2nack + '21Z1-'
    
    message2print = "{(Step 22B0/70) try" + "; function2callback-try-by-set2image4tpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22B/70) - FUNCTION2CALLBACK - EXTRA
        # -----------------------------------------------------------------------------------------               
        tpu2image4xi['FUNCTION2CALLBACK'] = task4imageset2tpu['FUNCTION2CALLBACK']
        
        message2print = "{(Step 22B1/70) set" + "; function2callback-set-by-set2image4tpu to: " + str(tpu2image4xi['FUNCTION2CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #  
        code2ack  = code2ack + '22B1-'
    except: 
        message2print = "{(Step 22Z2/70) yielded" + "; function2callback-tpu2image4xi-yielded-by-set2image4tpu: " + str(tpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        code2nack  = code2nack + '22Z2-' 
    
    message2print = "{(Step 22C0/70) try" + "; platform2callback-try-by-set2image4tpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22C/70) - PLATFORM - EXTRA
        # -----------------------------------------------------------------------------------------               
        tpu2image4xi['PLATFORM'] = task4imageset2tpu['PLATFORM']
        
        message2print = "{(Step 2C1/70) set" + "; platform-set-by-set2image4tpu to: " + str(tpu2image4xi['PLATFORM'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # yielded  
        code2ack  = code2ack + '2C1-'
    except: 
        message2print = "{(Step 22Z3/70) yielded" + "; platform-tpu2image4xi-yielded-by-set2image4tpu: " + str(tpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '22Z3-' 
        
        
      
    message2print = "{(Step 23A/70) try" + "; context2tpu-try-by-set2image4tpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)       
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 23/70) - CONTEXT - REQUIRED
        # -----------------------------------------------------------------------------------------               
        tpu2image4xi['CONTEXT'] = task4imageset2tpu['CONTEXT']
        
        message2print = "{(Step 23B/70) set" + "; context-set-by-set2image4tpu to: " + str(tpu2image4xi['CONTEXT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'COUNT2REFERENCE': 0,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4tpu-failed ; time: 07:13:59 09/06/11 Pacific Daylight Time}',
        #                    'CLASS': 'create2context4tpu',
        #                    'METHOD': 'create2context4tpu'},
        #  'FUNCTION2CALLBACK': <function trace2context4tpu at 0x0000000003770148>,
        #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
        #  'INFO2PRIVATE': None,
        #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 'NACK2RETURN': ' ',
        #  'SIZE2PRIVATE': 0,
        #  'PROPERTIES': None}; time: 07:13:59 09/06/11 Pacific Daylight Time} 

        
        code2ack  = code2ack + '23B-'
    except: 
        message2print = "{(Step 23Z1/70) yielded" + "; context-tpu2image4xi-yielded-by-set2image4tpu: " + str(tpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
    
        code2nack  = code2nack + '23Z1-' 
        
    message2print = "{(Step 24A/70) try" + "; device-try-by-set2image4tpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 24/70) - DEVICE - EXTRA
        # -----------------------------------------------------------------------------------------               
        tpu2image4xi['DEVICE'] = task4imageset2tpu['DEVICE']
        
        message2print = "{(Step 24B/70) set" + "; device-set-by-set2image4tpu to: " + str(tpu2image4xi['DEVICE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'RESOLUTION2TIMER4PROFILING': 318,
        #  'ID2PLATFORM': '0000000001509568',
        #  'PROFILING2QUEUE': 'ACK',
        #  'EXECUTE2KERNELS4OPENCL': 'ACK',
        #  'NATIVE2WIDTHLONG4VECTOR': 2,
        #  'MAX2UNITS4COMPUTE': 4,
        #  'SIZE2MEMORY4GLOBAL': 3221225472,
        #  'NATIVE2WIDTHFLOAT4VECTOR': 4,
        #  'MAX2SIZE4WORKGROUP': 1024,
        #  'PREFERRED2WIDTHCHAR4VECTOR': 16,
        #  'SIZE2CACHE': 65536,
        #  'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK',
        #  'TYPE4CACHE': 'Read/Write',
        #  'PREFERRED2WIDTHINT4VECTOR': 4,
        #  'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1,
        #  'NATIVE2WIDTHINT4VECTOR': 4,
        #  'SIZE2BUFFER4CONSTANT': 65536,
        #  'EXECUTE2FUNCTION4NATIVE': 'ACK',
        #  'PREFERRED2WIDTHDOUBLE4VECTOR': 0,
        #  'MAX2NUMBER4CONSTANTARGS': 8,
        #  'VERSION2DRIVER': '2.0',
        #  'EXTENSIONS': 'cl_amd_fp64 cl_khr_global_int32_base_atomics ...
        #  'AVAILABLE2COMPILER': 'ACK',
        #  'MAX2DIMENSIONS4WORKITEMS': 3,
        #  'SUPPORT2CORRECTION4ERROR': 0,
        #  'BITS2ADDRESS': 64,
        #  'AVAILABLE': 'ACK',
        #  'SIZE2MEMORY4LOCAL': 32768,
        #  'VENDOR': 'AuthenticAMD',
        #  'SUPPORT2IMAGE': 'NACK',
        #  'MAX2SIZE4KERNELARGUMENT': 4096,
        #  'ACK2RETURN': ' 1C-1D-1E-1F-1D-2A-3A-4A-5A-6A-7A-8A-9A-10A-11A-...
        #  'OUT-OF-ORDER2QUEUE': 'NACK',
        #  'ALIGNMENT2ADDRESS4BASE': 1024,
        #  'PROFILE': 'FULL_PROFILE',
        #  'TYPE2MEMORY4LOCAL': 'Global',
        #  'PREFERRED2WIDTHFLOAT4VECTOR': 4,
        #  'PREFERRED2WIDTHLONG4VECTOR': 2,
        #  'MAX2FREQUENCY4CLOCK': 3210,
        #  'PREFERRED2WIDTHSHORT4VECTOR': 8,
        #  'ID': 4033,
        #  'MAX2WORKITEM2': 1024,
        #  'NATIVE2WIDTHDOUBLE4VECTOR': 0,
        #  'ROUND2ZERO': 'ACK',
        #  'NAME': 'AMD64 Family 16 Model 4 Stepping 2, AuthenticAMD',
        #  'MAX2WORKITEM1': 1024,
        #  'MAX2WORKITEM0': 1024,
        #  'SIZE2LINE4CACHE': 64,
        #  'MAX2ALLOCATION4MEMORY': 1073741824,
        #  'NATIVE2WIDTHCHAR4VECTOR': 16,
        #  'QUIET2NANS': 'ACK',
        #  'NATIVE2WIDTHSHORT4VECTOR': 8,
        #  'ROUND2NEAREST4EVEN': 'ACK',
        #  'VERSION': 'OpenCL 1.1 ATI-Stream-v2.3 (451)',
        #  'NACK2RETURN': ' ',
        #  'MEMORY2UNIFIED4HOSTANDDEVICE': 1,
        #  'ENDIANESS2DEVICE': 'Little',
        #  'MINIMUM2ALIGNMENT4DATATYPE': 128,
        #  'ROUND2+VE3INFINITY': 'ACK',
        #  'TYPE': 'CL_DEVICE_TYPE_TPU',
        #  'DENORMS2SINGLEPRECISION4FLOATINGPOINTCAPABILITY': 'ACK'}; time: 07:13:59 09/06/11 Pacific Daylight Time}

        
        code2ack  = code2ack + '24B-'
    except: 
        message2print = "{(Step 24Z1/70) yielded" + "; device-tpu2image4xi-yielded-by-set2image4tpu: " + str(tpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '24Z1-'
                 
    message2print = "{(Step 25A/70) try" + "; code2error-try-by-set2image4tpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 25/70) - ERROR CODE - REQUIRED
        # -----------------------------------------------------------------------------------------               
        tpu2image4xi['CODE2ERROR'] = task4imageset2tpu['CODE2ERROR']
        
        message2print = "{(Step 25B/70) set" + "; code2error-set-by-set2image4tpu to: " + str(tpu2image4xi['CODE2ERROR'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0 
        code2ack  = code2ack + '25B-'
    except: 
        message2print = "{(Step 25Z1/70) yielded" + "; code2error-tpu2image4xi-yielded-by-set2image4tpu: " + str(tpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '25Z1-'                
                
          
    message2print = "{(Step 26A/70) try" + "; queue-try-by-set2image4tpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 26/70) - QUEUE - EXTRA
        # -----------------------------------------------------------------------------------------               
        tpu2image4xi['QUEUE'] = task4imageset2tpu['QUEUE']
        
        message2print = "{(Step 26B/70) set" + "; queue-set-by-set2image4tpu to: " + str(tpu2image4xi['QUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <queue4xi.Queue4Xi object at 0x0000000003D4A710> 
        code2ack  = code2ack + '26B-'
    except: 
        message2print = "{(Step 26Z1/70) yielded" + "; queue-tpu2image4xi-yielded-by-set2image4tpu: " + str(tpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '26Z1-' 
                                
    message2print = "{(Step 27A/70) try" + "; pipe-try-by-set2image4tpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                     
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 27/70) - PIPE - EXTRA
        # -----------------------------------------------------------------------------------------               
        tpu2image4xi['PIPE'] = task4imageset2tpu['PIPE']
        
        message2print = "{(Step 27B/70) set" + "; pipe-set-by-set2image4tpu to: " + str(tpu2image4xi['PIPE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # (<read-write PipeConnection, handle 520>, <read-write PipeConnection, handle 524>)  
        
        code2ack  = code2ack + '27B-'
    except: 
        message2print = "{(Step 27Z1/70) yielded" + "; pipe-tpu2image4xi-yielded-by-set2image4tpu: " + str(tpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '27Z1-'   

    message2print = "{(Step 28A/70) try" + "; commandqueue-try-by-set2image4tpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (28) COMMAND QUEUE - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        tpu2image4xi['COMMANDQUEUE']  = task4imageset2tpu['COMMANDQUEUE']  
 
        message2print = "{(Step 28B/70) set" + "; commandqueue-set-by-set2image4tpu to: " + str(tpu2image4xi['COMMANDQUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
        
        code2ack  = code2ack + '28B-'
    except: 
        message2print = "{(Step 28Z1/70) yielded" + "; commandqueue-yielded-by-set2image4tpu: " + str(tpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '28Z1-'    

    message2print = "{(Step 29A/70) try" + "; bufferinfo-try-by-set2image4tpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (29) BUFFER INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        tpu2image4xi['BUFFER']  = task4imageset2tpu['BUFFER']  
 
        message2print = "{(Step 29B/70) set" + "; bufferinfo-set-by-set2image4tpu to: " + str(tpu2image4xi['BUFFER'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        #  
        code2ack  = code2ack + '29B-'
    except: 
        message2print = "{(Step 29Z1/70) yielded" + "; bufferinfo-yielded-by-set2image4tpu: " + str(tpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        code2nack  = code2nack + '29Z1-'            

    message2print = "{(Step 30A/70) try" + "; eventinfo-try-by-set2image4tpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (30) EVENT INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        tpu2image4xi['EVENT']  = task4imageset2tpu['EVENT']  
 
        message2print = "{(Step 30B/70) set" + "; eventinfo-set-by-set2image4tpu to: " + str(tpu2image4xi['EVENT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '30B-'
    except: 
        message2print = "{(Step 30Z1/70) yielded" + "; eventinfo-yielded-by-set2image4tpu: " + str(tpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        
        code2nack  = code2nack + '30Z1-'            
                

    message2print = "{(Step 31A/70) try" + "; flags-try-by-set2image4tpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (31) FLAGS - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        tpu2image4xi['FLAGS']  = task4imageset2tpu['FLAGS']  
 
        message2print = "{(Step 31B/70) set" + "; flags-set-by-set2image4tpu to: " + str(tpu2image4xi['FLAGS'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '31B-'
    except: 
        message2print = "{(Step 31Z1/70) yielded" + "; flags-yielded-by-set2image4tpu: " + str(tpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        
        code2nack  = code2nack + '31Z1-'     

    message2print = "{(Step 34A/70) try" + "; width-try-by-set2image4tpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (34) WIDTH - REQUIRED                              
        # ---------------------------------------------------------------------------------------------      
        tpu2image4xi['WIDTH']  = task4imageset2tpu['WIDTH']  
 
        message2print = "{(Step 34H/70) set" + "; width-set-by-set2image4tpu to: " + str(tpu2image4xi['WIDTH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '34H-'
    except: 
        message2print = "{(Step 34Z1/70) yielded" + "; width-yielded-by-set2image4tpu: " + str(tpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '34Z1-'   


    message2print = "{(Step 35A/70) try" + "; height-try-by-set2image4tpu: None"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (35) HEIGHT - REQUIRED                              
        # ---------------------------------------------------------------------------------------------      
        tpu2image4xi['HEIGHT']  = task4imageset2tpu['HEIGHT']  
 
        message2print = "{(Step 35H/70) set" + "; height-set-by-set2image4tpu to: " + str(tpu2image4xi['HEIGHT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        #  
        code2ack  = code2ack + '35H-'
    except: 
        message2print = "{(Step 35Z1/70) yielded" + "; height-yielded-by-set2image4tpu: " + str(tpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded 
        code2nack  = code2nack + '35Z1-'   
 

    message2print = "{(Step 36A/70) try" + "; rowpitch-try-by-set2image4tpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (36) ROWPITCH - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        tpu2image4xi['ROWPITCH']  = task4imageset2tpu['ROWPITCH']  
 
        message2print = "{(Step 36B/70) set" + "; rowpitch-set-by-set2image4tpu to: " + str(tpu2image4xi['ROWPITCH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '36B-'
    except: 
        message2print = "{(Step 36Z1/70) yielded" + "; rowpitch-yielded-by-set2image4tpu: " + str(tpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded 
        code2nack  = code2nack + '36Z1-'

    message2print = "{(Step 37A/70) try" + "; pointer-try-by-set2image4tpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (37) POINTER - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        tpu2image4xi['POINTER']  = task4imageset2tpu['POINTER']  
 
        message2print = "{(Step 37B/70) set" + "; pointer-set-by-set2image4tpu to: " + str(tpu2image4xi['POINTER'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '37B-'
    except: 
        message2print = "{(Step 37Z1/70) yielded" + "; pointer-yielded-by-set2image4tpu: " + str(tpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded 
        code2nack  = code2nack + '37Z1-'

    message2print = "{(Step 38A/70) try" + "; format-try-by-set2image4tpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (38) FORMAT - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        tpu2image4xi['FORMAT']  = task4imageset2tpu['FORMAT']  
 
        message2print = "{(Step 38B/70) set" + "; format-set-by-set2image4tpu to: " + str(tpu2image4xi['FORMAT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '38B-'
    except: 
        message2print = "{(Step 38Z1/70) yielded" + "; format-yielded-by-set2image4tpu: " + str(tpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded 
        code2nack  = code2nack + '38Z1-'
                        
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    tpu2image4xi ['ACK2RETURN']  = code2ack
    tpu2image4xi ['NACK2RETURN'] = code2nack 
    
    message2print = "{(Step 70/70) returning" + "; tpu2image4xi-returning-by-set2image4tpu: " + str(tpu2image4xi) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'CODE2ERROR': 0, 'ACK2RETURN': ' 23B-24B-25B-26B-27B-28B-', 'ROWPITCH': 800, 'DATA2CALLBACK': ...
    
    
    return tpu2image4xi

           
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "copy2image4tpu" usage = "
#                                                                                                              
#   # -----------------------------------------------------------------------------------------------
#   # (16) ESTABLISH CONTROL & COMMAND WITH COPY2IMAGE4TPU                      
#   # -----------------------------------------------------------------------------------------------    
#   print('(Step 16A/700) OF TPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH COPY2IMAGE4TPU ', time4xi())    
#   response4copy2image = {} 
#   task4copy2image = None
#   try:                               
#       response4copy2image = copy2image4tpu(task4copy2image)                                                                                                          
#       print('(Step 16B/700) OF TPU2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH COPY2IMAGE4TPU : ', response4copy2image)
#       queue2copy4tpu      = response4copy2image['QUEUE']
#       channel2copypublish4tpu = response4copy2image['PUBLISHERCHANNEL']
#       channel2copyconsume4tpu = response4copy2image['CONSUMERCHANNEL']
#   except:
#       print('(Step 16Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH COPY2IMAGE4TPU ', time4xi()) 
#   # -----------------------------------------------------------------------------------------------
#   # (17) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2IMAGE4TPU TO RECEIVE                 
#   # -----------------------------------------------------------------------------------------------      
#   print('(Step 17A/700) OF TPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4TPU ', time4xi())     
#   try:                               
#       message2copyimagesend4tpu = "<copy4xi>copy2copy4tpu</copy4xi>"
#       channel2copypublish4tpu = response4copy2image['PUBLISHERCHANNEL']
#       channel2copypublish4tpu.send(message2copyimagesend4tpu)
#       print("(Step 17B/700) OF TPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4TPU: ", channel2copypublish4tpu, " MESSAGE2SEND4TPU: " , message2copyimagesend4tpu)                                                                                                
#   except:
#       print('(Step 17Z1/700) OF TPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4TPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------------------
#   # (18) LET MANOEUVRE2IMAGE4TPU TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # -----------------------------------------------------------------------------------------------       
#   print("(Step 18C/700) OF TPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4TPU ", time4xi()) 
#   try:        
#       response2manoeuvre4tpu = manoeuvre2copyimage4tpu(queue2copy4tpu, channel2copyconsume4tpu, channel2copypublish4tpu)
#       print("(Step 18D/700) OF TPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4TPU ", manoeuvre2copyimage4tpu)
#   except:
#       print("(Step 18Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4TPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # -----------------------------------------------------------------------------------------------
#   # (19) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 19A/700) OF TPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4TPU ', time4xi())     
#   try:
#       channel2copyconsume4tpu = response4copy2image['CONSUMERCHANNEL']                              
#       message2copyimagereceive4tpu = channel2copyconsume4tpu.recv()
#       print("(Step 19B/700) OF TPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4TPU: ", channel2copyconsume4tpu, " CONTROL & COMMAND: ", message2copyimagereceive4tpu)                                                                                                  
#       # <roger4xi>roger2copy4tpu</roger4xi>
#   except:
#       print('(Step 19Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPY2IMAGE4TPU ', time4xi())         
#   # -----------------------------------------------------------------------------------------------
#   # (20) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2IMAGE4TPU TO RECEIVE                
#   # -----------------------------------------------------------------------------------------------      
#   print('(Step 20A/700) OF TPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4TPU ', time4xi())     
#   try:                               
#       message2copyimagesend4tpu = "<copy4xi>copy2copy4tpu</copy4xi>"
#       channel2copypublish4tpu = response4copy2image['PUBLISHERCHANNEL']
#       channel2copypublish4tpu.send(message2copyimagesend4tpu)
#       print("(Step 20B/700) OF TPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4TPU: ", channel2copypublish4tpu, " MESSAGE2SEND4TPU: " , message2copyimagesend4tpu)                                                                                                
#       # <copy4xi>copy2copy4tpu</copy4xi> 
#   except:
#        print('(Step 20Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4TPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------------------
#   # (21) LET MANOEUVRE2IMAGE4TPU TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------------------       
#   print("(Step 21A/700) OF TPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4TPU ", time4xi()) 
#   try:        
#       response2manoeuvre4tpu = manoeuvre2copyimage4tpu(queue2copy4tpu, channel2copyconsume4tpu, channel2copypublish4tpu)
#       print("(Step 21B/700) OF TPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4TPU ", manoeuvre2copyimage4tpu)
#   except:
#       print("(Step 21Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4TPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (22) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 22A/700) OF TPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4TPU ', time4xi())     
#   try:
#       channel2copyconsume4tpu = response4copy2image['CONSUMERCHANNEL']                              
#       message2copyimagereceive4tpu = channel2copyconsume4tpu.recv()
#       print("(Step 22B/700) OF TPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4TPU: ", channel2copyconsume4tpu, " CONTROL & COMMAND: ", message2copyimagereceive4tpu)                                                                                                  
#       # <copy4xi>copy2copy4tpu</copy4xi> 
#   except:
#       print('(Step 22Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPY2IMAGE4TPU ', time4xi())          
#   # ----------------------------------------------------------------------------------------------
#   # (24) PUT IMAGE PATH TO QUEUE
#   # ----------------------------------------------------------------------------------------------                          
#   print("(Step 24A/700) OF TPU2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue2copy4dragon.put(path2dragon)             
#       print("(Step 24B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#       # ..//..//..//..//..//web//images//png//dragonxi.png  
#   except:
#       print("(Step 24Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # -----------------------------------------------------------------------------------------------
#   # (25) LET MANOEUVRE2IMAGE4TPU TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -----------------------------------------------------------------------------------------------       
#   print("(Step 25A/700) OF TPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4TPU ", time4xi()) 
#   try:        
#       response2manoeuvre4tpu = manoeuvre2copyimage4tpu(queue2copy4tpu, channel2copyconsume4tpu, channel2copypublish4tpu)
#       print("(Step 25B/700) OF TPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4TPU ", manoeuvre2copyimage4tpu)
#   except:
#       print("(Step 25Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4TPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (26) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 26A/700) OF TPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2copyimagereceive4tpu = channel2copyconsume4tpu.recv()
#       print("(Step 26D/700) OF TPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4TPU: ", channel2copyconsume4tpu, " CONTROL & COMMAND: ", message2copyimagereceive4tpu)                                                                                                   
#       # <ack4xi>ack2imageinfo4tpu</ack4xi>
#   except:
#       print('(Step 26Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -----------------------------------------------------------------------------------------------
#   (27) LET MANOEUVRE2IMAGE4TPU GO TO STATE 0               
#   # -----------------------------------------------------------------------------------------------       
#   print("(Step 27A/700) OF TPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4TPU ", time4xi()) 
#   try:        
#       response2manoeuvre4tpu = manoeuvre2copyimage4tpu(queue2copy4tpu, channel2copyconsume4tpu, channel2copypublish4tpu)
#       print("(Step 27B/700) OF TPU2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4TPU ", response2manoeuvre4tpu)
#   except:
#       print("(Step 27Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4TPU ", time4xi())   

#">

# -------------------------------------------------------------------------------------------------   
def copy2image4tpu(task4imagecopy2tpu=None):

    '''
    copies 2D images for TPU
    '''
    global tpu2image4xi                                                                                             # 2D-IMAGES FOR TPU  
    global control2copyimage4tpu                                                                                    # COMMAND AND CONTROL FOR TPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "copy2image4tpu"
    method2print = "copy2image4tpu"

    message2print = "{(Step 0A/33) started" + "; copy2image4tpu-started: " + str(copy2image4tpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                               
    # <function copy2image4tpu at 0x00000000038538C8>
    
    message2print = "{(Step 0B/33) passed" + "; task4imagecopy2tpu-to-copy2image4tpu: " + str(task4imagecopy2tpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE TPU2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if tpu2image4xi is None:
            tpu2image4xi = create2image4tpu()                                                                       # INITIALIZE IMAGE FOR TPU 
            message2print = "{(Step 0C/70) created" + "; tpu2image4xi-created-by-copy2image4tpu to: " + str(tpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        else:
            message2print = "{(Step 0D/70) skipped" + "; create2image4tpu-skipped-by-copy2image4tpu: " + str(copy2image4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # <function copy2image4tpu at 0x00000000038538C8>                                 
    except:
        try:
            tpu2image4xi = create2image4tpu()
            message2print = "{(Step 0E/70) set" + "; tpu2image4xi-set-by-copy2image4tpu to: " + str(tpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            tpu2image4xi = {}
            message2print = "{(Step 0Z/70) yielded" + "; tpu2image4xi-yielded-by-copy2image4tpu: " + str(tpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '0Z-' 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    try:             
        if task4imagecopy2tpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR COPY - BUILD TASK! 
            # -----------------------------------------------------------------------------------------    
            
            task4imagecopy2tpu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1A/70) initialized" + "; task4imagecopy2tpu-created-by-copy2image4tpu to: " + str(task4imagecopy2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {} 
            
            # ---------------------------------------------------------------------------------------------
            # (1B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4tpu = trace2image4tpu                                                                 # CALLBACK FUNCTION FOR TPU
            
            message2print = "{(Step 1B1/70) initialized" + "; function2callback4tpu-created-by-copy2image4tpu to: " + str(function2callback4tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # <function trace2image4tpu at 0x000000000384E9C8> 
            
            code2ack  = code2ack + '1B1-'
           
            args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "copy2image4tpu"                                                            # MODULE OF TPUIZED OPENCL MODULE
            args2callback['METHOD']  = "copy2image4tpu"                                                            # FUNCTION OF TPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2image4tpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR TPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-copy2image4tpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {'MESSAGE': '{(Step 1Z1/33) failed; copy2image4tpu-failed ; time: 07:01:44 10/04/11 Pacific Daylight Time}',
            #  'CLASS': 'copy2image4tpu',
            #  'METHOD': 'copy2image4tpu'}; time: 07:01:44 10/04/11 Pacific Daylight Time}


            code2ack  = code2ack + '1B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (1C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagecopy2tpu['DATA2CALLBACK'] = args2callback                                                    # USER DATA PASSED TO TPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagecopy2tpu['FUNCTION2CALLBACK'] = function2callback4tpu                                        # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '1C-'
            message2print = "{(Step 1C/70) initialized" + "; task4imagecopy2tpu-created-by-copy2image4tpu to: " + str(task4imagecopy2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # {'FUNCTION2CALLBACK': <function trace2image4tpu at 0x000000000384E9C8>,
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; copy2image4tpu-failed ; time: 07:01:44 10/04/11 Pacific Daylight Time}',
            #  'CLASS': 'copy2image4tpu',
            #  'METHOD': 'copy2image4tpu'}}; time: 07:01:44 10/04/11 Pacific Daylight Time}

            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-copy2image4tpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # {}
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-copy2image4tpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 07:01:44 10/04/11 Pacific Daylight Time}
                 
                code2ack  = code2ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-copy2image4tpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '2Z-'
                 
            task4imagecopy2tpu['PLATFORM'] = platform2xi                                                           # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2tpu = {}
            message2print = "{(Step 3A/70) trying" + "; context2tpu-trying-by-copy2image4tpu: " + str(context2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2tpu = get2context4tpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context2tpu-got-by-copy2image4tpu: " + str(context2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) ...
                
                code2ack  = code2ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context2tpu-failed-by-copy2image4tpu: " + str(context2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '3Z-'
                
            task4imagecopy2tpu ['CONTEXT'] = context2tpu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2tpu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device2tpu-trying-by-copy2image4tpu: " + str(device2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2tpu = get2tpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device2tpu-got-by-copy2image4tpu: " + str(device2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK', ...
                
                code2ack  = code2ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device2tpu-failed-by-copy2image4tpu: " + str(device2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '4Z-'
            
            task4imagecopy2tpu ['DEVICE'] = device2tpu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) IMAGE SOURCE OBJECT - REQUIRED                         
            # ---------------------------------------------------------------------------------------------         
            imagesourceinfo2tpu = {}
             
            message2print = "{(Step 5A/70) trying" + "; imagesourceinfo2tpu-trying-by-copy2image4tpu: " + str(imagesourceinfo2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imagesourceinfo2tpu = get2imageinfo4tpu()                                                                         
                
                message2print = "{(Step 5B/70) got" + "; imagesourceinfo2tpu-got-by-copy2image4tpu: " + str(imagesourceinfo2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'SIZE2QUEUE': 2147483646, 
                #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
                #  'ORDER': 'CL_RGBA'}, 
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004472BE0>, 
                #  'WIDTH': 800, 
                #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
                
                code2ack  = code2ack + '5B-'  
            except:
                message2print = "{(Step 5Z/70) failed" + "; imagesourceinfo2tpu-failed-by-copy2image4tpu: " + str(imagesourceinfo2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '5Z-'          
                      
            task4imagecopy2tpu ['IMAGESOURCE'] = imagesourceinfo2tpu         
                   
            # ---------------------------------------------------------------------------------------------
            # (6) IMAGE DESTINATION OBJECT - REQUIRED                        
            # ---------------------------------------------------------------------------------------------                         
            imagedestinationinfo2tpu = {}
             
            message2print = "{(Step 6A/70) trying" + "; imagedestinationinfo2tpu-trying-by-copy2image4tpu: " + str(imagedestinationinfo2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imagedestinationinfo2tpu = get2imageinfo4tpu()                                                                         
                
                message2print = "{(Step 6B/70) got" + "; imagedestinationinfo2tpu-got-by-copy2image4tpu: " + str(imagedestinationinfo2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'SIZE2QUEUE': 2147483646, 
                #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
                #  'ORDER': 'CL_RGBA'}, 
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004472BE0>, 
                #  'WIDTH': 800, 
                #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
                # note: source and destination will be the same !             
                code2ack  = code2ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; imagedestinationinfo2tpu-failed-by-copy2image4tpu: " + str(imagedestinationinfo2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '6Z-'          
                      
            task4imagecopy2tpu ['IMAGEDESTINATION'] = imagedestinationinfo2tpu     
            
            # ---------------------------------------------------------------------------------------------
            # (7) QUEUE - REQUIRED BY TPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2tpu = None
            
            message2print = "{(Step 7A/70) trying" + "; queue2tpu-trying-by-copy2image4tpu: " + str(queue2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2tpu = Queue4Xi()                                                                              # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 7B/70) got" + "; queue2tpu-got-by-copy2image4tpu: " + str(queue2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # <queue4xi.Queue4Xi object at 0x000000000453EB38>
                
                code2ack  = code2ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; queue2tpu-failed-by-copy2image4tpu: " + str(queue2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '7Z-'
                
            task4imagecopy2tpu ['QUEUE'] = queue2tpu

            
            # ---------------------------------------------------------------------------------------------
            # (8) PIPE - REQUIRED BY TPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2tpu = None
           
            message2print = "{(Step 8A/70) trying" + "; pipe2tpu-trying-by-copy2image4tpu: " + str(pipe2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2tpu = Pipe4Xi()                                                                                # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 8B/70) got" + "; pipe2tpu-got-by-copy2image4tpu: " + str(pipe2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # (<read-write PipeConnection, handle 612>, <read-write PipeConnection, handle 616>)
                
                code2ack  = code2ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; pipe2tpu-failed-by-copy2image4tpu: " + str(pipe2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '8Z-'
                
            task4imagecopy2tpu ['PIPE'] = pipe2tpu 
            
            # ---------------------------------------------------------------------------------------------
            # (9) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2tpu = {}
           
            message2print = "{(Step 9A/70) trying" + "; commandqueueinfo2tpu-trying-by-copy2image4tpu: " + str(commandqueueinfo2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2tpu = get2commandqueueinfo4tpu()                                                   # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 9B/70) got" + "; commandqueueinfo2tpu-got-by-copy2image4tpu: " + str(commandqueueinfo2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 
                #  'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4tpu-failed ; time: 07:01:44 10/04/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2commandqueue4tpu', 
                #  'METHOD': 'create2commandqueue4tpu'}, 
                #  'FUNCTION2CALLBACK': <function trace2commandqueue4tpu at 0x00000000037EDA48> ...
                
                code2ack  = code2ack + '9B-'  
            except:
                message2print = "{(Step 9Z/70) failed" + "; commandqueueinfo2tpu-failed-by-copy2image4tpu: " + str(commandqueueinfo2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '9Z-'
        
            task4imagecopy2tpu ['COMMANDQUEUE'] = commandqueueinfo2tpu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (10) BUFFER - AUTOMATIC BY TPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2tpu = {}
           
            message2print = "{(Step 10A/70) trying" + "; bufferinfo2tpu-trying-by-copy2image4tpu: " + str(bufferinfo2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2tpu = get2bufferinfo4tpu()                                                               # SYSTEM CREATES BUFFER FOR TPU, IF NECESSARY
                message2print = "{(Step 10B/70) got" + "; bufferinfo2tpu-got-by-copy2image4tpu: " + str(bufferinfo2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE' ...
                
                code2ack  = code2ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; bufferinfo2tpu-failed-by-copy2image4tpu: " + str(bufferinfo2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '10Z-'
        
            task4imagecopy2tpu ['BUFFER'] = bufferinfo2tpu 
            
            # ---------------------------------------------------------------------------------------------
            # (11) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2tpu = {}
           
            message2print = "{(Step 11A/70) trying" + "; eventinfo2tpu-trying-by-copy2image4tpu: " + str(eventinfo2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2tpu = get2eventinfo4tpu()                                                                    
                message2print = "{(Step 11B/70) got" + "; eventinfo2tpu-got-by-copy2image4tpu: " + str(eventinfo2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0,
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4tpu-failed ; time: 07:01:44 10/04/11 Pacific Daylight Time}',
                #  'CLASS': 'create2event4tpu',
                #  'METHOD': 'create2event4tpu'},
                #  'FUNCTION2CALLBACK': <function trace2event4tpu at 0x0000000003803F48>,
                #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-6B-7B-8B-9B-10B-11B-12B-14B-15B-16B-',
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x000000000423B668>,
                #  'NACK2RETURN': ' ',
                #  'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
                
                code2ack  = code2ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; eventinfo2tpu-failed-by-copy2image4tpu: " + str(eventinfo2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '11Z-'
        
            task4imagecopy2tpu ['EVENT'] = eventinfo2tpu 
            
            
            # ---------------------------------------------------------------------------------------------
            # (12) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2tpu = {}
           
            message2print = "{(Step 12A/70) trying" + "; eventwaitlist2tpu-trying-by-copy2image4tpu: " + str(eventwaitlist2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2tpu = {}                                                                   
                message2print = "{(Step 12B/70) set" + "; eventwaitlist2tpu-set-by-copy2image4tpu: " + str(eventwaitlist2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {} 
                
                code2ack  = code2ack + '12B-' 
            except:
                message2print = "{(Step 12Z/70) failed" + "; eventwaitlist2tpu-failed-by-copy2image4tpu: " + str(eventwaitlist2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '12Z-'
        
            task4imagecopy2tpu ['EVENTWAITLIST'] = eventwaitlist2tpu                     

            
            # ---------------------------------------------------------------------------------------------
            # (13) EVENTWAITLISTQUANTITY - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2tpu = 0
            
            message2print = "{(Step 13A/70) trying" + "; eventwaitlistquantity2tpu-trying-by-copy2image4tpu: " + str(eventwaitlistquantity2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2tpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2tpu = eventwaitlistquantity2tpu + 1    
                
                message2print = "{(Step 13B/70) set" + "; eventwaitlistquantity2tpu-set-by-copy2image4tpu: " + str(eventwaitlistquantity2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0  
                
                code2ack  = code2ack + '13B-' 
            except:
                message2print = "{(Step 13Z/70) failed" + "; eventwaitlistquantity2tpu-failed-by-copy2image4tpu: " + str(eventwaitlistquantity2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '13Z-'
        
            task4imagecopy2tpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2tpu       
                                                 
            
            # ---------------------------------------------------------------------------------------------
            # (14) ORIGIN FOR SOURCE - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin4source2tpu = {}
            
            message2print = "{(Step 14A/70) trying" + "; origin4source2tpu-trying-by-copy2image4tpu: " + str(origin4source2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin4source2tpu['X'] = 0 
                origin4source2tpu['Y'] = 0  
                origin4source2tpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 14B/70) set" + "; origin4source2tpu-set-by-copy2image4tpu to: " + str(origin4source2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; origin4source2tpu-failed-by-copy2image4tpu: " + str(origin4source2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '14Z-'
        
            task4imagecopy2tpu ['SOURCEORIGIN'] = origin4source2tpu
            
            # ---------------------------------------------------------------------------------------------
            # (15) ORIGIN FOR DESTINATION - REQUIRED                             
            # ---------------------------------------------------------------------------------------------              
            origin4destination2tpu = {}
            
            message2print = "{(Step 15A/70) trying" + "; origin4destination2tpu-trying-by-copy2image4tpu: " + str(origin4destination2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin4destination2tpu['X'] = 100 
                origin4destination2tpu['Y'] = 100  
                origin4destination2tpu['Z'] = 100                                                                                                                          
                message2print = "{(Step 15B/70) set" + "; origin4destination2tpu-set-by-copy2image4tpu to: " + str(origin4destination2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; origin4destination2tpu-failed-by-copy2image4tpu: " + str(origin4destination2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '15Z-'
        
            task4imagecopy2tpu ['DESTINATIONORIGIN'] = origin4destination2tpu
                        
            
            
            # --------------------------------------------------------------------------------------------------------
            # (16) REGION - REGION OF A IMAGE TO BE COPIED                         
            # --------------------------------------------------------------------------------------------------------             
            region4copy2tpu = {}                                                                                                         
           
            message2print = "{(Step 16A/70) trying" + "; region4copy2tpu-trying-by-copy2image4tpu: " + str(region4copy2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                region4copy2tpu['X'] = 10 
                region4copy2tpu['Y'] = 10  
                region4copy2tpu['Z'] = 10                                                                           
                message2print = "{(Step 16B/70) set" + "; region4copy2tpu-set-by-copy2image4tpu to: " + str(region4copy2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 

                code2ack  = code2ack + '16B-'                
                 
            except:
                message2print = "{(Step 16Z/70) failed" + "; region4copy2tpu-failed-by-copy2image4tpu: " + str(region4copy2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '16Z-'
            
            task4imagecopy2tpu ['REGION'] = region4copy2tpu              
            
            # ---------------------------------------------------------------------------------------------
            # EOF PARAMETER PROVIDING                
            # ---------------------------------------------------------------------------------------------                                                                                                                               
        else:
            pass
            
        # ---------------------------------------------------------------------------------------------
        # CHECK PARAMETERS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagecopy2tpu.keys()
        message2print = "{(Step 20/70) using" + "; keys2task-using-by-copy2image4tpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # ['BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DESTINATIONORIGIN',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGEDESTINATION',
        #  'IMAGESOURCE',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION',
        #  'SOURCEORIGIN']; time: 01:21:58 10/05/11 Pacific Daylight Time}
 
        if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4tpu = trace2image4tpu                                                            # CALLBACK FUNCTION FOR TPU
            
            message2print = "{(Step 21B1/70) initialized" + "; function2callback4tpu-created-by-copy2image4tpu to: " + str(function2callback4tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '21B1-'
           
            args2callback = {}                                                                                 # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "copy2image4tpu"                                                        # MODULE OF TPUIZED OPENCL MODULE
            args2callback['METHOD']  = "copy2image4tpu"                                                        # FUNCTION OF TPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2image4tpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR TPUIZED OPENCL TO RETURN    
            
            message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-copy2image4tpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
    
            code2ack  = code2ack + '21B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagecopy2tpu['DATA2CALLBACK'] = args2callback                                                # USER DATA PASSED TO TPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagecopy2tpu['FUNCTION2CALLBACK'] = function2callback4tpu                                    # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
                
            code2ack  = code2ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4imagecopy2tpu-created-by-copy2image4tpu to: " + str(task4imagecopy2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
                 
        if 'PLATFORM' in keys2task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-copy2image4tpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
                
            try:
                platform2xi = get2platformids4xi()                                                              # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-copy2image4tpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
                code2ack  = code2ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-copy2image4tpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '22Z-'
                 
            task4imagecopy2tpu['PLATFORM'] = platform2xi                                                       # PLATFORM OF SYSTEM 
          
        if 'CONTEXT' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2tpu = {}
            message2print = "{(Step 23A/70) trying" + "; context2tpu-trying-by-copy2image4tpu: " + str(context2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2tpu = get2context4tpu()                                                                 # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context2tpu-got-by-copy2image4tpu: " + str(context2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context2tpu-failed-by-copy2image4tpu: " + str(context2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '23Z-'
                
            task4imagecopy2tpu ['CONTEXT'] = context2tpu           
        
        if 'DEVICE' in keys2task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2tpu = {}
            
            message2print = "{(Step 24A/70) trying" + "; device2tpu-trying-by-copy2image4tpu: " + str(device2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2tpu = get2tpu4xi()                                                                       # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device2tpu-got-by-copy2image4tpu: " + str(device2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device2tpu-failed-by-copy2image4tpu: " + str(device2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '24Z-'
            
            task4imagecopy2tpu ['DEVICE'] = device2tpu
        
        if 'IMAGESOURCE' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) SOURCE IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            sourceimageinfo2tpu = {}
             
            message2print = "{(Step 25A/70) trying" + "; sourceimageinfo2tpu-trying-by-copy2image4tpu: " + str(sourceimageinfo2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                sourceimageinfo2tpu = get2imageinfo4tpu()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; sourceimageinfo2tpu-got-by-copy2image4tpu: " + str(sourceimageinfo2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; sourceimageinfo2tpu-failed-by-copy2image4tpu: " + str(sourceimageinfo2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '25Z-'          
                      
            task4imagecopy2tpu ['IMAGESOURCE'] = sourceimageinfo2tpu         
        
        if 'IMAGEDESTINATION' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (26) DESTINATION IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            destinationimageinfo2tpu = {}
             
            message2print = "{(Step 26A/70) trying" + "; destinationimageinfo2tpu-trying-by-copy2image4tpu: " + str(destinationimageinfo2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                destinationimageinfo2tpu = get2imageinfo4tpu()                                                                         
                
                message2print = "{(Step 26B/70) got" + "; destinationimageinfo2tpu-got-by-copy2image4tpu: " + str(destinationimageinfo2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; destinationimageinfo2tpu-failed-by-copy2image4tpu: " + str(destinationimageinfo2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '26Z-'          
                      
            task4imagecopy2tpu ['IMAGEDESTINATION'] = destinationimageinfo2tpu    
                                    
        if 'QUEUE' in keys2task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (27) QUEUE - REQUIRED BY TPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2tpu = None
            
            message2print = "{(Step 27A/70) trying" + "; queue2tpu-trying-by-copy2image4tpu: " + str(queue2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2tpu = Queue4Xi()                                                                          # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 27B/70) got" + "; queue2tpu-got-by-copy2image4tpu: " + str(queue2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; queue2tpu-failed-by-copy2image4tpu: " + str(queue2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '27Z-'
                
            task4imagecopy2tpu ['QUEUE'] = queue2tpu

        if 'PIPE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) PIPE - REQUIRED BY TPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2tpu = None
           
            message2print = "{(Step 28A/70) trying" + "; pipe2tpu-trying-by-copy2image4tpu: " + str(pipe2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2tpu = Pipe4Xi()                                                                            # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 28B/70) got" + "; pipe2tpu-got-by-copy2image4tpu: " + str(pipe2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; pipe2tpu-failed-by-copy2image4tpu: " + str(pipe2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '28Z-'
                
            task4imagecopy2tpu ['PIPE'] = pipe2tpu 
            
        if 'COMMANDQUEUE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2tpu = {}
           
            message2print = "{(Step 29A/70) trying" + "; commandqueueinfo2tpu-trying-by-copy2image4tpu: " + str(commandqueueinfo2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2tpu = get2commandqueueinfo4tpu()                                               # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 29B/70) got" + "; commandqueueinfo2tpu-got-by-copy2image4tpu: " + str(commandqueueinfo2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '29B-'  
            except:
                message2print = "{(Step 29Z/70) failed" + "; commandqueueinfo2tpu-failed-by-copy2image4tpu: " + str(commandqueueinfo2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '29Z-'
        
            task4imagecopy2tpu ['COMMANDQUEUE'] = commandqueueinfo2tpu
        
        
        if 'BUFFER' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (30) BUFFER - AUTOMATIC BY TPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2tpu = {}
           
            message2print = "{(Step 30A/70) trying" + "; bufferinfo2tpu-trying-by-copy2image4tpu: " + str(bufferinfo2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2tpu = get2bufferinfo4tpu()                                                           # SYSTEM CREATES BUFFER FOR TPU, IF NECESSARY
                message2print = "{(Step 30B/70) got" + "; bufferinfo2tpu-got-by-copy2image4tpu: " + str(bufferinfo2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '30B-' 
            except:
                message2print = "{(Step 30Z/70) failed" + "; bufferinfo2tpu-failed-by-copy2image4tpu: " + str(bufferinfo2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '30Z-'
        
            task4imagecopy2tpu ['BUFFER'] = bufferinfo2tpu
             
        if 'EVENT' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2tpu = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventinfo2tpu-trying-by-copy2image4tpu: " + str(eventinfo2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2tpu = get2eventinfo4tpu()                                                                    
                message2print = "{(Step 31B/70) got" + "; eventinfo2tpu-got-by-copy2image4tpu: " + str(eventinfo2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventinfo2tpu-failed-by-copy2image4tpu: " + str(eventinfo2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '31Z-'
        
            task4imagecopy2tpu ['EVENT'] = eventinfo2tpu 
        
             
        if 'EVENTWAITLIST' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2tpu = {}
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlist2tpu-trying-by-copy2image4tpu: " + str(eventwaitlist2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2tpu = {}                                                                   
                message2print = "{(Step 32B/70) set" + "; eventwaitlist2tpu-set-by-copy2image4tpu: " + str(eventwaitlist2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlist2tpu-failed-by-copy2image4tpu: " + str(eventwaitlist2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '32Z-'
        
            task4imagecopy2tpu ['EVENTWAITLIST'] = eventwaitlist2tpu                     
              
        if 'EVENTWAITLISTQUANTITY' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (33) EVENTWAITLISTQUANTITY - REQUIRED                              
            # ---------------------------------------------------------------------------------------------
            eventwaitlistquantity2tpu = 0
            
            message2print = "{(Step 33A/70) trying" + "; eventwaitlistquantity2tpu-trying-by-copy2image4tpu: " + str(eventwaitlistquantity2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2tpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2tpu = eventwaitlistquantity2tpu + 1    
                
                message2print = "{(Step 33B/70) set" + "; eventwaitlistquantity2tpu-set-by-copy2image4tpu: " + str(eventwaitlistquantity2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 
                
                code2ack  = code2ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; eventwaitlistquantity2tpu-failed-by-copy2image4tpu: " + str(eventwaitlistquantity2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '33Z-'
                    
            task4imagecopy2tpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2tpu       
                                             
                    
        if 'SOURCEORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) SOURCEORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            source4origin2tpu = {}
            
            message2print = "{(Step 34A/70) trying" + "; source4origin2tpu-trying-by-copy2image4tpu: " + str(source4origin2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                source4origin2tpu['X'] = 0 
                source4origin2tpu['Y'] = 0  
                source4origin2tpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; source4origin2tpu-set-by-copy2image4tpu to: " + str(source4origin2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; source4origin2tpu-failed-by-copy2image4tpu: " + str(source4origin2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '34Z-'
        
            task4imagecopy2tpu ['SOURCEORIGIN'] = source4origin2tpu
             
        if 'DESTINATIONORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (35) DESTINATIONORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            destination4origin2tpu = {}
            
            message2print = "{(Step 35A/70) trying" + "; destination4origin2tpu-trying-by-copy2image4tpu: " + str(destination4origin2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                destination4origin2tpu['X'] = 100 
                destination4origin2tpu['Y'] = 100  
                destination4origin2tpu['Z'] = 100                                                                                                                          
                message2print = "{(Step 35B/70) set" + "; destination4origin2tpu-set-by-copy2image4tpu to: " + str(destination4origin2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '35B-' 
            except:
                message2print = "{(Step 35Z/70) failed" + "; destination4origin2tpu-failed-by-copy2image4tpu: " + str(destination4origin2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '35Z-'
        
            task4imagecopy2tpu ['DESTINATIONSOURCEORIGIN'] = destination4origin2tpu                
             
        if 'REGION' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (36) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2tpu = {}
            
            message2print = "{(Step 36A/70) trying" + "; region2tpu-trying-by-copy2image4tpu: " + str(region2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2tpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO WRITE 
                region2tpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO WRITE
                region2tpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO WRITE                                                                                                                          
                message2print = "{(Step 36B/70) set" + "; region2tpu-set-by-copy2image4tpu to: " + str(region2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; region2tpu-failed-by-copy2image4tpu: " + str(region2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '36Z-'
        
            task4imagecopy2tpu ['REGION'] = region2tpu
        
        # ---------------------------------------------------------------------------------------------
        #  EOF PARAMETER CHECKING AND PROVIDING FOR MISSING ONES
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagecopy2tpu.keys()
        message2print = "{(Step 39/70) set" + "; keys2task-set-by-copy2image4tpu to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DESTINATIONORIGIN',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGEDESTINATION',
        #  'IMAGESOURCE',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION',
        #  'SOURCEORIGIN']; time: 01:28:44 10/05/11 Pacific Daylight Time}
    except: 
        message2print = "{(Step 39Z/70) yielded" + "; task4imagecopy2tpu-yielded-by-copy2image4tpu: " + str(task4imagecopy2tpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '39Z-' 

    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2copyimage4tpu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2tpu = False
    channels2tpu = ()
    message2print = "{(Step 40A/70) trying" + "channels2tpu-trying-by-copy2image4tpu: " + str(channels2tpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel2tpu, publisherchannel2tpu = task4imagecopy2tpu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2tpu-got-by-copy2image4tpu: " + str(consumerchannel2tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 612> 

        message2print = "{(Step 40C/70) got" + "; publisherchannel2tpu-got-by-copy2image4tpu: " + str(publisherchannel2tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 616>
                              
        ack4channels2tpu = True
        
        control2copyimage4tpu['CONSUMERCHANNEL']  = consumerchannel2tpu
        control2copyimage4tpu['PUBLISHERCHANNEL'] = publisherchannel2tpu
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2tpu = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2tpu = False
    queue2tpu = None
    message2print = "{(Step 41A/70) trying" + "queue2tpu-trying-by-copy2image4tpu: " + str(queue2tpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2tpu = task4imagecopy2tpu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2tpu-got-by-copy2image4tpu: " + str(queue2tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <queue4xi.Queue4Xi object at 0x0000000004721CF8>
              
        ack4queue2tpu = True
        control2copyimage4tpu['QUEUE']  = queue2tpu        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2tpu = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4tpu = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4tpu-trying-by-copy2image4tpu: " + str(dummy2image4tpu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4tpu = DummyProcess4Xi(target=manoeuvre2copyimage4tpu,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue2tpu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2tpu, publisherchannel2tpu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4tpu-created-by-copy2image4tpu: " + str(dummy2image4tpu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'
        
    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-copy2image4tpu: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-12B-13B-14B-15B-16B-40D-41C- 
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-copy2image4tpu: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #  
        
    message2print = "{(Step 70C/70) returning" + "; control2copyimage4tpu-returning-by-copy2image4tpu: " + str(control2copyimage4tpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004721CF8>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 616>,
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 612>}; time: 01:28:44 10/05/11 Pacific Daylight Time}
   
    return control2copyimage4tpu
                    
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "copy2imagetobuffer4tpu" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   # (60) ESTABLISH CONTROL & COMMAND WITH COPY2IMAGETOBUFFER4TPU                      
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 60A/700) OF TPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH COPY2IMAGETOBUFFER4TPU ', time4xi())    
#   response4copy2imagetobuffer = {} 
#   task4copy2imagetobuffer = None
#   try:                               
#       response4copy2imagetobuffer = copy2imagetobuffer4tpu(task4copy2imagetobuffer)                                                                                                          
#       print('(Step 60B/700) OF TPU2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH COPY2IMAGETOBUFFER4TPU : ', response4copy2imagetobuffer)
#       queue2copyimagetobuffer4tpu      = response4copy2imagetobuffer['QUEUE']
#       channel2copyimagetobufferpublish4tpu = response4copy2imagetobuffer['PUBLISHERCHANNEL']
#       channel2copyimagetobufferconsume4tpu = response4copy2imagetobuffer['CONSUMERCHANNEL']
#   except:
#       print('(Step 60Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH COPY2IMAGETOBUFFER4TPU ', time4xi()) 
#   # --------------------------------------------------------------------------------------------------
#   # (61) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2COPYIMAGETOBUFFER4TPU TO RECEIVE                 
#   # --------------------------------------------------------------------------------------------------      
#   print('(Step 61A/700) OF TPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4TPU ', time4xi())     
#   try:                               
#       message2copyimagetobuffersend4tpu = "<copy4xi>copy2copyimagetobuffer4tpu</copy4xi>"
#       channel2copyimagetobufferpublish4tpu = response4copy2imagetobuffer['PUBLISHERCHANNEL']
#       channel2copyimagetobufferpublish4tpu.send(message2copyimagetobuffersend4tpu)
#       print("(Step 61B/700) OF TPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4TPU: ", channel2copyimagetobufferpublish4tpu, " MESSAGE2SEND4TPU: " , message2copyimagetobuffersend4tpu)                                                                                                
#   except:
#       print('(Step 61Z1/700) OF TPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4TPU ', time4xi())                                                
#   # ------------------------------------------------------------------------------------------
#   # (62) LET MANOEUVRE2COPYIMAGETOBUFFER4TPU TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # ------------------------------------------------------------------------------------------       
#   print("(Step 62A/700) OF TPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4TPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4tpu = manoeuvre2copyimagetobuffer4tpu(queue2copyimagetobuffer4tpu, channel2copyimagetobufferconsume4tpu, channel2copyimagetobufferpublish4tpu)
#       print("(Step 62B/700) OF TPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4TPU ", manoeuvre2copyimagetobuffer4tpu)
#   except:
#       print("(Step 62Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4TPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # ---------------------------------------------------------------------------------------------
#   # (63) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 63A/700) OF TPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4TPU ', time4xi())     
#   try:
#       channel2copyimagetobufferconsume4tpu = response4copy2imagetobuffer['CONSUMERCHANNEL']                              
#       message2copyimagereceive4tpu = channel2copyimagetobufferconsume4tpu.recv()
#       print("(Step 63B/700) OF TPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4TPU: ", channel2copyimagetobufferconsume4tpu, " CONTROL & COMMAND: ", message2copyimagereceive4tpu)                                                                                                  
#       # 
#   except:
#       print('(Step 63Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPY2IMAGETOBUFFER4TPU ', time4xi())         
#   # ---------------------------------------------------------------------------------------------------
#   # (64) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2COPYIMAGETOBUFFER4TPU TO RECEIVE                
#   # ---------------------------------------------------------------------------------------------------      
#   print('(Step 64A/700) OF TPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4TPU ', time4xi())     
#   try:                               
#       message2copyimagetobuffersend4tpu = "<copy4xi>copy2copyimagetobuffer4tpu</copy4xi>"
#       channel2copyimagetobufferpublish4tpu = response4copy2imagetobuffer['PUBLISHERCHANNEL']
#       channel2copyimagetobufferpublish4tpu.send(message2copyimagetobuffersend4tpu)
#       print("(Step 64B/700) OF TPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4TPU: ", channel2copyimagetobufferpublish4tpu, " MESSAGE2SEND4TPU: " , message2copyimagetobuffersend4tpu)                                                                                                
#       #  
#   except:
#       print('(Step 64Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4TPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------
#   # (65) LET MANOEUVRE2COPYIMAGETOBUFFER4TPU TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------       
#   print("(Step 65A/700) OF TPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4TPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4tpu = manoeuvre2copyimagetobuffer4tpu(queue2copyimagetobuffer4tpu, channel2copyimagetobufferconsume4tpu, channel2copyimagetobufferpublish4tpu)
#       print("(Step 65B/700) OF TPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4TPU ", manoeuvre2copyimagetobuffer4tpu)
#   except:
#       print("(Step 65Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4TPU ", time4xi())     
#   # ---------------------------------------------------------------------------------------
#   # (66) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 66A/700) OF TPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4TPU ', time4xi())     
#   try:
#       channel2copyimagetobufferconsume4tpu = response4copy2imagetobuffer['CONSUMERCHANNEL']                              
#       message2copyimagereceive4tpu = channel2copyimagetobufferconsume4tpu.recv()
#       print("(Step 66B/700) OF TPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4TPU: ", channel2copyimagetobufferconsume4tpu, " CONTROL & COMMAND: ", message2copyimagereceive4tpu)                                                                                                  
#       # 
#   except:
#       print('(Step 66Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPY2IMAGETOBUFFER4TPU ', time4xi())          
#   # -----------------------------------------------------------------------------------------
#   # (67) PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 67A/700) OF TPU2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue2copyimagetobuffer4dragon.put(path2dragon)             
#       print("(Step 67B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#       # ..//..//..//..//..//web//images//png//dragonxi.png  
#   except:
#       print("(Step 67Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # ------------------------------------------------------------------------------------------------------
#   # (68) LET MANOEUVRE2COPYIMAGETOBUFFER4TPU TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 68A/700) OF TPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4TPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4tpu = manoeuvre2copyimagetobuffer4tpu(queue2copyimagetobuffer4tpu, channel2copyimagetobufferconsume4tpu, channel2copyimagetobufferpublish4tpu)
#       print("(Step 68B/700) OF TPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4TPU ", manoeuvre2copyimagetobuffer4tpu)
#   except:
#       print("(Step 68Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4TPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (69) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 69A/700) OF TPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2copyimagereceive4tpu = channel2copyimagetobufferconsume4tpu.recv()
#       print("(Step 69D/700) OF TPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4TPU: ", channel2copyimagetobufferconsume4tpu, " CONTROL & COMMAND: ", message2copyimagereceive4tpu)                                                                                                   
#   except:
#       print('(Step 69Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -------------------------------------------------------------------------------------------------------
#   # (70) LET MANOEUVRE2COPYIMAGETOBUFFER4TPU GO TO STATE 0               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 70A/700) OF TPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4TPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4tpu = manoeuvre2copyimagetobuffer4tpu(queue2copyimagetobuffer4tpu, channel2copyimagetobufferconsume4tpu, channel2copyimagetobufferpublish4tpu)
#       print("(Step 70B/700) OF TPU2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4TPU ", response2manoeuvreimagecopytobuffer4tpu)
#   except:
#       print("(Step 70Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4TPU ", time4xi())   

#">

# -------------------------------------------------------------------------------------------------   
def copy2imagetobuffer4tpu(task4copyimagetobuffer2tpu=None):

    '''
    copies 2D images to buffer for TPU
    '''
    global tpu2image4xi                                                                                             # 2D-IMAGES FOR TPU  
    global tpu2buffer4xi                                                                                            # BUFFER FOR TPU
    
    global control2copyimagetobuffer4tpu                                                                            # COMMAND AND CONTROL FOR TPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "copy2imagetobuffer4tpu"
    method2print = "copy2imagetobuffer4tpu"

    message2print = "{(Step 1A/33) started" + "; copy2imagetobuffer4tpu-started: " + str(copy2imagetobuffer4tpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                 
    # <function copy2imagetobuffer4tpu at 0x000000000390B948>
    
    message2print = "{(Step 1B/33) passed" + "; task4copyimagetobuffer2tpu-to-copy2imagetobuffer4tpu: " + str(task4copyimagetobuffer2tpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE TPU2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if tpu2image4xi is None:
            tpu2image4xi = create2image4tpu()                                                                              # INITIALIZE IMAGE FOR TPU 
            message2print = "{(Step 1C/70) created" + "; tpu2image4xi-created-by-copy2imagetobuffer4tpu to: " + str(tpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4tpu-skipped-by-copy2imagetobuffer4tpu: " + str(copy2imagetobuffer4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # create2image4tpu-skipped-by-copy2imagetobuffer4tpu                                 
    except:
        try:
            tpu2image4xi = create2image4tpu()
            message2print = "{(Step 1E/70) set" + "; tpu2image4xi-set-by-copy2imagetobuffer4tpu to: " + str(tpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            tpu2image4xi = {}
            message2print = "{(Step 1Z1/70) yielded" + "; tpu2image4xi-yielded-by-copy2imagetobuffer4tpu: " + str(tpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z1-' 
    
    # -----------------------------------------------------------------------------------------
    # CREATE TPU2BUFFER4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if tpu2buffer4xi is None:
            tpu2buffer4xi = create2buffer4tpu()                                                                     # INITIALIZE BUFFER FOR TPU 
            message2print = "{(Step 1F/70) created" + "; tpu2buffer4xi-created-by-copy2imagetobuffer4tpu to: " + str(tpu2buffer4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1G/70) skipped" + "; create2buffer4tpu-skipped-by-copy2imagetobuffer4tpu: " + str(copy2imagetobuffer4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                                  
    except:
        try:
            tpu2buffer4xi = create2buffer4tpu()
            message2print = "{(Step 1H/70) set" + "; tpu2buffer4xi-set-by-copy2imagetobuffer4tpu to: " + str(tpu2buffer4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'CODE2ERROR': 0,
            #  'COUNT2REFERENCE': 0,
            #   'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4tpu-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}',
            #   'CLASS': 'create2buffer4tpu',
            #   'METHOD': 'create2buffer4tpu'},
            #   'FUNCTION2CALLBACK': <function trace2buffer4tpu at 0x0000000004782B48>,
            #   'POINTER2BUFFER': <mmap.mmap object at 0x000000000423BD98>,
            #   'FLAGS': 0,
            #   'CONTEXT': {'COUNT2REFERENCE': 0,
            #   'DATA2CALLBACK': ...
            
        except:
            tpu2buffer4xi = {}
            message2print = "{(Step 1Z2/70) yielded" + "; tpu2buffer4xi-yielded-by-copy2imagetobuffer4tpu: " + str(tpu2buffer4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z2-' 
                            
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - ANALYZE TASK          
    # -----------------------------------------------------------------------------------------          
              
    if task4copyimagetobuffer2tpu is None:
        # -----------------------------------------------------------------------------------------
        # NO PARAMETERS GIVEN BY CALLER FOR COPY - BUILD TASK! 
        # -----------------------------------------------------------------------------------------    
            
        task4copyimagetobuffer2tpu = {}                                                                                                                                                                                       
            
        message2print = "{(Step 1A/70) initialized" + "; task4copyimagetobuffer2tpu-created-by-copy2imagetobuffer4tpu to: " + str(task4copyimagetobuffer2tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # 
            
        # ---------------------------------------------------------------------------------------------
        # (1B) TRACE - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        function2callback4tpu = trace2image4tpu                                                                 # CALLBACK FUNCTION FOR TPU
            
        message2print = "{(Step 1B1/70) initialized" + "; function2callback4tpu-created-by-copy2imagetobuffer4tpu to: " + str(function2callback4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <function trace2image4tpu at 0x0000000004797548> 
            
        code2ack  = code2ack + '1B1-'
           
        args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
        args2callback['CLASS']   = "copy2imagetobuffer4tpu"                                                            # MODULE OF TPUIZED OPENCL MODULE
        args2callback['METHOD']  = "copy2imagetobuffer4tpu"                                                            # FUNCTION OF TPUIZED OPENCL MODULE
        args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2imagetobuffer4tpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR TPUIZED OPENCL TO RETURN    

        message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-copy2imagetobuffer4tpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # {'MESSAGE': '{(Step 1Z1/33) failed; copy2imagetobuffer4tpu-failed; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
        #  'CLASS': 'copy2imagetobuffer4tpu', 
        #  'METHOD': 'copy2imagetobuffer4tpu'}; time: 03:40:47 10/04/11 Pacific Daylight Time}


        code2ack  = code2ack + '1B2-'
            
        # ---------------------------------------------------------------------------------------------
        # (1C) CALLBACK - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        task4copyimagetobuffer2tpu['DATA2CALLBACK'] = args2callback                                                    # USER DATA PASSED TO TPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
        task4copyimagetobuffer2tpu['FUNCTION2CALLBACK'] = function2callback4tpu                                        # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
        
        code2ack  = code2ack + '1C-'
        message2print = "{(Step 1C/70) initialized" + "; task4copyimagetobuffer2tpu-created-by-copy2imagetobuffer4tpu to: " + str(task4copyimagetobuffer2tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # {'FUNCTION2CALLBACK': <function trace2image4tpu at 0x0000000004797548>,
        #   'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; copy2imagetobuffer4tpu-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
        #  'CLASS': 'copy2imagetobuffer4tpu', 
        #  'METHOD': 'copy2imagetobuffer4tpu'}}; time: 03:40:47 10/04/11 Pacific Daylight Time}

            
        # ---------------------------------------------------------------------------------------------
        # (2) PLATFORM - AUTOMATIC                   
        # ---------------------------------------------------------------------------------------------             
        platform2xi = {}
        message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-copy2imagetobuffer4tpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # {}
            
        try:
            platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
            message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-copy2imagetobuffer4tpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 03:40:47 10/04/11 Pacific Daylight Time}
                 
            code2ack  = code2ack + '2B-'
        except:
            message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-copy2imagetobuffer4tpu: " + str(failed)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                                
            code2nack  = code2nack + '2Z-'
                 
        task4copyimagetobuffer2tpu['PLATFORM'] = platform2xi                                                           # PLATFORM OF SYSTEM 
              
            
        # ---------------------------------------------------------------------------------------------
        # (3) CONTEXT - AUTOMATIC                 
        # ---------------------------------------------------------------------------------------------    
        context2tpu = {}
        message2print = "{(Step 3A/70) trying" + "; context2tpu-trying-by-copy2imagetobuffer4tpu: " + str(context2tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            context2tpu = get2context4tpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
            message2print = "{(Step 3B/70) got" + "; context2tpu-got-by-copy2imagetobuffer4tpu: " + str(context2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'COUNT2REFERENCE': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4tpu-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2context4tpu', 
            #  'METHOD': 'create2context4tpu'}, 
            #  'FUNCTION2CALLBACK': <function trace2context4tpu at 0x000000000394F348>, 
            #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 
            #  'INFO2PRIVATE': None, 
            #  'PLATFORM': {'list2platforms': ['DragonXi'],'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
            #  'NACK2RETURN': ' ', 
            #  'SIZE2PRIVATE': 0, 
            #  'PROPERTIES': None}; time: 03:40:47 10/04/11 Pacific Daylight Time}
                
            code2ack  = code2ack + '3B-'         
        except:
            message2print = "{(Step 3Z/70) failed" + "; context2tpu-failed-by-copy2imagetobuffer4tpu: " + str(context2tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z-'
            
        task4copyimagetobuffer2tpu ['CONTEXT'] = context2tpu           
            
        # ---------------------------------------------------------------------------------------------
        # (4) DEVICE - AUTOMATIC                       
        # ---------------------------------------------------------------------------------------------    
        device2tpu = {}
            
        message2print = "{(Step 4A/70) trying" + "; device2tpu-trying-by-copy2imagetobuffer4tpu: " + str(device2tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            device2tpu = get2tpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
            
            message2print = "{(Step 4B/70) got" + "; device2tpu-got-by-copy2imagetobuffer4tpu: " + str(device2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'RESOLUTION2TIMER4PROFILING': 318, 
            #  'ID2PLATFORM': '0000000001509568', 
            #  'PROFILING2QUEUE': 'ACK', 
            #  'EXECUTE2KERNELS4OPENCL': 'ACK', 
            #  'NATIVE2WIDTHLONG4VECTOR': 2, 
            #  'MAX2UNITS4COMPUTE': 4, 
            #  'SIZE2MEMORY4GLOBAL': 3221225472, 
            #  'NATIVE2WIDTHFLOAT4VECTOR': 4, 
            #  'MAX2SIZE4WORKGROUP': 1024, 
            #  'PREFERRED2WIDTHCHAR4VECTOR': 16, 
            #  'SIZE2CACHE': 65536, 
            #  'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK', 
            #  'TYPE4CACHE': 'Read/Write', 
            #  'PREFERRED2WIDTHINT4VECTOR': 4, 
            #  'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1, 
            #  'NATIVE2WIDTHINT4VECTOR': 4, 
            #  'SIZE2BUFFER4CONSTANT': 65536, 
            #  'EXECUTE2FUNCTION4NATIVE': 'ACK', 
            #  'PREFERRED2WIDTHDOUBLE4VECTOR': 0, 
            #  'MAX2NUMBER4CONSTANTARGS': 8, 
            #  'VERSION2DRIVER': '2.0', 
            #  'EXTENSIONS': 'cl_amd_fp64 ...
            #  'AVAILABLE2COMPILER': 'ACK', 
            #  'MAX2DIMENSIONS4WORKITEMS': 3, 
            #  'SUPPORT2CORRECTION4ERROR': 0, 
            #  'BITS2ADDRESS': 64, 
            #  'AVAILABLE': 'ACK', 
            #  'SIZE2MEMORY4LOCAL': 32768, 
            #  'VENDOR': 'AuthenticAMD', 
            #  'SUPPORT2IMAGE': 'NACK', 
            #  'MAX2SIZE4KERNELARGUMENT': 4096, 
            #  'ACK2RETURN': ' 1C-1D-1E-1F-1D-2A-...
            #  'OUT-OF-ORDER2QUEUE': 'NACK', 
            #  'ALIGNMENT2ADDRESS4BASE': 1024, 
            #  'PROFILE': 'FULL_PROFILE', 
            #  'TYPE2MEMORY4LOCAL': 'Global', 
            #  'PREFERRED2WIDTHFLOAT4VECTOR': 4, 
            #  'PREFERRED2WIDTHLONG4VECTOR': 2, 
            #  'MAX2FREQUENCY4CLOCK': 3210, 
            #  'PREFERRED2WIDTHSHORT4VECTOR': 8, 
            #  'ID': 4033, 
            #  'MAX2WORKITEM2': 1024, 
            #  'NATIVE2WIDTHDOUBLE4VECTOR': 0, 
            #  'ROUND2ZERO': 'ACK', 
            #  'NAME': 'AMD64 Family 16 Model 4 Stepping 2, 
            #   AuthenticAMD', 
            #  'MAX2WORKITEM1': 1024, 
            #  'MAX2WORKITEM0': 1024, 
            #  'SIZE2LINE4CACHE': 64, 
            #  'MAX2ALLOCATION4MEMORY': 1073741824, 
            #  'NATIVE2WIDTHCHAR4VECTOR': 16, 
            #  'QUIET2NANS': 'ACK', 
            #  'NATIVE2WIDTHSHORT4VECTOR': 8, 
            #  'ROUND2NEAREST4EVEN': 'ACK', 
            #  'VERSION': 'OpenCL 1.1 ATI-Stream-v2.3 (451)', 
            #  'NACK2RETURN': ' ', 
            #  'MEMORY2UNIFIED4HOSTANDDEVICE': 1, 
            #  'ENDIANESS2DEVICE': 'Little', 
            #  'MINIMUM2ALIGNMENT4DATATYPE': 128, 
            #  'ROUND2+VE3INFINITY': 'ACK', 
            #  'TYPE': 'CL_DEVICE_TYPE_TPU', 
            #  'DENORMS2SINGLEPRECISION4FLOATINGPOINTCAPABILITY': 'ACK'}; time: 03:40:47 10/04/11 Pacific Daylight Time}
            
            code2ack  = code2ack + '4B-'  
        except:
            message2print = "{(Step 4Z/70) failed" + "; device2tpu-failed-by-copy2imagetobuffer4tpu: " + str(device2tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z-'
        
        task4copyimagetobuffer2tpu ['DEVICE'] = device2tpu
        
               
        # ---------------------------------------------------------------------------------------------
        # (5) IMAGE SOURCE OBJECT - REQUIRED                         
        # ---------------------------------------------------------------------------------------------         
        imagesourceinfo2tpu = {}
         
        message2print = "{(Step 5A/70) trying" + "; imagesourceinfo2tpu-trying-by-copy2imagetobuffer4tpu: " + str(imagesourceinfo2tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            imagesourceinfo2tpu = get2imageinfo4tpu()                                                                         
            
            message2print = "{(Step 5B/70) got" + "; imagesourceinfo2tpu-got-by-copy2imagetobuffer4tpu: " + str(imagesourceinfo2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'SIZE2QUEUE': 2147483646, 
            #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
            #  'ORDER': 'CL_RGBA'}, 
            #  'QUEUE': <queue4xi.Queue4Xi object at 0x00000000042514E0>, 
            #  'WIDTH': 800, 
            #  'CONTEXT': ...

            code2ack  = code2ack + '5B-'  
        except:
            message2print = "{(Step 5Z/70) failed" + "; imagesourceinfo2tpu-failed-by-copy2imagetobuffer4tpu: " + str(imagesourceinfo2tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '5Z-'          
                  
        task4copyimagetobuffer2tpu ['IMAGESOURCE'] = imagesourceinfo2tpu         
               
        # ---------------------------------------------------------------------------------------------
        # (6) BUFFER DESTINATION OBJECT - REQUIRED                        
        # ---------------------------------------------------------------------------------------------                         
        bufferdestinationinfo2tpu = {}
         
        message2print = "{(Step 6A/70) trying" + "; bufferdestinationinfo2tpu-trying-by-copy2imagetobuffer4tpu: " + str(bufferdestinationinfo2tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            bufferdestinationinfo2tpu = get2bufferinfo4tpu()                                                                         
            
            message2print = "{(Step 6B/70) got" + "; bufferdestinationinfo2tpu-got-by-copy2imagetobuffer4tpu: " + str(bufferdestinationinfo2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'CODE2ERROR': 0, 
            #  'COUNT2REFERENCE': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4tpu-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2buffer4tpu', 
            #  'METHOD': 'create2buffer4tpu'}, 
            #  'FUNCTION2CALLBACK': <function trace2buffer4tpu at 0x0000000004782B48>, 
            #  'POINTER2BUFFER': <mmap.mmap object at 0x000000000423BD98>, 
            #  'FLAGS': 0, 
            #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
            code2ack  = code2ack + '6B-'  
        except:
            message2print = "{(Step 6Z/70) failed" + "; bufferdestinationinfo2tpu-failed-by-copy2imagetobuffer4tpu: " + str(bufferdestinationinfo2tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '6Z-'          
                  
        task4copyimagetobuffer2tpu ['BUFFERDESTINATION'] = bufferdestinationinfo2tpu     
        
        # ---------------------------------------------------------------------------------------------
        # (7) QUEUE - REQUIRED BY TPUXI                          
        # ---------------------------------------------------------------------------------------------         
        queue2copyimagetobuffer4tpu = None
        
        message2print = "{(Step 7A/70) trying" + "; queue2copyimagetobuffer4tpu-trying-by-copy2imagetobuffer4tpu: " + str(queue2copyimagetobuffer4tpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            queue2copyimagetobuffer4tpu = Queue4Xi()                                                                              # THE SYSTEM CREATES QUEUE    
            
            message2print = "{(Step 7B/70) got" + "; queue2copyimagetobuffer4tpu-got-by-copy2imagetobuffer4tpu: " + str(queue2copyimagetobuffer4tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # <queue4xi.Queue4Xi object at 0x00000000042B2208> 
            
            code2ack  = code2ack + '7B-'  
        except:
            message2print = "{(Step 7Z/70) failed" + "; queue2copyimagetobuffer4tpu-failed-by-copy2imagetobuffer4tpu: " + str(queue2copyimagetobuffer4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '7Z-'
            
        task4copyimagetobuffer2tpu ['QUEUE'] = queue2copyimagetobuffer4tpu
         
        # ---------------------------------------------------------------------------------------------
        # (8) PIPE - REQUIRED BY TPUXI                              
        # ---------------------------------------------------------------------------------------------         
        pipe2tpu = None
       
        message2print = "{(Step 8A/70) trying" + "; pipe2tpu-trying-by-copy2imagetobuffer4tpu: " + str(pipe2tpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            pipe2tpu = Pipe4Xi()                                                                                # THE SYSTEM CREATES PIPE    
            
            message2print = "{(Step 8B/70) got" + "; pipe2tpu-got-by-copy2imagetobuffer4tpu: " + str(pipe2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # (<read-write PipeConnection, handle 660>, <read-write PipeConnection, handle 664>)
            
            code2ack  = code2ack + '8B-'  
        except:
            message2print = "{(Step 8Z/70) failed" + "; pipe2tpu-failed-by-copy2imagetobuffer4tpu: " + str(pipe2tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '8Z-'
            
        task4copyimagetobuffer2tpu ['PIPE'] = pipe2tpu 
        
        # ---------------------------------------------------------------------------------------------
        # (9) COMMAND QUEUE - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        commandqueueinfo2tpu = {}
       
        message2print = "{(Step 9A/70) trying" + "; commandqueueinfo2tpu-trying-by-copy2imagetobuffer4tpu: " + str(commandqueueinfo2tpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
                    
        try:
            commandqueueinfo2tpu = get2commandqueueinfo4tpu()                                                   # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
            message2print = "{(Step 9B/70) got" + "; commandqueueinfo2tpu-got-by-copy2imagetobuffer4tpu: " + str(commandqueueinfo2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 
            #  'COUNT2REFERENCE': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4tpu-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2commandqueue4tpu', 
            #  'METHOD': 'create2commandqueue4tpu'}, 
            #  'FUNCTION2CALLBACK': <function trace2commandqueue4tpu at 0x000000000395EA48>, 
            #  'ACK2RETURN': ...
            
            code2ack  = code2ack + '9B-'  
        except:
            message2print = "{(Step 9Z/70) failed" + "; commandqueueinfo2tpu-failed-by-copy2imagetobuffer4tpu: " + str(commandqueueinfo2tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '9Z-'
    
        task4copyimagetobuffer2tpu ['COMMANDQUEUE'] = commandqueueinfo2tpu
        
        
        
        # ---------------------------------------------------------------------------------------------
        # (11) EVENT - REQUIRED                            
        # ---------------------------------------------------------------------------------------------             
        eventinfo2tpu = {}
       
        message2print = "{(Step 11A/70) trying" + "; eventinfo2tpu-trying-by-copy2imagetobuffer4tpu: " + str(eventinfo2tpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventinfo2tpu = get2eventinfo4tpu()                                                                    
            message2print = "{(Step 11B/70) got" + "; eventinfo2tpu-got-by-copy2imagetobuffer4tpu: " + str(eventinfo2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4tpu-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2event4tpu', 
            #  'METHOD': 'create2event4tpu'}, 
            #  'FUNCTION2CALLBACK': <function trace2event4tpu at 0x0000000003971F48>, 
            #  'ACK2RETURN': ' 1D-1E-...
            code2ack  = code2ack + '11B-' 
        except:
            message2print = "{(Step 11Z/70) failed" + "; eventinfo2tpu-failed-by-copy2imagetobuffer4tpu: " + str(eventinfo2tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '11Z-'
    
        task4copyimagetobuffer2tpu ['EVENT'] = eventinfo2tpu 
        
        
        # ---------------------------------------------------------------------------------------------
        # (12) EVENTWAITLIST - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlist2tpu = {}
       
        message2print = "{(Step 12A/70) trying" + "; eventwaitlist2tpu-trying-by-copy2imagetobuffer4tpu: " + str(eventwaitlist2tpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventwaitlist2tpu = {}                                                                   
            message2print = "{(Step 12B/70) set" + "; eventwaitlist2tpu-set-by-copy2imagetobuffer4tpu: " + str(eventwaitlist2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {}
            
            code2ack  = code2ack + '12B-' 
        except:
            message2print = "{(Step 12Z/70) failed" + "; eventwaitlist2tpu-failed-by-copy2imagetobuffer4tpu: " + str(eventwaitlist2tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '12Z-'
    
        task4copyimagetobuffer2tpu ['EVENTWAITLIST'] = eventwaitlist2tpu                     
         
        # ---------------------------------------------------------------------------------------------
        # (13) EVENTWAITLISTQUANTITY - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlistquantity2tpu = 0
       
        message2print = "{(Step 13A/70) trying" + "; eventwaitlistquantity2tpu-trying-by-copy2imagetobuffer4tpu: " + str(eventwaitlistquantity2tpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            keys2event = eventwaitlist2tpu.keys()
            for key2event in keys2event:
                eventwaitlistquantity2tpu = eventwaitlistquantity2tpu + 1                                                                      
            message2print = "{(Step 13B/70) set" + "; eventwaitlistquantity2tpu-set-by-copy2imagetobuffer4tpu: " + str(eventwaitlistquantity2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 0 
            
            code2ack  = code2ack + '13B-' 
        except:
            message2print = "{(Step 13Z/70) failed" + "; eventwaitlistquantity2tpu-failed-by-copy2imagetobuffer4tpu: " + str(eventwaitlistquantity2tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '13Z-'
    
        task4copyimagetobuffer2tpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2tpu       
                                             
        
        # ---------------------------------------------------------------------------------------------
        # (14) ORIGIN FOR SOURCE - REQUIRED                             
        # ---------------------------------------------------------------------------------------------             
        origin4source2tpu = {}
        
        message2print = "{(Step 14A/70) trying" + "; origin4source2tpu-trying-by-copy2imagetobuffer4tpu: " + str(origin4source2tpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:                                                                                    
            origin4source2tpu['X'] = 0 
            origin4source2tpu['Y'] = 0  
            origin4source2tpu['Z'] = 0                                                                                                                          
            message2print = "{(Step 14B/70) set" + "; origin4source2tpu-set-by-copy2imagetobuffer4tpu to: " + str(origin4source2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'Y': 0, 'X': 0, 'Z': 0} 
            code2ack  = code2ack + '14B-' 
        except:
            message2print = "{(Step 14Z/70) failed" + "; origin4source2tpu-failed-by-copy2imagetobuffer4tpu: " + str(origin4source2tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '14Z-'
    
        task4copyimagetobuffer2tpu ['SOURCEORIGIN'] = origin4source2tpu
        
       
        
        # --------------------------------------------------------------------------------------------------------
        # (16) REGION - REGION OF A IMAGE TO BE COPIED                         
        # --------------------------------------------------------------------------------------------------------             
        region4copy2tpu = {}                                                                                                         
       
        message2print = "{(Step 16A/70) trying" + "; region4copy2tpu-trying-by-copy2imagetobuffer4tpu: " + str(region4copy2tpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            region4copy2tpu['X'] = 10 
            region4copy2tpu['Y'] = 10  
            region4copy2tpu['Z'] = 10                                                                           
            message2print = "{(Step 16B/70) set" + "; region4copy2tpu-set-by-copy2imagetobuffer4tpu to: " + str(region4copy2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'Y': 10, 'X': 10, 'Z': 10} 
            code2ack  = code2ack + '16B-'                
             
        except:
            message2print = "{(Step 16Z/70) failed" + "; region4copy2tpu-failed-by-copy2imagetobuffer4tpu: " + str(region4copy2tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '16Z-'
        
        task4copyimagetobuffer2tpu ['REGION'] = region4copy2tpu              
        
        # --------------------------------------------------------------------------------------------------------
        # (17) OFFSET - WHERE TO BEGIN COPYING DATA INTO DESTINATION BUFFER                     
        # --------------------------------------------------------------------------------------------------------             
        offset4copy2tpu = 0                                                                                                         
        message2print = "{(Step 17A/70) set" + "; offset4copy2tpu-set-by-copy2imagetobuffer4tpu to: " + str(offset4copy2tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 0
        code2ack  = code2ack + '17A-'                
        task4copyimagetobuffer2tpu ['OFFSET'] = offset4copy2tpu            
                    
        # ---------------------------------------------------------------------------------------------
        # EOF PARAMETER PROVIDING                
        # ---------------------------------------------------------------------------------------------                                                                                                                               
    else:
        pass
                    
    # ---------------------------------------------------------------------------------------------
    # ANALYZE TASK - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
    # ---------------------------------------------------------------------------------------------
    keys2task = task4copyimagetobuffer2tpu.keys()
    message2print = "{(Step 20/70) got" + "; keys2task-got-by-copy2imagetobuffer4tpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)            
    # ['BUFFERDESTINATION',
    #  'COMMANDQUEUE',
    #  'CONTEXT',
    #  'DATA2CALLBACK',
    #  'DEVICE',
    #  'EVENT',
    #  'EVENTWAITLIST',
    #  'EVENTWAITLISTQUANTITY',
    #  'FUNCTION2CALLBACK',
    #  'IMAGESOURCE',
    #  'OFFSET',
    #  'PIPE',
    #  'PLATFORM',
    #  'QUEUE',
    #  'REGION',
    #  'SOURCEORIGIN']; time: 01:28:44 10/05/11 Pacific Daylight Time}


    if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
        pass
    else:                
        # ---------------------------------------------------------------------------------------------
        # (21B) TRACE - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        function2callback4tpu = trace2image4tpu                                                             # CALLBACK FUNCTION FOR TPU
        
        message2print = "{(Step 21B1/70) initialized" + "; function2callback4tpu-created-by-copy2imagetobuffer4tpu to: " + str(function2callback4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
            
        code2ack  = code2ack + '21B1-'
           
        args2callback = {}                                                                                  # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
        args2callback['CLASS']   = "copy2imagetobuffer4tpu"                                                        # MODULE OF TPUIZED OPENCL MODULE
        args2callback['METHOD']  = "copy2imagetobuffer4tpu"                                                        # FUNCTION OF TPUIZED OPENCL MODULE
        args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2imagetobuffer4tpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR TPUIZED OPENCL TO RETURN    

        message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-copy2imagetobuffer4tpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # 

        code2ack  = code2ack + '21B2-'
        
        # ---------------------------------------------------------------------------------------------
        # (21C) CALLBACK - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        task4copyimagetobuffer2tpu['DATA2CALLBACK'] = args2callback                                                # USER DATA PASSED TO TPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
        task4copyimagetobuffer2tpu['FUNCTION2CALLBACK'] = function2callback4tpu                                    # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
        
        code2ack  = code2ack + '21C-'
        message2print = "{(Step 21C/70) initialized" + "; task4copyimagetobuffer2tpu-created-by-copy2imagetobuffer4tpu to: " + str(task4copyimagetobuffer2tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #
             
    if 'PLATFORM' in keys2task:
        pass
    else:  
        # ---------------------------------------------------------------------------------------------
        # (22) PLATFORM - AUTOMATIC                   
        # ---------------------------------------------------------------------------------------------             
        platform2xi = {}
        message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-copy2imagetobuffer4tpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        try:
            platform2xi = get2platformids4xi()                                                              # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
            message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-copy2imagetobuffer4tpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
                 
            code2ack  = code2ack + '22B-'
        except:
            message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-copy2imagetobuffer4tpu: " + str(failed)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                                
            code2nack  = code2nack + '22Z-'
                 
        task4copyimagetobuffer2tpu['PLATFORM'] = platform2xi                                                       # PLATFORM OF SYSTEM 
          
    if 'CONTEXT' in keys2task:
        pass
    else:              
        # ---------------------------------------------------------------------------------------------
        # (23) CONTEXT - AUTOMATIC                 
        # ---------------------------------------------------------------------------------------------    
        context2tpu = {}
        message2print = "{(Step 23A/70) trying" + "; context2tpu-trying-by-copy2imagetobuffer4tpu: " + str(context2tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            context2tpu = get2context4tpu()                                                                 # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
            
            message2print = "{(Step 23B/70) got" + "; context2tpu-got-by-copy2imagetobuffer4tpu: " + str(context2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
            
            code2ack  = code2ack + '23B-'         
        except:
            message2print = "{(Step 23Z/70) failed" + "; context2tpu-failed-by-copy2imagetobuffer4tpu: " + str(context2tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '23Z-'
            
        task4copyimagetobuffer2tpu ['CONTEXT'] = context2tpu           
    
    if 'DEVICE' in keys2task:
        pass
    else:                   
        # ---------------------------------------------------------------------------------------------
        # (24) DEVICE - AUTOMATIC                       
        # ---------------------------------------------------------------------------------------------    
        device2tpu = {}
        
        message2print = "{(Step 24A/70) trying" + "; device2tpu-trying-by-copy2imagetobuffer4tpu: " + str(device2tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            device2tpu = get2tpu4xi()                                                                       # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
            
            message2print = "{(Step 24B/70) got" + "; device2tpu-got-by-copy2imagetobuffer4tpu: " + str(device2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
                
            code2ack  = code2ack + '24B-'  
        except:
            message2print = "{(Step 24Z/70) failed" + "; device2tpu-failed-by-copy2imagetobuffer4tpu: " + str(device2tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '24Z-'
        
        task4copyimagetobuffer2tpu ['DEVICE'] = device2tpu
    
    if 'IMAGESOURCE' in keys2task:
        pass
    else:                      
        # ---------------------------------------------------------------------------------------------
        # (25) SOURCE IMAGE - REQUIRED                            
        # ---------------------------------------------------------------------------------------------         
        sourceimageinfo2tpu = {}
         
        message2print = "{(Step 25A/70) trying" + "; sourceimageinfo2tpu-trying-by-copy2imagetobuffer4tpu: " + str(sourceimageinfo2tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            sourceimageinfo2tpu = get2imageinfo4tpu()                                                                         
            
            message2print = "{(Step 25B/70) got" + "; sourceimageinfo2tpu-got-by-copy2imagetobuffer4tpu: " + str(sourceimageinfo2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            #                 
            code2ack  = code2ack + '25B-'  
        except:
            message2print = "{(Step 25Z/70) failed" + "; sourceimageinfo2tpu-failed-by-copy2imagetobuffer4tpu: " + str(sourceimageinfo2tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '25Z-'          
                  
        task4copyimagetobuffer2tpu ['IMAGESOURCE'] = sourceimageinfo2tpu         
    
    if 'BUFFERDESTINATION' in keys2task:
        pass
    else:                      
        # ---------------------------------------------------------------------------------------------
        # (26) DESTINATION BUFFER - REQUIRED                            
        # ---------------------------------------------------------------------------------------------         
        destinationbufferinfo2tpu = {}
         
        message2print = "{(Step 26A/70) trying" + "; destinationbufferinfo2tpu-trying-by-copy2imagetobuffer4tpu: " + str(destinationbufferinfo2tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            destinationbufferinfo2tpu = get2bufferinfo4tpu()                                                                         
            
            message2print = "{(Step 26B/70) got" + "; destinationbufferinfo2tpu-got-by-copy2imagetobuffer4tpu: " + str(destinationbufferinfo2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            #                 
            code2ack  = code2ack + '26B-'  
        except:
            message2print = "{(Step 26Z/70) failed" + "; destinationbufferinfo2tpu-failed-by-copy2imagetobuffer4tpu: " + str(destinationbufferinfo2tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '26Z-'          
                  
        task4copyimagetobuffer2tpu ['BUFFERDESTINATION'] = destinationbufferinfo2tpu    
                                
    if 'QUEUE' in keys2task:
        pass
    else:   
        # ---------------------------------------------------------------------------------------------
        # (27) QUEUE - REQUIRED BY TPUXI                          
        # ---------------------------------------------------------------------------------------------         
        queue2copyimagetobuffer4tpu = None
        
        message2print = "{(Step 27A/70) trying" + "; queue2copyimagetobuffer4tpu-trying-by-copy2imagetobuffer4tpu: " + str(queue2copyimagetobuffer4tpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            queue2copyimagetobuffer4tpu = Queue4Xi()                                                                          # THE SYSTEM CREATES QUEUE    
            
            message2print = "{(Step 27B/70) got" + "; queue2copyimagetobuffer4tpu-got-by-copy2imagetobuffer4tpu: " + str(queue2copyimagetobuffer4tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
            
            code2ack  = code2ack + '27B-'  
        except:
            message2print = "{(Step 27Z/70) failed" + "; queue2copyimagetobuffer4tpu-failed-by-copy2imagetobuffer4tpu: " + str(queue2copyimagetobuffer4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '27Z-'
            
        task4copyimagetobuffer2tpu ['QUEUE'] = queue2copyimagetobuffer4tpu
        
    if 'PIPE' in keys2task:
        pass
    else:              
        # ---------------------------------------------------------------------------------------------
        # (28) PIPE - REQUIRED BY TPUXI                              
        # ---------------------------------------------------------------------------------------------         
        pipe2tpu = None
       
        message2print = "{(Step 28A/70) trying" + "; pipe2tpu-trying-by-copy2imagetobuffer4tpu: " + str(pipe2tpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            pipe2tpu = Pipe4Xi()                                                                            # THE SYSTEM CREATES PIPE    
            
            message2print = "{(Step 28B/70) got" + "; pipe2tpu-got-by-copy2imagetobuffer4tpu: " + str(pipe2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
            
            code2ack  = code2ack + '28B-'  
        except:
            message2print = "{(Step 28Z/70) failed" + "; pipe2tpu-failed-by-copy2imagetobuffer4tpu: " + str(pipe2tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '28Z-'
            
        task4copyimagetobuffer2tpu ['PIPE'] = pipe2tpu 
        
    if 'COMMANDQUEUE' in keys2task:
        pass
    else:              
        # ---------------------------------------------------------------------------------------------
        # (28) COMMAND QUEUE - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        commandqueueinfo2tpu = {}
       
        message2print = "{(Step 29A/70) trying" + "; commandqueueinfo2tpu-trying-by-copy2imagetobuffer4tpu: " + str(commandqueueinfo2tpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
                    
        try:
            commandqueueinfo2tpu = get2commandqueueinfo4tpu()                                               # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
            message2print = "{(Step 29B/70) got" + "; commandqueueinfo2tpu-got-by-copy2imagetobuffer4tpu: " + str(commandqueueinfo2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code2ack  = code2ack + '29B-'  
        except:
            message2print = "{(Step 29Z/70) failed" + "; commandqueueinfo2tpu-failed-by-copy2imagetobuffer4tpu: " + str(commandqueueinfo2tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '29Z-'
    
        task4copyimagetobuffer2tpu ['COMMANDQUEUE'] = commandqueueinfo2tpu
    
         
    if 'EVENT' in keys2task:
        pass
    else:               
        # ---------------------------------------------------------------------------------------------
        # (31) EVENT - REQUIRED                            
        # ---------------------------------------------------------------------------------------------             
        eventinfo2tpu = {}
       
        message2print = "{(Step 31A/70) trying" + "; eventinfo2tpu-trying-by-copy2imagetobuffer4tpu: " + str(eventinfo2tpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventinfo2tpu = get2eventinfo4tpu()                                                                    
            message2print = "{(Step 31B/70) got" + "; eventinfo2tpu-got-by-copy2imagetobuffer4tpu: " + str(eventinfo2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code2ack  = code2ack + '31B-' 
        except:
            message2print = "{(Step 31Z/70) failed" + "; eventinfo2tpu-failed-by-copy2imagetobuffer4tpu: " + str(eventinfo2tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '31Z-'
    
        task4copyimagetobuffer2tpu ['EVENT'] = eventinfo2tpu 
    
         
    if 'EVENTWAITLIST' in keys2task:
        pass
    else:               
        # ---------------------------------------------------------------------------------------------
        # (32) EVENTWAITLIST - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlist2tpu = {}
       
        message2print = "{(Step 32A/70) trying" + "; eventwaitlist2tpu-trying-by-copy2imagetobuffer4tpu: " + str(eventwaitlist2tpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventwaitlist2tpu = {}                                                                   
            message2print = "{(Step 32B/70) set" + "; eventwaitlist2tpu-set-by-copy2imagetobuffer4tpu: " + str(eventwaitlist2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code2ack  = code2ack + '32B-' 
        except:
            message2print = "{(Step 32Z/70) failed" + "; eventwaitlist2tpu-failed-by-copy2imagetobuffer4tpu: " + str(eventwaitlist2tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '32Z-'
    
        task4copyimagetobuffer2tpu ['EVENTWAITLIST'] = eventwaitlist2tpu                     
          
    if 'EVENTWAITLISTQUANTITY' in keys2task:
        pass
    else:               
        # ---------------------------------------------------------------------------------------------
        # (33) EVENTWAITLISTQUANTITY - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlistquantity2tpu = 0
       
        message2print = "{(Step 33A/70) trying" + "; eventwaitlistquantity2tpu-trying-by-copy2imagetobuffer4tpu: " + str(eventwaitlistquantity2tpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            keys2event = eventwaitlist2tpu.keys()
            for key2event in keys2event:
                eventwaitlistquantity2tpu = eventwaitlistquantity2tpu + 1                                                                   
            message2print = "{(Step 33B/70) set" + "; eventwaitlistquantity2tpu-set-by-copy2imagetobuffer4tpu: " + str(eventwaitlistquantity2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 0 
            
            code2ack  = code2ack + '33B-' 
        except:
            message2print = "{(Step 33Z/70) failed" + "; eventwaitlistquantity2tpu-failed-by-copy2imagetobuffer4tpu: " + str(eventwaitlistquantity2tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '33Z-'
    
        task4copyimagetobuffer2tpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2tpu       
                                         
                
    if 'SOURCEORIGIN' in keys2task:
        pass
    else:                    
        # ---------------------------------------------------------------------------------------------
        # (34) SOURCEORIGIN - REQUIRED                             
        # ---------------------------------------------------------------------------------------------             
        source4origin2tpu = {}
        
        message2print = "{(Step 34A/70) trying" + "; source4origin2tpu-trying-by-copy2imagetobuffer4tpu: " + str(source4origin2tpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:                                                                                    
            source4origin2tpu['X'] = 0 
            source4origin2tpu['Y'] = 0  
            source4origin2tpu['Z'] = 0                                                                                                                          
            message2print = "{(Step 34B/70) set" + "; source4origin2tpu-set-by-copy2imagetobuffer4tpu to: " + str(source4origin2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            code2ack  = code2ack + '34B-' 
        except:
            message2print = "{(Step 34Z/70) failed" + "; source4origin2tpu-failed-by-copy2imagetobuffer4tpu: " + str(source4origin2tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '34Z-'
    
        task4copyimagetobuffer2tpu ['SOURCEORIGIN'] = source4origin2tpu
             
   
             
    if 'REGION' in keys2task:
        pass
    else:            
        # ---------------------------------------------------------------------------------------------
        # (36) REGION - REQUIRED                         
        # ---------------------------------------------------------------------------------------------             
        region2tpu = {}
        
        message2print = "{(Step 36A/70) trying" + "; region2tpu-trying-by-copy2imagetobuffer4tpu: " + str(region2tpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:                                                                                    
            region2tpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO WRITE 
            region2tpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO WRITE
            region2tpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO WRITE                                                                                                                          
            message2print = "{(Step 36B/70) set" + "; region2tpu-set-by-copy2imagetobuffer4tpu to: " + str(region2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            code2ack  = code2ack + '36B-' 
        except:
            message2print = "{(Step 36Z/70) failed" + "; region2tpu-failed-by-copy2imagetobuffer4tpu: " + str(region2tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '36Z-'
    
        task4copyimagetobuffer2tpu ['REGION'] = region2tpu
    
         
    if 'OFFSET' in keys2task:
        pass
    else:            
        # ---------------------------------------------------------------------------------------------
        # (37) OFFSET - WHERE TO BEGIN COPYING DATA INTO DESTINATION BUFFER - REQUIRED                         
        # ---------------------------------------------------------------------------------------------             
        offset2tpu = 0
        message2print = "{(Step 37A/70) set" + "; offset2tpu-set-by-copy2imagetobuffer4tpu to: " + str(offset2tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        task4copyimagetobuffer2tpu ['OFFSET'] = offset2tpu        
        code2ack  = code2ack + '37A-' 
        
    # ---------------------------------------------------------------------------------------------
    # EOF TASK
    # ---------------------------------------------------------------------------------------------
    keys2task = task4copyimagetobuffer2tpu.keys()
    message2print = "{(Step 39/70) set" + "; keys2task-set-by-copy2imagetobuffer4tpu to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)            
    # ['BUFFERDESTINATION',
    #  'COMMANDQUEUE',
    #  'CONTEXT',
    #  'DATA2CALLBACK',
    #  'DEVICE',
    #  'EVENT',
    #  'EVENTWAITLIST',
    #  'EVENTWAITLISTQUANTITY',
    #  'FUNCTION2CALLBACK',
    #  'IMAGESOURCE',
    #  'OFFSET',
    #  'PIPE',
    #  'PLATFORM',
    #  'QUEUE',
    #  'REGION',
    #  'SOURCEORIGIN']; time: 01:28:44 10/05/11 Pacific Daylight Time}
               


    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2copyimagetobuffer4tpu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2tpu = False
    channels2tpu = ()
    message2print = "{(Step 40A/70) trying" + "channels2tpu-trying-by-copy2imagetobuffer4tpu: " + str(channels2tpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel2copyimagetobuffer4tpu, publisherchannel2copyimagetobuffer4tpu = task4copyimagetobuffer2tpu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2copyimagetobuffer4tpu-got-by-copy2imagetobuffer4tpu: " + str(consumerchannel2copyimagetobuffer4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 660>

        message2print = "{(Step 40C/70) got" + "; publisherchannel2copyimagetobuffer4tpu-got-by-copy2imagetobuffer4tpu: " + str(publisherchannel2copyimagetobuffer4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 664>                     
        ack4channels2tpu = True
        
        control2copyimagetobuffer4tpu['CONSUMERCHANNEL']  = consumerchannel2copyimagetobuffer4tpu
        
        
        control2copyimagetobuffer4tpu['PUBLISHERCHANNEL'] = publisherchannel2copyimagetobuffer4tpu
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2tpu = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2copyimagetobuffer4tpu = False
    queue2copyimagetobuffer4tpu = None
    message2print = "{(Step 41A/70) trying" + "queue2copyimagetobuffer4tpu-trying-by-copy2imagetobuffer4tpu: " + str(queue2copyimagetobuffer4tpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2copyimagetobuffer4tpu = task4copyimagetobuffer2tpu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2copyimagetobuffer4tpu-got-by-copy2imagetobuffer4tpu: " + str(queue2copyimagetobuffer4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <queue4xi.Queue4Xi object at 0x0000000004465E48>
              
        ack4queue2copyimagetobuffer4tpu = True
        control2copyimagetobuffer4tpu['QUEUE']  = queue2copyimagetobuffer4tpu        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2copyimagetobuffer4tpu = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4tpu = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4tpu-trying-by-copy2imagetobuffer4tpu: " + str(dummy2image4tpu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4tpu = DummyProcess4Xi(target=manoeuvre2copyimagetobuffer4tpu,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue2copyimagetobuffer4tpu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2copyimagetobuffer4tpu, publisherchannel2copyimagetobuffer4tpu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4tpu-created-by-copy2imagetobuffer4tpu: " + str(dummy2image4tpu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'
        
    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-copy2imagetobuffer4tpu: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-11B-12B-13B-14B-16B-17A-40D-41C- 
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-copy2imagetobuffer4tpu: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #  
        
    message2print = "{(Step 70C/70) returning" + "; control2copyimagetobuffer4tpu-returning-by-copy2imagetobuffer4tpu: " + str(control2copyimagetobuffer4tpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004465E48>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 664>,
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 660>}; time: 01:28:44 10/05/11 Pacific Daylight Time}

        
    return control2copyimagetobuffer4tpu
                    
#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "copy2buffertoimage4tpu" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   # (80) ESTABLISH CONTROL & COMMAND WITH COPYBUFFERTOIMAGE4TPU                      
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 80A/700) OF TPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH COPYBUFFERTOIMAGE4TPU ', time4xi())    
#   response4copy2buffertoimage = {} 
#   task4copy2buffertoimage = None
#   try:                               
#       response4copy2buffertoimage = copy2buffertoimage4tpu(task4copy2buffertoimage)                                                                                                          
#       print('(Step 80B/700) OF TPU2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH COPYBUFFERTOIMAGE4TPU : ', response4copy2buffertoimage)
#       queue2copybuffertoimage4tpu      = response4copy2buffertoimage['QUEUE']
#       channel2copybuffertoimagepublish4tpu = response4copy2buffertoimage['PUBLISHERCHANNEL']
#       channel2copybuffertoimageconsume4tpu = response4copy2buffertoimage['CONSUMERCHANNEL']
#   except:
#       print('(Step 80Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH COPYBUFFERTOIMAGE4TPU ', time4xi()) 
#   # --------------------------------------------------------------------------------------------------
#   # (81) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2COPYIMAGETOBUFFER4TPU TO RECEIVE                 
#   # --------------------------------------------------------------------------------------------------      
#   print('(Step 81A/700) OF TPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4TPU ', time4xi())     
#   try:                               
#       message2copybuffertoimagesend4tpu = "<copy4xi>copy2copybuffertoimage4tpu</copy4xi>"
#       channel2copybuffertoimagepublish4tpu = response4copy2buffertoimage['PUBLISHERCHANNEL']
#       channel2copybuffertoimagepublish4tpu.send(message2copybuffertoimagesend4tpu)
#       print("(Step 81B/700) OF TPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4TPU: ", channel2copybuffertoimagepublish4tpu, " MESSAGE2SEND4TPU: " , message2copybuffertoimagesend4tpu)                                                                                                
#   except:
#       print('(Step 81Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4TPU ', time4xi())                                                
#   # ------------------------------------------------------------------------------------------
#   # (82) LET MANOEUVRE2COPYIMAGETOBUFFER4TPU TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # ------------------------------------------------------------------------------------------       
#   print("(Step 82C/700) OF TPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4TPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4tpu = manoeuvre2copybuffertoimage4tpu(queue2copybuffertoimage4tpu, channel2copybuffertoimageconsume4tpu, channel2copybuffertoimagepublish4tpu)
#       print("(Step 82D/700) OF TPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4TPU ", manoeuvre2copybuffertoimage4tpu)
#   except:
#       print("(Step 82Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4TPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # ---------------------------------------------------------------------------------------------
#   # (83) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 83A/700) OF TPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4TPU ', time4xi())     
#   try:
#       channel2copybuffertoimageconsume4tpu = response4copy2buffertoimage['CONSUMERCHANNEL']                              
#       message2copyimagereceive4tpu = channel2copybuffertoimageconsume4tpu.recv()
#       print("(Step 83B/700) OF TPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4TPU: ", channel2copybuffertoimageconsume4tpu, " CONTROL & COMMAND: ", message2copyimagereceive4tpu)                                                                                                  
#       # 
#   except:
#       print('(Step 83Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPYBUFFERTOIMAGE4TPU ', time4xi())         
#   # ---------------------------------------------------------------------------------------------------
#   # (84) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2COPYIMAGETOBUFFER4TPU TO RECEIVE                
#   # ---------------------------------------------------------------------------------------------------      
#   print('(Step 84A/700) OF TPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4TPU ', time4xi())     
#   try:                               
#       message2copybuffertoimagesend4tpu = "<copy4xi>copy2copybuffertoimage4tpu</copy4xi>"
#       channel2copybuffertoimagepublish4tpu = response4copy2buffertoimage['PUBLISHERCHANNEL']
#       channel2copybuffertoimagepublish4tpu.send(message2copybuffertoimagesend4tpu)
#       print("(Step 84B/700) OF TPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4TPU: ", channel2copybuffertoimagepublish4tpu, " MESSAGE2SEND4TPU: " , message2copybuffertoimagesend4tpu)                                                                                                
#       #  
#   except:
#        print('(Step 84Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4TPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------
#   # (85) LET MANOEUVRE2COPYIMAGETOBUFFER4TPU TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------       
#   print("(Step 85A/700) OF TPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4TPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4tpu = manoeuvre2copybuffertoimage4tpu(queue2copybuffertoimage4tpu, channel2copybuffertoimageconsume4tpu, channel2copybuffertoimagepublish4tpu)
#       print("(Step 85B/700) OF TPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4TPU ", manoeuvre2copybuffertoimage4tpu)
#   except:
#       print("(Step 85Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4TPU ", time4xi())     
#   # ---------------------------------------------------------------------------------------
#   # (86) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 86A/700) OF TPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4TPU ', time4xi())     
#   try:
#       channel2copybuffertoimageconsume4tpu = response4copy2buffertoimage['CONSUMERCHANNEL']                              
#       message2copyimagereceive4tpu = channel2copybuffertoimageconsume4tpu.recv()
#       print("(Step 86B/700) OF TPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4TPU: ", channel2copybuffertoimageconsume4tpu, " CONTROL & COMMAND: ", message2copyimagereceive4tpu)                                                                                                  
#       # 
#   except:
#       print('(Step 86Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPYBUFFERTOIMAGE4TPU ', time4xi())          
#   # -----------------------------------------------------------------------------------------
#   # (87) PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 87A/700) OF TPU2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue2copybuffertoimage4dragon.put(path2dragon)             
#       print("(Step 87B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#       # ..//..//..//..//..//web//images//png//dragonxi.png  
#   except:
#       print("(Step 87Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # ------------------------------------------------------------------------------------------------------
#   # (88) LET MANOEUVRE2COPYIMAGETOBUFFER4TPU TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 88A/700) OF TPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4TPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4tpu = manoeuvre2copybuffertoimage4tpu(queue2copybuffertoimage4tpu, channel2copybuffertoimageconsume4tpu, channel2copybuffertoimagepublish4tpu)
#       print("(Step 88B/700) OF TPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4TPU ", manoeuvre2copybuffertoimage4tpu)
#   except:
#       print("(Step 88Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4TPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (89) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 89A/700) OF TPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2copyimagereceive4tpu = channel2copybuffertoimageconsume4tpu.recv()
#       print("(Step 89D/700) OF TPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4TPU: ", channel2copybuffertoimageconsume4tpu, " CONTROL & COMMAND: ", message2copyimagereceive4tpu)                                                                                                   
#   except:
#       print('(Step 89Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -------------------------------------------------------------------------------------------------------
#   # (90) LET MANOEUVRE2COPYIMAGETOBUFFER4TPU GO TO STATE 0               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 90A/700) OF TPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4TPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4tpu = manoeuvre2copybuffertoimage4tpu(queue2copybuffertoimage4tpu, channel2copybuffertoimageconsume4tpu, channel2copybuffertoimagepublish4tpu)
#       print("(Step 90B/700) OF TPU2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4TPU ", response2manoeuvreimagecopytobuffer4tpu)
#   except:
#       print("(Step 90Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4TPU ", time4xi())   

#">

# -------------------------------------------------------------------------------------------------   
def copy2buffertoimage4tpu(task4copybuffertoimage2tpu=None):

    '''
    copies buffer to image for TPU
    '''
    global tpu2image4xi                                                                                             # 2D-IMAGES FOR TPU  
    global tpu2buffer4xi                                                                                            # BUFFER FOR TPU
    
    global control2copybuffertoimage4tpu                                                                            # COMMAND AND CONTROL FOR TPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "copy2buffertoimage4tpu"
    method2print = "copy2buffertoimage4tpu"

    message2print = "{(Step 1A/33) started" + "; copy2buffertoimage4tpu-started: " + str(copy2buffertoimage4tpu)  + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                                
    # <function copy2buffertoimage4tpu at 0x0000000003869548>
    
    message2print = "{(Step 1B/33) passed" + "; task4copybuffertoimage2tpu-to-copy2buffertoimage4tpu: " + str(task4copybuffertoimage2tpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE TPU2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if tpu2image4xi is None:
            tpu2image4xi = create2image4tpu()                                                                              # INITIALIZE IMAGE FOR TPU 
            message2print = "{(Step 1C/70) created" + "; tpu2image4xi-created-by-copy2buffertoimage4tpu to: " + str(tpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4tpu-skipped-by-copy2buffertoimage4tpu: " + str(copy2buffertoimage4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # skipped                                  
    except:
        try:
            tpu2image4xi = create2image4tpu()
            message2print = "{(Step 1E/70) set" + "; tpu2image4xi-set-by-copy2buffertoimage4tpu to: " + str(tpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            tpu2image4xi = {}
            message2print = "{(Step 1Z1/70) yielded" + "; tpu2image4xi-yielded-by-copy2buffertoimage4tpu: " + str(tpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z1-' 
    
    # -----------------------------------------------------------------------------------------
    # CREATE TPU2BUFFER4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if tpu2buffer4xi is None:
            tpu2buffer4xi = create2buffer4tpu()                                                                     # INITIALIZE BUFFER FOR TPU 
            message2print = "{(Step 1F/70) created" + "; tpu2buffer4xi-created-by-copy2buffertoimage4tpu to: " + str(tpu2buffer4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1G/70) skipped" + "; create2buffer4tpu-skipped-by-copy2buffertoimage4tpu: " + str(copy2buffertoimage4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # skipped                                  
    except:
        try:
            tpu2buffer4xi = create2buffer4tpu()
            message2print = "{(Step 1H/70) set" + "; tpu2buffer4xi-set-by-copy2buffertoimage4tpu to: " + str(tpu2buffer4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            tpu2buffer4xi = {}
            message2print = "{(Step 1Z2/70) yielded" + "; tpu2buffer4xi-yielded-by-copy2buffertoimage4tpu: " + str(tpu2buffer4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z2-' 
                            
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - ANALYZE TASKS           
    # -----------------------------------------------------------------------------------------          
    if task4copybuffertoimage2tpu is None:
        # -----------------------------------------------------------------------------------------
        # NO PARAMETERS GIVEN BY CALLER FOR COPY - BUILD TASK! 
        # -----------------------------------------------------------------------------------------    
        
        task4copybuffertoimage2tpu = {}                                                                                                                                                                                       
        
        message2print = "{(Step 1A/70) initialized" + "; task4copybuffertoimage2tpu-created-by-copy2buffertoimage4tpu to: " + str(task4copybuffertoimage2tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # 
        
        # ---------------------------------------------------------------------------------------------
        # (1B) TRACE - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        function2callback4tpu = trace2image4tpu                                                                 # CALLBACK FUNCTION FOR TPU
        
        message2print = "{(Step 1B1/70) initialized" + "; function2callback4tpu-created-by-copy2buffertoimage4tpu to: " + str(function2callback4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <function trace2image4tpu at 0x0000000004645548>
        
        code2ack  = code2ack + '1B1-'
       
        args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
        args2callback['CLASS']   = "copy2buffertoimage4tpu"                                                     # MODULE OF TPUIZED OPENCL MODULE
        args2callback['METHOD']  = "copy2buffertoimage4tpu"                                                     # FUNCTION OF TPUIZED OPENCL MODULE
        args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2buffertoimage4tpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR TPUIZED OPENCL TO RETURN    

        message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-copy2buffertoimage4tpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # {'MESSAGE': '{(Step 1Z1/33) failed; copy2buffertoimage4tpu-failed ; time: 04:41:15 10/04/11 Pacific Daylight Time}',
        #  'CLASS': 'copy2buffertoimage4tpu', 
        #  'METHOD': 'copy2buffertoimage4tpu'}; time: 04:41:15 10/04/11 Pacific Daylight Time}

        code2ack  = code2ack + '1B2-'
            
        # ---------------------------------------------------------------------------------------------
        # (1C) CALLBACK - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        task4copybuffertoimage2tpu['DATA2CALLBACK'] = args2callback                                             # USER DATA PASSED TO TPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
        task4copybuffertoimage2tpu['FUNCTION2CALLBACK'] = function2callback4tpu                                 # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
        code2ack  = code2ack + '1C-'
        message2print = "{(Step 1C/70) initialized" + "; task4copybuffertoimage2tpu-created-by-copy2buffertoimage4tpu to: " + str(task4copybuffertoimage2tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # {'FUNCTION2CALLBACK': <function trace2image4tpu at 0x0000000004645548>, 
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; copy2buffertoimage4tpu-failed ; time: 04:41:15 10/04/11 Pacific Daylight Time}', 
        #  'CLASS': 'copy2buffertoimage4tpu', 
        #  'METHOD': 'copy2buffertoimage4tpu'}}; time: 04:41:15 10/04/11 Pacific Daylight Time}

            
        # ---------------------------------------------------------------------------------------------
        # (2) PLATFORM - AUTOMATIC                   
        # ---------------------------------------------------------------------------------------------             
        platform2xi = {}
        message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-copy2buffertoimage4tpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # {}
            
        try:
            platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
            message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-copy2buffertoimage4tpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'list2platforms': ['DragonXi'], 
            #  'ack2return': ' 2C-2E-2F-2G-', 
            #  'nack2return': ' ', 
            #  'available2platform': 1, 
            #  'max2entries': 1}; time: 04:41:15 10/04/11 Pacific Daylight Time}
                 
            code2ack  = code2ack + '2B-'
        except:
            message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-copy2buffertoimage4tpu: " + str(failed)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                                
            code2nack  = code2nack + '2Z-'
                 
        task4copybuffertoimage2tpu['PLATFORM'] = platform2xi                                                    # PLATFORM OF SYSTEM 
              
            
        # ---------------------------------------------------------------------------------------------
        # (3) CONTEXT - AUTOMATIC                 
        # ---------------------------------------------------------------------------------------------    
        context2tpu = {}
        message2print = "{(Step 3A/70) trying" + "; context2tpu-trying-by-copy2buffertoimage4tpu: " + str(context2tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            context2tpu = get2context4tpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
            message2print = "{(Step 3B/70) got" + "; context2tpu-got-by-copy2buffertoimage4tpu: " + str(context2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
                
            code2ack  = code2ack + '3B-'         
        except:
            message2print = "{(Step 3Z/70) failed" + "; context2tpu-failed-by-copy2buffertoimage4tpu: " + str(context2tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z-'
                
        task4copybuffertoimage2tpu ['CONTEXT'] = context2tpu           
            
        # ---------------------------------------------------------------------------------------------
        # (4) DEVICE - AUTOMATIC                       
        # ---------------------------------------------------------------------------------------------    
        device2tpu = {}
            
        message2print = "{(Step 4A/70) trying" + "; device2tpu-trying-by-copy2buffertoimage4tpu: " + str(device2tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            device2tpu = get2tpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
            
            message2print = "{(Step 4B/70) got" + "; device2tpu-got-by-copy2buffertoimage4tpu: " + str(device2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'RESOLUTION2TIMER4PROFILING': 318, 
            #  'ID2PLATFORM': '0000000001509568', 
            #  'PROFILING2QUEUE': 'ACK', 
            #  'EXECUTE2KERNELS4OPENCL': 'ACK', 
            #  'NATIVE2WIDTHLONG4VECTOR': 2, 
            #  'MAX2UNITS4COMPUTE': 4, 
            #  'SIZE2MEMORY4GLOBAL': 3221225472, ...
               
            code2ack  = code2ack + '4B-'  
        except:
            message2print = "{(Step 4Z/70) failed" + "; device2tpu-failed-by-copy2buffertoimage4tpu: " + str(device2tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z-'
            
        task4copybuffertoimage2tpu ['DEVICE'] = device2tpu
            
                   
        # ---------------------------------------------------------------------------------------------
        # (5) IMAGE DESTINATION OBJECT - REQUIRED                         
        # ---------------------------------------------------------------------------------------------         
        imagedestinationinfo2tpu = {}
             
        message2print = "{(Step 5A/70) trying" + "; imagedestinationinfo2tpu-trying-by-copy2buffertoimage4tpu: " + str(imagedestinationinfo2tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            imagedestinationinfo2tpu = get2imageinfo4tpu()                                                                         
            
            message2print = "{(Step 5B/70) got" + "; imagedestinationinfo2tpu-got-by-copy2buffertoimage4tpu: " + str(imagedestinationinfo2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'SIZE2QUEUE': 2147483646, 
            #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
            #  'ORDER': 'CL_RGBA'}, 
            #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004105358>, 
            #  'WIDTH': 800, 
            #  'CONTEXT': {'COUNT2REFERENCE': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4tpu-failed ; time: 04:41:15 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2context4tpu', 
            #  'METHOD': 'create2context4tpu'}, 
            #  'FUNCTION2CALLBACK': <function trace2context4tpu at 0x00000000037FF348>, 
            #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 
            #  'INFO2PRIVATE': None, 
            #  'PLATFORM': {'list2platforms': ['DragonXi'],'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
            #  'NACK2RETURN': ' ', 
            #  'SIZE2PRIVATE': 0, 
            #  'PROPERTIES': None}, 
            #  'PIPE': (<read-write PipeConnection, handle 524>, <read-write PipeConnection, handle 528>), 
            #  'COUNT2REFERENCE': 1, 
            #  'FUNCTION2CALLBACK': <function trace2image4tpu at 0x0000000004645548>, 
            #  'COMMANDQUEUE': ...
            
            code2ack  = code2ack + '5B-'  
        except:
            message2print = "{(Step 5Z/70) failed" + "; imagedestinationinfo2tpu-failed-by-copy2buffertoimage4tpu: " + str(imagedestinationinfo2tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '5Z-'          
                  
        task4copybuffertoimage2tpu ['IMAGEDESTINATION'] = imagedestinationinfo2tpu         
               
        # ---------------------------------------------------------------------------------------------
        # (6) BUFFER SOURCE OBJECT - REQUIRED                        
        # ---------------------------------------------------------------------------------------------                         
        buffersourceinfo2tpu = {}
         
        message2print = "{(Step 6A/70) trying" + "; buffersourceinfo2tpu-trying-by-copy2buffertoimage4tpu: " + str(buffersourceinfo2tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            buffersourceinfo2tpu = get2bufferinfo4tpu()                                                                         
            
            message2print = "{(Step 6B/70) got" + "; buffersourceinfo2tpu-got-by-copy2buffertoimage4tpu: " + str(buffersourceinfo2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'CODE2ERROR': 0, 
            #  'COUNT2REFERENCE': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4tpu-failed ; time: 04:41:15 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2buffer4tpu', 
            #  'METHOD': 'create2buffer4tpu'}, 
            #  'FUNCTION2CALLBACK': <function trace2buffer4tpu at 0x0000000004631B48>, 
            #  'POINTER2BUFFER': <mmap.mmap object at 0x00000000040EBD98>, 
            #  'FLAGS': 0, 
            #  'CONTEXT': ...
            
            code2ack  = code2ack + '6B-'  
        except:
            message2print = "{(Step 6Z/70) failed" + "; buffersourceinfo2tpu-failed-by-copy2buffertoimage4tpu: " + str(buffersourceinfo2tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '6Z-'          
                  
        task4copybuffertoimage2tpu ['BUFFERSOURCE'] = buffersourceinfo2tpu     
        
        # ---------------------------------------------------------------------------------------------
        # (7) QUEUE - REQUIRED BY TPUXI                          
        # ---------------------------------------------------------------------------------------------         
        queue2copybuffertoimage4tpu = None
        
        message2print = "{(Step 7A/70) trying" + "; queue2copybuffertoimage4tpu-trying-by-copy2buffertoimage4tpu: " + str(queue2copybuffertoimage4tpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            queue2copybuffertoimage4tpu = Queue4Xi()                                                                              # THE SYSTEM CREATES QUEUE    
            
            message2print = "{(Step 7B/70) got" + "; queue2copybuffertoimage4tpu-got-by-copy2buffertoimage4tpu: " + str(queue2copybuffertoimage4tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # <queue4xi.Queue4Xi object at 0x000000000492B208> 
            
            code2ack  = code2ack + '7B-'  
        except:
            message2print = "{(Step 7Z/70) failed" + "; queue2copybuffertoimage4tpu-failed-by-copy2buffertoimage4tpu: " + str(queue2copybuffertoimage4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '7Z-'
            
        task4copybuffertoimage2tpu ['QUEUE'] = queue2copybuffertoimage4tpu
         
        # ---------------------------------------------------------------------------------------------
        # (8) PIPE - REQUIRED BY TPUXI                              
        # ---------------------------------------------------------------------------------------------         
        pipe2tpu = None
       
        message2print = "{(Step 8A/70) trying" + "; pipe2tpu-trying-by-copy2buffertoimage4tpu: " + str(pipe2tpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            pipe2tpu = Pipe4Xi()                                                                                # THE SYSTEM CREATES PIPE    
            
            message2print = "{(Step 8B/70) got" + "; pipe2tpu-got-by-copy2buffertoimage4tpu: " + str(pipe2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # (<read-write PipeConnection, handle 708>, <read-write PipeConnection, handle 712>) 
            
            code2ack  = code2ack + '8B-'  
        except:
            message2print = "{(Step 8Z/70) failed" + "; pipe2tpu-failed-by-copy2buffertoimage4tpu: " + str(pipe2tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '8Z-'
            
        task4copybuffertoimage2tpu ['PIPE'] = pipe2tpu 
        
        # ---------------------------------------------------------------------------------------------
        # (9) COMMAND QUEUE - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        commandqueueinfo2tpu = {}
       
        message2print = "{(Step 9A/70) trying" + "; commandqueueinfo2tpu-trying-by-copy2buffertoimage4tpu: " + str(commandqueueinfo2tpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
                    
        try:
            commandqueueinfo2tpu = get2commandqueueinfo4tpu()                                                   # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
            message2print = "{(Step 9B/70) got" + "; commandqueueinfo2tpu-got-by-copy2buffertoimage4tpu: " + str(commandqueueinfo2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': ...
            
            code2ack  = code2ack + '9B-'  
        except:
            message2print = "{(Step 9Z/70) failed" + "; commandqueueinfo2tpu-failed-by-copy2buffertoimage4tpu: " + str(commandqueueinfo2tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '9Z-'
    
        task4copybuffertoimage2tpu ['COMMANDQUEUE'] = commandqueueinfo2tpu
        
        
        
        # ---------------------------------------------------------------------------------------------
        # (11) EVENT - REQUIRED                            
        # ---------------------------------------------------------------------------------------------             
        eventinfo2tpu = {}
       
        message2print = "{(Step 11A/70) trying" + "; eventinfo2tpu-trying-by-copy2buffertoimage4tpu: " + str(eventinfo2tpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventinfo2tpu = get2eventinfo4tpu()                                                                    
            message2print = "{(Step 11B/70) got" + "; eventinfo2tpu-got-by-copy2buffertoimage4tpu: " + str(eventinfo2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 'DATA2CALLBACK': ...
            
            code2ack  = code2ack + '11B-' 
        except:
            message2print = "{(Step 11Z/70) failed" + "; eventinfo2tpu-failed-by-copy2buffertoimage4tpu: " + str(eventinfo2tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '11Z-'
    
        task4copybuffertoimage2tpu ['EVENT'] = eventinfo2tpu 
        
        
        # ---------------------------------------------------------------------------------------------
        # (12) EVENTWAITLIST - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlist2tpu = {}
       
        message2print = "{(Step 12A/70) trying" + "; eventwaitlist2tpu-trying-by-copy2buffertoimage4tpu: " + str(eventwaitlist2tpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventwaitlist2tpu = {}                                                                   
            message2print = "{(Step 12B/70) set" + "; eventwaitlist2tpu-set-by-copy2buffertoimage4tpu: " + str(eventwaitlist2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {}
            
            code2ack  = code2ack + '12B-' 
        except:
            message2print = "{(Step 12Z/70) failed" + "; eventwaitlist2tpu-failed-by-copy2buffertoimage4tpu: " + str(eventwaitlist2tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '12Z-'
    
        task4copybuffertoimage2tpu ['EVENTWAITLIST'] = eventwaitlist2tpu                     
         
        # ---------------------------------------------------------------------------------------------
        # (13) EVENTWAITLISTQUANTITY - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlistquantity2tpu = 0
       
        message2print = "{(Step 13A/70) trying" + "; eventwaitlistquantity2tpu-trying-by-copy2buffertoimage4tpu: " + str(eventwaitlistquantity2tpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            keys2event = eventwaitlist2tpu.keys()
            for key2event in keys2event:
                eventwaitlistquantity2tpu = eventwaitlistquantity2tpu + 1                                                                     
            message2print = "{(Step 13B/70) set" + "; eventwaitlistquantity2tpu-set-by-copy2buffertoimage4tpu: " + str(eventwaitlistquantity2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 0 
            
            code2ack  = code2ack + '13B-' 
        except:
            message2print = "{(Step 13Z/70) failed" + "; eventwaitlistquantity2tpu-failed-by-copy2buffertoimage4tpu: " + str(eventwaitlistquantity2tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '13Z-'
    
        task4copybuffertoimage2tpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2tpu       
                                             
        
        # ---------------------------------------------------------------------------------------------
        # (15) ORIGIN FOR DESTINATION - REQUIRED                             
        # ---------------------------------------------------------------------------------------------              
        origin4destination2tpu = {}
        
        message2print = "{(Step 15A/70) trying" + "; origin4destination2tpu-trying-by-copy2buffertoimage4tpu: " + str(origin4destination2tpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:                                                                                    
            origin4destination2tpu['X'] = 100 
            origin4destination2tpu['Y'] = 100  
            origin4destination2tpu['Z'] = 100                                                                                                                          
            message2print = "{(Step 15B/70) set" + "; origin4destination2tpu-set-by-copy2buffertoimage4tpu to: " + str(origin4destination2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'Y': 100, 'X': 100, 'Z': 100}
            code2ack  = code2ack + '15B-' 
        except:
            message2print = "{(Step 15Z/70) failed" + "; origin4destination2tpu-failed-by-copy2buffertoimage4tpu: " + str(origin4destination2tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '15Z-'
    
        task4copybuffertoimage2tpu ['DESTINATIONORIGIN'] = origin4destination2tpu
                    
        
        
        # --------------------------------------------------------------------------------------------------------
        # (16) REGION - REGION OF A IMAGE TO BE COPIED                         
        # --------------------------------------------------------------------------------------------------------             
        region4copy2tpu = {}                                                                                                         
       
        message2print = "{(Step 16A/70) trying" + "; region4copy2tpu-trying-by-copy2buffertoimage4tpu: " + str(region4copy2tpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            region4copy2tpu['X'] = 10 
            region4copy2tpu['Y'] = 10  
            region4copy2tpu['Z'] = 10                                                                           
            message2print = "{(Step 16B/70) set" + "; region4copy2tpu-set-by-copy2buffertoimage4tpu to: " + str(region4copy2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'Y': 10, 'X': 10, 'Z': 10}
            code2ack  = code2ack + '16B-'                
             
        except:
            message2print = "{(Step 16Z/70) failed" + "; region4copy2tpu-failed-by-copy2buffertoimage4tpu: " + str(region4copy2tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '16Z-'
        
        task4copybuffertoimage2tpu ['REGION'] = region4copy2tpu              
        
        # --------------------------------------------------------------------------------------------------------
        # (17) OFFSET - WHERE TO BEGIN COPYING DATA FROM SOURCE BUFFER                   
        # --------------------------------------------------------------------------------------------------------             
        offset4copy2tpu = 0                                                                                                         
        message2print = "{(Step 17A/70) set" + "; offset4copy2tpu-set-by-copy2buffertoimage4tpu to: " + str(offset4copy2tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 0
        
        task4copybuffertoimage2tpu ['OFFSET'] = offset4copy2tpu            
                                
        # ---------------------------------------------------------------------------------------------
        # EOF PARAMETER PROVIDING                
        # ---------------------------------------------------------------------------------------------                                                                                                                               
        
        # ---------------------------------------------------------------------------------------------
        # ANALYZE TASKS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys2task = task4copybuffertoimage2tpu.keys()
        message2print = "{(Step 20/70) got" + "; keys2task-got-by-copy2buffertoimage4tpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BUFFERSOURCE',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DESTINATIONORIGIN',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGEDESTINATION',
        #  'OFFSET',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION']; time: 04:41:15 10/04/11 Pacific Daylight Time}
 
        if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4tpu = trace2image4tpu                                                             # CALLBACK FUNCTION FOR TPU
            
            message2print = "{(Step 21B1/70) initialized" + "; function2callback4tpu-created-by-copy2buffertoimage4tpu to: " + str(function2callback4tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '21B1-'
           
            args2callback = {}                                                                                  # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "copy2buffertoimage4tpu"                                                        # MODULE OF TPUIZED OPENCL MODULE
            args2callback['METHOD']  = "copy2buffertoimage4tpu"                                                        # FUNCTION OF TPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2buffertoimage4tpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR TPUIZED OPENCL TO RETURN    

            message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-copy2buffertoimage4tpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 

            code2ack  = code2ack + '21B2-'
        
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4copybuffertoimage2tpu['DATA2CALLBACK'] = args2callback                                                # USER DATA PASSED TO TPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4copybuffertoimage2tpu['FUNCTION2CALLBACK'] = function2callback4tpu                                    # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4copybuffertoimage2tpu-created-by-copy2buffertoimage4tpu to: " + str(task4copybuffertoimage2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
             
        if 'PLATFORM' in keys2task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-copy2buffertoimage4tpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            try:
                platform2xi = get2platformids4xi()                                                              # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-copy2buffertoimage4tpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
                code2ack  = code2ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-copy2buffertoimage4tpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '22Z-'
                 
            task4copybuffertoimage2tpu['PLATFORM'] = platform2xi                                                       # PLATFORM OF SYSTEM 
          
        if 'CONTEXT' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2tpu = {}
            message2print = "{(Step 23A/70) trying" + "; context2tpu-trying-by-copy2buffertoimage4tpu: " + str(context2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2tpu = get2context4tpu()                                                                 # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context2tpu-got-by-copy2buffertoimage4tpu: " + str(context2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context2tpu-failed-by-copy2buffertoimage4tpu: " + str(context2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '23Z-'
                
            task4copybuffertoimage2tpu ['CONTEXT'] = context2tpu           
        
        if 'DEVICE' in keys2task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2tpu = {}
            
            message2print = "{(Step 24A/70) trying" + "; device2tpu-trying-by-copy2buffertoimage4tpu: " + str(device2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2tpu = get2tpu4xi()                                                                       # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device2tpu-got-by-copy2buffertoimage4tpu: " + str(device2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device2tpu-failed-by-copy2buffertoimage4tpu: " + str(device2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '24Z-'
            
            task4copybuffertoimage2tpu ['DEVICE'] = device2tpu
        
        if 'IMAGEDESTINATION' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) DESTINATION IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            destinationimageinfo2tpu = {}
             
            message2print = "{(Step 25A/70) trying" + "; destinationimageinfo2tpu-trying-by-copy2buffertoimage4tpu: " + str(destinationimageinfo2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                destinationimageinfo2tpu = get2imageinfo4tpu()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; destinationimageinfo2tpu-got-by-copy2buffertoimage4tpu: " + str(destinationimageinfo2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; destinationimageinfo2tpu-failed-by-copy2buffertoimage4tpu: " + str(destinationimageinfo2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '25Z-'          
                      
            task4copybuffertoimage2tpu ['IMAGEDESTINATION'] = destinationimageinfo2tpu         
        
        if 'BUFFERSOURCE' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (26) SOURCE BUFFER - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            sourcebufferinfo2tpu = {}
             
            message2print = "{(Step 26A/70) trying" + "; sourcebufferinfo2tpu-trying-by-copy2buffertoimage4tpu: " + str(sourcebufferinfo2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                sourcebufferinfo2tpu = get2bufferinfo4tpu()                                                                         
                
                message2print = "{(Step 26B/70) got" + "; sourcebufferinfo2tpu-got-by-copy2buffertoimage4tpu: " + str(sourcebufferinfo2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; sourcebufferinfo2tpu-failed-by-copy2buffertoimage4tpu: " + str(sourcebufferinfo2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '26Z-'          
                      
            task4copybuffertoimage2tpu ['BUFFERSOURCE'] = sourcebufferinfo2tpu    
                                    
        if 'QUEUE' in keys2task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (27) QUEUE - REQUIRED BY TPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2copybuffertoimage4tpu = None
            
            message2print = "{(Step 27A/70) trying" + "; queue2copybuffertoimage4tpu-trying-by-copy2buffertoimage4tpu: " + str(queue2copybuffertoimage4tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2copybuffertoimage4tpu = Queue4Xi()                                                                          # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 27B/70) got" + "; queue2copybuffertoimage4tpu-got-by-copy2buffertoimage4tpu: " + str(queue2copybuffertoimage4tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; queue2copybuffertoimage4tpu-failed-by-copy2buffertoimage4tpu: " + str(queue2copybuffertoimage4tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '27Z-'
                
            task4copybuffertoimage2tpu ['QUEUE'] = queue2copybuffertoimage4tpu
            
        if 'PIPE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) PIPE - REQUIRED BY TPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2tpu = None
           
            message2print = "{(Step 28A/70) trying" + "; pipe2tpu-trying-by-copy2buffertoimage4tpu: " + str(pipe2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2tpu = Pipe4Xi()                                                                            # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 28B/70) got" + "; pipe2tpu-got-by-copy2buffertoimage4tpu: " + str(pipe2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; pipe2tpu-failed-by-copy2buffertoimage4tpu: " + str(pipe2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '28Z-'
                
            task4copybuffertoimage2tpu ['PIPE'] = pipe2tpu 
            
        if 'COMMANDQUEUE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2tpu = {}
           
            message2print = "{(Step 29A/70) trying" + "; commandqueueinfo2tpu-trying-by-copy2buffertoimage4tpu: " + str(commandqueueinfo2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2tpu = get2commandqueueinfo4tpu()                                               # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 29B/70) got" + "; commandqueueinfo2tpu-got-by-copy2buffertoimage4tpu: " + str(commandqueueinfo2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '29B-'  
            except:
                message2print = "{(Step 29Z/70) failed" + "; commandqueueinfo2tpu-failed-by-copy2buffertoimage4tpu: " + str(commandqueueinfo2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '29Z-'
        
            task4copybuffertoimage2tpu ['COMMANDQUEUE'] = commandqueueinfo2tpu
        
             
        if 'EVENT' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2tpu = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventinfo2tpu-trying-by-copy2buffertoimage4tpu: " + str(eventinfo2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2tpu = get2eventinfo4tpu()                                                                    
                message2print = "{(Step 31B/70) got" + "; eventinfo2tpu-got-by-copy2buffertoimage4tpu: " + str(eventinfo2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventinfo2tpu-failed-by-copy2buffertoimage4tpu: " + str(eventinfo2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '31Z-'
        
            task4copybuffertoimage2tpu ['EVENT'] = eventinfo2tpu 
        
             
        if 'EVENTWAITLIST' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2tpu = {}
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlist2tpu-trying-by-copy2buffertoimage4tpu: " + str(eventwaitlist2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2tpu = {}                                                                   
                message2print = "{(Step 32B/70) set" + "; eventwaitlist2tpu-set-by-copy2buffertoimage4tpu: " + str(eventwaitlist2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlist2tpu-failed-by-copy2buffertoimage4tpu: " + str(eventwaitlist2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '32Z-'
        
            task4copybuffertoimage2tpu ['EVENTWAITLIST'] = eventwaitlist2tpu                     
              
        if 'EVENTWAITLISTQUANTITY' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (33) EVENTWAITLISTQUANTITY - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2tpu = 0
           
            message2print = "{(Step 33A/70) trying" + "; eventwaitlistquantity2tpu-trying-by-copy2buffertoimage4tpu: " + str(eventwaitlistquantity2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2tpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2tpu = eventwaitlistquantity2tpu + 1                                                                     
                message2print = "{(Step 33B/70) set" + "; eventwaitlistquantity2tpu-set-by-copy2buffertoimage4tpu: " + str(eventwaitlistquantity2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 
                
                code2ack  = code2ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; eventwaitlistquantity2tpu-failed-by-copy2buffertoimage4tpu: " + str(eventwaitlistquantity2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '33Z-'
        
            task4copybuffertoimage2tpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2tpu       
                                             
  
             
        if 'DESTINATIONORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) DESTINATIONORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            destination4origin2tpu = {}
            
            message2print = "{(Step 34A/70) trying" + "; destination4origin2tpu-trying-by-copy2buffertoimage4tpu: " + str(destination4origin2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                destination4origin2tpu['X'] = 0 
                destination4origin2tpu['Y'] = 0  
                destination4origin2tpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; destination4origin2tpu-set-by-copy2buffertoimage4tpu to: " + str(destination4origin2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; destination4origin2tpu-failed-by-copy2buffertoimage4tpu: " + str(destination4origin2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '34Z-'
        
            task4copybuffertoimage2tpu ['DESTINATIONORIGIN'] = destination4origin2tpu
             
     
             
        if 'REGION' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (36) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2tpu = {}
            
            message2print = "{(Step 36A/70) trying" + "; region2tpu-trying-by-copy2buffertoimage4tpu: " + str(region2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2tpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO WRITE 
                region2tpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO WRITE
                region2tpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO WRITE                                                                                                                          
                message2print = "{(Step 36B/70) set" + "; region2tpu-set-by-copy2buffertoimage4tpu to: " + str(region2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; region2tpu-failed-by-copy2buffertoimage4tpu: " + str(region2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '36Z-'
        
            task4copybuffertoimage2tpu ['REGION'] = region2tpu
        
             
        if 'OFFSET' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (37) OFFSET - WHERE TO BEGIN COPYING DATA FROM SOURCE BUFFER - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            offset2tpu = 0
            message2print = "{(Step 37A/70) set" + "; offset2tpu-set-by-copy2buffertoimage4tpu to: " + str(offset2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            task4copybuffertoimage2tpu ['OFFSET'] = offset2tpu        
            code2ack  = code2ack + '37A-' 
                    
        # ---------------------------------------------------------------------------------------------
        # EOF TASK ANALYSIS
        # ---------------------------------------------------------------------------------------------
        keys2task = task4copybuffertoimage2tpu.keys()
        message2print = "{(Step 39/70) set" + "; keys2task-set-by-copy2buffertoimage4tpu to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BUFFERSOURCE',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DESTINATIONORIGIN',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGEDESTINATION',
        #  'OFFSET',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION']; time: 04:52:11 10/04/11 Pacific Daylight Time}
                


    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2copybuffertoimage4tpu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2tpu = False
    channels2tpu = ()
    message2print = "{(Step 40A/70) trying" + "channels2tpu-trying-by-copy2buffertoimage4tpu: " + str(channels2tpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel2copybuffertoimage4tpu, publisherchannel2copybuffertoimage4tpu = task4copybuffertoimage2tpu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2copybuffertoimage4tpu-got-by-copy2buffertoimage4tpu: " + str(consumerchannel2copybuffertoimage4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 704>

        message2print = "{(Step 40C/70) got" + "; publisherchannel2copybuffertoimage4tpu-got-by-copy2buffertoimage4tpu: " + str(publisherchannel2copybuffertoimage4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 708>                      
        ack4channels2tpu = True
        
        control2copybuffertoimage4tpu['CONSUMERCHANNEL']  = consumerchannel2copybuffertoimage4tpu
        
        
        control2copybuffertoimage4tpu['PUBLISHERCHANNEL'] = publisherchannel2copybuffertoimage4tpu
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2tpu = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2copybuffertoimage4tpu = False
    queue2copybuffertoimage4tpu = None
    message2print = "{(Step 41A/70) trying" + "queue2copybuffertoimage4tpu-trying-by-copy2buffertoimage4tpu: " + str(queue2copybuffertoimage4tpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2copybuffertoimage4tpu = task4copybuffertoimage2tpu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2copybuffertoimage4tpu-got-by-copy2buffertoimage4tpu: " + str(queue2copybuffertoimage4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <queue4xi.Queue4Xi object at 0x0000000003DF2198>
              
        ack4queue2copybuffertoimage4tpu = True
        control2copybuffertoimage4tpu['QUEUE']  = queue2copybuffertoimage4tpu        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2copybuffertoimage4tpu = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4tpu = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4tpu-trying-by-copy2buffertoimage4tpu: " + str(dummy2image4tpu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4tpu = DummyProcess4Xi(target=manoeuvre2copybuffertoimage4tpu,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue2copybuffertoimage4tpu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2copybuffertoimage4tpu, publisherchannel2copybuffertoimage4tpu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4tpu-created-by-copy2buffertoimage4tpu: " + str(dummy2image4tpu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'
        
    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-copy2buffertoimage4tpu: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-11B-12B-13B-15B-16B-40D-41C- 
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-copy2buffertoimage4tpu: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # 
        
    message2print = "{(Step 70C/70) returning" + "; control2copybuffertoimage4tpu-returning-by-copy2buffertoimage4tpu: " + str(control2copybuffertoimage4tpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003DF2198>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 708>,
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 704>}; time: 04:52:11 10/04/11 Pacific Daylight Time}

        
    return control2copybuffertoimage4tpu
                    
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "read2image4tpu" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   # (16) ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4TPU                      
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 16A/700) OF TPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4TPU ', time4xi())    
#   respon4read2image = {} 
#   task4read2image = None
#   try:                               
#       response4read2image = read2image4tpu(task4read2image)                                                                                                          
#       print('(Step 16B/700) OF TPU2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH READ2IMAGE4TPU : ', response4read2image)
#       queue2read4tpu      = response4read2image['QUEUE']
#       channel2readpublish4tpu = response4read2image['PUBLISHERCHANNEL']
#       channel2readconsume4tpu = response4read2image['CONSUMERCHANNEL']
#   except:
#       print('(Step 16Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4TPU ', time4xi()) 
#   # --------------------------------------------------------------------------------------------------
#   # (17) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2IMAGE4TPU TO RECEIVE                 
#   # --------------------------------------------------------------------------------------------------      
#   print('(Step 17A/700) OF TPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4TPU ', time4xi())     
#   try:                               
#       message2readsend4tpu = "<copy4xi>copy2read4tpu</copy4xi>"
#       channel2readpublish4tpu = response4read2image['PUBLISHERCHANNEL']
#       channel2readpublish4tpu.send(message2readsend4tpu)
#       print("(Step 17B/700) OF TPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4TPU: ", channel2readpublish4tpu, " MESSAGE2SEND4TPU: " , message2readsend4tpu)                                                                                                
#   except:
#       print('(Step 17Z1/700) OF TPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4TPU ', time4xi())                                                
#   # ------------------------------------------------------------------------------------------
#   # (18) LET MANOEUVRE2IMAGE4TPU TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # ------------------------------------------------------------------------------------------       
#   print("(Step 18C/700) OF TPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4TPU ", time4xi()) 
#   try:        
#       response2manoeuvre4tpu = manoeuvre2readimage4tpu(queue2read4tpu, channel2readconsume4tpu, channel2readpublish4tpu)
#       print("(Step 18D/700) OF TPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4TPU ", manoeuvre2readimage4tpu)
#   except:
#       print("(Step 18Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4TPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # ---------------------------------------------------------------------------------------------
#   # (19) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 19A/700) OF TPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4TPU ', time4xi())     
#   try:
#       channel2readconsume4tpu = response4read2image['CONSUMERCHANNEL']                              
#       message2readreceive4tpu = channel2readconsume4tpu.recv()
#       print("(Step 19B/700) OF TPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4TPU: ", channel2readconsume4tpu, " CONTROL & COMMAND: ", message2readreceive4tpu)                                                                                                  
#       # <roger4xi>roger2read4tpu</roger4xi>
#   except:
#       print('(Step 19Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH READ2IMAGE4TPU ', time4xi())         
#   # ---------------------------------------------------------------------------------------------------
#   # (20) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2IMAGE4TPU TO RECEIVE                
#   # ---------------------------------------------------------------------------------------------------      
#   print('(Step 20A/700) OF TPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4TPU ', time4xi())     
#   try:                               
#       message2readsend4tpu = "<ready4xi>ready2read4tpu</ready4xi>"
#       channel2readpublish4tpu = response4read2image['PUBLISHERCHANNEL']
#       channel2readpublish4tpu.send(message2readsend4tpu)
#       print("(Step 20B/700) OF TPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4TPU: ", channel2readpublish4tpu, " MESSAGE2SEND4TPU: " , message2readsend4tpu)                                                                                                
#       # <ready4xi>ready2read4tpu</ready4xi> 
#   except:
#        print('(Step 20Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4TPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------
#   # (21) LET MANOEUVRE2IMAGE4TPU TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------       
#   print("(Step 21A/700) OF TPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4TPU ", time4xi()) 
#   try:        
#       response2manoeuvre4tpu = manoeuvre2readimage4tpu(queue2read4tpu, channel2readconsume4tpu, channel2readpublish4tpu)
#       print("(Step 21B/700) OF TPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4TPU ", manoeuvre2readimage4tpu)
#   except:
#       print("(Step 21Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4TPU ", time4xi())     
#   # ---------------------------------------------------------------------------------------
#   # (22) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 22A/700) OF TPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4TPU ', time4xi())     
#   try:
#       channel2readconsume4tpu = response4read2image['CONSUMERCHANNEL']                              
#       message2readreceive4tpu = channel2readconsume4tpu.recv()
#       print("(Step 22B/700) OF TPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4TPU: ", channel2readconsume4tpu, " CONTROL & COMMAND: ", message2readreceive4tpu)                                                                                                  
#       # <ready4xi>ready2read4tpu</ready4xi> 
#   except:
#       print('(Step 22Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH READ2IMAGE4TPU ', time4xi())          
#   # -----------------------------------------------------------------------------------------
#   # (24) PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 24A/700) OF TPU2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue2read4dragon.put(path2dragon)             
#       print("(Step 24B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#       # ..//..//..//..//..//web//images//png//dragonxi.png  
#   except:
#       print("(Step 24Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # ------------------------------------------------------------------------------------------------------
#   # (25) LET MANOEUVRE2IMAGE4TPU TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 25A/700) OF TPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4TPU ", time4xi()) 
#   try:        
#       response2manoeuvre4tpu = manoeuvre2readimage4tpu(queue2read4tpu, channel2readconsume4tpu, channel2readpublish4tpu)
#       print("(Step 25B/700) OF TPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4TPU ", manoeuvre2readimage4tpu)
#   except:
#       print("(Step 25Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4TPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (26) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 26A/700) OF TPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2readreceive4tpu = channel2readconsume4tpu.recv()
#       print("(Step 26D/700) OF TPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4TPU: ", channel2readconsume4tpu, " CONTROL & COMMAND: ", message2readreceive4tpu)                                                                                                   
#       # <ack4xi>ack2imageinfo4tpu</ack4xi>
#   except:
#       print('(Step 26Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -------------------------------------------------------------------------------------------------------
#   # (27) LET MANOEUVRE2IMAGE4TPU GO TO STATE 0               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 27A/700) OF TPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4TPU ", time4xi()) 
#   try:        
#       response2manoeuvre4tpu = manoeuvre2readimage4tpu(queue2read4tpu, channel2readconsume4tpu, channel2readpublish4tpu)
#       print("(Step 27B/700) OF TPU2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4TPU ", response2manoeuvre4tpu)
#   except:
#       print("(Step 27Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4TPU ", time4xi())   
#">

# -------------------------------------------------------------------------------------------------   
def read2image4tpu(task4imageread2tpu=None):

    '''
    reads 2D images for TPU
    '''
    global tpu2image4xi                                                     # 2D-IMAGES FOR TPU  
    global control2readimage4tpu                                              # COMMAND AND CONTROL FOR TPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "read2image4tpu"
    method2print = "read2image4tpu"

    message2print = "{(Step 1A/33) started" + "; read2image4tpu-started: " + str(read2image4tpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function read2image4tpu at 0x00000000039595C8> 
    
    message2print = "{(Step 1B/33) passed" + "; task4imageread2tpu-to-read2image4tpu: " + str(task4imageread2tpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE TPU2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if tpu2image4xi is None:
            tpu2image4xi = create2image4tpu()                                                                              # INITIALIZE IMAGE FOR TPU 
            message2print = "{(Step 1C/70) created" + "; tpu2image4xi-created-by-read2image4tpu to: " + str(tpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4tpu-skipped-by-read2image4tpu: " + str(read2image4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # skipped                                 
    except:
        try:
            tpu2image4xi = create2image4tpu()
            message2print = "{(Step 1E/70) set" + "; tpu2image4xi-set-by-read2image4tpu to: " + str(tpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            tpu2image4xi = {}
            message2print = "{(Step 1Z/70) yielded" + "; tpu2image4xi-yielded-by-read2image4tpu: " + str(tpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z-' 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    try:             
        if task4imageread2tpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR READ - BUILD TASK! 
            # -----------------------------------------------------------------------------------------    
            
            task4imageread2tpu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1A/70) initialized" + "; task4imageread2tpu-created-by-read2image4tpu to: " + str(task4imageread2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
            
            # ---------------------------------------------------------------------------------------------
            # (1B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4tpu = trace2image4tpu                                                              # CALLBACK FUNCTION FOR TPU
            
            message2print = "{(Step 1B1/70) initialized" + "; function2callback4tpu-created-by-read2image4tpu to: " + str(function2callback4tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # <function trace2image4tpu at 0x0000000004737548>;
            
            code2ack  = code2ack + '1B1-'
           
            args2callback = {}                                                                                   # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "read2image4tpu"                                                          # MODULE OF TPUIZED OPENCL MODULE
            args2callback['METHOD']  = "read2image4tpu"                                                          # FUNCTION OF TPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; read2image4tpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR TPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-read2image4tpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {'MESSAGE': '{(Step 1Z1/33) failed; read2image4tpu-failed ; time: 05:06:38 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'read2image4tpu', 
            #  'METHOD': 'read2image4tpu'}; time: 05:06:38 10/04/11 Pacific Daylight Time} 

            code2ack  = code2ack + '1B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (1C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imageread2tpu['DATA2CALLBACK'] = args2callback                                                   # USER DATA PASSED TO TPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imageread2tpu['FUNCTION2CALLBACK'] = function2callback4tpu                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '1C-'
            message2print = "{(Step 1C/70) initialized" + "; task4imageread2tpu-created-by-read2image4tpu to: " + str(task4imageread2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # {'FUNCTION2CALLBACK': <function trace2image4tpu at 0x0000000004737548>, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; read2image4tpu-failed ; time: 05:06:38 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'read2image4tpu', 
            #  'METHOD': 'read2image4tpu'}}; time: 05:06:38 10/04/11 Pacific Daylight Time}
             
            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-read2image4tpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # {}
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-read2image4tpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}
                 
                code2ack  = code2ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-read2image4tpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '2Z-'
                 
            task4imageread2tpu['PLATFORM'] = platform2xi                                                          # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2tpu = {}
            message2print = "{(Step 3A/70) trying" + "; context2tpu-trying-by-read2image4tpu: " + str(context2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2tpu = get2context4tpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context2tpu-got-by-read2image4tpu: " + str(context2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': ...
    
                code2ack  = code2ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context2tpu-failed-by-read2image4tpu: " + str(context2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '3Z-'
                
            task4imageread2tpu ['CONTEXT'] = context2tpu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2tpu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device2tpu-trying-by-read2image4tpu: " + str(device2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2tpu = get2tpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device2tpu-got-by-read2image4tpu: " + str(device2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'RESOLUTION2TIMER4PROFILING': 318, 
                #  'ID2PLATFORM': '0000000001509568', 
                #  'PROFILING2QUEUE': 'ACK', 
                #  'EXECUTE2KERNELS4OPENCL': 'ACK', 
                #  'NATIVE2WIDTHLONG4VECTOR': 2 ...
                
                code2ack  = code2ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device2tpu-failed-by-read2image4tpu: " + str(device2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '4Z-'
            
            task4imageread2tpu ['DEVICE'] = device2tpu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2tpu = {}
             
            message2print = "{(Step 5A/70) trying" + "; imageinfo2tpu-trying-by-read2image4tpu: " + str(imageinfo2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2tpu = get2imageinfo4tpu()                                                                         
                
                message2print = "{(Step 5B/70) got" + "; imageinfo2tpu-got-by-read2image4tpu: " + str(imageinfo2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'SIZE2QUEUE': 2147483646, 
                #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
                #  'ORDER': 'CL_RGBA'}, 
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x00000000041F9320>, 
                #  'WIDTH': 800, 
                #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
                
                code2ack  = code2ack + '5B-'  
            except:
                message2print = "{(Step 5Z/70) failed" + "; imageinfo2tpu-failed-by-read2image4tpu: " + str(imageinfo2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '5Z-'          
                      
            task4imageread2tpu ['IMAGE'] = imageinfo2tpu         
                        

            # ---------------------------------------------------------------------------------------------
            # (6) QUEUE - REQUIRED BY TPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2tpu = None
            
            message2print = "{(Step 6A/70) trying" + "; queue2tpu-trying-by-read2image4tpu: " + str(queue2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2tpu = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 6B/70) got" + "; queue2tpu-got-by-read2image4tpu: " + str(queue2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # <queue4xi.Queue4Xi object at 0x0000000004607160> 
                
                code2ack  = code2ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; queue2tpu-failed-by-read2image4tpu: " + str(queue2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '6Z-'
                
            task4imageread2tpu ['QUEUE'] = queue2tpu

            
            # ---------------------------------------------------------------------------------------------
            # (7) PIPE - REQUIRED BY TPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2tpu = None
           
            message2print = "{(Step 7A/70) trying" + "; pipe2tpu-trying-by-read2image4tpu: " + str(pipe2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2tpu = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 7B/70) got" + "; pipe2tpu-got-by-read2image4tpu: " + str(pipe2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # (<read-write PipeConnection, handle 408>, <read-write PipeConnection, handle 412>) 
                
                code2ack  = code2ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; pipe2tpu-failed-by-read2image4tpu: " + str(pipe2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '7Z-'
                
            task4imageread2tpu ['PIPE'] = pipe2tpu 
            
            # ---------------------------------------------------------------------------------------------
            # (8) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2tpu = {}
           
            message2print = "{(Step 8A/70) trying" + "; commandqueueinfo2tpu-trying-by-read2image4tpu: " + str(commandqueueinfo2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2tpu = get2commandqueueinfo4tpu()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 8B/70) got" + "; commandqueueinfo2tpu-got-by-read2image4tpu: " + str(commandqueueinfo2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 
                #  'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': ...
                
                code2ack  = code2ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; commandqueueinfo2tpu-failed-by-read2image4tpu: " + str(commandqueueinfo2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '8Z-'
        
            task4imageread2tpu ['COMMANDQUEUE'] = commandqueueinfo2tpu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (9) BUFFER - AUTOMATIC BY TPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2tpu = {}
           
            message2print = "{(Step 9A/70) trying" + "; bufferinfo2tpu-trying-by-read2image4tpu: " + str(bufferinfo2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2tpu = get2bufferinfo4tpu()                                                                   # SYSTEM CREATES BUFFER FOR TPU, IF NECESSARY
                message2print = "{(Step 9B/70) got" + "; bufferinfo2tpu-got-by-read2image4tpu: " + str(bufferinfo2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
                
                code2ack  = code2ack + '9B-' 
            except:
                message2print = "{(Step 9Z/70) failed" + "; bufferinfo2tpu-failed-by-read2image4tpu: " + str(bufferinfo2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '9Z-'
        
            task4imageread2tpu ['BUFFER'] = bufferinfo2tpu 
            
            # ---------------------------------------------------------------------------------------------
            # (10) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2tpu = {}
           
            message2print = "{(Step 10A/70) trying" + "; eventinfo2tpu-trying-by-read2image4tpu: " + str(eventinfo2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2tpu = get2eventinfo4tpu()                                                                    
                message2print = "{(Step 10B/70) got" + "; eventinfo2tpu-got-by-read2image4tpu: " + str(eventinfo2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'DATA2CALLBACK': {'MESSAGE': ...
                
                code2ack  = code2ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; eventinfo2tpu-failed-by-read2image4tpu: " + str(eventinfo2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '10Z-'
        
            task4imageread2tpu ['EVENT'] = eventinfo2tpu 
            
            
            # ---------------------------------------------------------------------------------------------
            # (11) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2tpu = {}
           
            message2print = "{(Step 11A/70) trying" + "; eventwaitlist2tpu-trying-by-read2image4tpu: " + str(eventwaitlist2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2tpu = {}                                                                   
                message2print = "{(Step 11B/70) set" + "; eventwaitlist2tpu-set-by-read2image4tpu: " + str(eventwaitlist2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {}
                
                code2ack  = code2ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; eventwaitlist2tpu-failed-by-read2image4tpu: " + str(eventwaitlist2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '11Z-'
        
            task4imageread2tpu ['EVENTWAITLIST'] = eventwaitlist2tpu                     

            
            # ---------------------------------------------------------------------------------------------
            # (12) EVENTWAITLISTQUANTITY - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2tpu = 0
           
            message2print = "{(Step 12A/70) trying" + "; eventwaitlistquantity2tpu-trying-by-read2image4tpu: " + str(eventwaitlistquantity2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2tpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2tpu = eventwaitlistquantity2tpu + 1                                                                   
                message2print = "{(Step 12B/70) set" + "; eventwaitlistquantity2tpu-set-by-read2image4tpu: " + str(eventwaitlistquantity2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 
                
                code2ack  = code2ack + '12B-' 
            except:
                message2print = "{(Step 12Z/70) failed" + "; eventwaitlistquantity2tpu-failed-by-read2image4tpu: " + str(eventwaitlistquantity2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '12Z-'
        
            task4imageread2tpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2tpu       
                                                 
            
            # ---------------------------------------------------------------------------------------------
            # (13) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2tpu = ' '                                                                   
     
            message2print = "{(Step 13A/70) trying" + "; blocking2tpu-trying-by-read2image4tpu: " + str(blocking2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2tpu = 'CL_FALSE'                                                          # the read command is non-blocking                                                              
                message2print = "{(Step 13B/70) set" + "; blocking2tpu-set-by-read2image4tpu to: " + str(blocking2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # CL_FALSE
                code2ack  = code2ack + '13B-' 
            except:
                message2print = "{(Step 13Z/70) failed" + "; blocking2tpu-failed-by-read2image4tpu: " + str(blocking2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '13Z-'
        
            task4imageread2tpu ['BLOCKING'] = blocking2tpu             
            
            # ---------------------------------------------------------------------------------------------
            # (14) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2tpu = {}
            
            message2print = "{(Step 14A/70) trying" + "; origin2tpu-trying-by-read2image4tpu: " + str(origin2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2tpu['X'] = 0 
                origin2tpu['Y'] = 0  
                origin2tpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 14B/70) set" + "; origin2tpu-set-by-read2image4tpu to: " + str(origin2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'Y': 0, 'X': 0, 'Z': 0} 
                code2ack  = code2ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; origin2tpu-failed-by-read2image4tpu: " + str(origin2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '14Z-'
        
            task4imageread2tpu ['ORIGIN'] = origin2tpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (15) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2tpu = {}
            
            message2print = "{(Step 15A/70) trying" + "; region2tpu-trying-by-read2image4tpu: " + str(region2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2tpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO READ 
                region2tpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO READ 
                region2tpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO READ                                                                                                                          
                message2print = "{(Step 15B/70) set" + "; region2tpu-set-by-read2image4tpu to: " + str(region2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'Y': 90, 'X': 160, 'Z': 0}
                code2ack  = code2ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; region2tpu-failed-by-read2image4tpu: " + str(region2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '15Z-'
        
            task4imageread2tpu ['REGION'] = region2tpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (16) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.


            rowpitch2tpu = 0
            size2element = 8                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 10                                                                                # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 8                                                                                  # ONE DIRECTIONS IN 2D
            width2tpu = 800
            
            message2print = "{(Step 16A/70) trying" + "; rowpitch2tpu-trying-by-read2image4tpu: " + str(rowpitch2tpu)  + "; time: " + time4xi() + "}"  
            print4xi(class2print, method2print, message2print) 
            # 
                     
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2tpu = int(width2tpu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 16B/70) set" + "; rowpitch2tpu-set-by-read2image4tpu to: " + str(rowpitch2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #  8000                                                                                          
                 
                code2ack  = code2ack + '16B-' 
            except:
                message2print = "{(Step 16Z/70) failed" + "; rowpitch2tpu-failed-by-read2image4tpu: " + str(rowpitch2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '16Z-'
        
            task4imageread2tpu ['ROWPITCH'] = rowpitch2tpu     
            
            # ----------------------------------------------------------------------------------------------------
            # (17) POINTER - REQUIRED - THE POINTER TO A BUFFER IN HOST MEMORY WHERE IMAGE DATA IS TO BE READ FROM                          
            # ----------------------------------------------------------------------------------------------------             
            pointer2tpu = None
            height2tpu = 800
            size2tpu = int(width2tpu * height2tpu)                                                                  # CALCULATED MEMORY
            if size2tpu == 0:
                size2tpu = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 17A/70) trying" + "; pointer2tpu-trying-by-read2image4tpu: " + str(pointer2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena2tpu = Arena4Xi(size2tpu)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 17B/70) got" + "; arena2tpu-got-by-read2image4tpu: " + str(arena2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # <arena4xi.Arena4Xi object at 0x0000000004449EF0>
                
                size2arena = arena2tpu.size                                                                         # 
                              
                message2print = "{(Step 17C/70) got" + "; size2arena-got-by-read2buffer4tpu: " + str(size2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 640000
                
                buffer2arena = arena2tpu.buffer
                
                message2print = "{(Step 17D/70) got" + "; buffer2arena-got-by-read2buffer4tpu: " + str(buffer2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # <mmap.mmap object at 0x00000000041A0C90>
                
                code2ack  = code2ack + '17D-' 
                
                if size2arena < size2tpu:
                    size2tpu = size2arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 17E/70) set" + "; size2tpu-set-by-read2buffer4tpu to: " + str(size2tpu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code2ack  = code2ack + '17E-'
                else:
                    pass
                    
                pointer2tpu = buffer2arena
                message2print = "{(Step 17F/70) set" + "; pointer2tpu-set-by-read2buffer4tpu to: " + str(pointer2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # <mmap.mmap object at 0x00000000041A0C90>
                
                code2ack  = code2ack + '17F-'       
            except:
                message2print = "{(Step 17Z/70) failed" + "; pointer2tpu-failed-by-read2image4tpu: " + str(pointer2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '17Z-'
        
            task4imageread2tpu ['POINTER'] = pointer2tpu                            
            task4imageread2tpu ['SIZE']    = size2tpu                                                               # AUTOMATIC BY TPUXI
            
            
            # --------------------------------------------------------------------------------------------------------
            # (18) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING READ                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2tpu = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 18A/70) trying" + "; slicepitch2tpu-trying-by-read2image4tpu: " + str(slicepitch2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2tpu = 0                                                          
                message2print = "{(Step 18B/70) set" + "; slicepitch2tpu-set-by-read2image4tpu to: " + str(slicepitch2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 

                code2ack  = code2ack + '18B-'                
                 
            except:
                message2print = "{(Step 18Z/70) failed" + "; slicepitch2tpu-failed-by-read2image4tpu: " + str(slicepitch2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '18Z-'
            
            task4imageread2tpu ['SLICEPITCH'] = slicepitch2tpu                  
            
            # ---------------------------------------------------------------------------------------------
            # EOF PARAMETER PROVIDING                
            # ---------------------------------------------------------------------------------------------                                                                                                                               
        else:
            pass
        # ---------------------------------------------------------------------------------------------
        # ANALYSE TASKS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imageread2tpu.keys()
        message2print = "{(Step 20/70) got" + "; keys2task-got-by-read2image4tpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'POINTER',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SIZE',
        #  'SLICEPITCH']; time: 05:25:42 10/04/11 Pacific Daylight Time}

        if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4tpu = trace2image4tpu                                                              # CALLBACK FUNCTION FOR TPU
            
            message2print = "{(Step 21B1/70) initialized" + "; function2callback4tpu-created-by-read2image4tpu to: " + str(function2callback4tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '21B1-'
           
            args2callback = {}                                                                                   # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "read2image4tpu"                                                          # MODULE OF TPUIZED OPENCL MODULE
            args2callback['METHOD']  = "read2image4tpu"                                                          # FUNCTION OF TPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; read2image4tpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR TPUIZED OPENCL TO RETURN    

            message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-read2image4tpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 

            code2ack  = code2ack + '21B2-'
        
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imageread2tpu['DATA2CALLBACK'] = args2callback                                                   # USER DATA PASSED TO TPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imageread2tpu['FUNCTION2CALLBACK'] = function2callback4tpu                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4imageread2tpu-created-by-read2image4tpu to: " + str(task4imageread2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
             
        if 'PLATFORM' in keys2task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-read2image4tpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-read2image4tpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
                code2ack  = code2ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-read2image4tpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '22Z-'
                 
            task4imageread2tpu['PLATFORM'] = platform2xi                                                          # PLATFORM OF SYSTEM 
          
        if 'CONTEXT' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2tpu = {}
            message2print = "{(Step 23A/70) trying" + "; context2tpu-trying-by-read2image4tpu: " + str(context2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2tpu = get2context4tpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context2tpu-got-by-read2image4tpu: " + str(context2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context2tpu-failed-by-read2image4tpu: " + str(context2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '23Z-'
                
            task4imageread2tpu ['CONTEXT'] = context2tpu           
        
        if 'DEVICE' in keys2task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2tpu = {}
            
            message2print = "{(Step 24A/70) trying" + "; device2tpu-trying-by-read2image4tpu: " + str(device2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2tpu = get2tpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device2tpu-got-by-read2image4tpu: " + str(device2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device2tpu-failed-by-read2image4tpu: " + str(device2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '24Z-'
            
            task4imageread2tpu ['DEVICE'] = device2tpu
        
        if 'IMAGE' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2tpu = {}
             
            message2print = "{(Step 25A/70) trying" + "; imageinfo2tpu-trying-by-read2image4tpu: " + str(imageinfo2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2tpu = get2imageinfo4tpu()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; imageinfo2tpu-got-by-read2image4tpu: " + str(imageinfo2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; imageinfo2tpu-failed-by-read2image4tpu: " + str(imageinfo2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '25Z-'          
                      
            task4imageread2tpu ['IMAGE'] = imageinfo2tpu         
                    
        if 'QUEUE' in keys2task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (26) QUEUE - REQUIRED BY TPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2tpu = None
            
            message2print = "{(Step 26A/70) trying" + "; queue2tpu-trying-by-read2image4tpu: " + str(queue2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2tpu = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 26B/70) got" + "; queue2tpu-got-by-read2image4tpu: " + str(queue2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; queue2tpu-failed-by-read2image4tpu: " + str(queue2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '26Z-'
                
            task4imageread2tpu ['QUEUE'] = queue2tpu
        
        if 'PIPE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (27) PIPE - REQUIRED BY TPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2tpu = None
           
            message2print = "{(Step 27A/70) trying" + "; pipe2tpu-trying-by-read2image4tpu: " + str(pipe2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2tpu = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 27B/70) got" + "; pipe2tpu-got-by-read2image4tpu: " + str(pipe2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; pipe2tpu-failed-by-read2image4tpu: " + str(pipe2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '27Z-'
                
            task4imageread2tpu ['PIPE'] = pipe2tpu 
            
        if 'COMMANDQUEUE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2tpu = {}
           
            message2print = "{(Step 28A/70) trying" + "; commandqueueinfo2tpu-trying-by-read2image4tpu: " + str(commandqueueinfo2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2tpu = get2commandqueueinfo4tpu()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 28B/70) got" + "; commandqueueinfo2tpu-got-by-read2image4tpu: " + str(commandqueueinfo2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; commandqueueinfo2tpu-failed-by-read2image4tpu: " + str(commandqueueinfo2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '28Z-'
        
            task4imageread2tpu ['COMMANDQUEUE'] = commandqueueinfo2tpu
        
        
        if 'BUFFER' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (29) BUFFER - AUTOMATIC BY TPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2tpu = {}
           
            message2print = "{(Step 29A/70) trying" + "; bufferinfo2tpu-trying-by-read2image4tpu: " + str(bufferinfo2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2tpu = get2bufferinfo4tpu()                                                                   # SYSTEM CREATES BUFFER FOR TPU, IF NECESSARY
                message2print = "{(Step 29B/70) got" + "; bufferinfo2tpu-got-by-read2image4tpu: " + str(bufferinfo2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '29B-' 
            except:
                message2print = "{(Step 29Z/70) failed" + "; bufferinfo2tpu-failed-by-read2image4tpu: " + str(bufferinfo2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '29Z-'
        
            task4imageread2tpu ['BUFFER'] = bufferinfo2tpu
             
        if 'EVENT' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (30) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2tpu = {}
           
            message2print = "{(Step 30A/70) trying" + "; eventinfo2tpu-trying-by-read2image4tpu: " + str(eventinfo2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2tpu = get2eventinfo4tpu()                                                                    
                message2print = "{(Step 30B/70) got" + "; eventinfo2tpu-got-by-read2image4tpu: " + str(eventinfo2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '30B-' 
            except:
                message2print = "{(Step 30Z/70) failed" + "; eventinfo2tpu-failed-by-read2image4tpu: " + str(eventinfo2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '30Z-'
        
            task4imageread2tpu ['EVENT'] = eventinfo2tpu 
        
             
        if 'EVENTWAITLIST' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2tpu = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventwaitlist2tpu-trying-by-read2image4tpu: " + str(eventwaitlist2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2tpu = {}                                                                   
                message2print = "{(Step 31B/70) set" + "; eventwaitlist2tpu-set-by-read2image4tpu: " + str(eventwaitlist2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventwaitlist2tpu-failed-by-read2image4tpu: " + str(eventwaitlist2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '31Z-'
        
            task4imageread2tpu ['EVENTWAITLIST'] = eventwaitlist2tpu                     
              
        if 'EVENTWAITLISTQUANTITY' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLISTQUANTITY - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2tpu = 0
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlistquantity2tpu-trying-by-read2image4tpu: " + str(eventwaitlistquantity2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2tpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2tpu = eventwaitlistquantity2tpu + 1                                                                      
                message2print = "{(Step 32B/70) set" + "; eventwaitlistquantity2tpu-set-by-read2image4tpu: " + str(eventwaitlistquantity2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0
                
                code2ack  = code2ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlistquantity2tpu-failed-by-read2image4tpu: " + str(eventwaitlistquantity2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '32Z-'
        
            task4imageread2tpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2tpu       
                                                 
                 
        if 'BLOCKING' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (33) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2tpu = ' '                                                                   
            message2print = "{(Step 33A/70) trying" + "; blocking2tpu-trying-by-read2image4tpu: " + str(blocking2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2tpu = 'CL_FALSE'                                                          # the read command is non-blocking                                                              
                message2print = "{(Step 33B/70) set" + "; blocking2tpu-set-by-read2image4tpu to: " + str(blocking2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; blocking2tpu-failed-by-read2image4tpu: " + str(blocking2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '33Z-'
        
            task4imageread2tpu ['BLOCKING'] = blocking2tpu             
             
        if 'ORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2tpu = {}
            
            message2print = "{(Step 34A/70) trying" + "; origin2tpu-trying-by-read2image4tpu: " + str(origin2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2tpu['X'] = 0 
                origin2tpu['Y'] = 0  
                origin2tpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; origin2tpu-set-by-read2image4tpu to: " + str(origin2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; origin2tpu-failed-by-read2image4tpu: " + str(origin2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '34Z-'
        
            task4imageread2tpu ['ORIGIN'] = origin2tpu
            
             
        if 'REGION' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (35) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2tpu = {}
            
            message2print = "{(Step 35A/70) trying" + "; region2tpu-trying-by-read2image4tpu: " + str(region2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                            
            try:                                                                                    
                region2tpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO READ 
                region2tpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO READ 
                region2tpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO READ                                                                                                                          
                message2print = "{(Step 35B/70) set" + "; region2tpu-set-by-read2image4tpu to: " + str(region2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '35B-' 
            except:
                message2print = "{(Step 35Z/70) failed" + "; region2tpu-failed-by-read2image4tpu: " + str(region2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '35Z-'
        
            task4imageread2tpu ['REGION'] = region2tpu
        
            
             
        if 'ROWPITCH' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (36) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.
     
            rowpitch2tpu = 0
            size2element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 1                                                                                  # ONE DIRECTIONS IN 2D
            
            message2print = "{(Step 36A/70) trying" + "; rowpitch2tpu-trying-by-read2image4tpu: " + str(rowpitch2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2tpu = int(width2tpu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 36B/70) set" + "; rowpitch2tpu-set-by-read2image4tpu to: " + str(rowpitch2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #                                                                                             
                 
                code2ack  = code2ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; rowpitch2tpu-failed-by-read2image4tpu: " + str(rowpitch2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '36Z-'
        
            task4imageread2tpu ['ROWPITCH'] = rowpitch2tpu     
             
        if 'POINTER' in keys2task:
            pass
        else:                
            # ----------------------------------------------------------------------------------------------------
            # (37) POINTER - REQUIRED - THE POINTER TO A BUFFER IN HOST MEMORY WHERE IMAGE DATA IS TO BE READ FROM                          
            # ----------------------------------------------------------------------------------------------------             
            pointer2tpu = None
            size2tpu = int(width2tpu * height2tpu)                                                                  # CALCULATED MEMORY
            if size2tpu == 0:
                size2tpu = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 37A/70) trying" + "; pointer2tpu-trying-by-read2image4tpu: " + str(pointer2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena2tpu = Arena4Xi(size2tpu)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 37B/70) got" + "; arena2tpu-got-by-read2image4tpu: " + str(arena2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                size2arena = arena2tpu.size                                                                         # 
                              
                message2print = "{(Step 37C/70) got" + "; size2arena-got-by-read2buffer4tpu: " + str(size2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                buffer2arena = arena2tpu.buffer
                
                message2print = "{(Step 37D/70) got" + "; buffer2arena-got-by-read2buffer4tpu: " + str(buffer2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                
                code2ack  = code2ack + '37D-' 
                
                if size2arena < size2tpu:
                    size2tpu = size2arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 37E/70) set" + "; size2tpu-set-by-read2buffer4tpu to: " + str(size2tpu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code2ack  = code2ack + '37E-'
                else:
                    pass
                    
                pointer2tpu = buffer2arena
                message2print = "{(Step 37F/70) set" + "; pointer2tpu-set-by-read2buffer4tpu to: " + str(pointer2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                code2ack  = code2ack + '37F-'       
            except:
                message2print = "{(Step 37Z/70) failed" + "; pointer2tpu-failed-by-read2image4tpu: " + str(pointer2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '37Z-'
        
            task4imageread2tpu ['POINTER'] = pointer2tpu                            
            task4imageread2tpu ['SIZE']    = size2tpu                                                               # AUTOMATIC BY TPUXI
        
             
        if 'SLICEPITCH' in keys2task:
            pass
        else:               
            # --------------------------------------------------------------------------------------------------------
            # (38) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING READ                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2tpu = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 38A/70) trying" + "; slicepitch2tpu-trying-by-read2image4tpu: " + str(slicepitch2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2tpu = 0                                                          
                message2print = "{(Step 38B/70) set" + "; slicepitch2tpu-set-by-read2image4tpu to: " + str(slicepitch2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '38B-'                
                 
            except:
                message2print = "{(Step 38Z/70) failed" + "; slicepitch2tpu-failed-by-read2image4tpu: " + str(slicepitch2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '38Z-'
            
            task4imageread2tpu ['SLICEPITCH'] = slicepitch2tpu
                              
        # ---------------------------------------------------------------------------------------------
        # EOF TASK ANALYSIS
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imageread2tpu.keys()
        message2print = "{(Step 39/70) set" + "; keys2task-set-by-read2image4tpu to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'POINTER',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SIZE',
        #  'SLICEPITCH']; time: 05:25:42 10/04/11 Pacific Daylight Time}                
    except: 
        message2print = "{(Step 39Z/70) yielded" + "; task4imageread2tpu-yielded-by-read2image4tpu: " + str(task4imageread2tpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '39Z-' 

    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2readimage4tpu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2tpu = False
    channels2tpu = ()
    message2print = "{(Step 40A/70) trying" + "channels2tpu-trying-by-read2image4tpu: " + str(channels2tpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # (<read-write PipeConnection, handle 520>, <read-write PipeConnection, handle 524>) 
        consumerchannel2tpu, publisherchannel2tpu = task4imageread2tpu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2tpu-got-by-read2image4tpu: " + str(consumerchannel2tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 

        message2print = "{(Step 40C/70) got" + "; publisherchannel2tpu-got-by-read2image4tpu: " + str(publisherchannel2tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        #                      
        ack4channels2tpu = True
        
        control2readimage4tpu['CONSUMERCHANNEL']  = consumerchannel2tpu
        
        
        control2readimage4tpu['PUBLISHERCHANNEL'] = publisherchannel2tpu
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2tpu = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2tpu = False
    queue2tpu = None
    message2print = "{(Step 41A/70) trying" + "; queue2tpu-trying-by-read2image4tpu: " + str(queue2tpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2tpu = task4imageread2tpu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2tpu-got-by-read2image4tpu: " + str(queue2tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
              
        ack4queue2tpu = True
        control2readimage4tpu['QUEUE']  = queue2tpu        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2tpu = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4tpu = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4tpu-trying-by-read2image4tpu: " + str(dummy2image4tpu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4tpu = DummyProcess4Xi(target=manoeuvre2readimage4tpu,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue2tpu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2tpu, publisherchannel2tpu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4tpu-created-by-read2image4tpu: " + str(dummy2image4tpu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'

    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-read2image4tpu: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-12B-13B-14B-15B-16B-17D-17F-18B-40D-41C-
     
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-read2image4tpu: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #   
        
    message2print = "{(Step 70C/70) returning" + "; control2readimage4tpu-returning-by-read2image4tpu: " + str(control2readimage4tpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003FE3B70>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 416>, 
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 412>}
        
    return control2readimage4tpu
                    
#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "map2image4tpu" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   # ESTABLISH CONTROL & COMMAND                 
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 92A/700) OF TPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH MAP2IMAGE4TPU ', time4xi())           
#                    
#   response4map2image = {} 
#   task4map2image = None                                                                                  # WE ONLY WANT TO ESTABLISH CONTROL AND CONTROL FIRST
#
#   try:                               
#       response4map2image = map2image4tpu(task4map2image)                                                                                                          
#       print('(Step 92B/700) OF TPU2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH MAP2IMAGE4TPU : ', response4map2image)
#    
#       queue4map2image4tpu      = response4map2image['QUEUE']
#       channel4map2imagepublish4tpu = response4map2image['PUBLISHERCHANNEL']
#       channel4map2imageconsume4tpu = response4map2image['CONSUMERCHANNEL']
#   except:
#       print('(Step 92Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH MAP2IMAGE4TPU ', time4xi()) 
#        
#   # -----------------------------------------------------------------------------------------------------------
#   # SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2COPYIMAGE4TPU TO RECEIVE                 
#   # -----------------------------------------------------------------------------------------------------------      
#   print('(Step 93A/700) OF TPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4TPU ', time4xi())      
#   try:                               
#       message2send4tpu = "<copy4xi>copy2mapimage4tpu</copy4xi>"
#       channel4map2imagepublish4tpu.send(message2send4tpu)
#       print("(Step 93B/700) OF TPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4TPU: ", channel4map2imagepublish4tpu, " MESSAGE2SEND4TPU: " , message2send4tpu)                                                                                                
#   except:
#       print('(Step 93Z1/700) OF TPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4TPU ', time4xi())   
# 
#   # ---------------------------------------------------------------------------------------------------------------------
#   # SIMULATE MANOEUVRE2COPYIMAGE4TPU  - LET IT TO RECEIVE COPY AND TO SEND ROGER VIA PIPE                
#   # ---------------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4tpu is True:
#       print("(Step 94A/700) OF TPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4TPU ", time4xi())   
#       try:        
#           response2manoeuvre4copy = manoeuvre2mapimage4tpu(queue4map2image4tpu, channel4map2imageconsume4tpu, channel4map2imagepublish4tpu)
#           print("(Step 94B/700) OF TPU2IMAGE4XI5TEST - PERFORMED MANOEUVRE2COPYIMAGE4TPU, RESPONSE2MANOEUVRE4COPY: ", response2manoeuvre4copy)
#        except:
#           print("(Step 94Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4TPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#
#   # ---------------------------------------------------------------------------------------------
#   # RECEIVE CONTROL & COMMAND ROGER VIA PIPE                  
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 95A/700) OF TPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4TPU ', time4xi())      
#   try:
#       message2receive4tpu = channel4map2imageconsume4tpu.recv()
#       print("(Step 95B/700) OF TPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4TPU: ", channel4map2imageconsume4tpu, " CONTROL & COMMAND: ", message2receive4tpu)                                                                                                  
#   except:
#       print('(Step 95Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH MAP2IMAGE4TPU ', time4xi())          
#   
#   # ------------------------------------------------------------------------------------------------------------
#   # SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2COPYIMAGE4TPU TO RECEIVE                
#   # ------------------------------------------------------------------------------------------------------------      
#   print('(Step 96A/700) OF TPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4TPU ', time4xi())     
#   try:                               
#       message2send4tpu = "<ready4xi>ready2mapimage4tpu</ready4xi>"
#       channel4map2imagepublish4tpu.send(message2send4tpu)
#       print("(Step 96B/700) OF TPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4TPU: ", channel4map2imagepublish4tpu, " MESSAGE2SEND4TPU: " , message2send4tpu)                                                                                                
#   except:
#       print('(Step 84Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4TPU ', time4xi())                                                
# 
#   # -------------------------------------------------------------------------------------------------------
#   # SIMULATE MANOEUVRE2COPYIMAGE4TPU  - LET IT RECEIVE READY AND SEND READY                
#   # -------------------------------------------------------------------------------------------------------       
#   if ack2simulate4tpu is True:
#       print("(Step 97A/700) OF TPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4TPU ", time4xi())  
#       try:        
#           response2manoeuvre4copy = manoeuvre2mapimage4tpu(queue4map2image4tpu, channel4map2imageconsume4tpu, channel4map2imagepublish4tpu)
#           print("(Step 97B/700) OF TPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGE4TPU ", manoeuvre2copyimage4tpu)
#       except:
#           print("(Step 97Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4TPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#            
#    
#   # ---------------------------------------------------------------------------------------
#   # RECEIVE CONTROL & COMMAND READY VIA PIPE                
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 98A/700) OF TPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4TPU ', time4xi())     
#   try:
#       message2receive4tpu = channel4map2imageconsume4tpu.recv()
#       print("(Step 98B/700) OF TPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4TPU: ", channel4map2imageconsume4tpu, " CONTROL & COMMAND: ", message2receive4tpu)                                                                                                  
#   except:
#       print('(Step 86Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH MAP2IMAGE4TPU ', time4xi())          
#                         
#   # -----------------------------------------------------------------------------------------
#   # IMAGE TPUXI COPY OPERATIONS 
#   # -----------------------------------------------------------------------------------------         
#   print('(Step 99A/700) OF TPU2IMAGE4XI5TEST - TRYING FILE OPERATIONS WITH IMAGE TPUXI ', time4xi())       
#   try:        
#       path2dragon = '..//..//..//..//..//web//images//png//dragonxi.png'
#       file2dragon = open(path2dragon)
#       print("(Step 99B/700) OF TPU2IMAGE4XI5TEST - FILENO: ", file2tpu.fileno( ))  
#       print("(Step 99C/700) OF TPU2IMAGE4XI5TEST - MODE: ", file2tpu.mode)
#       print("(Step 99D/700) OF TPU2IMAGE4XI5TEST - NAME: ", file2tpu.name)
#       offset2tpu = 10
#       file2tpu.seek(offset2tpu)
#       print("(Step 99E/700) OF TPU2IMAGE4XI5TEST - PERFORMED SEEK WITH OFFSET: ", offset2tpu)
#       file2tpu.close()
#       print("(Step 99F/700) OF DRAGON2IMAGE4XI5TEST - CLOSED IMAGE FILE: ", path2dragon)  
#   except:
#       print("(Step 87Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO PERFORM FILE OPERATIONS WITH IMAGE: ", path2dragon)        
# 
#   # -----------------------------------------------------------------------------------------
#   # PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 100A/700) OF TPU2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue4map2image4dragon.put(path2dragon)             
#       print("(Step 100B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#   except:
#       print("(Step 88Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   
#   # -------------------------------------------------------------------------------------------------------------------------------------
#   # SIMULATE MANOEUVRE2COPYIMAGE4TPU  - STATE III - GETS FROM QUEUE AND SENDS EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4tpu is True:
#       print("(Step 101A/700) OF TPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4TPU ", time4xi()) 
#       try:        
#           response2manoeuvre4copy = manoeuvre2mapimage4tpu(queue4map2image4tpu, channel4map2imageconsume4tpu, channel4map2imagepublish4tpu)
#           print("(Step 101B/700) OF TPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGE4TPU ", manoeuvre2copyimage4tpu)
#       except:
#           print("(Step 101Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4TPU ", time4xi())     
#   else:
#       pass # trust activated thread                     
#                 
#                                  
#   # ----------------------------------------------------------------------------------------------------
#   # RECEIVE ACK OR NACK ABOUT IMAGE OPERATION CONTROL & COMMAND VIA PIPE                
#   # ----------------------------------------------------------------------------------------------------          
#   print('(Step 102A/700) OF TPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2receive4tpu = channel4map2imageconsume4tpu.recv()
#       print("(Step 102B/700) OF TPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4TPU: ", channel4map2imageconsume4tpu, " CONTROL & COMMAND: ", message2receive4tpu)                                                                                                   
#   except:
#       print('(Step 90Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#
#   
#   # ----------------------------------------------------------------------------------------------------------------
#   # SIMULATE MANOEUVRE2COPYIMAGE4TPU  - LET MANOEUVRE2COPYIMAGE4TPU TO GO TO STATE 0               
#   # ----------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4tpu is True:
#       print("(Step 103A/700) OF TPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4TPU ", time4xi())  
#       try:        
#           response2manoeuvre4copy = manoeuvre2mapimage4tpu(queue4map2image4tpu, channel4map2imageconsume4tpu, channel4map2imagepublish4tpu)
#           print("(Step 103B/700) OF TPU2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4TPU ", response2manoeuvre4copy)
#       except:
#           print("(Step 103Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4TPU ", time4xi())     
#   else:
#       pass # trust activated thread  
#                                           
#">

# -------------------------------------------------------------------------------------------------   
def map2image4tpu(task4imagemap2tpu=None):

    '''
    maps 2D images for TPU
    '''
    global tpu2image4xi                                                                                             # 2D-IMAGES FOR TPU  
    global control2mapimage4tpu                                                                                     # COMMAND AND CONTROL FOR TPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "map2image4tpu"
    method2print = "map2image4tpu"

    message2print = "{(Step 1A/33) started" + "; map2image4tpu-started: " + str(map2image4tpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # 
    
    message2print = "{(Step 1B/33) passed" + "; task4imagemap2tpu-to-map2image4tpu: " + str(task4imagemap2tpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE TPU2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if tpu2image4xi is None:
            tpu2image4xi = create2image4tpu()                                                                              # INITIALIZE IMAGE FOR TPU 
            message2print = "{(Step 1C/70) created" + "; tpu2image4xi-created-by-map2image4tpu to: " + str(tpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4tpu-skipped-by-map2image4tpu: " + str(map2image4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                                
    except:
        try:
            tpu2image4xi = create2image4tpu()
            message2print = "{(Step 1E/70) set" + "; tpu2image4xi-set-by-map2image4tpu to: " + str(tpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            tpu2image4xi = {}
            message2print = "{(Step 1Z/70) yielded" + "; tpu2image4xi-yielded-by-map2image4tpu: " + str(tpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z-' 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    try:             
        if task4imagemap2tpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR MAP - BUILD TASK! 
            # -----------------------------------------------------------------------------------------    
            
            task4imagemap2tpu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1A/70) initialized" + "; task4imagemap2tpu-created-by-map2image4tpu to: " + str(task4imagemap2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
            
            # ---------------------------------------------------------------------------------------------
            # (1B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4tpu = trace2image4tpu                                                              # CALLBACK FUNCTION FOR TPU
            
            message2print = "{(Step 1B1/70) initialized" + "; function2callback4tpu-created-by-map2image4tpu to: " + str(function2callback4tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '1B1-'
           
            args2callback = {}                                                                                   # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "map2image4tpu"                                                          # MODULE OF TPUIZED OPENCL MODULE
            args2callback['METHOD']  = "map2image4tpu"                                                          # FUNCTION OF TPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; map2image4tpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR TPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-map2image4tpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
            code2ack  = code2ack + '1B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (1C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagemap2tpu['DATA2CALLBACK'] = args2callback                                                   # USER DATA PASSED TO TPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagemap2tpu['FUNCTION2CALLBACK'] = function2callback4tpu                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '1C-'
            message2print = "{(Step 1C/70) initialized" + "; task4imagemap2tpu-created-by-map2image4tpu to: " + str(task4imagemap2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-map2image4tpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-map2image4tpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                code2ack  = code2ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-map2image4tpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '2Z-'
                 
            task4imagemap2tpu['PLATFORM'] = platform2xi                                                          # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2tpu = {}
            message2print = "{(Step 3A/70) trying" + "; context2tpu-trying-by-map2image4tpu: " + str(context2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2tpu = get2context4tpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context2tpu-got-by-map2image4tpu: " + str(context2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context2tpu-failed-by-map2image4tpu: " + str(context2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '3Z-'
                
            task4imagemap2tpu ['CONTEXT'] = context2tpu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2tpu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device2tpu-trying-by-map2image4tpu: " + str(device2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2tpu = get2tpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device2tpu-got-by-map2image4tpu: " + str(device2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                code2ack  = code2ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device2tpu-failed-by-map2image4tpu: " + str(device2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '4Z-'
            
            task4imagemap2tpu ['DEVICE'] = device2tpu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2tpu = {}
             
            message2print = "{(Step 5A/70) trying" + "; imageinfo2tpu-trying-by-map2image4tpu: " + str(imageinfo2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2tpu = get2imageinfo4tpu()                                                                         
                
                message2print = "{(Step 5B/70) got" + "; imageinfo2tpu-got-by-map2image4tpu: " + str(imageinfo2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                code2ack  = code2ack + '5B-'  
            except:
                message2print = "{(Step 5Z/70) failed" + "; imageinfo2tpu-failed-by-map2image4tpu: " + str(imageinfo2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '5Z-'          
                      
            task4imagemap2tpu ['IMAGE'] = imageinfo2tpu         
                        

            # ---------------------------------------------------------------------------------------------
            # (6) QUEUE - REQUIRED BY TPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2tpu = None
            
            message2print = "{(Step 6A/70) trying" + "; queue2tpu-trying-by-map2image4tpu: " + str(queue2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2tpu = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 6B/70) got" + "; queue2tpu-got-by-map2image4tpu: " + str(queue2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; queue2tpu-failed-by-map2image4tpu: " + str(queue2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '6Z-'
                
            task4imagemap2tpu ['QUEUE'] = queue2tpu

            
            # ---------------------------------------------------------------------------------------------
            # (7) PIPE - REQUIRED BY TPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2tpu = None
           
            message2print = "{(Step 7A/70) trying" + "; pipe2tpu-trying-by-map2image4tpu: " + str(pipe2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2tpu = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 7B/70) got" + "; pipe2tpu-got-by-map2image4tpu: " + str(pipe2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; pipe2tpu-failed-by-map2image4tpu: " + str(pipe2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '7Z-'
                
            task4imagemap2tpu ['PIPE'] = pipe2tpu 
            
            # ---------------------------------------------------------------------------------------------
            # (8) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2tpu = {}
           
            message2print = "{(Step 8A/70) trying" + "; commandqueueinfo2tpu-trying-by-map2image4tpu: " + str(commandqueueinfo2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2tpu = get2commandqueueinfo4tpu()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 8B/70) got" + "; commandqueueinfo2tpu-got-by-map2image4tpu: " + str(commandqueueinfo2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; commandqueueinfo2tpu-failed-by-map2image4tpu: " + str(commandqueueinfo2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '8Z-'
        
            task4imagemap2tpu ['COMMANDQUEUE'] = commandqueueinfo2tpu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (9) BUFFER - AUTOMATIC BY TPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2tpu = {}
           
            message2print = "{(Step 9A/70) trying" + "; bufferinfo2tpu-trying-by-map2image4tpu: " + str(bufferinfo2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2tpu = get2bufferinfo4tpu()                                                                   # SYSTEM CREATES BUFFER FOR TPU, IF NECESSARY
                message2print = "{(Step 9B/70) got" + "; bufferinfo2tpu-got-by-map2image4tpu: " + str(bufferinfo2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '9B-' 
            except:
                message2print = "{(Step 9Z/70) failed" + "; bufferinfo2tpu-failed-by-map2image4tpu: " + str(bufferinfo2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '9Z-'
        
            task4imagemap2tpu ['BUFFER'] = bufferinfo2tpu 
            
            # ---------------------------------------------------------------------------------------------
            # (10) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2tpu = {}
           
            message2print = "{(Step 10A/70) trying" + "; eventinfo2tpu-trying-by-map2image4tpu: " + str(eventinfo2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2tpu = get2eventinfo4tpu()                                                                    
                message2print = "{(Step 10B/70) got" + "; eventinfo2tpu-got-by-map2image4tpu: " + str(eventinfo2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; eventinfo2tpu-failed-by-map2image4tpu: " + str(eventinfo2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '10Z-'
        
            task4imagemap2tpu ['EVENT'] = eventinfo2tpu 
            
            
            # ---------------------------------------------------------------------------------------------
            # (11) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2tpu = {}
           
            message2print = "{(Step 11A/70) trying" + "; eventwaitlist2tpu-trying-by-map2image4tpu: " + str(eventwaitlist2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2tpu = {}                                                                   
                message2print = "{(Step 11B/70) set" + "; eventwaitlist2tpu-set-by-map2image4tpu: " + str(eventwaitlist2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; eventwaitlist2tpu-failed-by-map2image4tpu: " + str(eventwaitlist2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '11Z-'
        
            task4imagemap2tpu ['EVENTWAITLIST'] = eventwaitlist2tpu                     

            
            # ---------------------------------------------------------------------------------------------
            # (12) EVENTWAITLISTQUANTITY - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2tpu = 0
           
            message2print = "{(Step 12A/70) trying" + "; eventwaitlistquantity2tpu-trying-by-map2image4tpu: " + str(eventwaitlistquantity2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2tpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2tpu = eventwaitlistquantity2tpu + 1                                                                      
                message2print = "{(Step 12B/70) set" + "; eventwaitlistquantity2tpu-set-by-map2image4tpu: " + str(eventwaitlistquantity2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 
                
                code2ack  = code2ack + '12B-' 
            except:
                message2print = "{(Step 12Z/70) failed" + "; eventwaitlistquantity2tpu-failed-by-map2image4tpu: " + str(eventwaitlistquantity2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '12Z-'
        
            task4imagemap2tpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2tpu       
                                                 
            
            # ---------------------------------------------------------------------------------------------
            # (13) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2tpu = ' '                                                                   
     
            message2print = "{(Step 13A/70) trying" + "; blocking2tpu-trying-by-map2image4tpu: " + str(blocking2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2tpu = 'CL_FALSE'                                                          # the map command is non-blocking                                                              
                message2print = "{(Step 13B/70) set" + "; blocking2tpu-set-by-map2image4tpu to: " + str(blocking2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '13B-' 
            except:
                message2print = "{(Step 13Z/70) failed" + "; blocking2tpu-failed-by-map2image4tpu: " + str(blocking2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '13Z-'
        
            task4imagemap2tpu ['BLOCKING'] = blocking2tpu             
            
            # ---------------------------------------------------------------------------------------------
            # (14) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2tpu = {}
            
            message2print = "{(Step 14A/70) trying" + "; origin2tpu-trying-by-map2image4tpu: " + str(origin2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2tpu['X'] = 0 
                origin2tpu['Y'] = 0  
                origin2tpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 14B/70) set" + "; origin2tpu-set-by-map2image4tpu to: " + str(origin2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                 
                code2ack  = code2ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; origin2tpu-failed-by-map2image4tpu: " + str(origin2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '14Z-'
        
            task4imagemap2tpu ['ORIGIN'] = origin2tpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (15) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2tpu = {}
            
            message2print = "{(Step 15A/70) trying" + "; region2tpu-trying-by-map2image4tpu: " + str(region2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2tpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO MAP 
                region2tpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO MAP 
                region2tpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO MAP                                                                                                                          
                message2print = "{(Step 15B/70) set" + "; region2tpu-set-by-map2image4tpu to: " + str(region2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; region2tpu-failed-by-map2image4tpu: " + str(region2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '15Z-'
        
            task4imagemap2tpu ['REGION'] = region2tpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (16) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.


            rowpitch2tpu = 0
            size2element = 8                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 10                                                                                # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 8                                                                                  # ONE DIRECTIONS IN 2D
            width2tpu = 800
            
            message2print = "{(Step 16A/70) trying" + "; rowpitch2tpu-trying-by-map2image4tpu: " + str(rowpitch2tpu)  + "; time: " + time4xi() + "}"  
            print4xi(class2print, method2print, message2print) 
            # 
                     
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2tpu = int(width2tpu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 16B/70) set" + "; rowpitch2tpu-set-by-map2image4tpu to: " + str(rowpitch2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #  8000                                                                                          
                 
                code2ack  = code2ack + '16B-' 
            except:
                message2print = "{(Step 16Z/70) failed" + "; rowpitch2tpu-failed-by-map2image4tpu: " + str(rowpitch2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '16Z-'
        
            task4imagemap2tpu ['ROWPITCH'] = rowpitch2tpu     
            
            # --------------------------------------------------------------------------------------------------------
            # (18) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING MAP                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2tpu = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 18A/70) trying" + "; slicepitch2tpu-trying-by-map2image4tpu: " + str(slicepitch2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2tpu = 0                                                          
                message2print = "{(Step 18B/70) set" + "; slicepitch2tpu-set-by-map2image4tpu to: " + str(slicepitch2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 

                code2ack  = code2ack + '18B-'                
                 
            except:
                message2print = "{(Step 18Z/70) failed" + "; slicepitch2tpu-failed-by-map2image4tpu: " + str(slicepitch2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '18Z-'
            
            task4imagemap2tpu ['SLICEPITCH'] = slicepitch2tpu                  
            
            # ---------------------------------------------------------------------------------------------
            # (19) FLAGS - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            flags2tpu = 'CL_MAP_READ'                                                                
     
            message2print = "{(Step 19A/70) trying" + "; flags2tpu-trying-by-map2image4tpu: " + str(flags2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                flags2tpu = 'CL_MAP_WRITE'  # THE REGION SPECIFIED BY (ORIGIN, REGION) IN THE IMAGE OBJECT IS BEING MAPPED FOR WRITING                                                                                                                     
                message2print = "{(Step 19B/70) set" + "; flags2tpu-set-by-map2image4tpu to: " + str(flags2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '19B-' 
            except:
                message2print = "{(Step 19Z/70) failed" + "; flags2tpu-failed-by-map2image4tpu: " + str(flags2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '19Z-'
        
            task4imagemap2tpu ['FLAGS'] = flags2tpu 
                                  
            # ---------------------------------------------------------------------------------------------
            # EOF PARAMETER PROVIDING                
            # ---------------------------------------------------------------------------------------------                                                                                                                               
        else:
            pass
            
        # ---------------------------------------------------------------------------------------------
        # ANALYSE TASKS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagemap2tpu.keys()
        message2print = "{(Step 20/70) got" + "; keys2task-got-by-map2image4tpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FLAGS',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SLICEPITCH']; time: 01:28:44 10/05/11 Pacific Daylight Time}
 

        if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4tpu = trace2image4tpu                                                              # CALLBACK FUNCTION FOR TPU
            
            message2print = "{(Step 21B1/70) initialized" + "; function2callback4tpu-created-by-map2image4tpu to: " + str(function2callback4tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '21B1-'
           
            args2callback = {}                                                                                   # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "map2image4tpu"                                                          # MODULE OF TPUIZED OPENCL MODULE
            args2callback['METHOD']  = "map2image4tpu"                                                          # FUNCTION OF TPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; map2image4tpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR TPUIZED OPENCL TO RETURN    

            message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-map2image4tpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 

            code2ack  = code2ack + '21B2-'
        
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagemap2tpu['DATA2CALLBACK'] = args2callback                                                   # USER DATA PASSED TO TPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagemap2tpu['FUNCTION2CALLBACK'] = function2callback4tpu                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4imagemap2tpu-created-by-map2image4tpu to: " + str(task4imagemap2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
             
        if 'PLATFORM' in keys2task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-map2image4tpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-map2image4tpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
                code2ack  = code2ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-map2image4tpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '22Z-'
                 
            task4imagemap2tpu['PLATFORM'] = platform2xi                                                          # PLATFORM OF SYSTEM 
          
        if 'CONTEXT' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2tpu = {}
            message2print = "{(Step 23A/70) trying" + "; context2tpu-trying-by-map2image4tpu: " + str(context2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2tpu = get2context4tpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context2tpu-got-by-map2image4tpu: " + str(context2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context2tpu-failed-by-map2image4tpu: " + str(context2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '23Z-'
                
            task4imagemap2tpu ['CONTEXT'] = context2tpu           
        
        if 'DEVICE' in keys2task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2tpu = {}
            
            message2print = "{(Step 24A/70) trying" + "; device2tpu-trying-by-map2image4tpu: " + str(device2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2tpu = get2tpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device2tpu-got-by-map2image4tpu: " + str(device2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device2tpu-failed-by-map2image4tpu: " + str(device2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '24Z-'
            
            task4imagemap2tpu ['DEVICE'] = device2tpu
        
        if 'IMAGE' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2tpu = {}
             
            message2print = "{(Step 25A/70) trying" + "; imageinfo2tpu-trying-by-map2image4tpu: " + str(imageinfo2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2tpu = get2imageinfo4tpu()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; imageinfo2tpu-got-by-map2image4tpu: " + str(imageinfo2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; imageinfo2tpu-failed-by-map2image4tpu: " + str(imageinfo2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '25Z-'          
                      
            task4imagemap2tpu ['IMAGE'] = imageinfo2tpu         
                    
        if 'QUEUE' in keys2task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (26) QUEUE - REQUIRED BY TPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2tpu = None
            
            message2print = "{(Step 26A/70) trying" + "; queue2tpu-trying-by-map2image4tpu: " + str(queue2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2tpu = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 26B/70) got" + "; queue2tpu-got-by-map2image4tpu: " + str(queue2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; queue2tpu-failed-by-map2image4tpu: " + str(queue2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '26Z-'
                
            task4imagemap2tpu ['QUEUE'] = queue2tpu
        
        if 'PIPE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (27) PIPE - REQUIRED BY TPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2tpu = None
           
            message2print = "{(Step 27A/70) trying" + "; pipe2tpu-trying-by-map2image4tpu: " + str(pipe2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2tpu = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 27B/70) got" + "; pipe2tpu-got-by-map2image4tpu: " + str(pipe2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; pipe2tpu-failed-by-map2image4tpu: " + str(pipe2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '27Z-'
                
            task4imagemap2tpu ['PIPE'] = pipe2tpu 
            
        if 'COMMANDQUEUE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2tpu = {}
           
            message2print = "{(Step 28A/70) trying" + "; commandqueueinfo2tpu-trying-by-map2image4tpu: " + str(commandqueueinfo2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2tpu = get2commandqueueinfo4tpu()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 28B/70) got" + "; commandqueueinfo2tpu-got-by-map2image4tpu: " + str(commandqueueinfo2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; commandqueueinfo2tpu-failed-by-map2image4tpu: " + str(commandqueueinfo2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '28Z-'
        
            task4imagemap2tpu ['COMMANDQUEUE'] = commandqueueinfo2tpu
        
        
        if 'BUFFER' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (29) BUFFER - AUTOMATIC BY TPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2tpu = {}
           
            message2print = "{(Step 29A/70) trying" + "; bufferinfo2tpu-trying-by-map2image4tpu: " + str(bufferinfo2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2tpu = get2bufferinfo4tpu()                                                                   # SYSTEM CREATES BUFFER FOR TPU, IF NECESSARY
                message2print = "{(Step 29B/70) got" + "; bufferinfo2tpu-got-by-map2image4tpu: " + str(bufferinfo2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '29B-' 
            except:
                message2print = "{(Step 29Z/70) failed" + "; bufferinfo2tpu-failed-by-map2image4tpu: " + str(bufferinfo2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '29Z-'
        
            task4imagemap2tpu ['BUFFER'] = bufferinfo2tpu
             
        if 'EVENT' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (30) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2tpu = {}
           
            message2print = "{(Step 30A/70) trying" + "; eventinfo2tpu-trying-by-map2image4tpu: " + str(eventinfo2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2tpu = get2eventinfo4tpu()                                                                    
                message2print = "{(Step 30B/70) got" + "; eventinfo2tpu-got-by-map2image4tpu: " + str(eventinfo2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '30B-' 
            except:
                message2print = "{(Step 30Z/70) failed" + "; eventinfo2tpu-failed-by-map2image4tpu: " + str(eventinfo2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '30Z-'
        
            task4imagemap2tpu ['EVENT'] = eventinfo2tpu 
        
             
        if 'EVENTWAITLIST' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2tpu = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventwaitlist2tpu-trying-by-map2image4tpu: " + str(eventwaitlist2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2tpu = {}                                                                   
                message2print = "{(Step 31B/70) set" + "; eventwaitlist2tpu-set-by-map2image4tpu: " + str(eventwaitlist2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventwaitlist2tpu-failed-by-map2image4tpu: " + str(eventwaitlist2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '31Z-'
        
            task4imagemap2tpu ['EVENTWAITLIST'] = eventwaitlist2tpu                     
              
        if 'EVENTWAITLISTQUANTITY' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLISTQUANTITY - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2tpu = 0
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlistquantity2tpu-trying-by-map2image4tpu: " + str(eventwaitlistquantity2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2tpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2tpu = eventwaitlistquantity2tpu + 1                                                                   
                message2print = "{(Step 32B/70) set" + "; eventwaitlistquantity2tpu-set-by-map2image4tpu: " + str(eventwaitlistquantity2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlistquantity2tpu-failed-by-map2image4tpu: " + str(eventwaitlistquantity2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '32Z-'
        
            task4imagemap2tpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2tpu       
                                                 
                 
        if 'BLOCKING' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (33) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2tpu = ' '                                                                   
            message2print = "{(Step 33A/70) trying" + "; blocking2tpu-trying-by-map2image4tpu: " + str(blocking2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2tpu = 'CL_FALSE'                                                          # the map command is non-blocking                                                              
                message2print = "{(Step 33B/70) set" + "; blocking2tpu-set-by-map2image4tpu to: " + str(blocking2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; blocking2tpu-failed-by-map2image4tpu: " + str(blocking2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '33Z-'
        
            task4imagemap2tpu ['BLOCKING'] = blocking2tpu             
             
        if 'ORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2tpu = {}
            
            message2print = "{(Step 34A/70) trying" + "; origin2tpu-trying-by-map2image4tpu: " + str(origin2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2tpu['X'] = 0 
                origin2tpu['Y'] = 0  
                origin2tpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; origin2tpu-set-by-map2image4tpu to: " + str(origin2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; origin2tpu-failed-by-map2image4tpu: " + str(origin2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '34Z-'
        
            task4imagemap2tpu ['ORIGIN'] = origin2tpu
            
             
        if 'REGION' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (35) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2tpu = {}
            
            message2print = "{(Step 35A/70) trying" + "; region2tpu-trying-by-map2image4tpu: " + str(region2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                            
            try:                                                                                    
                region2tpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO MAP 
                region2tpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO MAP 
                region2tpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO MAP                                                                                                                          
                message2print = "{(Step 35B/70) set" + "; region2tpu-set-by-map2image4tpu to: " + str(region2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '35B-' 
            except:
                message2print = "{(Step 35Z/70) failed" + "; region2tpu-failed-by-map2image4tpu: " + str(region2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '35Z-'
        
            task4imagemap2tpu ['REGION'] = region2tpu
        
            
             
        if 'ROWPITCH' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (36) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.
     
            rowpitch2tpu = 0
            size2element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 1                                                                                  # ONE DIRECTIONS IN 2D
            
            message2print = "{(Step 36A/70) trying" + "; rowpitch2tpu-trying-by-map2image4tpu: " + str(rowpitch2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2tpu = int(width2tpu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 36B/70) set" + "; rowpitch2tpu-set-by-map2image4tpu to: " + str(rowpitch2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #                                                                                             
                 
                code2ack  = code2ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; rowpitch2tpu-failed-by-map2image4tpu: " + str(rowpitch2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '36Z-'
        
            task4imagemap2tpu ['ROWPITCH'] = rowpitch2tpu     
   
        if 'SLICEPITCH' in keys2task:
            pass
        else:               
            # --------------------------------------------------------------------------------------------------------
            # (38) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING MAP                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2tpu = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 38A/70) trying" + "; slicepitch2tpu-trying-by-map2image4tpu: " + str(slicepitch2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2tpu = 0                                                          
                message2print = "{(Step 38B/70) set" + "; slicepitch2tpu-set-by-map2image4tpu to: " + str(slicepitch2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '38B-'                
                 
            except:
                message2print = "{(Step 38Z/70) failed" + "; slicepitch2tpu-failed-by-map2image4tpu: " + str(slicepitch2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '38Z-'
            
            task4imagemap2tpu ['SLICEPITCH'] = slicepitch2tpu
   
        if 'FLAGS' in keys2task:
            pass
        else:               
            # --------------------------------------------------------------------------------------------------------
            # (39) FLAGS - REQUIRED                   
            # --------------------------------------------------------------------------------------------------------             
            flags2tpu = 'CL_MAP_READ'                                                                                                             
           
            message2print = "{(Step 39A/70) trying" + "; flags2tpu-trying-by-map2image4tpu: " + str(flags2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                flags2tpu = 'CL_MAP_WRITE'                                                          
                message2print = "{(Step 39B/70) set" + "; flags2tpu-set-by-map2image4tpu to: " + str(flags2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '39B-'                
                 
            except:
                message2print = "{(Step 39Z/70) failed" + "; flags2tpu-failed-by-map2image4tpu: " + str(flags2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '39Z-'
            
            task4imagemap2tpu ['FLAGS'] = flags2tpu
                                          
        # ---------------------------------------------------------------------------------------------
        # EOF TASK ANALYSIS
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagemap2tpu.keys()
        message2print = "{(Step 40A/70) set" + "; keys2task-set-by-map2image4tpu to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FLAGS',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SLICEPITCH']; time: 01:28:44 10/05/11 Pacific Daylight Time}

                       
    except: 
        message2print = "{(Step 40Z/70) yielded" + "; task4imagemap2tpu-yielded-by-map2image4tpu: " + str(task4imagemap2tpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '40Z-' 

    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2mapimage4tpu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2tpu = False
    channels2tpu = ()
    message2print = "{(Step 40A/70) trying" + "channels2tpu-trying-by-map2image4tpu: " + str(channels2tpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel2tpu, publisherchannel2tpu = task4imagemap2tpu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2tpu-got-by-map2image4tpu: " + str(consumerchannel2tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 748>

        message2print = "{(Step 40C/70) got" + "; publisherchannel2tpu-got-by-map2image4tpu: " + str(publisherchannel2tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 752>                     
        ack4channels2tpu = True
        
        control2mapimage4tpu['CONSUMERCHANNEL']  = consumerchannel2tpu
        
        
        control2mapimage4tpu['PUBLISHERCHANNEL'] = publisherchannel2tpu
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2tpu = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2tpu = False
    queue2tpu = None
    message2print = "{(Step 41A/70) trying" + "queue2tpu-trying-by-map2image4tpu: " + str(queue2tpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2tpu = task4imagemap2tpu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2tpu-got-by-map2image4tpu: " + str(queue2tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <queue4xi.Queue4Xi object at 0x0000000004B3FB70> 
              
        ack4queue2tpu = True
        control2mapimage4tpu['QUEUE']  = queue2tpu        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2tpu = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4tpu = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4tpu-trying-by-map2image4tpu: " + str(dummy2image4tpu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4tpu = DummyProcess4Xi(target=manoeuvre2mapimage4tpu,                                            # in this module to handle queue and pipe channels
        #                                   args =(queue2tpu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2tpu, publisherchannel2tpu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4tpu-created-by-map2image4tpu: " + str(dummy2image4tpu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'
        
    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-map2image4tpu: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-map2image4tpu: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-12B-13B-14B-15B-16B-18B-19B-40D-41C- 
        
    message2print = "{(Step 70C/70) returning" + "; control2mapimage4tpu-returning-by-map2image4tpu: " + str(control2mapimage4tpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004B3FB70>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 752>,
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 748>}; time: 01:28:44 10/05/11 Pacific Daylight Time}

        
    return control2mapimage4tpu
                    
#</function4xi> 

# -----------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2copyimage4tpu" usage = "
#   # ---------------------------------------------------------------------------------------------------------------------
#   # CALL MANOEUVRE2COPYIMAGE4TPU - LET IT TO RECEIVE COPY MESSAGE AND TO SEND ROGER MESSAGE VIA PIPE                
#   # ---------------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4tpu is True:
#       print("(Step 42A/700) OF TPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4TPU ", time4xi())   
#       try:        
#           response2manoeuvre4copy = manoeuvre2copyimage4tpu(queue2copy4tpu, channel2copyconsume4tpu, channel2copypublish4tpu)
#           print("(Step 42B/700) OF TPU2IMAGE4XI5TEST - PERFORMED MANOEUVRE2COPYIMAGE4TPU, RESPONSE2MANOEUVRE4COPY: ", response2manoeuvre4copy)
#           # {'MESSAGE2SEND4TPU': '<roger4xi>roger2copy4tpu</roger4xi>', 'ACK2INFO4TPU': False, 'MESSAGE2GET4TPU': '<default4xi>default2copy4tpu</default4xi>', 
#           #  'MESSAGE2RECEIVE4TPU': '<copy4xi>copy2copy4tpu</copy4xi>', 
#           'CODE2ACK4TPU': ' 0D-1C-', 'MESSAGE2PUT4TPU': '<default4xi>default2copy4tpu</default4xi>', 'CODE2NACK4TPU': ' ', 'STATE2MANOEUVRE4TPU': 1}
#        except:
#           print("(Step 42Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4TPU ", time4xi())     
#   else:
#       pass # trust activated thread
#">            

# ----------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2copyimage4tpu(queue2copymanoeuvre4tpu, consumerchannel2copymanoeuvre4tpu, publisherchannel2copymanoeuvre4tpu):
    '''
    manoeuvres copy control and command
    '''
    global state2copyimage4tpu                                                                                           # STATE FOR COMMAND AND CONTROL
    global max2copyimagestate4tpu                                                                                        # MAX FOR STATE
    global ack2copyimageinfo4tpu                                                                                         # ACK FOR IMAGE INFO
    global message2copyimageget4tpu                                 
    global message2copyimageput4tpu
    global message2copyimagereceive4tpu
    global message2copyimagesend4tpu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2copyimage4tpu" 
    method2print = "manoeuvre2copyimage4tpu" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2copyimage4tpu-started: " + str(manoeuvre2copyimage4tpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function manoeuvre2copyimage4tpu at 0x0000000003DB8C48>  
    
    message2print = "{(Step 0B/33) passed" + "; queue2copymanoeuvre4tpu-to-manoeuvre2copyimage4tpu: " + str(queue2copymanoeuvre4tpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <queue4xi.Queue4Xi object at 0x0000000003AD9400>
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2copymanoeuvre4tpu-to-manoeuvre2copyimage4tpu: " + str(consumerchannel2copymanoeuvre4tpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <copy-write PipeConnection, handle 408>
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2copymanoeuvre4tpu-to-manoeuvre2copyimage4tpu: " + str(publisherchannel2copymanoeuvre4tpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <copy-write PipeConnection, handle 412>     
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4copy = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2copyimage4tpu-try-by-manoeuvre2copyimage4tpu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2copyimage4tpu is None:
            state2copyimage4tpu = 0
            message2print = "{(Step 0F/33) initialized" + "; state2copyimage4tpu-initialized-by-manoeuvre2copyimage4tpu to: " + str(state2copyimage4tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2copyimage4tpu-used-by-manoeuvre2copyimage4tpu: " + str(state2copyimage4tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2copyimage4tpu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2copyimage4tpu-initialized-by-manoeuvre2copyimage4tpu to: " + str(state2copyimage4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2copyimageinfo4tpu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2copyimageinfo4tpu-initialized-by-manoeuvre2copyimage4tpu to: " + str(ack2copyimageinfo4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2copyimageget4tpu     = '<default4xi>default2copy4tpu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2copyimageget4tpu-initialized-by-manoeuvre2copyimage4tpu to: " + str(message2copyimageget4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2copy4tpu</default4xi>
        message2copyimageput4tpu     = '<default4xi>default2copy4tpu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2copyimageput4tpu-initialized-by-manoeuvre2copyimage4tpu to: " + str(message2copyimageput4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4tpu</default4xi>
        message2copyimagereceive4tpu = '<default4xi>default2copy4tpu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2copyimagereceive4tpu-initialized-by-manoeuvre2copyimage4tpu to: " + str(message2copyimagereceive4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4tpu</default4xi>
        message2copyimagesend4tpu    = '<default4xi>default2copy4tpu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2copyimagesend4tpu-initialized-by-manoeuvre2copyimage4tpu to: " + str(message2copyimagesend4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4tpu</default4xi>
        max2copyimagestate4tpu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2copyimagestate4tpu-initialized-by-manoeuvre2copyimage4tpu to: " + str( max2copyimagestate4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2copyimage4tpu = state2copyimage4tpu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2copyimage4tpu-added-by-manoeuvre2copyimage4tpu: " + str(state2copyimage4tpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4  
    if state2copyimage4tpu > max2copyimagestate4tpu:
        state2copyimage4tpu = 0
        message2print = "{(Step 0I/33) set" + "; state2copyimage4tpu-set-by-manoeuvre2copyimage4tpu to: " + str(state2copyimage4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2copyimageinfo4tpu        = False
        message2print = "{(Step 0J/33) set" + "; ack2copyimageinfo4tpu-set-by-manoeuvre2copyimage4tpu to: " + str(ack2copyimageinfo4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2copyimageget4tpu     = '<default4xi>default2copy4tpu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2copyimageget4tpu-set-by-manoeuvre2copyimage4tpu to: " + str(message2copyimageget4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2copy4tpu</default4xi>
        message2copyimageput4tpu     = '<default4xi>default2copy4tpu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2copyimageput4tpu-set-by-manoeuvre2copyimage4tpu to: " + str(message2copyimageput4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4tpu</default4xi>
        message2copyimagereceive4tpu = '<default4xi>default2copy4tpu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2copyimagereceive4tpu-set-by-manoeuvre2copyimage4tpu to: " + str(message2copyimagereceive4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4tpu</default4xi>
        message2copyimagesend4tpu    = '<default4xi>default2copy4tpu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2copyimagesend4tpu-set-by-manoeuvre2copyimage4tpu to: " + str(message2copyimagesend4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4tpu</default4xi>
        max2copyimagestate4tpu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2copyimagestate4tpu-set-by-manoeuvre2copyimage4tpu to: " + str( max2copyimagestate4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimage4tpu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2copyimage4tpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copyimagereceive4tpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copyimagereceive4tpu = consumerchannel2copymanoeuvre4tpu.recv()
            message2print = "{(Step 1B/33) received" + "; message2copyimagereceive4tpu-received-by-manoeuvre2copyimage4tpu: " + str(message2copyimagereceive4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <copy4xi>copy2copy4tpu</copy4xi>
                         
            if message2copyimagereceive4tpu == '<copy4xi>copy2copy4tpu</copy4xi>':
                message2copyimagesend4tpu = '<roger4xi>roger2copy4tpu</roger4xi>'
            else:
                message2copyimagesend4tpu = '<negative4xi><negative2copy4tpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copymanoeuvre4tpu.send(message2copyimagesend4tpu)
            
            message2print = "{(Step 1C/33) sent" + "; message2copyimagesend4tpu-sent-by-manoeuvre2copyimage4tpu: " + str(message2copyimagesend4tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <roger4xi>roger2copy4tpu</roger4xi>
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2copyimagereceive4tpu-failed-by-manoeuvre2copyimage4tpu: " + str(message2copyimagereceive4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimage4tpu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2copyimage4tpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copyimagereceive4tpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copyimagereceive4tpu = consumerchannel2copymanoeuvre4tpu.recv()
            message2print = "{(Step 2B/33) received" + "; message2copyimagereceive4tpu-received-by-manoeuvre2copyimage4tpu: " + str(message2copyimagereceive4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <ready4xi>ready2copy4tpu</ready4xi>
                         
            if message2copyimagereceive4tpu == '<ready4xi>ready2copy4tpu</ready4xi>':
                message2copyimagesend4tpu = '<ready4xi>ready2copy4tpu</ready4xi>'
            else:
                message2copyimagesend4tpu = '<negative4xi><negative2copy4tpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND READY OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copymanoeuvre4tpu.send(message2copyimagesend4tpu)
            
            message2print = "{(Step 2C/33) sent" + "; message2copyimagesend4tpu-sent-by-manoeuvre2copyimage4tpu: " + str(message2copyimagesend4tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2copyimagereceive4tpu-failed-by-manoeuvre2copyimage4tpu: " + str(message2copyimagereceive4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimage4tpu == 3:    
        info2image4tpu = None
        message2print = "{(Step 3A/70) try" + "; info2image4tpu-try-by-manoeuvre2copyimage4tpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4tpu = queue2copymanoeuvre4tpu.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4tpu-got-by-manoeuvre2copyimage4tpu: " + str(info2image4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # ..//..//..//..//..//web//images//png//dragonxi.png
            if info2image4tpu is None:
                code2nack  = code2nack + '3Z1-'
                ack2copyimageinfo4tpu = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2copyimageinfo4tpu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2copymanoeuvre4tpu-failed-by-manoeuvre2copyimage4tpu: " + str(queue2copymanoeuvre4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2copyimagesend4tpu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2copyimageinfo4tpu is True:
                message2copyimagesend4tpu = '<ack4xi>ack2imageinfo4tpu</ack4xi>'
            else:
                message2copyimagesend4tpu = '<nack4xi>nack2imageinfo4tpu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2copyimagesend4tpu-set-by-manoeuvre2copyimage4tpu to: " + str(message2copyimagesend4tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <ack4xi>ack2imageinfo4tpu</ack4xi>       
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copyimagesend4tpu-failed-by-manoeuvre2copyimage4tpu: " + str(message2copyimagesend4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2copymanoeuvre4tpu-calling-by-manoeuvre2copyimage4tpu: " + str(publisherchannel2copymanoeuvre4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <copy-write PipeConnection, handle 412>
            message2print = "{{(Step 4D/70) passing" + "; message2copyimagesend4tpu-passing-to-send-by-manoeuvre2copyimage4tpu: " + str(message2copyimagesend4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copyimagesend4tpu-failed-by-manoeuvre2copyimage4tpu: " + str(message2copyimagesend4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2copymanoeuvre4tpu.send(message2copyimagesend4tpu)
            
            message2print = "{(Step 4E/33) sent" + "; message2copyimagesend4tpu-sent-by-manoeuvre2copyimage4tpu: " + str(message2copyimagesend4tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ack4xi>ack2imageinfo4tpu</ack4xi>            
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2copyimagesend4tpu-failed-by-manoeuvre2copyimage4tpu: " + str(message2copyimagesend4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4copy['ACK2INFO4TPU']        = ack2copyimageinfo4tpu
    response2manouvre4copy['CODE2ACK4TPU']        = code2ack
    response2manouvre4copy['CODE2NACK4TPU']       = code2nack
    response2manouvre4copy['MESSAGE2GET4TPU']     = message2copyimageget4tpu
    response2manouvre4copy['MESSAGE2PUT4TPU']     = message2copyimageput4tpu
    response2manouvre4copy['MESSAGE2RECEIVE4TPU'] = message2copyimagereceive4tpu
    response2manouvre4copy['MESSAGE2SEND4TPU']    = message2copyimagesend4tpu
    response2manouvre4copy['STATE2MANOEUVRE4TPU'] = state2copyimage4tpu
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4copy-returning-by-manoeuvre2copyimage4tpu: " + str(response2manouvre4copy) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 {'MESSAGE2SEND4TPU': '<roger4xi>roger2copy4tpu</roger4xi>',
    #   'ACK2INFO4TPU': False,
    #   'MESSAGE2GET4TPU': '<default4xi>default2copy4tpu</default4xi>',
    #   'MESSAGE2RECEIVE4TPU': '<copy4xi>copy2copy4tpu</copy4xi>',
    #   'CODE2ACK4TPU': ' 0D-1C-',
    #   'MESSAGE2PUT4TPU': '<default4xi>default2copy4tpu</default4xi>',
    #   'CODE2NACK4TPU': ' ',
    #   'STATE2MANOEUVRE4TPU': 1}; time: 05:25:15 10/03/11 Pacific Daylight Time}

 
    #2 {'MESSAGE2SEND4TPU': '<ready4xi>ready2copy4tpu</ready4xi>',
    #   'ACK2INFO4TPU': False,
    #   'MESSAGE2GET4TPU': '<default4xi>default2copy4tpu</default4xi>',
    #   'MESSAGE2RECEIVE4TPU': '<ready4xi>ready2copy4tpu</ready4xi>',
    #   'CODE2ACK4TPU': ' 0D-2C-',
    #   'MESSAGE2PUT4TPU': '<default4xi>default2copy4tpu</default4xi>',
    #   'CODE2NACK4TPU': ' ',
    #   'STATE2MANOEUVRE4TPU': 2}; time: 05:25:15 10/03/11 Pacific Daylight Time}

   
    #3 {'MESSAGE2SEND4TPU': '<ack4xi>ack2imageinfo4tpu</ack4xi>',
    #   'ACK2INFO4TPU': True,
    #   'MESSAGE2GET4TPU': '<default4xi>default2copy4tpu</default4xi>',
    #   'MESSAGE2RECEIVE4TPU': '<ready4xi>ready2copy4tpu</ready4xi>',
    #   'CODE2ACK4TPU': ' 0D-3C-4E-',
    #   'MESSAGE2PUT4TPU': '<default4xi>default2copy4tpu</default4xi>',
    #   'CODE2NACK4TPU': ' ', 'STATE2MANOEUVRE4TPU': 3}; time: 05:25:15 10/03/11 Pacific Daylight Time}

    
    #4 {'MESSAGE2SEND4TPU': '<default4xi>default2copy4tpu</default4xi>',
    #   'ACK2INFO4TPU': False,
    #   'MESSAGE2GET4TPU': '<default4xi>default2copy4tpu</default4xi>',
    #   'MESSAGE2RECEIVE4TPU': '<default4xi>default2copy4tpu</default4xi>',
    #   'CODE2ACK4TPU': ' 0D-',
    #   'MESSAGE2PUT4TPU': '<default4xi>default2copy4tpu</default4xi>',
    #   'CODE2NACK4TPU': ' ', 'STATE2MANOEUVRE4TPU': 0}; time: 05:25:15 10/03/11 Pacific Daylight Time}

    return response2manouvre4copy           
          
#</function4xi>


# -----------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2copyimagetobuffer4tpu" usage = "
#   # ---------------------------------------------------------------------------------------------------------------------
#   # CALL MANOEUVRE2COPYIMAGE4TPU  - LET IT TO RECEIVE COPY AND TO SEND ROGER VIA PIPE                
#   # ---------------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4tpu is True:
#       print("(Step 62A/700) OF TPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4TPU ", time4xi())   
#       try:        
#           response2manoeuvre4copy = manoeuvre2copyimagetobuffer4tpu(queue4copy2imagetobuffer4tpu, channel4copy2imagetobufferconsume4tpu, channel4copy2imagetobufferpublish4tpu)
#           print("(Step 62B/700) OF TPU2IMAGE4XI5TEST - PERFORMED MANOEUVRE2COPYIMAGE4TPU, RESPONSE2MANOEUVRE4COPY: ", response2manoeuvre4copy)
#           # { 'MESSAGE2SEND4TPU': '<roger4xi>roger2copyimagetobuffer4tpu</roger4xi>',
#           #   'ACK2INFO4TPU': False,
#           #   'MESSAGE2GET4TPU': '<default4xi>default2copyimagetobuffer4tpu</default4xi>',
#           #   'MESSAGE2RECEIVE4TPU': '<copy4xi>copy2copyimagetobuffer4tpu</copy4xi>',
#           #   'CODE2ACK4TPU': ' 0D-1C-',
#           #   'MESSAGE2PUT4TPU': '<default4xi>default2copyimagetobuffer4tpu</default4xi>',
#           #   'CODE2NACK4TPU': ' ',
#           #   'STATE2MANOEUVRE4TPU': 1}
#        except:
#           print("(Step 62Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4TPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#">

# ----------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2copyimagetobuffer4tpu(queue2copyimagetobuffermanoeuvre4tpu, consumerchannel2copyimagetobuffermanoeuvre4tpu, publisherchannel2copyimagetobuffermanoeuvre4tpu):
    '''
    manoeuvres copy control and command for copy2imagetobuffer4tpu
    '''
    global state2copyimagetobuffer4tpu                                                                                           # STATE FOR COMMAND AND CONTROL
    global max2copyimagetobufferstate4tpu                                                                                        # MAX FOR STATE
    global ack2copyimagetobufferinfo4tpu                                                                                         # ACK FOR IMAGE INFO
    global message2copyimagetobufferget4tpu                                 
    global message2copyimagetobufferput4tpu
    global message2copyimagetobufferreceive4tpu
    global message2copyimagetobuffersend4tpu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2copyimagetobuffer4tpu" 
    method2print = "manoeuvre2copyimagetobuffer4tpu" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2copyimagetobuffer4tpu-started: " + str(manoeuvre2copyimagetobuffer4tpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # 
    
    message2print = "{(Step 0B/33) passed" + "; queue2copyimagetobuffermanoeuvre4tpu-to-manoeuvre2copyimagetobuffer4tpu: " + str(queue2copyimagetobuffermanoeuvre4tpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2copyimagetobuffermanoeuvre4tpu-to-manoeuvre2copyimagetobuffer4tpu: " + str(consumerchannel2copyimagetobuffermanoeuvre4tpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2copyimagetobuffermanoeuvre4tpu-to-manoeuvre2copyimagetobuffer4tpu: " + str(publisherchannel2copyimagetobuffermanoeuvre4tpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4copyimagetobuffer = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2copyimagetobuffer4tpu-try-by-manoeuvre2copyimagetobuffer4tpu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2copyimagetobuffer4tpu is None:
            state2copyimagetobuffer4tpu = 0
            message2print = "{(Step 0F/33) initialized" + "; state2copyimagetobuffer4tpu-initialized-by-manoeuvre2copyimagetobuffer4tpu to: " + str(state2copyimagetobuffer4tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2copyimagetobuffer4tpu-used-by-manoeuvre2copyimagetobuffer4tpu: " + str(state2copyimagetobuffer4tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2copyimagetobuffer4tpu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2copyimagetobuffer4tpu-initialized-by-manoeuvre2copyimagetobuffer4tpu to: " + str(state2copyimagetobuffer4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2copyimagetobufferinfo4tpu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2copyimagetobufferinfo4tpu-initialized-by-manoeuvre2copyimagetobuffer4tpu to: " + str(ack2copyimagetobufferinfo4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2copyimagetobufferget4tpu     = '<default4xi>default2copyimagetobuffer4tpu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2copyimagetobufferget4tpu-initialized-by-manoeuvre2copyimagetobuffer4tpu to: " + str(message2copyimagetobufferget4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message2copyimagetobufferput4tpu     = '<default4xi>default2copyimagetobuffer4tpu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2copyimagetobufferput4tpu-initialized-by-manoeuvre2copyimagetobuffer4tpu to: " + str(message2copyimagetobufferput4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copyimagetobufferreceive4tpu = '<default4xi>default2copyimagetobuffer4tpu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2copyimagetobufferreceive4tpu-initialized-by-manoeuvre2copyimagetobuffer4tpu to: " + str(message2copyimagetobufferreceive4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copyimagetobuffersend4tpu    = '<default4xi>default2copyimagetobuffer4tpu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2copyimagetobuffersend4tpu-initialized-by-manoeuvre2copyimagetobuffer4tpu to: " + str(message2copyimagetobuffersend4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max2copyimagetobufferstate4tpu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2copyimagetobufferstate4tpu-initialized-by-manoeuvre2copyimagetobuffer4tpu to: " + str( max2copyimagetobufferstate4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2copyimagetobuffer4tpu = state2copyimagetobuffer4tpu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2copyimagetobuffer4tpu-added-by-manoeuvre2copyimagetobuffer4tpu: " + str(state2copyimagetobuffer4tpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4 
     
    if state2copyimagetobuffer4tpu > max2copyimagetobufferstate4tpu:
        state2copyimagetobuffer4tpu = 0
        message2print = "{(Step 0I/33) set" + "; state2copyimagetobuffer4tpu-set-by-manoeuvre2copyimagetobuffer4tpu to: " + str(state2copyimagetobuffer4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        ack2copyimagetobufferinfo4tpu        = False
        message2print = "{(Step 0J/33) set" + "; ack2copyimagetobufferinfo4tpu-set-by-manoeuvre2copyimagetobuffer4tpu to: " + str(ack2copyimagetobufferinfo4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # 
        message2copyimagetobufferget4tpu     = '<default4xi>default2copyimagetobuffer4tpu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2copyimagetobufferget4tpu-set-by-manoeuvre2copyimagetobuffer4tpu to: " + str(message2copyimagetobufferget4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message2copyimagetobufferput4tpu     = '<default4xi>default2copyimagetobuffer4tpu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2copyimagetobufferput4tpu-set-by-manoeuvre2copyimagetobuffer4tpu to: " + str(message2copyimagetobufferput4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copyimagetobufferreceive4tpu = '<default4xi>default2copyimagetobuffer4tpu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2copyimagetobufferreceive4tpu-set-by-manoeuvre2copyimagetobuffer4tpu to: " + str(message2copyimagetobufferreceive4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copyimagetobuffersend4tpu    = '<default4xi>default2copyimagetobuffer4tpu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2copyimagetobuffersend4tpu-set-by-manoeuvre2copyimagetobuffer4tpu to: " + str(message2copyimagetobuffersend4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max2copyimagetobufferstate4tpu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2copyimagetobufferstate4tpu-set-by-manoeuvre2copyimagetobuffer4tpu to: " + str( max2copyimagetobufferstate4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimagetobuffer4tpu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2copyimagetobuffer4tpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copyimagetobufferreceive4tpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copyimagetobufferreceive4tpu = consumerchannel2copyimagetobuffermanoeuvre4tpu.recv()
            message2print = "{(Step 1B/33) received" + "; message2copyimagetobufferreceive4tpu-received-by-manoeuvre2copyimagetobuffer4tpu: " + str(message2copyimagetobufferreceive4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2copyimagetobufferreceive4tpu == '<copy4xi>copy2copyimagetobuffer4tpu</copy4xi>': 
                message2copyimagetobuffersend4tpu = '<roger4xi>roger2copyimagetobuffer4tpu</roger4xi>'
            else:
                message2copyimagetobuffersend4tpu = '<negative4xi><negative2copyimagetobuffer4tpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copyimagetobuffermanoeuvre4tpu.send(message2copyimagetobuffersend4tpu)
            
            message2print = "{(Step 1C/33) sent" + "; message2copyimagetobuffersend4tpu-sent-by-manoeuvre2copyimagetobuffer4tpu: " + str(message2copyimagetobuffersend4tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2copyimagetobufferreceive4tpu-failed-by-manoeuvre2copyimagetobuffer4tpu: " + str(message2copyimagetobufferreceive4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimagetobuffer4tpu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2copyimagetobuffer4tpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copyimagetobufferreceive4tpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copyimagetobufferreceive4tpu = consumerchannel2copyimagetobuffermanoeuvre4tpu.recv()
            message2print = "{(Step 2B/33) received" + "; message2copyimagetobufferreceive4tpu-received-by-manoeuvre2copyimagetobuffer4tpu: " + str(message2copyimagetobufferreceive4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2copyimagetobufferreceive4tpu == '<ready4xi>ready2copyimagetobuffer4tpu</ready4xi>':
                message2copyimagetobuffersend4tpu = '<ready4xi>ready2copyimagetobuffer4tpu</ready4xi>'
            else:
                message2copyimagetobuffersend4tpu = '<negative4xi><negative2copyimagetobuffer4tpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND READY OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copyimagetobuffermanoeuvre4tpu.send(message2copyimagetobuffersend4tpu)
            
            message2print = "{(Step 2C/33) sent" + "; message2copyimagetobuffersend4tpu-sent-by-manoeuvre2copyimagetobuffer4tpu: " + str(message2copyimagetobuffersend4tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2copyimagetobufferreceive4tpu-failed-by-manoeuvre2copyimagetobuffer4tpu: " + str(message2copyimagetobufferreceive4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimagetobuffer4tpu == 3:    
        info2image4tpu = None
        message2print = "{(Step 3A/70) try" + "; info2image4tpu-try-by-manoeuvre2copyimagetobuffer4tpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4tpu = queue2copyimagetobuffermanoeuvre4tpu.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4tpu-got-by-manoeuvre2copyimagetobuffer4tpu: " + str(info2image4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # ..//..//..//..//..//web//images//png//dragonxi.png
            if info2image4tpu is None:
                code2nack  = code2nack + '3Z1-'
                ack2copyimagetobufferinfo4tpu = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2copyimagetobufferinfo4tpu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2copyimagetobuffermanoeuvre4tpu-failed-by-manoeuvre2copyimagetobuffer4tpu: " + str(queue2copyimagetobuffermanoeuvre4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2copyimagetobuffersend4tpu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2copyimagetobufferinfo4tpu is True:
                message2copyimagetobuffersend4tpu = '<ack4xi>ack2imageinfo4tpu</ack4xi>'
            else:
                message2copyimagetobuffersend4tpu = '<nack4xi>nack2imageinfo4tpu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2copyimagetobuffersend4tpu-set-by-manoeuvre2copyimagetobuffer4tpu to: " + str(message2copyimagetobuffersend4tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <ack4xi>ack2imageinfo4tpu</ack4xi>       
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copyimagetobuffersend4tpu-failed-by-manoeuvre2copyimagetobuffer4tpu: " + str(message2copyimagetobuffersend4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2copyimagetobuffermanoeuvre4tpu-calling-by-manoeuvre2copyimagetobuffer4tpu: " + str(publisherchannel2copyimagetobuffermanoeuvre4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <copy-write PipeConnection, handle 412>
            message2print = "{{(Step 4D/70) passing" + "; message2copyimagetobuffersend4tpu-passing-to-send-by-manoeuvre2copyimagetobuffer4tpu: " + str(message2copyimagetobuffersend4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copyimagetobuffersend4tpu-failed-by-manoeuvre2copyimagetobuffer4tpu: " + str(message2copyimagetobuffersend4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2copyimagetobuffermanoeuvre4tpu.send(message2copyimagetobuffersend4tpu)
            
            message2print = "{(Step 4E/33) sent" + "; message2copyimagetobuffersend4tpu-sent-by-manoeuvre2copyimagetobuffer4tpu: " + str(message2copyimagetobuffersend4tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ack4xi>ack2imageinfo4tpu</ack4xi>            
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2copyimagetobuffersend4tpu-failed-by-manoeuvre2copyimagetobuffer4tpu: " + str(message2copyimagetobuffersend4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4copyimagetobuffer['ACK2INFO4TPU']        = ack2copyimagetobufferinfo4tpu
    response2manouvre4copyimagetobuffer['CODE2ACK4TPU']        = code2ack
    response2manouvre4copyimagetobuffer['CODE2NACK4TPU']       = code2nack
    response2manouvre4copyimagetobuffer['MESSAGE2GET4TPU']     = message2copyimagetobufferget4tpu
    response2manouvre4copyimagetobuffer['MESSAGE2PUT4TPU']     = message2copyimagetobufferput4tpu
    response2manouvre4copyimagetobuffer['MESSAGE2RECEIVE4TPU'] = message2copyimagetobufferreceive4tpu
    response2manouvre4copyimagetobuffer['MESSAGE2SEND4TPU']    = message2copyimagetobuffersend4tpu
    response2manouvre4copyimagetobuffer['STATE2MANOEUVRE4TPU'] = state2copyimagetobuffer4tpu
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4copyimagetobuffer-returning-by-manoeuvre2copyimagetobuffer4tpu: " + str(response2manouvre4copyimagetobuffer) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 {'MESSAGE2SEND4TPU': '<roger4xi>roger2copyimagetobuffer4tpu</roger4xi>',
    #   'ACK2INFO4TPU': False,
    #   'MESSAGE2GET4TPU': '<default4xi>default2copyimagetobuffer4tpu</default4xi>',
    #   'MESSAGE2RECEIVE4TPU': '<copy4xi>copy2copyimagetobuffer4tpu</copy4xi>',
    #   'CODE2ACK4TPU': ' 0D-1C-',
    #   'MESSAGE2PUT4TPU': '<default4xi>default2copyimagetobuffer4tpu</default4xi>',
    #   'CODE2NACK4TPU': ' ',
    #   'STATE2MANOEUVRE4TPU': 1}; time: 02:00:20 10/04/11 Pacific Daylight Time}

    
    #2 {'MESSAGE2SEND4TPU': '<ready4xi>ready2copyimagetobuffer4tpu</ready4xi>',
    #   'ACK2INFO4TPU': False,
    #   'MESSAGE2GET4TPU': '<default4xi>default2copyimagetobuffer4tpu</default4xi>',
    #   'MESSAGE2RECEIVE4TPU': '<ready4xi>ready2copyimagetobuffer4tpu</ready4xi>',
    #   'CODE2ACK4TPU': ' 0D-2C-',
    #   'MESSAGE2PUT4TPU': '<default4xi>default2copyimagetobuffer4tpu</default4xi>',
    #   'CODE2NACK4TPU': ' ',
    #   'STATE2MANOEUVRE4TPU': 2}; time: 02:10:33 10/04/11 Pacific Daylight Time}


    #3 {'MESSAGE2SEND4TPU': '<ack4xi>ack2imageinfo4tpu</ack4xi>',
    #   'ACK2INFO4TPU': True,
    #   'MESSAGE2GET4TPU': '<default4xi>default2copyimagetobuffer4tpu</default4xi>',
    #   'MESSAGE2RECEIVE4TPU': '<ready4xi>ready2copyimagetobuffer4tpu</ready4xi>',
    #   'CODE2ACK4TPU': ' 0D-3C-4E-',
    #   'MESSAGE2PUT4TPU': '<default4xi>default2copyimagetobuffer4tpu</default4xi>',
    #   'CODE2NACK4TPU': ' ',
    #   'STATE2MANOEUVRE4TPU': 3}; time: 02:10:33 10/04/11 Pacific Daylight Time}


    #4 {'MESSAGE2SEND4TPU': '<default4xi>default2copyimagetobuffer4tpu</default4xi>',
    #   'ACK2INFO4TPU': False,
    #   'MESSAGE2GET4TPU': '<default4xi>default2copyimagetobuffer4tpu</default4xi>',
    #   'MESSAGE2RECEIVE4TPU': '<default4xi>default2copyimagetobuffer4tpu</default4xi>',
    #   'CODE2ACK4TPU': ' 0D-',
    #   'MESSAGE2PUT4TPU': '<default4xi>default2copyimagetobuffer4tpu</default4xi>',
    #   'CODE2NACK4TPU': ' ',
    #   'STATE2MANOEUVRE4TPU': 0}; time: 02:10:33 10/04/11 Pacific Daylight Time}


    return response2manouvre4copyimagetobuffer           
          
#</function4xi>

# -----------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2copybuffertoimage4tpu" usage = "
#   # ---------------------------------------------------------------------------------------------------------------------
#   # SIMULATE MANOEUVRE2COPYIMAGE4TPU - LET IT TO RECEIVE COPY AND TO SEND ROGER VIA PIPE                
#   # ---------------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4tpu is True:
#       print("(Step 82A/700) OF TPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4TPU ", time4xi())   
#       try:        
#           response2manoeuvre4copy = manoeuvre2copybuffertoimage4tpu(queue4copybuffertoimage4tpu, channel4copybuffertoimageconsume4tpu, channel4copybuffertoimagepublish4tpu)
#           print("(Step 82B/700) OF TPU2IMAGE4XI5TEST - PERFORMED MANOEUVRE2COPYIMAGE4TPU, RESPONSE2MANOEUVRE4COPY: ", response2manoeuvre4copy)
#           # {'MESSAGE2SEND4TPU': '<roger4xi>roger2copybuffertoimage4tpu</roger4xi>',
#           #   'ACK2INFO4TPU': False,
#           #   'MESSAGE2GET4TPU': '<default4xi>default2copybuffertoimage4tpu</default4xi>',
#           #   'MESSAGE2RECEIVE4TPU': '<copy4xi>copy2copybuffertoimage4tpu</copy4xi>',
#           #   'CODE2ACK4TPU': ' 0D-1C-',
#           #   'MESSAGE2PUT4TPU': '<default4xi>default2copybuffertoimage4tpu</default4xi>',
#           #   'CODE2NACK4TPU': ' ',
#           #   'STATE2MANOEUVRE4TPU': 1}
#       except:
#           print("(Step 82Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4TPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#">

# ----------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2copybuffertoimage4tpu(queue2copybuffertoimagemanoeuvre4tpu, consumerchannel2copybuffertoimagemanoeuvre4tpu, publisherchannel2copybuffertoimagemanoeuvre4tpu):
    '''
    manoeuvres copy control and command for copy2buffertoimage4tpu
    '''
    global state2copybuffertoimage4tpu                                                                                           # STATE FOR COMMAND AND CONTROL
    global max2copybuffertoimagestate4tpu                                                                                        # MAX FOR STATE
    global ack2copybuffertoimageinfo4tpu                                                                                         # ACK FOR IMAGE INFO
    global message2copybuffertoimageget4tpu                                 
    global message2copybuffertoimageput4tpu
    global message2copybuffertoimagereceive4tpu
    global message2copybuffertoimagesend4tpu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2copybuffertoimage4tpu" 
    method2print = "manoeuvre2copybuffertoimage4tpu" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2copybuffertoimage4tpu-started: " + str(manoeuvre2copybuffertoimage4tpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # 
    
    message2print = "{(Step 0B/33) passed" + "; queue2copybuffertoimagemanoeuvre4tpu-to-manoeuvre2copybuffertoimage4tpu: " + str(queue2copybuffertoimagemanoeuvre4tpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2copybuffertoimagemanoeuvre4tpu-to-manoeuvre2copybuffertoimage4tpu: " + str(consumerchannel2copybuffertoimagemanoeuvre4tpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2copybuffertoimagemanoeuvre4tpu-to-manoeuvre2copybuffertoimage4tpu: " + str(publisherchannel2copybuffertoimagemanoeuvre4tpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4copybuffertoimage = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2copybuffertoimage4tpu-try-by-manoeuvre2copybuffertoimage4tpu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2copybuffertoimage4tpu is None:
            state2copybuffertoimage4tpu = 0
            message2print = "{(Step 0F/33) initialized" + "; state2copybuffertoimage4tpu-initialized-by-manoeuvre2copybuffertoimage4tpu to: " + str(state2copybuffertoimage4tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2copybuffertoimage4tpu-used-by-manoeuvre2copybuffertoimage4tpu: " + str(state2copybuffertoimage4tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2copybuffertoimage4tpu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2copybuffertoimage4tpu-initialized-by-manoeuvre2copybuffertoimage4tpu to: " + str(state2copybuffertoimage4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2copybuffertoimageinfo4tpu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2copybuffertoimageinfo4tpu-initialized-by-manoeuvre2copybuffertoimage4tpu to: " + str(ack2copybuffertoimageinfo4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2copybuffertoimageget4tpu     = '<default4xi>default2copybuffertoimage4tpu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2copybuffertoimageget4tpu-initialized-by-manoeuvre2copybuffertoimage4tpu to: " + str(message2copybuffertoimageget4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message2copybuffertoimageput4tpu     = '<default4xi>default2copybuffertoimage4tpu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2copybuffertoimageput4tpu-initialized-by-manoeuvre2copybuffertoimage4tpu to: " + str(message2copybuffertoimageput4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copybuffertoimagereceive4tpu = '<default4xi>default2copybuffertoimage4tpu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2copybuffertoimagereceive4tpu-initialized-by-manoeuvre2copybuffertoimage4tpu to: " + str(message2copybuffertoimagereceive4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copybuffertoimagesend4tpu    = '<default4xi>default2copybuffertoimage4tpu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2copybuffertoimagesend4tpu-initialized-by-manoeuvre2copybuffertoimage4tpu to: " + str(message2copybuffertoimagesend4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max2copybuffertoimagestate4tpu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2copybuffertoimagestate4tpu-initialized-by-manoeuvre2copybuffertoimage4tpu to: " + str( max2copybuffertoimagestate4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2copybuffertoimage4tpu = state2copybuffertoimage4tpu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2copybuffertoimage4tpu-added-by-manoeuvre2copybuffertoimage4tpu: " + str(state2copybuffertoimage4tpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4 
     
    if state2copybuffertoimage4tpu > max2copybuffertoimagestate4tpu:
        state2copybuffertoimage4tpu = 0
        message2print = "{(Step 0I/33) set" + "; state2copybuffertoimage4tpu-set-by-manoeuvre2copybuffertoimage4tpu to: " + str(state2copybuffertoimage4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        ack2copybuffertoimageinfo4tpu        = False
        message2print = "{(Step 0J/33) set" + "; ack2copybuffertoimageinfo4tpu-set-by-manoeuvre2copybuffertoimage4tpu to: " + str(ack2copybuffertoimageinfo4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # 
        message2copybuffertoimageget4tpu     = '<default4xi>default2copybuffertoimage4tpu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2copybuffertoimageget4tpu-set-by-manoeuvre2copybuffertoimage4tpu to: " + str(message2copybuffertoimageget4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message2copybuffertoimageput4tpu     = '<default4xi>default2copybuffertoimage4tpu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2copybuffertoimageput4tpu-set-by-manoeuvre2copybuffertoimage4tpu to: " + str(message2copybuffertoimageput4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copybuffertoimagereceive4tpu = '<default4xi>default2copybuffertoimage4tpu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2copybuffertoimagereceive4tpu-set-by-manoeuvre2copybuffertoimage4tpu to: " + str(message2copybuffertoimagereceive4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copybuffertoimagesend4tpu    = '<default4xi>default2copybuffertoimage4tpu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2copybuffertoimagesend4tpu-set-by-manoeuvre2copybuffertoimage4tpu to: " + str(message2copybuffertoimagesend4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max2copybuffertoimagestate4tpu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2copybuffertoimagestate4tpu-set-by-manoeuvre2copybuffertoimage4tpu to: " + str( max2copybuffertoimagestate4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copybuffertoimage4tpu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2copybuffertoimage4tpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copybuffertoimagereceive4tpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copybuffertoimagereceive4tpu = consumerchannel2copybuffertoimagemanoeuvre4tpu.recv()
            message2print = "{(Step 1B/33) received" + "; message2copybuffertoimagereceive4tpu-received-by-manoeuvre2copybuffertoimage4tpu: " + str(message2copybuffertoimagereceive4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2copybuffertoimagereceive4tpu == '<copy4xi>copy2copybuffertoimage4tpu</copy4xi>':
                message2copybuffertoimagesend4tpu = '<roger4xi>roger2copybuffertoimage4tpu</roger4xi>'
            else:
                message2copybuffertoimagesend4tpu = '<negative4xi><negative2copybuffertoimage4tpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copybuffertoimagemanoeuvre4tpu.send(message2copybuffertoimagesend4tpu)
            
            message2print = "{(Step 1C/33) sent" + "; message2copybuffertoimagesend4tpu-sent-by-manoeuvre2copybuffertoimage4tpu: " + str(message2copybuffertoimagesend4tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2copybuffertoimagereceive4tpu-failed-by-manoeuvre2copybuffertoimage4tpu: " + str(message2copybuffertoimagereceive4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copybuffertoimage4tpu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2copybuffertoimage4tpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copybuffertoimagereceive4tpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copybuffertoimagereceive4tpu = consumerchannel2copybuffertoimagemanoeuvre4tpu.recv()
            message2print = "{(Step 2B/33) received" + "; message2copybuffertoimagereceive4tpu-received-by-manoeuvre2copybuffertoimage4tpu: " + str(message2copybuffertoimagereceive4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2copybuffertoimagereceive4tpu == '<ready4xi>ready2copybuffertoimage4tpu</ready4xi>':
                message2copybuffertoimagesend4tpu = '<ready4xi>ready2copybuffertoimage4tpu</ready4xi>'
            else:
                message2copybuffertoimagesend4tpu = '<negative4xi><negative2copybuffertoimage4tpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND READY OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copybuffertoimagemanoeuvre4tpu.send(message2copybuffertoimagesend4tpu)
            
            message2print = "{(Step 2C/33) sent" + "; message2copybuffertoimagesend4tpu-sent-by-manoeuvre2copybuffertoimage4tpu: " + str(message2copybuffertoimagesend4tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2copybuffertoimagereceive4tpu-failed-by-manoeuvre2copybuffertoimage4tpu: " + str(message2copybuffertoimagereceive4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copybuffertoimage4tpu == 3:    
        info2image4tpu = None
        message2print = "{(Step 3A/70) try" + "; info2image4tpu-try-by-manoeuvre2copybuffertoimage4tpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4tpu = queue2copybuffertoimagemanoeuvre4tpu.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4tpu-got-by-manoeuvre2copybuffertoimage4tpu: " + str(info2image4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # ..//..//..//..//..//web//images//png//dragonxi.png
            if info2image4tpu is None:
                code2nack  = code2nack + '3Z1-'
                ack2copybuffertoimageinfo4tpu = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2copybuffertoimageinfo4tpu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2copybuffertoimagemanoeuvre4tpu-failed-by-manoeuvre2copybuffertoimage4tpu: " + str(queue2copybuffertoimagemanoeuvre4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2copybuffertoimagesend4tpu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2copybuffertoimageinfo4tpu is True:
                message2copybuffertoimagesend4tpu = '<ack4xi>ack2imageinfo4tpu</ack4xi>'
            else:
                message2copybuffertoimagesend4tpu = '<nack4xi>nack2imageinfo4tpu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2copybuffertoimagesend4tpu-set-by-manoeuvre2copybuffertoimage4tpu to: " + str(message2copybuffertoimagesend4tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <ack4xi>ack2imageinfo4tpu</ack4xi>       
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copybuffertoimagesend4tpu-failed-by-manoeuvre2copybuffertoimage4tpu: " + str(message2copybuffertoimagesend4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2copybuffertoimagemanoeuvre4tpu-calling-by-manoeuvre2copybuffertoimage4tpu: " + str(publisherchannel2copybuffertoimagemanoeuvre4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <copy-write PipeConnection, handle 412>
            message2print = "{{(Step 4D/70) passing" + "; message2copybuffertoimagesend4tpu-passing-to-send-by-manoeuvre2copybuffertoimage4tpu: " + str(message2copybuffertoimagesend4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copybuffertoimagesend4tpu-failed-by-manoeuvre2copybuffertoimage4tpu: " + str(message2copybuffertoimagesend4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2copybuffertoimagemanoeuvre4tpu.send(message2copybuffertoimagesend4tpu)
            
            message2print = "{(Step 4E/33) sent" + "; message2copybuffertoimagesend4tpu-sent-by-manoeuvre2copybuffertoimage4tpu: " + str(message2copybuffertoimagesend4tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ack4xi>ack2imageinfo4tpu</ack4xi>            
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2copybuffertoimagesend4tpu-failed-by-manoeuvre2copybuffertoimage4tpu: " + str(message2copybuffertoimagesend4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4copybuffertoimage['ACK2INFO4TPU']        = ack2copybuffertoimageinfo4tpu
    response2manouvre4copybuffertoimage['CODE2ACK4TPU']        = code2ack
    response2manouvre4copybuffertoimage['CODE2NACK4TPU']       = code2nack
    response2manouvre4copybuffertoimage['MESSAGE2GET4TPU']     = message2copybuffertoimageget4tpu
    response2manouvre4copybuffertoimage['MESSAGE2PUT4TPU']     = message2copybuffertoimageput4tpu
    response2manouvre4copybuffertoimage['MESSAGE2RECEIVE4TPU'] = message2copybuffertoimagereceive4tpu
    response2manouvre4copybuffertoimage['MESSAGE2SEND4TPU']    = message2copybuffertoimagesend4tpu
    response2manouvre4copybuffertoimage['STATE2MANOEUVRE4TPU'] = state2copybuffertoimage4tpu
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4copybuffertoimage-returning-by-manoeuvre2copybuffertoimage4tpu: " + str(response2manouvre4copybuffertoimage) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 {'MESSAGE2SEND4TPU': '<roger4xi>roger2copybuffertoimage4tpu</roger4xi>',
    #   'ACK2INFO4TPU': False,
    #   'MESSAGE2GET4TPU': '<default4xi>default2copybuffertoimage4tpu</default4xi>',
    #   'MESSAGE2RECEIVE4TPU': '<copy4xi>copy2copybuffertoimage4tpu</copy4xi>',
    #   'CODE2ACK4TPU': ' 0D-1C-',
    #   'MESSAGE2PUT4TPU': '<default4xi>default2copybuffertoimage4tpu</default4xi>',
    #   'CODE2NACK4TPU': ' ',
    #   'STATE2MANOEUVRE4TPU': 1}; time: 02:18:55 10/04/11 Pacific Daylight Time}

    
    #2 {'MESSAGE2SEND4TPU': '<ready4xi>ready2copybuffertoimage4tpu</ready4xi>',
    #   'ACK2INFO4TPU': False,
    #   'MESSAGE2GET4TPU': '<default4xi>default2copybuffertoimage4tpu</default4xi>',
    #   'MESSAGE2RECEIVE4TPU': '<ready4xi>ready2copybuffertoimage4tpu</ready4xi>',
    #   'CODE2ACK4TPU': ' 0D-2C-',
    #   'MESSAGE2PUT4TPU': '<default4xi>default2copybuffertoimage4tpu</default4xi>',
    #   'CODE2NACK4TPU': ' ',
    #   'STATE2MANOEUVRE4TPU': 2}; time: 02:18:55 10/04/11 Pacific Daylight Time}

    #3 {'MESSAGE2SEND4TPU': '<ack4xi>ack2imageinfo4tpu</ack4xi>',
    #   'ACK2INFO4TPU': True,
    #   'MESSAGE2GET4TPU': '<default4xi>default2copybuffertoimage4tpu</default4xi>',
    #   'MESSAGE2RECEIVE4TPU': '<ready4xi>ready2copybuffertoimage4tpu</ready4xi>',
    #   'CODE2ACK4TPU': ' 0D-3C-4E-',
    #   'MESSAGE2PUT4TPU': '<default4xi>default2copybuffertoimage4tpu</default4xi>',
    #   'CODE2NACK4TPU': ' ',
    #   'STATE2MANOEUVRE4TPU': 3}; time: 02:18:55 10/04/11 Pacific Daylight Time}
 

    #4 {'MESSAGE2SEND4TPU': '<default4xi>default2copybuffertoimage4tpu</default4xi>',
    #   'ACK2INFO4TPU': False,
    #   'MESSAGE2GET4TPU': '<default4xi>default2copybuffertoimage4tpu</default4xi>',
    #   'MESSAGE2RECEIVE4TPU': '<default4xi>default2copybuffertoimage4tpu</default4xi>',
    #   'CODE2ACK4TPU': ' 0D-',
    #   'MESSAGE2PUT4TPU': '<default4xi>default2copybuffertoimage4tpu</default4xi>',
    #   'CODE2NACK4TPU': ' ',
    #   'STATE2MANOEUVRE4TPU': 0}

    return response2manouvre4copybuffertoimage           
          
#</function4xi>



# -----------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2readimage4tpu" usage = "control function for read2image4tpu function in this module">

# ----------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2readimage4tpu(queue2readmanoeuvre4tpu, consumerchannel2readmanoeuvre4tpu, publisherchannel2readmanoeuvre4tpu):
    '''
    manoeuvres read control and command
    '''
    global state2read4tpu                                                                                           # STATE FOR COMMAND AND CONTROL
    global max2readstate4tpu                                                                                        # MAX FOR STATE
    global ack2readinfo4tpu                                                                                         # ACK FOR IMAGE INFO
    global message2readget4tpu                                 
    global message2readput4tpu
    global message2readreceive4tpu
    global message2readsend4tpu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2readimage4tpu" 
    method2print = "manoeuvre2readimage4tpu" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2readimage4tpu-started: " + str(manoeuvre2readimage4tpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function manoeuvre2readimage4tpu at 0x0000000003DB8C48>  
    
    message2print = "{(Step 0B/33) passed" + "; queue2readmanoeuvre4tpu-to-manoeuvre2readimage4tpu: " + str(queue2readmanoeuvre4tpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <queue4xi.Queue4Xi object at 0x0000000003AD9400>
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2readmanoeuvre4tpu-to-manoeuvre2readimage4tpu: " + str(consumerchannel2readmanoeuvre4tpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <read-write PipeConnection, handle 408>
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2readmanoeuvre4tpu-to-manoeuvre2readimage4tpu: " + str(publisherchannel2readmanoeuvre4tpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <read-write PipeConnection, handle 412>     
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4read = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2read4tpu-try-by-manoeuvre2readimage4tpu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2read4tpu is None:
            state2read4tpu = 0
            message2print = "{(Step 0F/33) initialized" + "; state2read4tpu-initialized-by-manoeuvre2readimage4tpu to: " + str(state2read4tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2read4tpu-used-by-manoeuvre2readimage4tpu: " + str(state2read4tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2read4tpu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2read4tpu-initialized-by-manoeuvre2readimage4tpu to: " + str(state2read4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2readinfo4tpu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2readinfo4tpu-initialized-by-manoeuvre2readimage4tpu to: " + str(ack2readinfo4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2readget4tpu     = '<default4xi>default2read4tpu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2readget4tpu-initialized-by-manoeuvre2readimage4tpu to: " + str(message2readget4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2read4tpu</default4xi>
        message2readput4tpu     = '<default4xi>default2read4tpu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2readput4tpu-initialized-by-manoeuvre2readimage4tpu to: " + str(message2readput4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4tpu</default4xi>
        message2readreceive4tpu = '<default4xi>default2read4tpu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2readreceive4tpu-initialized-by-manoeuvre2readimage4tpu to: " + str(message2readreceive4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4tpu</default4xi>
        message2readsend4tpu    = '<default4xi>default2read4tpu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2readsend4tpu-initialized-by-manoeuvre2readimage4tpu to: " + str(message2readsend4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4tpu</default4xi>
        max2readstate4tpu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2readstate4tpu-initialized-by-manoeuvre2readimage4tpu to: " + str( max2readstate4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2read4tpu = state2read4tpu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2read4tpu-added-by-manoeuvre2readimage4tpu: " + str(state2read4tpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4  
    if state2read4tpu > max2readstate4tpu:
        state2read4tpu = 0
        message2print = "{(Step 0I/33) set" + "; state2read4tpu-set-by-manoeuvre2readimage4tpu to: " + str(state2read4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2readinfo4tpu        = False
        message2print = "{(Step 0J/33) set" + "; ack2readinfo4tpu-set-by-manoeuvre2readimage4tpu to: " + str(ack2readinfo4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2readget4tpu     = '<default4xi>default2read4tpu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2readget4tpu-set-by-manoeuvre2readimage4tpu to: " + str(message2readget4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2read4tpu</default4xi>
        message2readput4tpu     = '<default4xi>default2read4tpu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2readput4tpu-set-by-manoeuvre2readimage4tpu to: " + str(message2readput4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4tpu</default4xi>
        message2readreceive4tpu = '<default4xi>default2read4tpu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2readreceive4tpu-set-by-manoeuvre2readimage4tpu to: " + str(message2readreceive4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4tpu</default4xi>
        message2readsend4tpu    = '<default4xi>default2read4tpu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2readsend4tpu-set-by-manoeuvre2readimage4tpu to: " + str(message2readsend4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4tpu</default4xi>
        max2readstate4tpu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2readstate4tpu-set-by-manoeuvre2readimage4tpu to: " + str( max2readstate4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2read4tpu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2readimage4tpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2readreceive4tpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2readreceive4tpu = consumerchannel2readmanoeuvre4tpu.recv()
            message2print = "{(Step 1B/33) received" + "; message2readreceive4tpu-received-by-manoeuvre2readimage4tpu: " + str(message2readreceive4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <copy4xi>copy2read4tpu</copy4xi>
                         
            if message2readreceive4tpu == '<copy4xi>copy2read4tpu</copy4xi>':
                message2readsend4tpu = '<roger4xi>roger2read4tpu</roger4xi>'
            else:
                message2readsend4tpu = '<negative4xi><negative2read4tpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2readmanoeuvre4tpu.send(message2readsend4tpu)
            
            message2print = "{(Step 1C/33) sent" + "; message2readsend4tpu-sent-by-manoeuvre2readimage4tpu: " + str(message2readsend4tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <roger4xi>roger2read4tpu</roger4xi>
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2readreceive4tpu-failed-by-manoeuvre2readimage4tpu: " + str(message2readreceive4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2read4tpu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2readimage4tpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2readreceive4tpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2readreceive4tpu = consumerchannel2readmanoeuvre4tpu.recv()
            message2print = "{(Step 2B/33) received" + "; message2readreceive4tpu-received-by-manoeuvre2readimage4tpu: " + str(message2readreceive4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <ready4xi>ready2read4tpu</ready4xi> 
                         
            if message2readreceive4tpu == '<ready4xi>ready2read4tpu</ready4xi>':
                message2readsend4tpu = '<ready4xi>ready2read4tpu</ready4xi>'
            else:
                message2readsend4tpu = '<negative4xi><negative2ready4tpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2readmanoeuvre4tpu.send(message2readsend4tpu)
            
            message2print = "{(Step 2C/33) sent" + "; message2readsend4tpu-sent-by-manoeuvre2readimage4tpu: " + str(message2readsend4tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ready4xi>ready2read4tpu</ready4xi>
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2readreceive4tpu-failed-by-manoeuvre2readimage4tpu: " + str(message2readreceive4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2read4tpu == 3:    
        info2image4tpu = None
        message2print = "{(Step 3A/70) try" + "; info2image4tpu-try-by-manoeuvre2readimage4tpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4tpu = queue2readmanoeuvre4tpu.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4tpu-got-by-manoeuvre2readimage4tpu: " + str(info2image4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # ..//..//..//..//..//web//images//png//dragonxi.png
            if info2image4tpu is None:
                code2nack  = code2nack + '3Z1-'
                ack2readinfo4tpu = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2readinfo4tpu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2readmanoeuvre4tpu-failed-by-manoeuvre2readimage4tpu: " + str(queue2readmanoeuvre4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2readsend4tpu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2readinfo4tpu is True:
                message2readsend4tpu = '<ack4xi>ack2imageinfo4tpu</ack4xi>'
            else:
                message2readsend4tpu = '<nack4xi>nack2imageinfo4tpu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2readsend4tpu-set-by-manoeuvre2readimage4tpu to: " + str(message2readsend4tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <ack4xi>ack2imageinfo4tpu</ack4xi>       
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2readsend4tpu-failed-by-manoeuvre2readimage4tpu: " + str(message2readsend4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2readmanoeuvre4tpu-calling-by-manoeuvre2readimage4tpu: " + str(publisherchannel2readmanoeuvre4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <read-write PipeConnection, handle 412>
            message2print = "{{(Step 4D/70) passing" + "; message2readsend4tpu-passing-to-send-by-manoeuvre2readimage4tpu: " + str(message2readsend4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2readsend4tpu-failed-by-manoeuvre2readimage4tpu: " + str(message2readsend4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2readmanoeuvre4tpu.send(message2readsend4tpu)
            
            message2print = "{(Step 4E/33) sent" + "; message2readsend4tpu-sent-by-manoeuvre2readimage4tpu: " + str(message2readsend4tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ack4xi>ack2imageinfo4tpu</ack4xi>            
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2readsend4tpu-failed-by-manoeuvre2readimage4tpu: " + str(message2readsend4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4read['ACK2INFO4TPU']        = ack2readinfo4tpu
    response2manouvre4read['CODE2ACK4TPU']        = code2ack
    response2manouvre4read['CODE2NACK4TPU']       = code2nack
    response2manouvre4read['MESSAGE2GET4TPU']     = message2readget4tpu
    response2manouvre4read['MESSAGE2PUT4TPU']     = message2readput4tpu
    response2manouvre4read['MESSAGE2RECEIVE4TPU'] = message2readreceive4tpu
    response2manouvre4read['MESSAGE2SEND4TPU']    = message2readsend4tpu
    response2manouvre4read['STATE2MANOEUVRE4TPU'] = state2read4tpu
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4read-returning-by-manoeuvre2readimage4tpu: " + str(response2manouvre4read) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 {'MESSAGE2SEND4TPU': '<roger4xi>roger2read4tpu</roger4xi>',
    #  'ACK2INFO4TPU': False,
    #  'MESSAGE2GET4TPU': '<default4xi>default2read4tpu</default4xi>',
    #  'MESSAGE2RECEIVE4TPU': '<copy4xi>copy2read4tpu</copy4xi>',
    #  'CODE2ACK4TPU': ' 0D-1C-',
    #  'MESSAGE2PUT4TPU': '<default4xi>default2read4tpu</default4xi>',
    #  'CODE2NACK4TPU': ' ', 'STATE2MANOEUVRE4TPU': 1}; time: 01:44:40 09/29/11 Pacific Daylight Time}
 
    #2 {'MESSAGE2SEND4TPU': '<ready4xi>ready2read4tpu</ready4xi>',
    #  'ACK2INFO4TPU': False,
    #  'MESSAGE2GET4TPU': '<default4xi>default2read4tpu</default4xi>',
    #  'MESSAGE2RECEIVE4TPU': '<ready4xi>ready2read4tpu</ready4xi>',
    #  'CODE2ACK4TPU': ' 0D-2C-',
    #  'MESSAGE2PUT4TPU': '<default4xi>default2read4tpu</default4xi>',
    #  'CODE2NACK4TPU': ' ', 'STATE2MANOEUVRE4TPU': 2}; time: 01:44:40 09/29/11 Pacific Daylight Time}
    
    #3 {'MESSAGE2SEND4TPU': '<ack4xi>ack2imageinfo4tpu</ack4xi>',
    #  'ACK2INFO4TPU': True,
    #  'MESSAGE2GET4TPU': '<default4xi>default2read4tpu</default4xi>',
    #  'MESSAGE2RECEIVE4TPU': '<ready4xi>ready2read4tpu</ready4xi>',
    #  'CODE2ACK4TPU': ' 0D-3C-4E-',
    #  'MESSAGE2PUT4TPU': '<default4xi>default2read4tpu</default4xi>',
    #  'CODE2NACK4TPU': ' ',
    #  'STATE2MANOEUVRE4TPU': 3}; time: 01:44:40 09/29/11 Pacific Daylight Time}
    
    #4 {'MESSAGE2SEND4TPU': '<default4xi>default2read4tpu</default4xi>',
    #  'ACK2INFO4TPU': False,
    #  'MESSAGE2GET4TPU': '<default4xi>default2read4tpu</default4xi>',
    #  'MESSAGE2RECEIVE4TPU': '<default4xi>default2read4tpu</default4xi>',
    #  'CODE2ACK4TPU': ' 0D-',
    #  'MESSAGE2PUT4TPU': '<default4xi>default2read4tpu</default4xi>',
    #  'CODE2NACK4TPU': ' ',
    #  'STATE2MANOEUVRE4TPU': 0}; time: 02:11:49 09/29/11 Pacific Daylight Time}


 
    
    
    return response2manouvre4read           
          
#</function4xi>


# -------------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2writeimage4tpu" usage = "control function for write2image4tpu function in this module">

# ------------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2writeimage4tpu(queue2writemanoeuvre4tpu, consumerchannel2writemanoeuvre4tpu, publisherchannel2writemanoeuvre4tpu):
    '''
    manoeuvres control and command
    '''
    global state2write4tpu                                                       # STATE FOR COMMAND AND CONTROL
    global max2writestate4tpu                                                    # MAX FOR STATE
    global ack2writeinfo4tpu                                                     # ACK FOR IMAGE INFO
    global message2writeget4tpu                                 
    global message2writeput4tpu
    global message2writereceive4tpu
    global message2writesend4tpu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2writeimage4tpu" 
    method2print = "manoeuvre2writeimage4tpu" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2writeimage4tpu-started: " + str(manoeuvre2writeimage4tpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                      
    # <function manoeuvre2writeimage4tpu at 0x0000000003DB8C48>  
    
    message2print = "{(Step 0B/33) passed" + "; queue2writemanoeuvre4tpu-to-manoeuvre2writeimage4tpu: " + str(queue2writemanoeuvre4tpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <queue4xi.Queue4Xi object at 0x000000000422D9E8>
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2writemanoeuvre4tpu-to-manoeuvre2writeimage4tpu: " + str(consumerchannel2writemanoeuvre4tpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # read-write PipeConnection, handle 568>
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2writemanoeuvre4tpu-to-manoeuvre2writeimage4tpu: " + str(publisherchannel2writemanoeuvre4tpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <read-write PipeConnection, handle 572> 
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4write = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2write4tpu-try-by-manoeuvre2writeimage4tpu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2write4tpu is None:
            state2write4tpu = 0
            message2print = "{(Step 0F/33) initialized" + "; state2write4tpu-initialized-by-manoeuvre2writeimage4tpu to: " + str(state2write4tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2write4tpu-used-by-manoeuvre2writeimage4tpu: " + str(state2write4tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2write4tpu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2write4tpu-initialized-by-manoeuvre2writeimage4tpu to: " + str(state2write4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2writeinfo4tpu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2writeinfo4tpu-initialized-by-manoeuvre2writeimage4tpu to: " + str(ack2writeinfo4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2writeget4tpu     = '<default4xi>default2write4tpu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2writeget4tpu-initialized-by-manoeuvre2writeimage4tpu to: " + str(message2writeget4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2write4tpu</default4xi>
        message2writeput4tpu     = '<default4xi>default2write4tpu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2writeput4tpu-initialized-by-manoeuvre2writeimage4tpu to: " + str(message2writeput4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4tpu</default4xi>
        message2writereceive4tpu = '<default4xi>default2write4tpu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2writereceive4tpu-initialized-by-manoeuvre2writeimage4tpu to: " + str(message2writereceive4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4tpu</default4xi>
        message2writesend4tpu    = '<default4xi>default2write4tpu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2writesend4tpu-initialized-by-manoeuvre2writeimage4tpu to: " + str(message2writesend4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4tpu</default4xi>
        max2writestate4tpu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2writestate4tpu-initialized-by-manoeuvre2writeimage4tpu to: " + str( max2writestate4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2write4tpu = state2write4tpu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2write4tpu-added-by-manoeuvre2writeimage4tpu: " + str(state2write4tpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4  
    if state2write4tpu > max2writestate4tpu:
        state2write4tpu = 0
        message2print = "{(Step 0I/33) set" + "; state2write4tpu-set-by-manoeuvre2writeimage4tpu to: " + str(state2write4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2writeinfo4tpu        = False
        message2print = "{(Step 0J/33) set" + "; ack2writeinfo4tpu-set-by-manoeuvre2writeimage4tpu to: " + str(ack2writeinfo4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2writeget4tpu     = '<default4xi>default2write4tpu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2writeget4tpu-set-by-manoeuvre2writeimage4tpu to: " + str(message2writeget4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2write4tpu</default4xi>
        message2writeput4tpu     = '<default4xi>default2write4tpu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2writeput4tpu-set-by-manoeuvre2writeimage4tpu to: " + str(message2writeput4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4tpu</default4xi>
        message2writereceive4tpu = '<default4xi>default2write4tpu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2writereceive4tpu-set-by-manoeuvre2writeimage4tpu to: " + str(message2writereceive4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4tpu</default4xi>
        message2writesend4tpu    = '<default4xi>default2write4tpu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2writesend4tpu-set-by-manoeuvre2writeimage4tpu to: " + str(message2writesend4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4tpu</default4xi>
        max2writestate4tpu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2writestate4tpu-set-by-manoeuvre2writeimage4tpu to: " + str( max2writestate4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2write4tpu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2writeimage4tpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2writereceive4tpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2writereceive4tpu = consumerchannel2writemanoeuvre4tpu.recv()
            message2print = "{(Step 1B/33) received" + "; message2writereceive4tpu-received-by-manoeuvre2writeimage4tpu: " + str(message2writereceive4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <copy4xi>copy2write4tpu</copy4xi>
                         
            if message2writereceive4tpu == '<copy4xi>copy2write4tpu</copy4xi>':
                message2writesend4tpu = '<roger4xi>roger2write4tpu</roger4xi>'
            else:
                message2writesend4tpu = '<negative4xi><negative2write4tpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2writemanoeuvre4tpu.send(message2writesend4tpu)
            
            message2print = "{(Step 1C/33) sent" + "; message2writesend4tpu-sent-by-manoeuvre2writeimage4tpu: " + str(message2writesend4tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <roger4xi>roger2write4tpu</roger4xi>
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2writereceive4tpu-failed-by-manoeuvre2writeimage4tpu: " + str(message2writereceive4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2write4tpu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2writeimage4tpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2writereceive4tpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2writereceive4tpu = consumerchannel2writemanoeuvre4tpu.recv()
            message2print = "{(Step 2B/33) received" + "; message2writereceive4tpu-received-by-manoeuvre2writeimage4tpu: " + str(message2writereceive4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <ready4xi>ready2write4tpu</ready4xi>
                         
            if message2writereceive4tpu == '<ready4xi>ready2write4tpu</ready4xi>':
                message2writesend4tpu = '<ready4xi>ready2write4tpu</ready4xi>'
            else:
                message2writesend4tpu = '<negative4xi><negative2ready4tpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2writemanoeuvre4tpu.send(message2writesend4tpu)
            
            message2print = "{(Step 2C/33) sent" + "; message2writesend4tpu-sent-by-manoeuvre2writeimage4tpu: " + str(message2writesend4tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ready4xi>ready2write4tpu</ready4xi>
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2writereceive4tpu-failed-by-manoeuvre2writeimage4tpu: " + str(message2writereceive4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2write4tpu == 3:    
        info2image4tpu = None
        message2print = "{(Step 3A/70) try" + "; info2image4tpu-try-by-manoeuvre2writeimage4tpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4tpu = queue2writemanoeuvre4tpu.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4tpu-got-by-manoeuvre2writeimage4tpu: " + str(info2image4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # ..//..//..//..//..//web//images//png//dragonxi.png
            if info2image4tpu is None:
                code2nack  = code2nack + '3Z1-'
                ack2writeinfo4tpu = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2writeinfo4tpu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2writemanoeuvre4tpu-failed-by-manoeuvre2writeimage4tpu: " + str(queue2writemanoeuvre4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2writesend4tpu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2writeinfo4tpu is True:
                message2writesend4tpu = '<ack4xi>ack2imageinfo4tpu</ack4xi>'
            else:
                message2writesend4tpu = '<nack4xi>nack2imageinfo4tpu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2writesend4tpu-set-by-manoeuvre2writeimage4tpu to: " + str(message2writesend4tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <ack4xi>ack2imageinfo4tpu</ack4xi>       
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2writesend4tpu-failed-by-manoeuvre2writeimage4tpu: " + str(message2writesend4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2writemanoeuvre4tpu-calling-by-manoeuvre2writeimage4tpu: " + str(publisherchannel2writemanoeuvre4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <read-write PipeConnection, handle 412>
            message2print = "{{(Step 4D/70) passing" + "; message2writesend4tpu-passing-to-send-by-manoeuvre2writeimage4tpu: " + str(message2writesend4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2writesend4tpu-failed-by-manoeuvre2writeimage4tpu: " + str(message2writesend4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2writemanoeuvre4tpu.send(message2writesend4tpu)
            
            message2print = "{(Step 4E/33) sent" + "; message2writesend4tpu-sent-by-manoeuvre2writeimage4tpu: " + str(message2writesend4tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ack4xi>ack2imageinfo4tpu</ack4xi>          
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2writesend4tpu-failed-by-manoeuvre2writeimage4tpu: " + str(message2writesend4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4write['ACK2INFO4TPU']        = ack2writeinfo4tpu
    response2manouvre4write['CODE2ACK4TPU']        = code2ack
    response2manouvre4write['CODE2NACK4TPU']       = code2nack
    response2manouvre4write['MESSAGE2GET4TPU']     = message2writeget4tpu
    response2manouvre4write['MESSAGE2PUT4TPU']     = message2writeput4tpu
    response2manouvre4write['MESSAGE2RECEIVE4TPU'] = message2writereceive4tpu
    response2manouvre4write['MESSAGE2SEND4TPU']    = message2writesend4tpu
    response2manouvre4write['STATE2MANOEUVRE4TPU'] = state2write4tpu
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4write-returning-by-manoeuvre2writeimage4tpu: " + str(response2manouvre4write) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1
    # {'MESSAGE2SEND4TPU': '<roger4xi>roger2write4tpu</roger4xi>', 
    #  'ACK2INFO4TPU': False, 'MESSAGE2GET4TPU': '<default4xi>default2write4tpu</default4xi>', 
    #  'MESSAGE2RECEIVE4TPU': '<copy4xi>copy2write4tpu</copy4xi>', 
    #  'CODE2ACK4TPU': ' 0D-1C-', 
    #  'MESSAGE2PUT4TPU': '<default4xi>default2write4tpu</default4xi>', 
    #  'CODE2NACK4TPU': ' ', 
    #  'STATE2MANOEUVRE4TPU': 1}; time: 00:11:53 10/03/11 Pacific Daylight Time}
    #2
    # {'MESSAGE2SEND4TPU': '<ready4xi>ready2write4tpu</ready4xi>', 
    #  'ACK2INFO4TPU': False, 'MESSAGE2GET4TPU': '<default4xi>default2write4tpu</default4xi>', 
    #  'MESSAGE2RECEIVE4TPU': '<ready4xi>ready2write4tpu</ready4xi>', 
    #  'CODE2ACK4TPU': ' 0D-2C-', 
    #  'MESSAGE2PUT4TPU': '<default4xi>default2write4tpu</default4xi>', 
    #  'CODE2NACK4TPU': ' ', 
    #  'STATE2MANOEUVRE4TPU': 2}; time: 00:11:53 10/03/11 Pacific Daylight Time}
    #3
    # {'MESSAGE2SEND4TPU': '<ack4xi>ack2imageinfo4tpu</ack4xi>', 
    #  'ACK2INFO4TPU': True, 'MESSAGE2GET4TPU': '<default4xi>default2write4tpu</default4xi>', 
    #  'MESSAGE2RECEIVE4TPU': '<ready4xi>ready2write4tpu</ready4xi>', 
    #  'CODE2ACK4TPU': ' 0D-3C-4E-', 
    #  'MESSAGE2PUT4TPU': '<default4xi>default2write4tpu</default4xi>', 
    #  'CODE2NACK4TPU': ' ', 
    #  'STATE2MANOEUVRE4TPU': 3}; time: 00:11:53 10/03/11 Pacific Daylight Time}

    return response2manouvre4write           
          
#</function4xi>

# -------------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2mapimage4tpu" usage = "control function for map2image4tpu function in this module">

# ------------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2mapimage4tpu(queue2mapmanoeuvre4tpu, consumerchannel2mapmanoeuvre4tpu, publisherchannel2mapmanoeuvre4tpu):
    '''
    manoeuvres control and command
    '''
    global state2map4tpu                                                       # STATE FOR COMMAND AND CONTROL
    global max2mapstate4tpu                                                    # MAX FOR STATE
    global ack2mapinfo4tpu                                                     # ACK FOR IMAGE INFO
    global message2mapget4tpu                                 
    global message2mapput4tpu
    global message2mapreceive4tpu
    global message2mapsend4tpu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2mapimage4tpu" 
    method2print = "manoeuvre2mapimage4tpu" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2mapimage4tpu-started: " + str(manoeuvre2mapimage4tpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                      
    #  
    
    message2print = "{(Step 0B/33) passed" + "; queue2mapmanoeuvre4tpu-to-manoeuvre2mapimage4tpu: " + str(queue2mapmanoeuvre4tpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2mapmanoeuvre4tpu-to-manoeuvre2mapimage4tpu: " + str(consumerchannel2mapmanoeuvre4tpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2mapmanoeuvre4tpu-to-manoeuvre2mapimage4tpu: " + str(publisherchannel2mapmanoeuvre4tpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4map = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2map4tpu-try-by-manoeuvre2mapimage4tpu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2map4tpu is None:
            state2map4tpu = 0
            message2print = "{(Step 0F/33) initialized" + "; state2map4tpu-initialized-by-manoeuvre2mapimage4tpu to: " + str(state2map4tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2map4tpu-used-by-manoeuvre2mapimage4tpu: " + str(state2map4tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2map4tpu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2map4tpu-initialized-by-manoeuvre2mapimage4tpu to: " + str(state2map4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2mapinfo4tpu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2mapinfo4tpu-initialized-by-manoeuvre2mapimage4tpu to: " + str(ack2mapinfo4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2mapget4tpu     = '<default4xi>default2map4tpu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2mapget4tpu-initialized-by-manoeuvre2mapimage4tpu to: " + str(message2mapget4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2map4tpu</default4xi>
        message2mapput4tpu     = '<default4xi>default2map4tpu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2mapput4tpu-initialized-by-manoeuvre2mapimage4tpu to: " + str(message2mapput4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4tpu</default4xi>
        message2mapreceive4tpu = '<default4xi>default2map4tpu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2mapreceive4tpu-initialized-by-manoeuvre2mapimage4tpu to: " + str(message2mapreceive4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4tpu</default4xi>
        message2mapsend4tpu    = '<default4xi>default2map4tpu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2mapsend4tpu-initialized-by-manoeuvre2mapimage4tpu to: " + str(message2mapsend4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4tpu</default4xi>
        max2mapstate4tpu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2mapstate4tpu-initialized-by-manoeuvre2mapimage4tpu to: " + str( max2mapstate4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2map4tpu = state2map4tpu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2map4tpu-added-by-manoeuvre2mapimage4tpu: " + str(state2map4tpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4  
    if state2map4tpu > max2mapstate4tpu:
        state2map4tpu = 0
        message2print = "{(Step 0I/33) set" + "; state2map4tpu-set-by-manoeuvre2mapimage4tpu to: " + str(state2map4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2mapinfo4tpu        = False
        message2print = "{(Step 0J/33) set" + "; ack2mapinfo4tpu-set-by-manoeuvre2mapimage4tpu to: " + str(ack2mapinfo4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2mapget4tpu     = '<default4xi>default2map4tpu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2mapget4tpu-set-by-manoeuvre2mapimage4tpu to: " + str(message2mapget4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2map4tpu</default4xi>
        message2mapput4tpu     = '<default4xi>default2map4tpu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2mapput4tpu-set-by-manoeuvre2mapimage4tpu to: " + str(message2mapput4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4tpu</default4xi>
        message2mapreceive4tpu = '<default4xi>default2map4tpu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2mapreceive4tpu-set-by-manoeuvre2mapimage4tpu to: " + str(message2mapreceive4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4tpu</default4xi>
        message2mapsend4tpu    = '<default4xi>default2map4tpu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2mapsend4tpu-set-by-manoeuvre2mapimage4tpu to: " + str(message2mapsend4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4tpu</default4xi>
        max2mapstate4tpu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2mapstate4tpu-set-by-manoeuvre2mapimage4tpu to: " + str( max2mapstate4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2map4tpu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2mapimage4tpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2mapreceive4tpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2mapreceive4tpu = consumerchannel2mapmanoeuvre4tpu.recv()
            message2print = "{(Step 1B/33) received" + "; message2mapreceive4tpu-received-by-manoeuvre2mapimage4tpu: " + str(message2mapreceive4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2mapreceive4tpu == '<copy4xi>copy2mapimage4tpu</copy4xi>':
                message2mapsend4tpu = '<roger4xi>roger2mapimage4tpu</roger4xi>'
            else:
                message2mapsend4tpu = '<negative4xi><negative2mapimage4tpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2mapmanoeuvre4tpu.send(message2mapsend4tpu)
            
            message2print = "{(Step 1C/33) sent" + "; message2mapsend4tpu-sent-by-manoeuvre2mapimage4tpu: " + str(message2mapsend4tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2mapreceive4tpu-failed-by-manoeuvre2mapimage4tpu: " + str(message2mapreceive4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2map4tpu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2mapimage4tpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2mapreceive4tpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2mapreceive4tpu = consumerchannel2mapmanoeuvre4tpu.recv()
            message2print = "{(Step 2B/33) received" + "; message2mapreceive4tpu-received-by-manoeuvre2mapimage4tpu: " + str(message2mapreceive4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2mapreceive4tpu == '<ready4xi>ready2mapimage4tpu</ready4xi>':
                message2mapsend4tpu = '<ready4xi>ready2mapimage4tpu</ready4xi>'
            else:
                message2mapsend4tpu = '<negative4xi><negative2ready4tpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2mapmanoeuvre4tpu.send(message2mapsend4tpu)
            
            message2print = "{(Step 2C/33) sent" + "; message2mapsend4tpu-sent-by-manoeuvre2mapimage4tpu: " + str(message2mapsend4tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2mapreceive4tpu-failed-by-manoeuvre2mapimage4tpu: " + str(message2mapreceive4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2map4tpu == 3:    
        info2image4tpu = None
        message2print = "{(Step 3A/70) try" + "; info2image4tpu-try-by-manoeuvre2mapimage4tpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4tpu = queue2mapmanoeuvre4tpu.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4tpu-got-by-manoeuvre2mapimage4tpu: " + str(info2image4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
            if info2image4tpu is None:
                code2nack  = code2nack + '3Z1-'
                ack2mapinfo4tpu = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2mapinfo4tpu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2mapmanoeuvre4tpu-failed-by-manoeuvre2mapimage4tpu: " + str(queue2mapmanoeuvre4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2mapsend4tpu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2mapinfo4tpu is True:
                message2mapsend4tpu = '<ack4xi>ack2imageinfo4tpu</ack4xi>'
            else:
                message2mapsend4tpu = '<nack4xi>nack2imageinfo4tpu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2mapsend4tpu-set-by-manoeuvre2mapimage4tpu to: " + str(message2mapsend4tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #      
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2mapsend4tpu-failed-by-manoeuvre2mapimage4tpu: " + str(message2mapsend4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2mapmanoeuvre4tpu-calling-by-manoeuvre2mapimage4tpu: " + str(publisherchannel2mapmanoeuvre4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # 
            message2print = "{{(Step 4D/70) passing" + "; message2mapsend4tpu-passing-to-send-by-manoeuvre2mapimage4tpu: " + str(message2mapsend4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2mapsend4tpu-failed-by-manoeuvre2mapimage4tpu: " + str(message2mapsend4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2mapmanoeuvre4tpu.send(message2mapsend4tpu)
            
            message2print = "{(Step 4E/33) sent" + "; message2mapsend4tpu-sent-by-manoeuvre2mapimage4tpu: " + str(message2mapsend4tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            #          
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2mapsend4tpu-failed-by-manoeuvre2mapimage4tpu: " + str(message2mapsend4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4map['ACK2INFO4TPU']        = ack2mapinfo4tpu
    response2manouvre4map['CODE2ACK4TPU']        = code2ack
    response2manouvre4map['CODE2NACK4TPU']       = code2nack
    response2manouvre4map['MESSAGE2GET4TPU']     = message2mapget4tpu
    response2manouvre4map['MESSAGE2PUT4TPU']     = message2mapput4tpu
    response2manouvre4map['MESSAGE2RECEIVE4TPU'] = message2mapreceive4tpu
    response2manouvre4map['MESSAGE2SEND4TPU']    = message2mapsend4tpu
    response2manouvre4map['STATE2MANOEUVRE4TPU'] = state2map4tpu
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4map-returning-by-manoeuvre2mapimage4tpu: " + str(response2manouvre4map) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 {'MESSAGE2SEND4TPU': '<roger4xi>roger2mapimage4tpu</roger4xi>', 
    #  'ACK2INFO4TPU': False, 
    #  'MESSAGE2GET4TPU': '<default4xi>default2map4tpu</default4xi>', 
    #  'MESSAGE2RECEIVE4TPU': '<copy4xi>copy2mapimage4tpu</copy4xi>', 
    #  'CODE2ACK4TPU': ' 0D-1C-', 
    #  'MESSAGE2PUT4TPU': '<default4xi>default2map4tpu</default4xi>', 
    #  'CODE2NACK4TPU': ' ', 
    #  'STATE2MANOEUVRE4TPU': 1}
     
    #2 {'MESSAGE2SEND4TPU': '<ready4xi>ready2mapimage4tpu</ready4xi>', 
    #  'ACK2INFO4TPU': False, 
    #  'MESSAGE2GET4TPU': '<default4xi>default2map4tpu</default4xi>', 
    #  'MESSAGE2RECEIVE4TPU': '<ready4xi>ready2mapimage4tpu</ready4xi>', 
    #  'CODE2ACK4TPU': ' 0D-2C-', 
    #  'MESSAGE2PUT4TPU': '<default4xi>default2map4tpu</default4xi>', 
    #  'CODE2NACK4TPU': ' ', 
    #  'STATE2MANOEUVRE4TPU': 2}; time: 06:49:48 10/04/11 Pacific Daylight Time}
     
    #3 {'MESSAGE2SEND4TPU': '<ack4xi>ack2imageinfo4tpu</ack4xi>', 
    #  'ACK2INFO4TPU': True, 
    #  'MESSAGE2GET4TPU': '<default4xi>default2map4tpu</default4xi>', 
    #  'MESSAGE2RECEIVE4TPU': '<ready4xi>ready2mapimage4tpu</ready4xi>', 
    #  'CODE2ACK4TPU': ' 0D-3C-4E-', 
    #  'MESSAGE2PUT4TPU': '<default4xi>default2map4tpu</default4xi>', 
    #  'CODE2NACK4TPU': ' ', 
    #  'STATE2MANOEUVRE4TPU': 3}; time: 06:49:48 10/04/11 Pacific Daylight Time}
    
    #4 {'MESSAGE2SEND4TPU': '<default4xi>default2map4tpu</default4xi>', 
    #  'ACK2INFO4TPU': False, 
    #  'MESSAGE2GET4TPU': '<default4xi>default2map4tpu</default4xi>', 
    #  'MESSAGE2RECEIVE4TPU': '<default4xi>default2map4tpu</default4xi>', 
    #  'CODE2ACK4TPU': ' 0D-', 
    #  'MESSAGE2PUT4TPU': '<default4xi>default2map4tpu</default4xi>', 
    #  'CODE2NACK4TPU': ' ', 'STATE2MANOEUVRE4TPU': 0}; time: 06:49:48 10/04/11 Pacific Daylight Time}

    return response2manouvre4map           
          
#</function4xi>

# -------------------------------------------------------------------------------------------------
#<function4xi function = "write2image4tpu" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   # (16) ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4TPU                      
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 16A/700) OF TPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4TPU ', time4xi())    
#   respon4write2image = {} 
#   task4write2image = None
#   try:                               
#       response4write2image = write2image4tpu(task4write2image)                                                                                                          
#       print('(Step 16B/700) OF TPU2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH READ2IMAGE4TPU : ', response4write2image)
#       queue2write4tpu      = response4write2image['QUEUE']
#       channel2publish4tpu = response4write2image['PUBLISHERCHANNEL']
#       channel2consume4tpu = response4write2image['CONSUMERCHANNEL']
#   except:
#       print('(Step 16Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4TPU ', time4xi()) 
#   # --------------------------------------------------------------------------------------------------
#   # (17) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2IMAGE4TPU TO RECEIVE                 
#   # --------------------------------------------------------------------------------------------------      
#   print('(Step 17A/700) OF TPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4TPU ', time4xi())     
#   try:                               
#       message2writesend4tpu = "<copy4xi>copy2write4tpu</copy4xi>"
#       channel2publish4tpu = response4write2image['PUBLISHERCHANNEL']
#       channel2publish4tpu.send(message2writesend4tpu)
#       print("(Step 17B/700) OF TPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4TPU: ", channel2publish4tpu, " MESSAGE2SEND4TPU: " , message2writesend4tpu)                                                                                                
#   except:
#       print('(Step 17Z1/700) OF TPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4TPU ', time4xi())                                                
#   # ------------------------------------------------------------------------------------------
#   # (18) LET MANOEUVRE2IMAGE4TPU TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # ------------------------------------------------------------------------------------------       
#   print("(Step 18C/700) OF TPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4TPU ", time4xi()) 
#   try:        
#       response2manoeuvre4tpu = manoeuvre2readimage4tpu(queue2write4tpu, channel2consume4tpu, channel2publish4tpu)
#       print("(Step 18D/700) OF TPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4TPU ", manoeuvre2readimage4tpu)
#   except:
#       print("(Step 18Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4TPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # ---------------------------------------------------------------------------------------------
#   # (19) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 19A/700) OF TPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4TPU ', time4xi())     
#   try:
#       channel2consume4tpu = response4write2image['CONSUMERCHANNEL']                              
#       message2writereceive4tpu = channel2consume4tpu.recv()
#       print("(Step 19B/700) OF TPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4TPU: ", channel2consume4tpu, " CONTROL & COMMAND: ", message2writereceive4tpu)                                                                                                  
#       # <roger4xi>roger2write4tpu</roger4xi>
#   except:
#       print('(Step 19Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH READ2IMAGE4TPU ', time4xi())         
#   # ---------------------------------------------------------------------------------------------------
#   # (20) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2IMAGE4TPU TO RECEIVE                
#   # ---------------------------------------------------------------------------------------------------      
#   print('(Step 20A/700) OF TPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4TPU ', time4xi())     
#   try:                               
#       message2writesend4tpu = "<write4xi>write2write4tpu</write4xi>"
#       channel2publish4tpu = response4write2image['PUBLISHERCHANNEL']
#       channel2publish4tpu.send(message2writesend4tpu)
#       print("(Step 20B/700) OF TPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4TPU: ", channel2publish4tpu, " MESSAGE2SEND4TPU: " , message2writesend4tpu)                                                                                                
#       # <write4xi>write2write4tpu</write4xi> 
#   except:
#        print('(Step 20Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4TPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------
#   # (21) LET MANOEUVRE2IMAGE4TPU TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------       
#   print("(Step 21A/700) OF TPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4TPU ", time4xi()) 
#   try:        
#       response2manoeuvre4tpu = manoeuvre2readimage4tpu(queue2write4tpu, channel2consume4tpu, channel2publish4tpu)
#       print("(Step 21B/700) OF TPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4TPU ", manoeuvre2readimage4tpu)
#   except:
#       print("(Step 21Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4TPU ", time4xi())     
#   # ---------------------------------------------------------------------------------------
#   # (22) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 22A/700) OF TPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4TPU ', time4xi())     
#   try:
#       channel2consume4tpu = response4write2image['CONSUMERCHANNEL']                              
#       message2writereceive4tpu = channel2consume4tpu.recv()
#       print("(Step 22B/700) OF TPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4TPU: ", channel2consume4tpu, " CONTROL & COMMAND: ", message2writereceive4tpu)                                                                                                  
#       # <write4xi>write2write4tpu</write4xi> 
#   except:
#       print('(Step 22Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH READ2IMAGE4TPU ', time4xi())          
#   # -----------------------------------------------------------------------------------------
#   # (24) PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 24A/700) OF TPU2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue2write4dragon.put(path2dragon)             
#       print("(Step 24B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#       # ..//..//..//..//..//web//images//png//dragonxi.png  
#   except:
#       print("(Step 24Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # ------------------------------------------------------------------------------------------------------
#   # (25) LET MANOEUVRE2IMAGE4TPU TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 25A/700) OF TPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4TPU ", time4xi()) 
#   try:        
#       response2manoeuvre4tpu = manoeuvre2readimage4tpu(queue2write4tpu, channel2consume4tpu, channel2publish4tpu)
#       print("(Step 25B/700) OF TPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4TPU ", manoeuvre2readimage4tpu)
#   except:
#       print("(Step 25Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4TPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (26) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 26A/700) OF TPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2writereceive4tpu = channel2consume4tpu.recv()
#       print("(Step 26D/700) OF TPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4TPU: ", channel2consume4tpu, " CONTROL & COMMAND: ", message2writereceive4tpu)                                                                                                   
#       # <ack4xi>ack2imageinfo4tpu</ack4xi>
#   except:
#       print('(Step 26Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -------------------------------------------------------------------------------------------------------
#   # (27) LET MANOEUVRE2IMAGE4TPU GO TO STATE 0               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 27A/700) OF TPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4TPU ", time4xi()) 
#   try:        
#       response2manoeuvre4tpu = manoeuvre2readimage4tpu(queue2write4tpu, channel2consume4tpu, channel2publish4tpu)
#       print("(Step 27B/700) OF TPU2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4TPU ", response2manoeuvre4tpu)
#   except:
#       print("(Step 27Z/700) OF TPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4TPU ", time4xi())   

#">

# -------------------------------------------------------------------------------------------------   
def write2image4tpu(task4imagewrite2tpu=None):

    '''
    writes 2D images for TPU
    '''
    global tpu2image4xi                                                     # 2D-IMAGES FOR TPU  
    global control2writeimage4tpu                                              # COMMAND AND CONTROL FOR TPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS        
    # -----------------------------------------------------------------------------------------          
    class2print = "write2image4tpu"
    method2print = "write2image4tpu"

    message2print = "{(Step 1A/33) started" + "; write2image4tpu-started: " + str(write2image4tpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function write2image4tpu at 0x00000000038E8EC8>
    
    message2print = "{(Step 1B/33) passed" + "; task4imagewrite2tpu-to-write2image4tpu: " + str(task4imagewrite2tpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE TPU2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if tpu2image4xi is None:
            tpu2image4xi = create2image4tpu()                                                                              # INITIALIZE IMAGE FOR TPU 
            message2print = "{(Step 1C/70) created" + "; tpu2image4xi-created-by-write2image4tpu to: " + str(tpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4tpu-skipped-by-write2image4tpu: " + str(write2image4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # create2image4tpu-skipped-by-write2image4tpu: <function write2image4tpu at 0x00000000038E8EC8>                                 
    except:
        try:
            tpu2image4xi = create2image4tpu()
            message2print = "{(Step 1E/70) set" + "; tpu2image4xi-set-by-write2image4tpu to: " + str(tpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            tpu2image4xi = {}
            message2print = "{(Step 1Z/70) yielded" + "; tpu2image4xi-yielded-by-write2image4tpu: " + str(tpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z-' 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    try:             
        if task4imagewrite2tpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR READ - BUILD TASK! 
            # -----------------------------------------------------------------------------------------    
            
            task4imagewrite2tpu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1A/70) initialized" + "; task4imagewrite2tpu-created-by-write2image4tpu to: " + str(task4imagewrite2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
            
            # ---------------------------------------------------------------------------------------------
            # (1B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4tpu = trace2image4tpu                                                                 # CALLBACK FUNCTION FOR TPU
            
            message2print = "{(Step 1B1/70) initialized" + "; function2callback4tpu-created-by-write2image4tpu to: " + str(function2callback4tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # <function trace2image4tpu at 0x00000000038E2A48>
            
            code2ack  = code2ack + '1B1-'
           
            args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "write2image4tpu"                                                            # MODULE OF TPUIZED OPENCL MODULE
            args2callback['METHOD']  = "write2image4tpu"                                                            # FUNCTION OF TPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; write2image4tpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR TPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-write2image4tpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {'MESSAGE': '{(Step 1Z1/33) failed; write2image4tpu-failed ; time: 02:32:29 10/05/11 Pacific Daylight Time}', 
            #  'CLASS': 'write2image4tpu', 
            #  'METHOD': 'write2image4tpu'}; time: 02:32:29 10/05/11 Pacific Daylight Time}

            code2ack  = code2ack + '1B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (1C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagewrite2tpu['DATA2CALLBACK'] = args2callback                                                    # USER DATA PASSED TO TPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagewrite2tpu['FUNCTION2CALLBACK'] = function2callback4tpu                                        # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '1C-'
            message2print = "{(Step 1C/70) initialized" + "; task4imagewrite2tpu-created-by-write2image4tpu to: " + str(task4imagewrite2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # {'FUNCTION2CALLBACK': <function trace2image4tpu at 0x00000000038E2A48>,
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; write2image4tpu-failed ; time: 02:32:29 10/05/11 Pacific Daylight Time}',
            #  'CLASS': 'write2image4tpu',
            #  'METHOD': 'write2image4tpu'}}; time: 02:32:29 10/05/11 Pacific Daylight Time}

            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-write2image4tpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # {}
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-write2image4tpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 02:32:29 10/05/11 Pacific Daylight Time}

                code2ack  = code2ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-write2image4tpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '2Z-'
                 
            task4imagewrite2tpu['PLATFORM'] = platform2xi                                                           # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2tpu = {}
            message2print = "{(Step 3A/70) trying" + "; context2tpu-trying-by-write2image4tpu: " + str(context2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2tpu = get2context4tpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context2tpu-got-by-write2image4tpu: " + str(context2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4tpu-failed ; time: 02:32:29 10/05/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2context4tpu', ...
   
                code2ack  = code2ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context2tpu-failed-by-write2image4tpu: " + str(context2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '3Z-'
                
            task4imagewrite2tpu ['CONTEXT'] = context2tpu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2tpu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device2tpu-trying-by-write2image4tpu: " + str(device2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2tpu = get2tpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device2tpu-got-by-write2image4tpu: " + str(device2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568' ...
               
                code2ack  = code2ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device2tpu-failed-by-write2image4tpu: " + str(device2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '4Z-'
            
            task4imagewrite2tpu ['DEVICE'] = device2tpu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2tpu = {}
             
            message2print = "{(Step 5A/70) trying" + "; imageinfo2tpu-trying-by-write2image4tpu: " + str(imageinfo2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2tpu = get2imageinfo4tpu()                                                                         
                
                message2print = "{(Step 5B/70) got" + "; imageinfo2tpu-got-by-write2image4tpu: " + str(imageinfo2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'SIZE2QUEUE': 2147483646, 
                #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
                #  'ORDER': 'CL_RGBA'}, 
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004494E10>, 
                #  'WIDTH': 800, 
                #  'CONTEXT': {'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4tpu-failed ; time: 02:32:29 10/05/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2context4tpu', 
                #  'METHOD': 'create2context4tpu'}, 
                #  'FUNCTION2CALLBACK': <function trace2context4tpu at 0x000000000387E348>, 
                #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 
                #  'INFO2PRIVATE': None, 
                #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
                #  'NACK2RETURN': ' ', 
                #  'SIZE2PRIVATE': 0, 
                #  'PROPERTIES': None}, 
                #  'PIPE': (<read-write PipeConnection, handle 520>, <read-write PipeConnection, handle 524>), 
                #  'COUNT2REFERENCE': 1, 
                #  'FUNCTION2CALLBACK': <function trace2image4tpu at 0x00000000038E2A48>, 
                #  'COMMANDQUEUE': ...

                code2ack  = code2ack + '5B-'  
            except:
                message2print = "{(Step 5Z/70) failed" + "; imageinfo2tpu-failed-by-write2image4tpu: " + str(imageinfo2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '5Z-'          
                      
            task4imagewrite2tpu ['IMAGE'] = imageinfo2tpu         
                        

            # ---------------------------------------------------------------------------------------------
            # (6) QUEUE - REQUIRED BY TPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2tpu = None
            
            message2print = "{(Step 6A/70) trying" + "; queue2tpu-trying-by-write2image4tpu: " + str(queue2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2tpu = Queue4Xi()                                                                              # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 6B/70) got" + "; queue2tpu-got-by-write2image4tpu: " + str(queue2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # <queue4xi.Queue4Xi object at 0x0000000003D4FA90>
                
                code2ack  = code2ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; queue2tpu-failed-by-write2image4tpu: " + str(queue2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '6Z-'
                
            task4imagewrite2tpu ['QUEUE'] = queue2tpu

            
            # ---------------------------------------------------------------------------------------------
            # (7) PIPE - REQUIRED BY TPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2tpu = None
           
            message2print = "{(Step 7A/70) trying" + "; pipe2tpu-trying-by-write2image4tpu: " + str(pipe2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2tpu = Pipe4Xi()                                                                                # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 7B/70) got" + "; pipe2tpu-got-by-write2image4tpu: " + str(pipe2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # (<read-write PipeConnection, handle 568>, <read-write PipeConnection, handle 572>)
                
                code2ack  = code2ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; pipe2tpu-failed-by-write2image4tpu: " + str(pipe2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '7Z-'
                
            task4imagewrite2tpu ['PIPE'] = pipe2tpu 
            
            # ---------------------------------------------------------------------------------------------
            # (8) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2tpu = {}
           
            message2print = "{(Step 8A/70) trying" + "; commandqueueinfo2tpu-trying-by-write2image4tpu: " + str(commandqueueinfo2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2tpu = get2commandqueueinfo4tpu()                                                   # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 8B/70) got" + "; commandqueueinfo2tpu-got-by-write2image4tpu: " + str(commandqueueinfo2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 
                #  'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4tpu-failed ; time: 02:32:29 10/05/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2commandqueue4tpu', 
                #  'METHOD': 'create2commandqueue4tpu'}, 
                #  'FUNCTION2CALLBACK': <function trace2commandqueue4tpu at 0x000000000388DA48> ...
                
                code2ack  = code2ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; commandqueueinfo2tpu-failed-by-write2image4tpu: " + str(commandqueueinfo2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '8Z-'
        
            task4imagewrite2tpu ['COMMANDQUEUE'] = commandqueueinfo2tpu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (9) BUFFER - AUTOMATIC BY TPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2tpu = {}
           
            message2print = "{(Step 9A/70) trying" + "; bufferinfo2tpu-trying-by-write2image4tpu: " + str(bufferinfo2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2tpu = get2bufferinfo4tpu()                                                               # SYSTEM CREATES BUFFER FOR TPU, IF NECESSARY
                message2print = "{(Step 9B/70) got" + "; bufferinfo2tpu-got-by-write2image4tpu: " + str(bufferinfo2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 
                #  'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4tpu-failed ; time: 02:32:28 10/05/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2buffer4tpu', 
                #  'METHOD': 'create2buffer4tpu'}, ...
                
                code2ack  = code2ack + '9B-' 
            except:
                message2print = "{(Step 9Z/70) failed" + "; bufferinfo2tpu-failed-by-write2image4tpu: " + str(bufferinfo2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '9Z-'
        
            task4imagewrite2tpu ['BUFFER'] = bufferinfo2tpu 
            
            # ---------------------------------------------------------------------------------------------
            # (10) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2tpu = {}
           
            message2print = "{(Step 10A/70) trying" + "; eventinfo2tpu-trying-by-write2image4tpu: " + str(eventinfo2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2tpu = get2eventinfo4tpu()                                                                    
                message2print = "{(Step 10B/70) got" + "; eventinfo2tpu-got-by-write2image4tpu: " + str(eventinfo2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #  {'CODE2ERROR': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4tpu-failed ; time: 02:32:28 10/05/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2event4tpu', 
                #  'METHOD': 'create2event4tpu'}, 
                #  'FUNCTION2CALLBACK': <function trace2event4tpu at 0x00000000038A3F48>, ...
                
                code2ack  = code2ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; eventinfo2tpu-failed-by-write2image4tpu: " + str(eventinfo2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '10Z-'
        
            task4imagewrite2tpu ['EVENT'] = eventinfo2tpu 
            
            
            # ---------------------------------------------------------------------------------------------
            # (11) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2tpu = {}
           
            message2print = "{(Step 11A/70) trying" + "; eventwaitlist2tpu-trying-by-write2image4tpu: " + str(eventwaitlist2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2tpu = {}                                                                   
                message2print = "{(Step 11B/70) set" + "; eventwaitlist2tpu-set-by-write2image4tpu: " + str(eventwaitlist2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {} 
                
                code2ack  = code2ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; eventwaitlist2tpu-failed-by-write2image4tpu: " + str(eventwaitlist2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '11Z-'
        
            task4imagewrite2tpu ['EVENTWAITLIST'] = eventwaitlist2tpu                     

            
            # ---------------------------------------------------------------------------------------------
            # (12) EVENTWAITLISTQUANTITY - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2tpu = 0
           
            message2print = "{(Step 12A/70) trying" + "; eventwaitlistquantity2tpu-trying-by-write2image4tpu: " + str(eventwaitlistquantity2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2tpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2tpu = eventwaitlistquantity2tpu + 1                                                                   
                message2print = "{(Step 12B/70) set" + "; eventwaitlistquantity2tpu-set-by-write2image4tpu: " + str(eventwaitlistquantity2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 
                
                code2ack  = code2ack + '12B-' 
            except:
                message2print = "{(Step 12Z/70) failed" + "; eventwaitlistquantity2tpu-failed-by-write2image4tpu: " + str(eventwaitlistquantity2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '12Z-'
        
            task4imagewrite2tpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2tpu       
                                                 
            
            # ---------------------------------------------------------------------------------------------
            # (13) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2tpu = ' '                                                                   
     
            message2print = "{(Step 13A/70) trying" + "; blocking2tpu-trying-by-write2image4tpu: " + str(blocking2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2tpu = 'CL_FALSE'                                                                           # the write command is non-blocking                                                              
                message2print = "{(Step 13B/70) set" + "; blocking2tpu-set-by-write2image4tpu to: " + str(blocking2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # CL_FALSE 
                code2ack  = code2ack + '13B-' 
            except:
                message2print = "{(Step 13Z/70) failed" + "; blocking2tpu-failed-by-write2image4tpu: " + str(blocking2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '13Z-'
        
            task4imagewrite2tpu ['BLOCKING'] = blocking2tpu             
            
            # ---------------------------------------------------------------------------------------------
            # (14) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2tpu = {}
            
            message2print = "{(Step 14A/70) trying" + "; origin2tpu-trying-by-write2image4tpu: " + str(origin2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2tpu['X'] = 0 
                origin2tpu['Y'] = 0  
                origin2tpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 14B/70) set" + "; origin2tpu-set-by-write2image4tpu to: " + str(origin2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'Y': 0, 'X': 0, 'Z': 0} 
                code2ack  = code2ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; origin2tpu-failed-by-write2image4tpu: " + str(origin2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '14Z-'
        
            task4imagewrite2tpu ['ORIGIN'] = origin2tpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (15) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2tpu = {}
            
            message2print = "{(Step 15A/70) trying" + "; region2tpu-trying-by-write2image4tpu: " + str(region2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2tpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO READ 
                region2tpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO READ 
                region2tpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO READ                                                                                                                          
                message2print = "{(Step 15B/70) set" + "; region2tpu-set-by-write2image4tpu to: " + str(region2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'Y': 90, 'X': 160, 'Z': 0}
                code2ack  = code2ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; region2tpu-failed-by-write2image4tpu: " + str(region2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '15Z-'
        
            task4imagewrite2tpu ['REGION'] = region2tpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (16) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.

            height2tpu = 800
            width2tpu = 800
            rowpitch2tpu = 0
            size2element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 1                                                                                  # ONE DIRECTIONS IN 2D
            
            message2print = "{(Step 16A/70) trying" + "; rowpitch2tpu-trying-by-write2image4tpu: " + str(rowpitch2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            #  
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2tpu = int(width2tpu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 16B/70) set" + "; rowpitch2tpu-set-by-write2image4tpu to: " + str(rowpitch2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 800                                                                                            
                 
                code2ack  = code2ack + '16B-' 
            except:
                message2print = "{(Step 16Z/70) failed" + "; rowpitch2tpu-failed-by-write2image4tpu: " + str(rowpitch2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '16Z-'
        
            task4imagewrite2tpu ['ROWPITCH'] = rowpitch2tpu     
            
            # ----------------------------------------------------------------------------------------------------
            # (17) POINTER - REQUIRED - THE POINTER TO A BUFFER IN HOST MEMORY WHERE IMAGE DATA IS TO BE READ FROM                          
            # ----------------------------------------------------------------------------------------------------             
            pointer2tpu = None
            size2tpu = int(width2tpu * height2tpu)                                                                  # CALCULATED MEMORY
            if size2tpu == 0:
                size2tpu = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 17A/70) trying" + "; pointer2tpu-trying-by-write2image4tpu: " + str(pointer2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena2tpu = Arena4Xi(size2tpu)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 17B/70) got" + "; arena2tpu-got-by-write2image4tpu: " + str(arena2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # <arena4xi.Arena4Xi object at 0x00000000041348D0>
                
                size2arena = arena2tpu.size                                                                         # 
                              
                message2print = "{(Step 17C/70) got" + "; size2arena-got-by-write2buffer4tpu: " + str(size2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 640000 
                
                buffer2arena = arena2tpu.buffer
                
                message2print = "{(Step 17D/70) got" + "; buffer2arena-got-by-write2buffer4tpu: " + str(buffer2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # <mmap.mmap object at 0x000000000421A450> 
                
                code2ack  = code2ack + '17D-' 
                
                if size2arena < size2tpu:
                    size2tpu = size2arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 17E/70) set" + "; size2tpu-set-by-write2buffer4tpu to: " + str(size2tpu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code2ack  = code2ack + '17E-'
                else:
                    pass
                    
                pointer2tpu = buffer2arena
                message2print = "{(Step 17F/70) set" + "; pointer2tpu-set-by-write2buffer4tpu to: " + str(pointer2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # <mmap.mmap object at 0x000000000421A450>
                
                code2ack  = code2ack + '17F-'       
            except:
                message2print = "{(Step 17Z/70) failed" + "; pointer2tpu-failed-by-write2image4tpu: " + str(pointer2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '17Z-'
        
            task4imagewrite2tpu ['POINTER'] = pointer2tpu                            
            task4imagewrite2tpu ['SIZE']    = size2tpu                                                               # AUTOMATIC BY TPUXI
            
            
            # --------------------------------------------------------------------------------------------------------
            # (18) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING READ                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2tpu = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 18A/70) trying" + "; slicepitch2tpu-trying-by-write2image4tpu: " + str(slicepitch2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2tpu = 0                                                          
                message2print = "{(Step 18B/70) set" + "; slicepitch2tpu-set-by-write2image4tpu to: " + str(slicepitch2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0

                code2ack  = code2ack + '18B-'                
                 
            except:
                message2print = "{(Step 18Z/70) failed" + "; slicepitch2tpu-failed-by-write2image4tpu: " + str(slicepitch2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '18Z-'
            
            task4imagewrite2tpu ['SLICEPITCH'] = slicepitch2tpu                  
            
            # ---------------------------------------------------------------------------------------------
            # EOF PARAMETER PROVIDING                
            # ---------------------------------------------------------------------------------------------                                                                                                                               
        else:
            pass
            
        # ---------------------------------------------------------------------------------------------
        # CHECK PARAMETERS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagewrite2tpu.keys()
        message2print = "{(Step 20/70) using" + "; keys2task-using-by-write2image4tpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'POINTER',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SIZE',
        #  'SLICEPITCH']; time: 02:45:31 10/05/11 Pacific Daylight Time}

        if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4tpu = trace2image4tpu                                                             # CALLBACK FUNCTION FOR TPU
            
            message2print = "{(Step 21B1/70) initialized" + "; function2callback4tpu-created-by-write2image4tpu to: " + str(function2callback4tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '21B1-'
           
            args2callback = {}                                                                                  # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "write2image4tpu"                                                        # MODULE OF TPUIZED OPENCL MODULE
            args2callback['METHOD']  = "write2image4tpu"                                                        # FUNCTION OF TPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; write2image4tpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR TPUIZED OPENCL TO RETURN    
            message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-write2image4tpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
    
            code2ack  = code2ack + '21B2-'
        
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagewrite2tpu['DATA2CALLBACK'] = args2callback                                                # USER DATA PASSED TO TPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagewrite2tpu['FUNCTION2CALLBACK'] = function2callback4tpu                                    # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4imagewrite2tpu-created-by-write2image4tpu to: " + str(task4imagewrite2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
             
        if 'PLATFORM' in keys2task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-write2image4tpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
                
            try:
                platform2xi = get2platformids4xi()                                                              # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-write2image4tpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                     
                code2ack  = code2ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-write2image4tpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '22Z-'
                     
            task4imagewrite2tpu['PLATFORM'] = platform2xi                                                       # PLATFORM OF SYSTEM 
              
        if 'CONTEXT' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2tpu = {}
            message2print = "{(Step 23A/70) trying" + "; context2tpu-trying-by-write2image4tpu: " + str(context2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2tpu = get2context4tpu()                                                                 # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context2tpu-got-by-write2image4tpu: " + str(context2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context2tpu-failed-by-write2image4tpu: " + str(context2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '23Z-'
                
            task4imagewrite2tpu ['CONTEXT'] = context2tpu           
        
        if 'DEVICE' in keys2task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2tpu = {}
            
            message2print = "{(Step 24A/70) trying" + "; device2tpu-trying-by-write2image4tpu: " + str(device2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2tpu = get2tpu4xi()                                                                       # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device2tpu-got-by-write2image4tpu: " + str(device2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device2tpu-failed-by-write2image4tpu: " + str(device2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '24Z-'
            
            task4imagewrite2tpu ['DEVICE'] = device2tpu
        
        if 'IMAGE' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2tpu = {}
             
            message2print = "{(Step 25A/70) trying" + "; imageinfo2tpu-trying-by-write2image4tpu: " + str(imageinfo2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2tpu = get2imageinfo4tpu()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; imageinfo2tpu-got-by-write2image4tpu: " + str(imageinfo2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; imageinfo2tpu-failed-by-write2image4tpu: " + str(imageinfo2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '25Z-'          
                      
            task4imagewrite2tpu ['IMAGE'] = imageinfo2tpu         
                    
        if 'QUEUE' in keys2task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (26) QUEUE - REQUIRED BY TPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2tpu = None
            
            message2print = "{(Step 26A/70) trying" + "; queue2tpu-trying-by-write2image4tpu: " + str(queue2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2tpu = Queue4Xi()                                                                          # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 26B/70) got" + "; queue2tpu-got-by-write2image4tpu: " + str(queue2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; queue2tpu-failed-by-write2image4tpu: " + str(queue2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '26Z-'
                
            task4imagewrite2tpu ['QUEUE'] = queue2tpu

        if 'PIPE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (27) PIPE - REQUIRED BY TPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2tpu = None
           
            message2print = "{(Step 27A/70) trying" + "; pipe2tpu-trying-by-write2image4tpu: " + str(pipe2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2tpu = Pipe4Xi()                                                                            # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 27B/70) got" + "; pipe2tpu-got-by-write2image4tpu: " + str(pipe2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; pipe2tpu-failed-by-write2image4tpu: " + str(pipe2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '27Z-'
                
            task4imagewrite2tpu ['PIPE'] = pipe2tpu 
            
        if 'COMMANDQUEUE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2tpu = {}
           
            message2print = "{(Step 28A/70) trying" + "; commandqueueinfo2tpu-trying-by-write2image4tpu: " + str(commandqueueinfo2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2tpu = get2commandqueueinfo4tpu()                                               # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 28B/70) got" + "; commandqueueinfo2tpu-got-by-write2image4tpu: " + str(commandqueueinfo2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; commandqueueinfo2tpu-failed-by-write2image4tpu: " + str(commandqueueinfo2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '28Z-'
        
            task4imagewrite2tpu ['COMMANDQUEUE'] = commandqueueinfo2tpu
            
            
        if 'BUFFER' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (29) BUFFER - AUTOMATIC BY TPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2tpu = {}
           
            message2print = "{(Step 29A/70) trying" + "; bufferinfo2tpu-trying-by-write2image4tpu: " + str(bufferinfo2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2tpu = get2bufferinfo4tpu()                                                           # SYSTEM CREATES BUFFER FOR TPU, IF NECESSARY
                message2print = "{(Step 29B/70) got" + "; bufferinfo2tpu-got-by-write2image4tpu: " + str(bufferinfo2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '29B-' 
            except:
                message2print = "{(Step 29Z/70) failed" + "; bufferinfo2tpu-failed-by-write2image4tpu: " + str(bufferinfo2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '29Z-'
        
            task4imagewrite2tpu ['BUFFER'] = bufferinfo2tpu
             
        if 'EVENT' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (30) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2tpu = {}
           
            message2print = "{(Step 30A/70) trying" + "; eventinfo2tpu-trying-by-write2image4tpu: " + str(eventinfo2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2tpu = get2eventinfo4tpu()                                                                    
                message2print = "{(Step 30B/70) got" + "; eventinfo2tpu-got-by-write2image4tpu: " + str(eventinfo2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '30B-' 
            except:
                message2print = "{(Step 30Z/70) failed" + "; eventinfo2tpu-failed-by-write2image4tpu: " + str(eventinfo2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '30Z-'
        
            task4imagewrite2tpu ['EVENT'] = eventinfo2tpu 
        
             
        if 'EVENTWAITLIST' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2tpu = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventwaitlist2tpu-trying-by-write2image4tpu: " + str(eventwaitlist2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2tpu = {}                                                                   
                message2print = "{(Step 31B/70) set" + "; eventwaitlist2tpu-set-by-write2image4tpu: " + str(eventwaitlist2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventwaitlist2tpu-failed-by-write2image4tpu: " + str(eventwaitlist2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '31Z-'
        
            task4imagewrite2tpu ['EVENTWAITLIST'] = eventwaitlist2tpu                     

                 
        if 'EVENTWAITLISTQUANTITY' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLISTQUANTITY - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2tpu = 0
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlistquantity2tpu-trying-by-write2image4tpu: " + str(eventwaitlistquantity2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2tpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2tpu = eventwaitlistquantity2tpu + 1                                                                     
                message2print = "{(Step 32B/70) set" + "; eventwaitlistquantity2tpu-set-by-write2image4tpu: " + str(eventwaitlistquantity2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlistquantity2tpu-failed-by-write2image4tpu: " + str(eventwaitlistquantity2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '32Z-'
        
            task4imagewrite2tpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2tpu       
                                                 
                 
        if 'BLOCKING' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (33) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2tpu = ' '                                                                   
            message2print = "{(Step 33A/70) trying" + "; blocking2tpu-trying-by-write2image4tpu: " + str(blocking2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2tpu = 'CL_FALSE'                                                                       # the write command is non-blocking                                                              
                message2print = "{(Step 33B/70) set" + "; blocking2tpu-set-by-write2image4tpu to: " + str(blocking2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; blocking2tpu-failed-by-write2image4tpu: " + str(blocking2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '33Z-'
        
            task4imagewrite2tpu ['BLOCKING'] = blocking2tpu             
                 
        if 'ORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2tpu = {}
            
            message2print = "{(Step 34A/70) trying" + "; origin2tpu-trying-by-write2image4tpu: " + str(origin2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2tpu['X'] = 0 
                origin2tpu['Y'] = 0  
                origin2tpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; origin2tpu-set-by-write2image4tpu to: " + str(origin2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; origin2tpu-failed-by-write2image4tpu: " + str(origin2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '34Z-'
        
            task4imagewrite2tpu ['ORIGIN'] = origin2tpu
                
                 
        if 'REGION' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (35) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2tpu = {}
            
            message2print = "{(Step 35A/70) trying" + "; region2tpu-trying-by-write2image4tpu: " + str(region2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2tpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO WRITE 
                region2tpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO WRITE
                region2tpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO WRITE                                                                                                                          
                message2print = "{(Step 35B/70) set" + "; region2tpu-set-by-write2image4tpu to: " + str(region2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '35B-' 
            except:
                message2print = "{(Step 35Z/70) failed" + "; region2tpu-failed-by-write2image4tpu: " + str(region2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '35Z-'
        
            task4imagewrite2tpu ['REGION'] = region2tpu
            
                
                 
        if 'ROWPITCH' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (36) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.
     
            rowpitch2tpu = 0
            size2element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 1                                                                                  # ONE DIRECTIONS IN 2D
            
            message2print = "{(Step 36A/70) trying" + "; rowpitch2tpu-trying-by-write2image4tpu: " + str(rowpitch2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2tpu = int(width2tpu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 36B/70) set" + "; rowpitch2tpu-set-by-write2image4tpu to: " + str(rowpitch2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #                                                                                             
                 
                code2ack  = code2ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; rowpitch2tpu-failed-by-write2image4tpu: " + str(rowpitch2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '36Z-'
        
            task4imagewrite2tpu ['ROWPITCH'] = rowpitch2tpu     
                 
        if 'POINTER' in keys2task:
            pass
        else:                
            # ----------------------------------------------------------------------------------------------------
            # (37) POINTER - REQUIRED - THE POINTER TO A BUFFER IN HOST MEMORY WHERE IMAGE DATA IS TO BE READ FROM                          
            # ----------------------------------------------------------------------------------------------------             
            pointer2tpu = None
            size2tpu = int(width2tpu * height2tpu)                                                                  # CALCULATED MEMORY
            if size2tpu == 0:
                size2tpu = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 37A/70) trying" + "; pointer2tpu-trying-by-write2image4tpu: " + str(pointer2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena2tpu = Arena4Xi(size2tpu)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 37B/70) got" + "; arena2tpu-got-by-write2image4tpu: " + str(arena2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                size2arena = arena2tpu.size                                                                         # 
                              
                message2print = "{(Step 37C/70) got" + "; size2arena-got-by-write2buffer4tpu: " + str(size2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                buffer2arena = arena2tpu.buffer
                
                message2print = "{(Step 37D/70) got" + "; buffer2arena-got-by-write2buffer4tpu: " + str(buffer2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                
                code2ack  = code2ack + '37D-' 
                
                if size2arena < size2tpu:
                    size2tpu = size2arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 37E/70) set" + "; size2tpu-set-by-write2buffer4tpu to: " + str(size2tpu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code2ack  = code2ack + '37E-'
                else:
                    pass
                    
                pointer2tpu = buffer2arena
                message2print = "{(Step 37F/70) set" + "; pointer2tpu-set-by-write2buffer4tpu to: " + str(pointer2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                code2ack  = code2ack + '37F-'       
            except:
                message2print = "{(Step 37Z/70) failed" + "; pointer2tpu-failed-by-write2image4tpu: " + str(pointer2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '37Z-'
        
            task4imagewrite2tpu ['POINTER'] = pointer2tpu                            
            task4imagewrite2tpu ['SIZE']    = size2tpu                                                               # AUTOMATIC BY TPUXI
        
             
        if 'SLICEPITCH' in keys2task:
            pass
        else:               
            # --------------------------------------------------------------------------------------------------------
            # (38) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING READ                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2tpu = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 38A/70) trying" + "; slicepitch2tpu-trying-by-write2image4tpu: " + str(slicepitch2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2tpu = 0                                                          
                message2print = "{(Step 38B/70) set" + "; slicepitch2tpu-set-by-write2image4tpu to: " + str(slicepitch2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '38B-'                
                 
            except:
                message2print = "{(Step 38Z/70) failed" + "; slicepitch2tpu-failed-by-write2image4tpu: " + str(slicepitch2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '38Z-'
            
            task4imagewrite2tpu ['SLICEPITCH'] = slicepitch2tpu                  

        # ---------------------------------------------------------------------------------------------
        # EOF PARAMETER CHECKING AND PROVIDING FOR MISSING ONES
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagewrite2tpu.keys()
        message2print = "{(Step 39/70) set" + "; keys2task-set-by-write2image4tpu to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'POINTER',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SIZE',
        #  'SLICEPITCH']; time: 02:45:31 10/05/11 Pacific Daylight Time}
             
    except: 
        message2print = "{(Step 39Z/70) yielded" + "; task4imagewrite2tpu-yielded-by-write2image4tpu: " + str(task4imagewrite2tpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '39Z-' 

    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2writeimage4tpu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2tpu = False
    channels2tpu = ()
    message2print = "{(Step 40A/70) trying" + "channels2tpu-trying-by-write2image4tpu: " + str(channels2tpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel2tpu, publisherchannel2tpu = task4imagewrite2tpu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2tpu-got-by-write2image4tpu: " + str(consumerchannel2tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 568>

        message2print = "{(Step 40C/70) got" + "; publisherchannel2tpu-got-by-write2image4tpu: " + str(publisherchannel2tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 572>                     
        ack4channels2tpu = True
        
        control2writeimage4tpu['CONSUMERCHANNEL']  = consumerchannel2tpu
        
        
        control2writeimage4tpu['PUBLISHERCHANNEL'] = publisherchannel2tpu
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2tpu = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2tpu = False
    queue2tpu = None
    message2print = "{(Step 41A/70) trying" + "queue2tpu-trying-by-write2image4tpu: " + str(queue2tpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2tpu = task4imagewrite2tpu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2tpu-got-by-write2image4tpu: " + str(queue2tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <queue4xi.Queue4Xi object at 0x0000000003C83A90> 
              
        ack4queue2tpu = True
        control2writeimage4tpu['QUEUE']  = queue2tpu        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2tpu = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4tpu = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4tpu-trying-by-write2image4tpu: " + str(dummy2image4tpu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4tpu = DummyProcess4Xi(target=manoeuvre2readimage4tpu,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue2tpu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2tpu, publisherchannel2tpu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4tpu-created-by-write2image4tpu: " + str(dummy2image4tpu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'
        
    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-write2image4tpu: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-12B-13B-14B-15B-16B-17D-17F-18B-40D-41C-
     
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-write2image4tpu: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #  
        
    message2print = "{(Step 70C/70) returning" + "; control2writeimage4tpu-returning-by-write2image4tpu: " + str(control2writeimage4tpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003C83A90>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 572>,
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 568>}; time: 02:45:31 10/05/11 Pacific Daylight Time}

    return control2writeimage4tpu
                    
#</function4xi> 


 
# -------------------------------------------------------------------------------------------------
#<function4xi function = "release2image4tpu" usage = "
#
#   task4release2image  = {} 
#   task4release2image  ['COUNT2REFERENCE'] = 1 
#   response4release2image = {}
#    
#   print('(Step 12A/33) OF TPU2IMAGE4XI5TEST - TRYING TO RELEASE IMAGE OF TPU DEVICE ', time4xi())   
#   try:                               
#       response4release2image = release2image4tpu(task4release2image)                                                                                             
#       print('(Step 12B/33) OF TPU2IMAGE4XI5TEST - RESPONSE4RELEASE2IMAGE : ', response4release2image)                           
#   except:
#       print('(Step 12Z1/33) OF TPU2IMAGE4XI5TEST *** FAILED TO RELEASE IMAGE OF TPU DEVICE ', time4xi())  
#   
#                 
#">

# -------------------------------------------------------------------------------------------------   
def release2image4tpu(task4release2tpu=None):

    '''
    releases tpu2image4xi
    '''
    global tpu2image4xi  
    global code2ack
    global code2nack
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "release2image4tpu"
    method2print = "release2image4tpu" 

    message2print = "{(Step 1A/33) started" + "; release2image4tpu-started: " + str(release2image4tpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function release2image4tpu at 0x0000000003A9E8C8>
    
    message2print = "{(Step 1B/33) passed" + "; task4release2tpu-to-release2image4tpu: " + str(task4release2tpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # {'COUNT2REFERENCE': 1} 

             
    code2ack  = ' '
    code2nack = ' '

    try:
        if tpu2image4xi is None:
            tpu2image4xi = create2image4tpu()                                                                       # INITIALIZE IMAGE                 
    except:
        try:
            tpu2image4xi = create2image4tpu()
        except:
            tpu2image4xi = {}
            message2print = "{(Step 2Z0/70) yielded" + "; tpu2image4xi-yielded-by-release2image4tpu: " + str(tpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '2Z0-' 
                
    message2print = "{(Step 1A/70) try" + "; release-try-by-release2image4tpu: None"  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    
    if task4release2tpu is None:
        task4release2tpu = {}
        task4release2tpu['COUNT2REFERENCE'] = 1
           
    try:
        # -----------------------------------------------------------------------------------------
        # RELEASE
        # -----------------------------------------------------------------------------------------               
        tpu2image4xi['COUNT2REFERENCE'] = tpu2image4xi['COUNT2REFERENCE'] - task4release2tpu['COUNT2REFERENCE']
        
        message2print = "{(Step 1A/70) calculated" + "; count2reference-calculated-by-release2image4tpu: " + str(tpu2image4xi['COUNT2REFERENCE']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
         
        if tpu2image4xi['COUNT2REFERENCE'] < 0:
            tpu2image4xi['COUNT2REFERENCE'] = 0
            
            message2print = "{(Step 1B/70) warning" + "; count2reference-warning-by-release2image4tpu: " + " ** " + str(tpu2image4xi['COUNT2REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)      
            # { release2image4tpu : release2image4tpu ; (Step 8Z1/70) warning; count2reference-warning-by-release2image4tpu:  ** 0 ** ; time: 23:41:37 09/06/11 Pacific Daylight Time}    
            code2nack  = code2nack + '1B-'
        else:
            code2ack  = code2ack + '1B-'
    except:
        try:
            tpu2image4xi['COUNT2REFERENCE'] = 0
            
            message2print = "{(Step 8Z1/70) warning" + "; count2reference-warning-by-release2image4tpu: " + " ** " + str(tpu2image4xi['COUNT2REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            code2nack  = code2nack + '8Z1-'
            
        except:
            message2print = "{(Step 8Z2/70) yielded" + "; tpu2image4xi-yielded-by-release2image4tpu: " + str(tpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '8Z2-'
                                                                                          
            
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    tpu2image4xi ['ACK2RETURN']  = code2ack
    tpu2image4xi ['NACK2RETURN'] = code2nack  
    
    message2print = "{(Step 70/70) returning" + "; tpu2image4xi-returning-by-release2image4tpu: " + str(tpu2image4xi) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'CODE2ERROR': 0,
    #  'ACK2RETURN': ' ',
    #  'ROWPITCH': 800,
    #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4tpu-failed ; time: 23:41:37 09/06/11 Pacific Daylight Time}',
    #  'CLASS': 'create2image4tpu',
    #  'METHOD': 'create2image4tpu'},
    #  'WIDTH': 800,
    #  'BUFFER': {'CODE2ERROR': 0, 
    #  'COUNT2REFERENCE': 0,
    
    #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4tpu-failed ; time: 23:41:37 09/06/11 Pacific Daylight Time}', 'CLASS': 'create2buffer4tpu', 'METHOD': 'create2buffer4tpu'},
    #  'FUNCTION2CALLBACK': <function trace2buffer4tpu at 0x0000000003A97348>,
    #  'POINTER2BUFFER': <mmap.mmap object at 0x0000000003CDE298>,
    #  'FLAGS': 0, 'CONTEXT': {'COUNT2REFERENCE': 0, ...
    
    
    return tpu2image4xi

#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "retain2image4tpu" usage = "
#   task4retain2image  = {}
#   task4retain2image  ['DIMENSION'] = 2 
#   task4retain2image  ['COUNT2REFERENCE'] = 1 
#   response4retain2Dimage = {}
#   print('(Step 13A/700) OF TPU2IMAGE4XI5TEST - TRYING TO RETAIN 2D IMAGE OF TPU DEVICE ', time4xi())  
#   try:                               
#       response4retain2Dimage = retain2image4tpu(task4retain2image)                                                                                                 
#       print('(Step 13B/700) OF TPU2IMAGE4XI5TEST - RESPONSE4RETAIN2DIMAGE : ', response4retain2Dimage)                          
#       # 
#   except:
#       print('(Step 13Z1/700) OF TPU2IMAGE4XI5TEST *** FAILED TO RETAIN 2D IMAGE OF TPU DEVICE ', time4xi()) 
#
#">

# -------------------------------------------------------------------------------------------------   
def retain2image4tpu(task4retain2tpu=None):

    '''
    retains either tpu2image4xi
    '''
    global tpu2image4xi 
    
     
    global code2ack
    global code2nack
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "retain2image4tpu"
    method2print = "retain2image4tpu"

    message2print = "{(Step 1A/33) started" + "; retain2image4tpu-started: " + str(retain2image4tpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function retain2image4tpu at 0x0000000003A9E948>   
    
    message2print = "{(Step 1B/33) passed" + "; task4retain2tpu-to-retain2image4tpu: " + str(task4retain2tpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # {'COUNT2REFERENCE': 1, 'DIMENSION': 2}
    # {'COUNT2REFERENCE': 1, 'DIMENSION': 3}
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # (Step 2/70) - PROVIDE TASKS, IF NECESSARY AND CHECK GIVEN TASKS 
    # -----------------------------------------------------------------------------------------    
    if task4retain2tpu is None:
        task4retain2tpu = {}
        task4retain2tpu['DIMENSION'] = 2
        task4retain2tpu['COUNT2REFERENCE'] = 1
        message2print = "{(Step 2A/33) set" + "; task4retain2tpu-set-by-retain2image4tpu to: " + str(task4retain2tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #           
    else:
        pass
        
    if 'DIMENSION' in task4retain2tpu.keys():
        if task4retain2tpu['DIMENSION'] == 2:
            pass
        else:
            task4retain2tpu['DIMENSION'] = 2 
            message2print = "{(Step 2B/33) set" + "; dimension-set-by-retain2image4tpu to: " + str(task4retain2tpu['DIMENSION'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                       
    else:
        task4retain2tpu['DIMENSION'] = 2
    
    # -----------------------------------------------------------------------------------------
    # (Step 3/70) - CHECK COUNT2REFERENCE
    # -----------------------------------------------------------------------------------------         
    if 'COUNT2REFERENCE' in task4retain2tpu.keys():
        pass
    else:
        task4retain2tpu['COUNT2REFERENCE'] = 1
        message2print = "{(Step 3A/33) set" + "; count2reference-set-by-retain2image4tpu to: " + str(task4retain2tpu['COUNT2REFERENCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
                                   
    if task4retain2tpu['COUNT2REFERENCE'] < 1:
        task4retain2tpu['COUNT2REFERENCE'] = 1
        message2print = "{(Step 3B/33) set" + "; count2reference-set-by-retain2image4tpu to: " + str(task4retain2tpu['COUNT2REFERENCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #           
        
    # -----------------------------------------------------------------------------------------
    # (Step 4/70) - INITIALIZE 2D IMAGE IF NECESSARY
    # -----------------------------------------------------------------------------------------
    try:
        if tpu2image4xi is None:
            tpu2image4xi = create2image4tpu()
            message2print = "{(Step 4A/70) created" + "; tpu2image4xi-created-by-retain2image4tpu: " + str(tpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                                                                                          
    except:
        try:
            tpu2image4xi = create2image4tpu()
            message2print = "{(Step 4B/70) created" + "; tpu2image4xi-created-by-retain2image4tpu: " + str(tpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                 
        except:
            tpu2image4xi = {}
            message2print = "{(Step 4Z/70) yielded" + "; tpu2image4xi-yielded-by-retain2image4tpu: " + str(tpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '4Z-' 


    # -----------------------------------------------------------------------------------------
    # (Step 5/70) - RETAIN
    # -----------------------------------------------------------------------------------------             
    try:
        tpu2image4xi['COUNT2REFERENCE'] = tpu2image4xi['COUNT2REFERENCE'] + task4retain2tpu['COUNT2REFERENCE']
        message2print = "{(Step 6A/70) calculated" + "; count2reference-calculated-by-retain2image4tpu: " + str(tpu2image4xi['COUNT2REFERENCE']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #              
        if tpu2image4xi['COUNT2REFERENCE'] <= 0:
            tpu2image4xi['COUNT2REFERENCE'] = 1
            message2print = "{(Step 6Z1/70) warning" + "; count2reference-warning-by-retain2image4tpu: " + " ** " + str(tpu2image4xi['COUNT2REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)      
            code2nack  = code2nack + '6Z1-'
        else:
            code2ack  = code2ack + '6C-'

    except:
        try:
            tpu2image4xi['COUNT2REFERENCE'] = 1
        
            message2print = "{(Step 7Z1/70) warning" + "; count2reference-warning-by-retain2image4tpu: " + " ** " + str(tpu2image4xi['COUNT2REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            code2nack  = code2nack + '7Z1-'
        
        except:
            message2print = "{(Step 7Z2/70) yielded" + "; tpu2image4xi-yielded-by-retain2image4tpu: " + str(tpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '7Z2-'
        
                                                                                          
            
    # -----------------------------------------------------------------------------------------
    # (Step 70/70) - RETURN           
    # -----------------------------------------------------------------------------------------
    
    tpu2image4xi ['ACK2RETURN']  = code2ack
    tpu2image4xi ['NACK2RETURN'] = code2nack     
    message2print = "{(Step 70A/70) returning" + "; tpu2image4xi-returning-by-retain2image4tpu: " + str(tpu2image4xi) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'CODE2ERROR': 0, 'ACK2RETURN': ' 6C-', 'ROWPITCH': 800, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4tpu-failed ; time: 23:25:02 10/05/11 Pacific Daylight Time}', 'CLASS': 'create2image4tpu', 'METHOD': 'create2image4tpu'}, 'WIDTH': 800, 'BUFFER': {'CODE2ERROR': 0, 
    #                   'COUNT2REFERENCE': 0, ...
    # 'COUNT2REFERENCE': 1, 
    # 'FLAGS': 0, 'CONTEXT': ...
    
    return tpu2image4xi

#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "get2imageinfo4tpu" usage = "
#   #1
#   print('(Step 3A/33) OF TPU2IMAGE4XI5TEST - TRYING TO GET 2D IMAGE INFO FOR TPU', time4xi())  
#   try:
#       imageinfo2tpu = get2imageinfo4tpu()                                                                       
#       print('(Step 3B/33) OF TPU2IMAGE4XI5TEST - GOT 2D IMAGE INFO FOR TPU: ', imageinfo2tpu) 
#   except:
#       print('(Step 3Z/33) OF TPU2IMAGE4XI5TEST *** FAILED TO GET 2D IMAGE INFO FOR TPU', time4xi())
#   #2
#   task4getinfo2image = {}
#   task4getinfo2image ['DIMENSION'] = 2
#   task4getinfo2image ['WIDTH'] = 0
#   task4getinfo2image ['HEIGHT'] = 0
#   response4getinfo2image = {}
#   print('(Step 12C/33) OF TPU2IMAGE4XI5TEST - TRYING TO GET SELECTED 3D IMAGE INFO OF TPU DEVICE ', time4xi())    
# 
#   try:                               
#       response4getinfo2image = get2imageinfo4tpu (task4getinfo2image)                                                                                             
#       print('(Step 12D/33) OF TPU2IMAGE4XI5TEST - RESPONSE4RELEASE2IMAGE : ', response4getinfo2image)                           
#       #
#   except:
#       print('(Step 12Z2/33) OF TPU2IMAGE4XI5TEST *** FAILED TO GET SELECTED 3D IMAGE INFO OF TPU DEVICE ', time4xi()) 
#            
#">

# -------------------------------------------------------------------------------------------------   
def get2imageinfo4tpu(task4getinfo2image=None):

    '''
    gets tpu2image4xi
    '''
    
    global tpu2image4xi
    
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33)           
    # -----------------------------------------------------------------------------------------          
    class2print = "get2imageinfo4tpu"
    method2print = "get2imageinfo4tpu"

    message2print = "{(Step 0A/33) started" + "; get2imageinfo4tpu-started: " + str(get2imageinfo4tpu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # <function get2imageinfo4tpu at 0x0000000003A9E9C8>                                                              
    
    message2print = "{(Step 0A/33) passed" + "; task4getinfo2image-passed-to-get2imageinfo4tpu: " + str(task4getinfo2image) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)   
    # {'WIDTH': 0, 'DIMENSION': 2, 'HEIGHT': 0}
    # {'WIDTH': 0, 'DEPTH': 0, 'DIMENSION': 3, 'HEIGHT': 0}
    # {'DOMAIN': 'ALL', 'DIMENSION': 2}
    
    image2tpu = {}
    try: 
        if tpu2image4xi is None:
            # -------------------------------------------------------------------------------------
            # INITIALIZE 2D IMAGE FOR TPU           
            # -------------------------------------------------------------------------------------           
            image2tpu = create2image4tpu()
            message2print = "{(Step 1B2/33) initialized" + "; image2tpu-initialized-by-get2imageinfo4tpu to: " + str(image2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
        else:
            message2print = "{(Step 1B2/33) using" + "; tpu2image4xi-using-by-get2imageinfo4tpu: " + str(tpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 
            #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-14B-15B-16B-17B-17D-18B-18C-21A1-22B1-22C1-23B-24B-25B-26B-27B-28B-29B-30B-31B-34H-35H-36B-37B-38B-', 
            #  'ROWPITCH': 800, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4tpu-failed ; time: 09:05:31 10/05/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2image4tpu', 
            #  'METHOD': 'create2image4tpu'}, 
            #  'WIDTH': 800, 
            #  'BUFFER': ...
            
            
        # -------------------------------------------------------------------------------------
        # RETURN THE WHOLE OR PART OF THE IMAGE INFO OF TPU          
        # -------------------------------------------------------------------------------------     
        dimension2xi = 0
        
        if task4getinfo2image is None:
            task4getinfo2image = {}
            task4getinfo2image ['DIMENSION'] = 2 
            task4getinfo2image ['DOMAIN'] = 'ALL'
            image2tpu = tpu2image4xi
            message2print = "{(Step 1C/33) using" + "; image2tpu-got-by-get2imageinfo4tpu: " + str(image2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'CODE2ERROR': 0, 
            #  'DOMAIN': 'ALL', 
            #  'CONTEXT': ...
            
        else:
            pass    

        keys2task4imageinfo = task4getinfo2image.keys()
        if 'DIMENSION' in keys2task4imageinfo:
            dimension2xi = task4getinfo2image['DIMENSION']
            if dimension2xi == 2:
                pass
            else:
                dimension2xi = 2    
        else:
            pass
            
        if 'DOMAIN' in keys2task4imageinfo:
            domain2xi = task4getinfo2image['DOMAIN']
            if domain2xi == 'ALL':
                pass
            else:
                domain2xi = 'ALL'

            keys2domain = tpu2image4xi.keys()
            for key2domain in keys2domain:
                task4getinfo2image[key2domain] = tpu2image4xi[key2domain]
                    
            message2print = "{(Step 1C/33) using" + "; task4getinfo2image-using-by-get2imageinfo4tpu: " + str(task4getinfo2image)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                          
        else:
            pass            
            
                    

        keys2task = task4getinfo2image.keys()
        for key2task in keys2task:
            try:
                message2print = "{(Step 2D2/33) using" + "; key2task-using-by-get2image4tpu: " + str(key2task)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                # WIDTH; DIMENSION; HEIGHT  
                image2tpu [key2task] = tpu2image4xi[key2task]
                
                message2print = "{(Step 2D2/33) set" + "; info-of-image2tpu-set-by-get2imageinfo4tpu to: " + str(image2tpu [key2task])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 800; 800  
            except:
                message2print = "{(Step 2Z/70) yielded" + "; key2task-yielded-by-get2image4tpu: " + str(key2task) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                               
                # DIMENSION;
                                   
                message2print = "{(Step 2Z2/70) using" + "; task4getinfo2image-using-by-get2image4tpu: " + str(task4getinfo2image)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                # 
                
                message2print = "{(Step 2Z3/70) using" + "; tpu2image4xi-using-by-get2image4tpu: " + str(tpu2image4xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #                           
    except:
        try:
            # -------------------------------------------------------------------------------------
            # INITIALIZE           
            # -------------------------------------------------------------------------------------           
            image2tpu = create2image4tpu()
            message2print = "{(Step 1F/33) initialized" + "; image2tpu-initialized-by-get2imageinfo4tpu to: " + str(image2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #
            if task4getinfo2image is None:
                image2tpu = tpu2image4xi
                message2print = "{(Step 1G/33) got" + "; image2tpu-got-by-get2imageinfo4tpu: " + str(image2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
            else:
                keys2task = task4getinfo2image.keys()
                for key2task in keys2task:
                    try:
                        message2print = "{(Step HD2/33) using" + "; key2task-using-by-get2image4tpu: " + str(key2task)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)  
                        # 
                        image2tpu [key2task] = tpu2image4xi[key2task]
                    
                        message2print = "{(Step HD2/33) set" + "; info-of-image2tpu-set-by-get2imageinfo4tpu to: " + str(image2tpu [key2task])  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        #   
                    except:
                        message2print = "{(Step HZ/70) yielded" + "; key2task-yielded-by-get2image4tpu: " + str(key2task) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)                            
        except:
            image2tpu = {}
            message2print = "{(Step 1Z3/70) yielded" + "; image2tpu-yielded-by-get2image4tpu: " + str(image2tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
                        
    message2print = "{(Step 33/33) returning" + "; image2tpu-returning-by-get2imageinfo4tpu: " + str(image2tpu) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)
    # 2D: {'WIDTH': 800, 'HEIGHT': 800}
    # 3D: {'WIDTH': 800, 'DEPTH': 800, 'HEIGHT': 800}
    # 2D ALL: {'CODE2ERROR': 0, 
    #  'ROWPITCH': 800, 
    #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4tpu-failed ; time: 09:05:31 10/05/11 Pacific Daylight Time}', 
    #  'CLASS': 'create2image4tpu', 
    #  'METHOD': 'create2image4tpu'}, 
    #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
    #  'ORDER': 'CL_RGBA'}, 
    #  'BUFFER': ...
    
    # 3D ALL: {'PIPE': (<read-write PipeConnection, handle 520>, <read-write PipeConnection, handle 524>), 
    #  'ROWPITCH': 800, 
    #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create3image4tpu-failed ; time: 09:05:31 10/05/11 Pacific Daylight Time}', 
    #  'CLASS': 'create3image4tpu', 
    #  'METHOD': 'create3image4tpu'}, 
    #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
    #  'ORDER': 'CL_RGBA'}, 
    #  'BUFFER': ....
    #  'HEIGHT': 800, 
    #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004504400>, 
    #  'WIDTH': 800, 
    #  'DEPTH': 800, 
    #  'FLAGS': 0, 
    #  'CONTEXT': ...
    
    return image2tpu
            
#</function4xi> 
 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "get2supportedimageformats4tpu" usage = "
#
#   print('(Step 3A/33) OF TPU2IMAGE4XI5TEST - TRYING TO GET SUPPORTED IMAGE FORMATS FOR TPU', time4xi())  
#   try:
#       imageformats = get2supportedimageformats4tpu()                                                                       
#       print('(Step 3B/33) OF TPU2IMAGE4XI5TEST - GOT SUPPORTED IMAGE FORMATS FOR TPU: ', imageformats) 
#   except:
#       print('(Step 3Z/33) OF TPU2IMAGE4XI5TEST *** FAILED TO GET SUPPORTED IMAGE FORMATS FOR TPU', time4xi())
#            
#">

# -------------------------------------------------------------------------------------------------   
def get2supportedimageformats4tpu(task4supportedimageformats2tpu=None):  

    '''
    gets supported image formats for tpu
    '''
    
    global tpu2image4xi

    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33)           
    # -----------------------------------------------------------------------------------------          
    class2print = "get2supportedimageformats4tpu"
    method2print = "get2supportedimageformats4tpu"

    message2print = "{(Step 0A/33) started" + "; get2supportedimageformats4tpu-started: " + str(get2supportedimageformats4tpu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
    # <function get2supportedimageformats4tpu at 0x0000000003BBE8C8>                                                          
    
    message2print = "{(Step 0B/33) passed" + "; task4supportedimageformats2tpu-passed-to-get2supportedimageformats4tpu: " + str(task4supportedimageformats2tpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)   
    # None; {'DIMENSION': 3} 
    
    
    if task4supportedimageformats2tpu is None:
        task4supportedimageformats2tpu = {} 
        task4supportedimageformats2tpu['DIMENSION'] = 2                                                             # 2D image is the default
    
    dimension2xi = 2
    try:
        keys2supportedimageformats = task4supportedimageformats2tpu.keys()    
        if 'DIMENSION' in keys2supportedimageformats:
            dimension2xi = task4supportedimageformats2tpu['DIMENSION']
            if ((dimension2xi <=0) or (dimension2xi > 2)): 
                dimension2xi = 2
        else:
            dimension2xi = 2
    except:
        pass        
    
    message2print = "{(Step 1A/33) using" + "; dimension2xi-using-by-dimension2xi: " + str(dimension2xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    # 2; 3
                        
    supportedimageformats2tpu = {} 
    try: 
        if tpu2image4xi is None:
            # -------------------------------------------------------------------------------------
            # INITIALIZE           
            # -------------------------------------------------------------------------------------           
            image2tpu = create2image4tpu()                                                                          # create tpu2image4xi
            message2print = "{(Step 1B/33) initialized" + "; image2tpu-initialized-by-get2supportedimageformats4tpu to: " + str(image2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                
        else:
            pass

                        
        # -------------------------------------------------------------------------------------
        # SUPPORTED IMAGE FORMATS FOR TPU       
        # -------------------------------------------------------------------------------------     
        supportedimageformats2tpu = tpu2image4xi['FORMAT']
        message2print = "{(Step 1E/33) got" + "; supportedimageformats2tpu-got-by-get2supportedimageformats4tpu: " + str(supportedimageformats2tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)           
        # {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'} 
            
    except:
        try:
            # -------------------------------------------------------------------------------------
            # INITIALIZE           
            # -------------------------------------------------------------------------------------           
            image2tpu = create2image4tpu()
            message2print = "{(Step 1F1/33) initialized" + "; image2tpu-initialized-by-get2supportedimageformats4tpu to: " + str(image2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
     
            
            # -------------------------------------------------------------------------------------
            # SUPPORTED IMAGE FORMATS FOR TPU       
            # -------------------------------------------------------------------------------------     
            supportedimageformats2tpu = tpu2image4xi['FORMAT']
            message2print = "{(Step 1H/33) got" + "; supportedimageformats2tpu-got-by-get2supportedimageformats4tpu: " + str(supportedimageformats2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)           
            #  
                                              
        except:
            message2print = "{(Step 1Z/70) yielded" + "; supportedimageformats2tpu-yielded-by-get2image4tpu: " + str(supportedimageformats2tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
                        
    message2print = "{(Step 33/33) returning" + "; supportedimageformats2tpu-returning-by-get2supportedimageformats4tpu: " + str(supportedimageformats2tpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # 2: {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'}
    # 3: {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'}
    
    return supportedimageformats2tpu
            
#</function4xi> 

 



# -------------------------------------------------------------------------------------------------
#<function4xi function = "wait2image4tpu" usage = "
#   print('=' *33)          
#   print('(Step 16A/700) OF TPU2IMAGE4XI5TEST - TRYING TO WAIT 2D IMAGE FOR TPU ', time4xi())   
#   print('=' *33)      
#   try:
#       response4wait2tpu = wait2image4tpu()
#       keys2response = response4wait2tpu.keys()
#       print('(Step 16B/700) OF TPU2IMAGE4XI5TEST - WAITED 2D IMAGE FOR TPU ', time4xi()) 
#       for key2response in sorted(keys2response):
#           print(key2response, ' : ', response4wait2tpu[key2response])
#           # ACK2RETURN  :   6C-
#           # BUFFER  :  {'CODE2ERROR': 0 ...
#           # CALLBACK  :  <function trace2image4tpu at 0x0000000004E93B48>
#           # CODE2ERROR  :  0
#           # COMMANDQUEUE  :  {'CODE2ERROR': 0, ...
#           # CONTEXT  :  {'COUNT2REFERENCE': 0 ...
#           # COUNT2REFERENCE  :  1
#           # DATA2CALLBACK  :  {'MESSAGE': '{(Step 1Z1/33) failed; create2image4tpu-failed ; time: 02:00:43 10/06/11 Pacific Daylight Time}', 'CLASS': 'create2image4tpu', 'METHOD': 'create2image4tpu'}
#           # DEPTH  :  0
#           # DEVICE  :  {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK',...
#           # EMPTY2QUEUE  :  True
#           # EVENT  :  {'CODE2ERROR': 0, 'DATA2CALLBACK':...
#           # FLAGS  :  0
#           # FORMAT  :  {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'}
#           # FULL2QUEUE  :  False
#           # FUNCTION2CALLBACK  :  <function trace2image4tpu at 0x0000000004E93B48>
#           # HEIGHT  :  800
#           # NACK2RETURN  :   
#           # PIPE  :  (<read-write PipeConnection, handle 556>, <read-write PipeConnection, handle 560>)
#           # PLATFORM  :  {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}
#           # POINTER  :  <mmap.mmap object at 0x0000000004351500>
#           # QUEUE  :  <queue4xi.Queue4Xi object at 0x00000000044E95C0>
#           # ROWPITCH  :  800
#           # SIZE2QUEUE  :  2147483646 
#   except:
#       print('(Step 16Z1/700) OF TPU2IMAGE4XI5TEST *** FAILED TO WAIT 2D IMAGE FOR TPU: ', wait2tpu)            
#">

# -------------------------------------------------------------------------------------------------   
def wait2image4tpu(task4wait2image=None):

    '''
    waits tpu2image4tpu
    '''
    
    global tpu2image4xi
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33)           
    # -----------------------------------------------------------------------------------------          
    class2print = "wait2image4tpu"
    method2print = "wait2image4tpu"

    message2print = "{(Step 1A1/33) started" + "; wait2image4tpu-started: " + str(wait2image4tpu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    #  
    
    message2print = "{(Step 1A2/33) passed" + "; task4wait2image-passed-by-wait2image4tpu to: " + str(task4wait2image)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 
    
    try: 
        if tpu2image4xi is None:
            # -------------------------------------------------------------------------------------
            # INITIALIZE - KEEP SYSTEM RUNNING           
            # -------------------------------------------------------------------------------------           
            tpu2image4xi = create2image4tpu()
            
            message2print = "{(Step 1B/33) created" + "; tpu2image4xi-created-by-wait2image4tpu: " + str(tpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #
                                                
            # -------------------------------------------------------------------------------------
            # WAIT         
            # -------------------------------------------------------------------------------------             
            queue2image4tpu = None
            try:
                queue2image4tpu = tpu2image4xi['QUEUE']
                message2print = "{(Step 1C/33) got" + "; queue2image4tpu-got-by-wait2image4tpu to: " + str(queue2image4tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #
            except:     
                message2print = "{(Step 1Z1/70) yielded" + "; tpu2xi-yielded-by-wait2image4tpu: " + str(tpu2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            
            size2image4tpu = 0
            try:                    
                size2image4tpu = queue2image4tpu.qsize()
                message2print = "{(Step 1D/33) got" + "; size2image4tpu-got-by-wait2image4tpu to: " + str(size2image4tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #
                               
                
            except:     
                message2print = "{(Step 1Z2/70) yielded" + "; tpu2xi-yielded-by-wait2image4tpu: " + str(tpu2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
             
            empty2image4tpu = True       
            try:                    
                empty2image4tpu= queue2image4tpu.empty()
                message2print = "{(Step 1E/33) got" + "; empty2image4tpu-got-by-wait2image4tpu to: " + str(empty2image4tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #
            except:     
                message2print = "{(Step 1Z3/70) yielded" + "; empty2image4tpu-yielded-by-wait2image4tpu: " + str(empty2image4tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
             
            full2image4tpu = False       
            try:                                
                full2image4tpu = queue2image4tpu.full()
                message2print = "{(Step 1F/33) got" + "; full2image4tpu-got-by-wait2image4tpu to: " + str(full2image4tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #                
            except:     
                message2print = "{(Step 1Z4/70) yielded" + "; full2image4tpu-yielded-by-wait2image4tpu: " + str(full2image4tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)   
                            
            message2print = "{(Step 1G/33) finished" + "; tpu2image4xi-finished-by-wait2image4tpu to: " + str(tpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #
                            
        else:
            # -------------------------------------------------------------------------------------
            # USE THE EXISTING IMAGE FOR TPU          
            # -------------------------------------------------------------------------------------     
            
            message2print = "{(Step 1H/33) used" + "; tpu2image4xi-used-by-wait2image4tpu: " + str(tpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
            # -------------------------------------------------------------------------------------
            # WAIT         
            # -------------------------------------------------------------------------------------             
            queue2image4tpu = None
            try:
                queue2image4tpu = tpu2image4xi['QUEUE']
                message2print = "{(Step 1I/33) got" + "; queue2image4tpu-got-by-wait2image4tpu to: " + str(queue2image4tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
            except:     
                message2print = "{(Step 1Z5/70) yielded" + "; tpu2xi-yielded-by-wait2image4tpu: " + str(tpu2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
            if task4wait2image is None:
                pass
            else:
                # -------------------------------------------------------------------------------------
                # IMAGE GIVEN AS PARAMETER          
                # -------------------------------------------------------------------------------------               
                message2print = "{(Step 1J1/33) passed" + "; task4wait2image-passed-by-wait2image4tpu to: " + str(task4wait2image)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                queue2task = {}    
                try:
                    queue2task = task4wait2image ['QUEUE']
                    message2print = "{(Step 1J2/33) set" + "; queue2task-set-by-wait2image4tpu to: " + str(queue2task)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                except:     
                    message2print = "{(Step 1Z61/70) yielded" + "; queue2task-yielded-by-wait2image4tpu: " + str(queue2task) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                
                try:                        
                    if queue2task == queue2image4tpu:
                        # -------------------------------------------------------------------------------------
                        # ACCEPTED - GIVEN QUEUE IS THE SAME          
                        # -------------------------------------------------------------------------------------                     
                        message2print = "{(Step 1K/33) accepted" + "; queue2task-accepted-by-wait2image4tpu: " + str(queue2task) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        # 
                    else:
                        try:
                            message2print = "{(Step 1L/33) warned" + "; queue2task-warned-by-wait2image4tpu: " + " ** " + str(queue2task)  + " ** " + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print) 
                            # 
                        
                            message2print = "{(Step 1M/33) warned" + "; queue2image4tpu-warned-by-wait2image4tpu: " + " ** " + str(queue2image4tpu)  + " ** " + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)    
                            # 
                        except:
                            pass
                                                     
                except:     
                    message2print = "{(Step 1Z62/70) yielded" + "; queue2task-yielded-by-wait2image4tpu: " + str(queue2task) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                   
            
            size2queue4tpu = 0        
            try:                    
                size2queue4tpu = queue2image4tpu.qsize()
                message2print = "{(Step 1N1/33) got" + "; size2queue4tpu-got-by-wait2image4tpu to: " + str(size2queue4tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                
                tpu2image4xi ['SIZE2QUEUE'] = size2queue4tpu
                message2print = "{(Step 1N2/33) set" + "; size2queue4image-got-by-wait2image4tpu to: " + str(tpu2image4xi ['SIZE2QUEUE'])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                  
                # 
            except:     
                message2print = "{(Step 1Z7/70) yielded" + "; size2queue4tpu-yielded-by-wait2image4tpu: " + str(size2queue4tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            
            empty2queue4tpu = False
            try:                    
                empty2queue4tpu = queue2image4tpu.empty()
                message2print = "{(Step 1O1/33) got" + "; empty2queue4tpu-got-by-wait2image4tpu to: " + str(empty2queue4tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                
                tpu2image4xi ['EMPTY2QUEUE'] = empty2queue4tpu
                message2print = "{(Step 1O2/33) set" + "; empty2queue4image-got-by-wait2image4tpu to: " + str(tpu2image4xi ['EMPTY2QUEUE'])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #               
            except:     
                message2print = "{(Step 1Z8/70) yielded" + "; empty2queue4tpu-yielded-by-wait2image4tpu: " + str(empty2queue4tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            
            full2queue4tpu = False
            try:                                
                full2queue4tpu = queue2image4tpu.full()
                message2print = "{(Step 1P1/33) got" + "; full2queue4tpu-got-by-wait2image4tpu to: " + str(full2queue4tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                 
                tpu2image4xi ['FULL2QUEUE'] = full2queue4tpu
                message2print = "{(Step 1P2/33) set" + "; full2queue4image-got-by-wait2image4tpu to: " + str(tpu2image4xi ['FULL2QUEUE'])  + "; time: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print)
                #                          
            except:     
                message2print = "{(Step 1Z9/70) yielded" + "; full2queue4tpu-yielded-by-wait2image4tpu: " + str(full2queue4tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            
            state2queue4tpu = None
            try:                                
                state2queue4tpu = queue2image4tpu.__getstate__()
                message2print = "{(Step 1P3/33) got" + "; state2queue4tpu-got-by-wait2image4tpu to: " + str(state2queue4tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                 
                tpu2image4xi ['STATE2QUEUE'] = state2queue4tpu
                message2print = "{(Step 1P4/33) set" + "; state2queue4image-set-by-wait2image4tpu to: " + str(tpu2image4xi ['STATE2QUEUE'])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #           
            except:     
                message2print = "{(Step 1Z10/70) yielded" + "; state2queue4tpu-yielded-by-wait2image4tpu: " + str(state2queue4tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)               
                
                                                            
            message2print = "{(Step 1Q/33) finished" + "; tpu2image4xi-finished-by-wait2image4tpu: " + str(tpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
    except:
        try:
            # -------------------------------------------------------------------------------------
            # INITIALIZE           
            # -------------------------------------------------------------------------------------           
            tpu2image4tpu = create2image4tpu()
            message2print = "{(Step 1R/33) got" + "; tpu2image4xi-got-by-wait2image4tpu to: " + str(tpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #  
            
            # -------------------------------------------------------------------------------------
            # WAIT         
            # -------------------------------------------------------------------------------------             
            queue2image4tpu = None
            try:
                queue2image4tpu = tpu2image4xi['QUEUE']
                message2print = "{(Step 1S/33) got" + "; queue2image4tpu-got-by-wait2image4tpu to: " + str(queue2image4tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #
            except:     
                message2print = "{(Step 1Z10/70) yielded" + "; queue2image4tpu-yielded-by-wait2image4tpu: " + str(queue2image4tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
            if task4wait2image is None:
                pass
            else:
                # -------------------------------------------------------------------------------------
                # IMAGE QUEUE GIVEN AS PARAMETER          
                # -------------------------------------------------------------------------------------               
                queue2task = None    
                try:
                    queue2task = task4wait2image ['QUEUE']
                    message2print = "{(Step 1T/33) passed" + "; queue2task-passed-by-wait2image4tpu to: " + str(queue2task)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    #
                    
                    if queue2task == queue2image4tpu:
                        # -------------------------------------------------------------------------------------
                        # ACCEPTED - GIVEN QUEUE IS THE SAME          
                        # -------------------------------------------------------------------------------------                     
                        message2print = "{(Step 1U/33) accepted" + "; queue2task-accepted-by-wait2image4tpu: " + str(queue2task) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        # 
                    else:
                        message2print = "{(Step 1X/33) warned" + "; queue2task-warned-by-wait2image4tpu: " + " ** " + str(queue2task)  + " ** " + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print) 
                        # 
                        
                        message2print = "{(Step 1Y1/33) warned" + "; queue2image4tpu-warned-by-wait2image4tpu: " + " ** " + str(queue2image4tpu)  + " ** " + "; time: " + time4xi() + "}" 
                        print4xi(class2print, method2print, message2print)    
                        #                      
                except:     
                    message2print = "{(Step 1Z11/70) yielded" + "; queue2task-yielded-by-wait2image4tpu: " + str(queue2task) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
            
            size2image4tpu = 0                             
            try:                    
                size2image4tpu = queue2image4tpu.qsize()
                message2print = "{(Step 1Y2/33) got" + "; size2image4tpu-got-by-wait2image4tpu to: " + str(size2image4tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #
            except:     
                message2print = "{(Step 1Z12/70) yielded" + "; size2image4tpu-yielded-by-wait2image4tpu: " + str(size2image4tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
             
            empty2image4tpu = True          
            try:                    
                empty2image4tpu = queue2image4tpu.empty()
                message2print = "{(Step 1Y3/33) got" + "; empty2image4tpu-got-by-wait2image4tpu to: " + str(empty2image4tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #
            except:     
                message2print = "{(Step 1Z13/70) yielded" + "; tpu2xi-yielded-by-wait2image4tpu: " + str(tpu2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
             
            full2image4tpu = False          
            try:                                
                full2image4tpu= queue2image4tpu.full()
                message2print = "{(Step 1Y4/33) got" + "; full2image4tpu-got-by-wait2image4tpu to: " + str(full2image4tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #                
            except:     
                message2print = "{(Step 1Z14/70) yielded" + "; full2image4tpu-yielded-by-wait2image4tpu: " + str(full2image4tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                                                        
            message2print = "{(Step 1Y5/33) finished" + "; tpu2image4xi-finished-by-wait2image4tpu: " + str(tpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                    
        except:
            tpu2image4tpu= {}
            message2print = "{(Step 1Z15/70) yielded" + "; tpu2image4xi-yielded-by-wait2image4tpu: " + str(tpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
                        
    message2print = "{(Step 33/33) returning" + "; tpu2image4xi-returning-by-wait2image4tpu: " + str(tpu2image4xi) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # {'SIZE2QUEUE': 2147483646, 
    #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
    #  'ORDER': 'CL_RGBA'}, 
    #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004420E10>, 
    #  'WIDTH': 800, 
    #  'CONTEXT': ...
    
    return tpu2image4xi
    
   
#</function4xi> 
                                               
# =================================================================================================
#<class4xi class = "TPU2Image4Xi" base = "object" child = "None"
#   synopsis = "offers methods for TPU 2D images" usage = "2D Image4Xi of TPU"
#>

# =================================================================================================
class TPU2Image4Xi(object): 
    ''' 
    TPU2Image4Xi offers methods for TPU 2D images
    '''            

    #                                        
    # ---------------------------------------------------------------------------------------------
    # sharable class variables
    # ---------------------------------------------------------------------------------------------      
    #<share4xi share = " 
    classvariable2tpu = cpu_corecount4xi()
    #"
    #/>
        
    # ---------------------------------------------------------------------------------------------
    # Public API of this class
    # ---------------------------------------------------------------------------------------------    
    #<expose4xi expose = "
    _exposed_ = (
            'create2image',                    # ACK 5.31   p76    
            'copy2image',                      # ACK 5.31   p88
            'copy2imagetobuffer',              # ACK 5.3.4  p90
            'copy2buffertoimage',              # ACK 5.3.4  p92
            'read2image',                      # ACK 5.3.3  p84
            'write2image',                     # ACK 5.3.3  p84            
            'map2image',                       # ACK 5.3.5  p94            
            'get2imageinfo',                   # ACK 5.3.6  p97
            'get2supportedimageformats',       # ACK 5.3.2  p82 nb: a separate method to get format whereas using get2imageinfo you can access any element by key such as 'FORMAT'   
            'retain2image',                    # ACK 5.4.1  p99 nb: for compability to OpenCL - you can get any element of dictionary by get2imageinfo
            'release2image',                   # ACK 5.4.1  p99
            'trace2image',                     # ACK modernized 
            'set2imagecallback',               # ACK modernized
            'set2image',                       # ACK modernized nb: with memorymap features you can process 2D images as files!                    
            'wait2image',                      # ACK modernized nb: checks Queue object and thereby generates delay   
            'sense',                           # ACK modernized  
            'reason',                          # ACK modernized  
            'ace')                             # ACK modernized           
    #"
    #/>
 
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "init-of-TPU2Image4Xi" usage = "
    #   print('(Step 5A/33) OF TPU2IMAGE4XI5TEST - CREATING IMAGE4TPU OBJECT ', time4xi())     
    #   try:
    #       imageobject2tpu = TPU2Image4Xi()
    #       print('(Step 5B/33) OF TPU2IMAGE4XI5TEST - CREATED IMAGE4TPU OBJECT: ', imageobject2tpu)                                  
    #   except:
    #       print('(Step 5Z/33) OF TPU2IMAGE4XI5TEST *** FAILED TO CREATE IMAGE4TPU OBJECT')
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def __init__(self, task4imagecreate2tpu = None):
        '''
        _init__ method initializes TPU2Image4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------
                    
        class2print = "TPU2Image4Xi"
        method2print = "__init__"
                     
        message2print = "{(Step 1A/33) started" + "; init-of-TPU2Image4Xi-started: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # 23:41:37 09/06/11 Pacific Daylight Time 

        message2print = "{(Step 1B/33) borning" + "; tpu2image2xi-of-TPU2Image4Xi-borning: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None 
                
        message2print = "{(Step 1C/33) inheriting" + "; object-inheriting-by-tpu2image2xi-of-TPU2Image4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # <class 'object'>
        
        message2print = "{(Step 1D/33) sharing" + "; classvariable2tpu-sharing-by-tpu2image2xi-of-TPU2Image4Xi: " + str(TPU2Image4Xi.classvariable2tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 4             
                                
        message2print = "{(Step 1E/33) passed" + "; task4imagecreate2tpu-passed-to-init-of-TPU2Image4Xi: " + str(task4imagecreate2tpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None                            
        
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)             
        # -----------------------------------------------------------------------------------------
        
        message2print = "{(Step 2A/33) passing" + "; task4imagecreate2tpu-passing-to-create2image4tpu-by-init-of-TPU2Image4Xi: " + str(task4imagecreate2tpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None  
        
        tpu2image2xi = create2image4tpu(task4imagecreate2tpu)
        
        message2print = "{(Step 2B/33) performed" + "; create2image4tpu-performed-by-init-of-TPU2Image4Xi: " + str(create2image4tpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # <function create2image4tpu at 0x0000000003A9E7C8>
                        
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)             
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 3A/33) activated" + "; tpu2image2xi-of-TPU2Image4Xi-activated: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # { TPU2Image4Xi : __init__ ; (Step 3A/33) activated; tpu2image2xi-of-TPU2Image4Xi-activated: ; time: 00:14:05 09/07/11 Pacific Daylight Time} 
                
        message2print = "{(Step 3B/33) inherited" + "; object-inherited-by-tpu2image2xi-of-TPU2Image4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # <class 'object'> 
        
        message2print = "{(Step 3C/33) shared" + "; classvariable2tpu-shared-by-tpu2image2xi-of-TPU2Image4Xi: " + str(TPU2Image4Xi.classvariable2tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 4           
       
        message2print = "{(Step 3D/33) returning" + "; init-of-TPU2Image4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # 00:14:05 09/07/11 Pacific Daylight Time
        

    #</methodize4xi>
    

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "create2image-of-TPU2Image4Xi" usage = "
    #   print('(Step 23A/33) OF TPU2IMAGE4XI5TEST - TRYING TO CREATE TPU IMAGE', time4xi()) 
    #   try:
    #       xi2tpu = TPU2Image4Xi()                            
    #       image2tpu = xi2tpu.create2image()                                  
    #       print('(Step 23B/33) OF TPU2IMAGE4XI5TEST - CREATED TPU IMAGE: ', image2tpu)
    #   except:
    #       print('(Step 23Z/33) OF TPU2IMAGE4XI5TEST *** FAILED TO CREATE TPU IMAGE ***', time4xi())                  
    #                     
    #">   
    
    # ---------------------------------------------------------------------------------------------   
    def create2image(self, task4image2create = None):
        '''
        create2image method creates 2D image objects of TPU2Image4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "TPU2Image4Xi"
        method2print = "create2image"
                     
        message2print = "{(Step 1A/33) started" + "; create2image-of-TPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4image2create-passed-to-create-of-TPU2Image4Xi: " + str(task4image2create) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                      
        # None 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4image2create-passing-to-create2image4tpu-by-create2image-of-TPU2Image4Xi: " + str(task4image2create) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # None      
  
        tpu2image2xi = create2image4tpu(task4image2create)

        message2print = "{(Step 2B/33) performed" + "; create2image4tpu-performed-by-create2image-of-TPU2Image4Xi: " + str(create2image4tpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function create2image4tpu at 0x00000000039DF7C8>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
        message2print = "{(Step 3/33) returning" + "; tpu2image2xi-returning-by-create2image-of-TPU2Image4Xi: " + str(tpu2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          # 
        # {'CODE2ERROR': 0,
        #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-14B-15B-16B-17B-17D-18B-18C-21A1-22B1-22C1-23B-24B-25B-26B-27B-28B-29B-30B-31B-34H-35H-36B-37B-38B-',
        #  'ROWPITCH': 800,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4tpu-failed ; time: 00:53:55 10/05/11 Pacific Daylight Time}',
        #  'CLASS': 'create2image4tpu',
        #  'METHOD': 'create2image4tpu'},
        #  'WIDTH': 800 ...

        return tpu2image2xi
                                
    #                        
    #</methodize4xi>
    

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "set2image-of-TPU2Image4Xi" usage = "
    #   print('(Step 1A/33) OF TPU2IMAGE4XI - TRYING TO SET TPU IMAGE')
    #   try:
    #       cc2xi = TPU2Image4Xi()                            
    #       image2tpu = cc2xi.set2image()                                  
    #       print('(Step 1B/33) OF TPU4XI5TEST - SET TPU IMAGE: ', image2tpu)                                                  
    #   except:
    #       print('(Step 1Z/33) OF TPU4XI5TEST *** FAILED TO SET TPU IMAGE ***')  
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def set2image(self, task4set2image = None):
        '''
        sets 2D image TPU objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "TPU2Image4Xi"
        method2print = "set2image"
                     
        message2print = "{(Step 1A/33) started" + "; set2image-of-TPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:02:22 09/07/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4set2image-passed-to-set2image-of-TPU2Image4Xi: " + str(task4set2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # {'CODE2ERROR': 0,
        #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004191E10>,
        #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
        
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4set2image-passing-to-set2image4tpu-by-set2image-of-TPU2Image4Xi: " + str(task4set2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # {'CODE2ERROR': 0, 
        #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004191E10>, 
        #  'CONTEXT': {'COUNT2REFERENCE': 0 ...

        tpu2image2xi = set2image4tpu(task4set2image)

        message2print = "{(Step 2B/33) performed" + "; set2image4tpu-performed-by-set2image-of-TPU2Image4Xi: " + str(set2image4tpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function set2image4tpu at 0x0000000003AB36C8> 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; tpu2image2xi-returning-by-set2image-of-TPU2Image4Xi: " + str(tpu2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0,
        #  'ACK2RETURN': ' 23B-24B-25B-26B-27B-28B-',
        #  'ROWPITCH': 800,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4tpu-failed ; time: 01:02:22 09/07/11 Pacific Daylight Time}', 'CLASS': 'create2image4tpu', 'METHOD': 'create2image4tpu'},
        #  'WIDTH': 800,
        #  'BUFFER': {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4tpu-failed ; time: 01:02:22 09/07/11 Pacific Daylight Time}',
        #  'CLASS': 'create2buffer4tpu',
        #  'METHOD': 'create2buffer4tpu'},
        #  'FUNCTION2CALLBACK': <function trace2buffer4tpu at 0x0000000003AA81C8>,
        #  'POINTER2BUFFER': <mmap.mmap object at 0x0000000003CC92F0>,
        #  'FLAGS': 0,
        #  'CONTEXT': {'COUNT2REFERENCE': 0 ...
                    
                
        return tpu2image2xi
                                
                            
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "get2imageinfo-of-TPU2Image4Xi" usage = "
    #   print('(Step 1A/33) OF TPU2IMAGE4XI - TRYING TO GET TPU 2D IMAGE')
    #   try:
    #       cc2xi = TPU2Image4Xi()                            
    #       image2tpu = cc2xi.get2imageinfo()                                  
    #       print('(Step 1B/33) OF TPU4XI5TEST - GOT TPU IMAGE: ', image2tpu)                                                  
    #   except:
    #       print('(Step 1Z/33) OF TPU4XI5TEST *** FAILED TO GET TPU 2D IMAGE ***')   
    # ">
    
    # ---------------------------------------------------------------------------------------------   
    def get2imageinfo(self, task4getinfo2image=None):
        '''
        get2imageinfo method gets 2D objects of TPU2Image4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "TPU2Image4Xi"
        method2print = "get2imageinfo"
                     
        message2print = "{(Step 1A/33) started" + "; get2imageinfo-of-TPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time 

        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # ----------------------------------------------------------------------------------------- 
        tpu2image2xi = get2imageinfo4tpu(task4getinfo2image)
        
        message2print = "{(Step 2A/33) returning" + "; tpu2image2xi-returning-by-get2imageinfo-of-TPU2Image4Xi: " + str(tpu2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0,
        #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-14B-15B-16B-17B-17D-18B-18C-21A1-22B1-22C1-23B-24B-25B-26B-27B-28B-29B-30B-31B-34H-35H-36B-37B-38B-',
        #  'ROWPITCH': 800,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4tpu-failed ; time: 00:14:05 09/07/11 Pacific Daylight Time}', 'CLASS': 'create2image4tpu', 'METHOD': 'create2image4tpu'},
        #  'WIDTH': 800,
        #  'BUFFER': {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4tpu-failed ; time: 00:14:05 09/07/11 Pacific Daylight Time}',
        #  'CLASS': 'create2buffer4tpu',
        #  'METHOD': 'create2buffer4tpu'},
        #  'FUNCTION2CALLBACK': <function trace2buffer4tpu at 0x00000000039D8348>,
        #  'POINTER2BUFFER': <mmap.mmap object at 0x00000000037D32F0>,
        #  'FLAGS': 0,
        #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
        # or when only two elemets required
        # {'WIDTH': 800, 'HEIGHT': 800}
        
        return tpu2image2xi
                            
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "get2supportedimageformats-of-TPU2Image4Xi" usage = "
    #  print('(Step 6B1/33) OF TPU2IMAGE4XI5TEST - TRYING TO GET SUPPORTED IMAGE FORMATS FOR TPU')
    #  try:
    #      cc2xi = TPU2Image4Xi()                            
    #      supportedimageformats2tpu = cc2xi.get2supportedimageformats()                                  
    #      print('(Step 6B2/33) OF TPU4XI5TEST - GOT SUPPORTED IMAGE FORMATS FOR TPU: ', supportedimageformats2tpu)
    #                                                   
    #  except:
    #      print('(Step 2Z/33) OF TPU4XI5TEST *** FAILED TO GET SUPPORTED IMAGE FORMATS FOR TPU***')     
    # ">
    
    # ---------------------------------------------------------------------------------------------   
    def get2supportedimageformats(self):
        '''
        get2supportedimageformats method gets supported imageformats by objects of TPU2Image4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "TPU2Image4Xi"
        method2print = "get2supportedimageformats"
                     
        message2print = "{(Step 1A/33) started" + "; get2supportedimageformats-of-TPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:43:00 09/07/11 Pacific Daylight Time

        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # ----------------------------------------------------------------------------------------- 
        supportedimageformats2tpu = get2supportedimageformats4tpu()
        
        message2print = "{(Step 2A/33) returning" + "; supportedimageformats2tpu-returning-by-get2imageinfo-of-TPU2Image4Xi: " + str(supportedimageformats2tpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'} 
        
        return supportedimageformats2tpu
                            
    #</methodize4xi>    
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "read2image-of-TPU2Image4Xi" usage = "
    #   print('(Step 127A/700) OF TPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH READ IMAGE OBJECT ', time4xi()) 
    #   response4read2tpu = {}
    #   try:
    #       xi2tpu = TPU2Image4Xi()    
    #       response4read2tpu = xi2tpu.read2image()
    #       print('(Step 127B/700) OF TPU2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH READ IMAGE OBJECT, RESPONSE4READ2TPU: ', response4read2tpu) 
    #   except:
    #       print('(Step 127Z1/700) OF TPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH READ IMAGE OBJECT ', time4xi())
    #   ... continue as with read2image4tpu function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def read2image(self, task4read2image = None):
        '''
        reads TPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "TPU2Image4Xi"
        method2print = "read2image"
                     
        message2print = "{(Step 1A/33) started" + "; read2image-of-TPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:20:02 10/05/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4read2image-passed-to-read2image-of-TPU2Image4Xi: " + str(task4read2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4read2image-passing-to-read2image4tpu-by-read2image-of-TPU2Image4Xi: " + str(task4read2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        response4readimage2tpu = read2image4tpu(task4read2image)

        message2print = "{(Step 2B/33) performed" + "; read2image4tpu-performed-by-read2image-of-TPU2Image4Xi: " + str(read2image4tpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function read2image4tpu at 0x000000000385DA48> 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4readimage2tpu-returning-by-read2image-of-TPU2Image4Xi: " + str(response4readimage2tpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x000000000417DD68>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 936>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 932>}; time: 00:20:02 10/05/11 Pacific Daylight Time}
  
        return response4readimage2tpu
                                                           
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "map2image-of-TPU2Image4Xi" usage = "
    #   print('(Step 127A/700) OF TPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH MAP IMAGE OBJECT ', time4xi()) 
    #   xi2tpu = TPU2Image4Xi() 
    #   ...
    #   response4map2tpu = {}
    #   try:
    #       response4map2tpu = xi2tpu.map2image()
    #       print('(Step 127B/700) OF TPU2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH MAP IMAGE OBJECT, RESPONSE4READ2TPU: ', response4map2tpu) 
    #   except:
    #       print('(Step 127Z1/700) OF TPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH MAP IMAGE OBJECT ', time4xi())
    #   ... continue as with map2image4tpu function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def map2image(self, task4map2image = None):
        '''
        maps TPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "TPU2Image4Xi"
        method2print = "map2image"
                     
        message2print = "{(Step 1A/33) started" + "; map2image-of-TPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:28:44 10/05/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4map2image-passed-to-map2image-of-TPU2Image4Xi: " + str(task4map2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4map2image-passing-to-map2image4tpu-by-map2image-of-TPU2Image4Xi: " + str(task4map2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        response4mapimage2tpu = map2image4tpu(task4map2image)

        message2print = "{(Step 2B/33) performed" + "; map2image4tpu-performed-by-map2image-of-TPU2Image4Xi: " + str(map2image4tpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function map2image4tpu at 0x000000000390BAC8> 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4mapimage2tpu-returning-by-map2image-of-TPU2Image4Xi: " + str(response4mapimage2tpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x00000000050E9C18>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 1100>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 1096>}; time: 01:28:44 10/05/11 Pacific Daylight Time}

        
        return response4mapimage2tpu
                                                           
    #</methodize4xi>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "release2image-of-TPU2Image4Xi" usage = "
    #
    # ---------------------------------------------------------------------------------------------
    # RELEASE 2D IMAGE BY DECREMENTING REFERENCE COUNT WITH 1             
    # ---------------------------------------------------------------------------------------------    
    #   task4release2image  = {} 
    #   task4release2image  ['COUNT2REFERENCE'] = 1 
    #   response4release2image = {}
    #   print('(Step 133A/700) OF TPU2IMAGE4XI5TEST - TRYING TO RELEASE 2D IMAGE OF TPU DEVICE ', time4xi())    
    # 
    #   try:                               
    #       response4release2Dimage = xi2tpu.release2image(task4release2image)                                                                                             
    #       print('(Step 133B/700) OF TPU2IMAGE4XI5TEST - RELEASED 2D IMAGE OF TPU DEVICE ', time4xi())                           
    #       keys2response = response4release2Dimage.keys()
    #       for key2response in sorted(keys2response):
    #           print(key2response, ' : ', response4release2Dimage[key2response])
    #           #
    #   except:
    #       print('(Step 133Z1/700) OF TPU2IMAGE4XI5TEST *** FAILED TO RELEASE IMAGE OF TPU DEVICE ', time4xi())  
    #   
    #   task4getinfo2image = {}
    #   task4getinfo2image ['COUNT2REFERENCE'] = 0
    #   response4getinfo2image = {}
    #   print('(Step 133C/700) OF TPU2IMAGE4XI5TEST - TRYING TO GET SELECTED 2D IMAGE INFO OF TPU DEVICE ', time4xi())     
    #   # 
    #   try:                               
    #       response4getinfo2Dimage = xi2tpu.get2imageinfo (task4getinfo2image)                                                                                             
    #       keys2response = response4getinfo2Dimage.keys()
    #       print('(Step 133D/700) OF TPU2IMAGE4XI5TEST - GOT SELECTED 2D IMAGE INFO OF TPU DEVICE ', time4xi())   
    #       for key2response in sorted(keys2response):
    #           print(key2response, ' : ', response4getinfo2Dimage[key2response])
    #           # 
    #   except:
    #       print('(Step 133Z2/700) OF TPU2IMAGE4XI5TEST *** FAILED TO GET SELECTED 2D IMAGE INFO OF TPU DEVICE ', time4xi())  
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def release2image(self, task4release2image = None):
        '''
        releases TPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "TPU2Image4Xi"
        method2print = "release2image"
                     
        message2print = "{(Step 1A/33) started" + "; release2image-of-TPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        message2print = "{(Step 1B/33) passed" + "; task4release2image-passed-to-release2image-of-TPU2Image4Xi: " + str(task4release2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4release2image-passing-to-release2image4tpu-by-release2image-of-TPU2Image4Xi: " + str(task4release2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
  
        tpu2image2xi = release2image4tpu(task4release2image)

        message2print = "{(Step 2B/33) performed" + "; release2image4tpu-performed-by-release2image-of-TPU2Image4Xi: " + str(release2image4tpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; tpu2image2xi-returning-by-release2image-of-TPU2Image4Xi: " + str(tpu2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        #                          
                
        return tpu2image2xi
                                                           
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "retain2image-of-TPU2Image4Xi" usage = "
    #
    # ---------------------------------------------------------------------------------------------
    # RETAIN 2D IMAGE BY DECREASING REFERENCE WITH 1             
    # ---------------------------------------------------------------------------------------------    
    #   task4retain2image  = {}
    #   task4retain2image  ['COUNT2REFERENCE'] = 1 
    #   response4retain2Dimage = {}
    #   print('(Step 134A/700) OF TPU2IMAGE4XI5TEST - TRYING TO RETAIN 2D IMAGE OF TPU DEVICE ', time4xi())   
    #   try:                               
    #       response4retain2Dimage = xi2tpu.retain2image(task4retain2image)                                                                                                 
    #       print('(Step 134B/700) OF TPU2IMAGE4XI5TEST - RESPONSE4RETAIN2DIMAGE : ', time4xi())                         
    #       keys2response = response4retain2Dimage.keys()
    #       for key2response in sorted(keys2response):
    #           print(key2response, ' : ', response4retain2Dimage[key2response])
    #           # 
    #   except:
    #       print('(Step 134Z1/700) OF TPU2IMAGE4XI5TEST *** FAILED TO RETAIN 2D IMAGE OF TPU DEVICE ', time4xi())
    #   task4getinfo2image = {}
    #   task4getinfo2image ['COUNT2REFERENCE'] = 0
    #   response4getinfo2image = {}
    #   print('(Step 134C/700) OF TPU2IMAGE4XI5TEST - TRYING TO GET SELECTED 2D IMAGE INFO OF TPU DEVICE ', time4xi())     
    #   # 
    #   try:                               
    #       response4getinfo2Dimage = xi2tpu.get2imageinfo (task4getinfo2image)                                                                                             
    #       keys2response = response4getinfo2Dimage.keys()
    #       print('(Step 134D/700) OF TPU2IMAGE4XI5TEST - GOT SELECTED 2D IMAGE INFO OF TPU DEVICE ', time4xi())   
    #       for key2response in sorted(keys2response):
    #           print(key2response, ' : ', response4getinfo2Dimage[key2response])
    #           # 
    #   except:
    #       print('(Step 134Z2/700) OF TPU2IMAGE4XI5TEST *** FAILED TO GET SELECTED 2D IMAGE INFO OF TPU DEVICE ', time4xi())  
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def retain2image(self, task4retain2image = None):
        '''
        retains TPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "TPU2Image4Xi"
        method2print = "retain2image"
                     
        message2print = "{(Step 1A/33) started" + "; retain2image-of-TPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4retain-passed-to-retain2image-of-TPU2Image4Xi: " + str(task4retain2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # {'COUNT2REFERENCE': 1}
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4retain-passing-to-retain2image4tpu-by-retain-of-TPU2Image4Xi: " + str(task4retain2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # {'COUNT2REFERENCE': 1}
  
        tpu2image2xi = retain2image4tpu(task4retain2image)

        message2print = "{(Step 2B/33) performed" + "; retain2image4tpu-performed-by-retain2image-of-TPU2Image4Xi: " + str(retain2image4tpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function retain2image4tpu at 0x00000000039DF948>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; tpu2image2xi-returning-by-retain2image-of-TPU2Image4Xi: " + str(tpu2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0, 
        #  'ACK2RETURN': ' 1D-', 
        #  'ROWPITCH': 800, 
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4tpu-failed ; time: 00:14:05 09/07/11 Pacific Daylight Time}', 
        #  'CLASS': 'create2image4tpu', 
        #  'METHOD': 'create2image4tpu'}, 
        #  'WIDTH': 800, 
        #  'BUFFER': {'CODE2ERROR': 0, 
        #  ...
        #  'COUNT2REFERENCE': 1,
        #  'FLAGS': 0,
        #  'CONTEXT': {'COUNT2REFERENCE': 0 ...
                         
                
        return tpu2image2xi
                                                           
    #</methodize4xi>   
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "wait2image-of-TPU2Image4Xi" usage = "
    #   print('=' *33)          
    #   print('(Step 126A/700) OF TPU2IMAGE4XI5TEST - TRYING TO WAIT 2D IMAGE OPERATION FOR TPU ', time4xi()) 
    #   print('=' *33)      
    #   wait2tpu = {}
    #   try:
    #       response4wait2Dimage = xi2tpu.wait2image()
    #       print('(Step 122B/700) OF TPU2IMAGE4XI5TEST - WAITED 2D IMAGE OPERATION FOR TPU ', time4xi()) 
    #       keys2response = response4wait2Dimage.keys()
    #       for key2response in sorted(keys2response):
    #           print(key2response, ' : ', response4wait2Dimage[key2response])
    #           # 
    #   except:
    #       print('(Step 126Z1/700) OF TPU2IMAGE4XI5TEST *** FAILED TO 2D IMAGE OPERATION FOR TPU ', time4xi())    
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def wait2image(self, task4wait2image = None):
        '''
        waits TPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "TPU2Image4Xi"
        method2print = "wait2image"
                     
        message2print = "{(Step 1A/33) started" + "; wait2image-of-TPU2Image4Xi-started: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4wait2image-passed-to-wait2image-of-TPU2Image4Xi: " + str(task4wait2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4wait-passing-to-wait2image4tpu-by-wait2image-of-TPU2Image4Xi: " + str(task4wait2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        tpu2image2xi = wait2image4tpu(task4wait2image)

        message2print = "{(Step 2B/33) performed" + "; wait2image4tpu-performed-by-wait2image-of-TPU2Image4Xi: " + str(wait2image4tpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # semaphore and lock analysis by multiprocessing4xi
        # <function wait2image4tpu at 0x00000000039DFA48> 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; tpu2image2xi-returning-by-wait2image-of-TPU2Image4Xi: " + str(tpu2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'SIZE2QUEUE': 2147483646,
        #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'},
        #  'QUEUE': <queue4xi.Queue4Xi object at 0x000000000434DDA0>,
        #  'WIDTH': 800,
        #  'CONTEXT': ...
                
        return tpu2image2xi
                                                           
    #</methodize4xi> 
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "write2image-of-TPU2Image4Xi" usage = "
    #   print('(Step 127A/700) OF TPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH WRITE IMAGE OBJECT', time4xi()) 
    #   response4write2tpu = {}
    #   try:
    #       response4write2tpu = xi2tpu.write2image()
    #       print('(Step 127B/700) OF TPU2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH WRITE IMAGE OBJECT, RESPONSE4WRITE2TPU: ', response4write2tpu) 
    #   except:
    #       print('(Step 127Z1/700) OF TPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH WRITE IMAGE OBJECT', time4xi()) 
    #
    #   ... continue as with write2image4tpu function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def write2image(self, task4write2image = None):
        '''
        writes TPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "TPU2Image4Xi"
        method2print = "write2image"
                     
        message2print = "{(Step 1A/33) started" + "; write2image-of-TPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 02:45:31 10/05/11 Pacific Daylight Time
            
        message2print = "{(Step 1B/33) passed" + "; task4write2image-passed-to-write2image-of-TPU2Image4Xi: " + str(task4write2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4write2image-passing-to-write2image4tpu-by-write2image-of-TPU2Image4Xi: " + str(task4write2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        response4writeimage2tpu = write2image4tpu(task4write2image)

        message2print = "{(Step 2B/33) performed" + "; write2image4tpu-performed-by-write2image-of-TPU2Image4Xi: " + str(write2image4tpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function write2image4tpu at 0x000000000380AEC8>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4writeimage2tpu-returning-by-write2image-of-TPU2Image4Xi: " + str(response4writeimage2tpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004D10E10>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 968>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 964>}; time: 02:45:31 10/05/11 Pacific Daylight Time}

                     
                
        return response4writeimage2tpu
                                                           
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "copy2image-of-TPU2Image4Xi" usage = "
    #   print('(Step 127A/700) OF TPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH WRITE IMAGE OBJECT', time4xi()) 
    #   response4copy2tpu = {}
    #   try:
    #       response4copy2tpu = xi2tpu.copy2image()
    #       print('(Step 127B/700) OF TPU2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH WRITE IMAGE OBJECT, RESPONSE4WRITE2TPU: ', response4copy2tpu) 
    #   except:
    #       print('(Step 127Z1/700) OF TPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH WRITE IMAGE OBJECT', time4xi()) 
    #
    #   ... continue as with copy2image4tpu function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def copy2image(self, task4copy2image = None):
        '''
        copies TPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "TPU2Image4Xi"
        method2print = "copy2image"
                     
        message2print = "{(Step 1A/33) started" + "; copy2image-of-TPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:28:44 10/05/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4copy2image-passed-to-copy2image-of-TPU2Image4Xi: " + str(task4copy2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4copy2image-passing-to-copy2image4tpu-by-copy2image-of-TPU2Image4Xi: " + str(task4copy2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None 
  
        response4copyimage2tpu = copy2image4tpu(task4copy2image)

        message2print = "{(Step 2B/33) performed" + "; copy2image4tpu-performed-by-copy2image-of-TPU2Image4Xi: " + str(copy2image4tpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function copy2image4tpu at 0x000000000390B8C8>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4copyimage2tpu-returning-by-copy2image-of-TPU2Image4Xi: " + str(response4copyimage2tpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004EC4128>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 1000>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 996>}; time: 01:28:44 10/05/11 Pacific Daylight Time}
        
        return response4copyimage2tpu
                                                           
    #</methodize4xi>
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "copy2imagetobuffer-of-TPU2Image4Xi" usage = "
    #   print('(Step 127A/700) OF TPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH COPY IMAGE TO BUFFER OBJECT', time4xi()) 
    #   response4copyimagetobuffer = {}
    #   try:
    #       response4copyimagetobuffer = xi2tpu.copy2imagetobuffer()
    #       print('(Step 127B/700) OF TPU2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH COPY IMAGE TO BUFFER  OBJECT, RESPONSE4WRITE2TPU: ', response4copyimagetobuffer) 
    #   except:
    #       print('(Step 127Z1/700) OF TPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH COPY IMAGE TO BUFFER ', time4xi()) 
    #
    #   ... continue as with copy2imagetobuffer4tpu function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def copy2imagetobuffer(self, task4copy2imagetobuffer = None):
        '''
        copies 2D images to buffer
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "TPU2Image4Xi"
        method2print = "copy2imagetobuffer"
                     
        message2print = "{(Step 1A/33) started" + "; copy2imagetobuffer-of-TPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:28:44 10/05/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4copy2imagetobuffer-passed-to-copy2imagetobuffer-of-TPU2Image4Xi: " + str(task4copy2imagetobuffer) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4copy2imagetobuffer-passing-to-copy2imagetobuffer4tpu-by-copy2imagetobuffer-of-TPU2Image4Xi: " + str(task4copy2imagetobuffer) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        response4copyimage2tpu = copy2imagetobuffer4tpu(task4copy2imagetobuffer)

        message2print = "{(Step 2B/33) performed" + "; copy2imagetobuffer4tpu-performed-by-copy2imagetobuffer-of-TPU2Image4Xi: " + str(copy2imagetobuffer4tpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function copy2imagetobuffer4tpu at 0x000000000390B948>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4copyimage2tpu-returning-by-copy2imagetobuffer-of-TPU2Image4Xi: " + str(response4copyimage2tpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004FE5FD0>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 1036>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 1032>}; time: 01:28:44 10/05/11 Pacific Daylight Time}
          
        return response4copyimage2tpu
                                                           
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "copy2buffertoimage-of-TPU2Image4Xi" usage = "
    #   print('(Step 127A/700) OF TPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH COPY BUFFER TO IMAGE OBJECT', time4xi()) 
    #   response4copybuffertoimage = {}
    #   try:
    #       response4copybuffertoimage = xi2tpu.copy2buffertoimage()
    #       print('(Step 127B/700) OF TPU2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH COPY BUFFER TO IMAGE  OBJECT, RESPONSE4WRITE2TPU: ', response4copybuffertoimage) 
    #   except:
    #       print('(Step 127Z1/700) OF TPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH COPY BUFFER TO IMAGE ', time4xi()) 
    #
    #   ... continue as with copy2buffertoimage4tpu function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def copy2buffertoimage(self, task4copy2buffertoimage = None):
        '''
        copies 2D images to buffer
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "TPU2Image4Xi"
        method2print = "copy2buffertoimage"
                     
        message2print = "{(Step 1A/33) started" + "; copy2buffertoimage-of-TPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:28:44 10/05/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4copy2buffertoimage-passed-to-copy2buffertoimage-of-TPU2Image4Xi: " + str(task4copy2buffertoimage) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4copy2buffertoimage-passing-to-copy2buffertoimage4tpu-by-copy2buffertoimage-of-TPU2Image4Xi: " + str(task4copy2buffertoimage) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        response4copyimage2tpu = copy2buffertoimage4tpu(task4copy2buffertoimage)

        message2print = "{(Step 2B/33) performed" + "; copy2buffertoimage4tpu-performed-by-copy2buffertoimage-of-TPU2Image4Xi: " + str(copy2buffertoimage4tpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function copy2buffertoimage4tpu at 0x000000000390B9C8> 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4copyimage2tpu-returning-by-copy2buffertoimage-of-TPU2Image4Xi: " + str(response4copyimage2tpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x000000000505A6D8>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 1068>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 1064>}; time: 01:28:44 10/05/11 Pacific Daylight Time}

                
        return response4copyimage2tpu
                                                           
    #</methodize4xi>    
                    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "trace2image-of-TPU2Image4Xi" usage = "
    #   print('=' *33)          
    #   print('(Step 15A/25) OF TPU2IMAGE4XI5TEST - TRYING TO TRACE TPU: ', time4xi())      
    #   print('=' *33)      
    #   trace2tpu = None
    #   try:
    #       trace2tpu = trace2image4tpu() 
    #       print('(Step 15B/25) OF TPU2IMAGE4XI5TEST - TRACE2TPU: ', trace2tpu)  
    #       # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4tpu-failed {}; time: 01:39:15 10/06/11 Pacific Daylight Time}', 'CLASS': 'trace2image4tpu', 'METHOD': 'trace2image4tpu'}
    #    except:
    #       print('(Step 15Z1/25) OF TPU2IMAGE4XI5TEST *** FAILED TO TRACE TPU: ', trace2tpu)     
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def trace2image(self, task4trace2image = None):
        '''
        traces TPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "TPU2Image4Xi"
        method2print = "trace2image"
                     
        message2print = "{(Step 1A/33) started" + "; trace2image-of-TPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4trace2image-passed-to-trace-of-TPU2Image4Xi: " + str(task4trace2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None   
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4trace2image-passing-to-trace2image4tpu-by-trace2image-of-TPU2Image4Xi: " + str(task4trace2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None  
  
        trace2xi = trace2image4tpu(task4trace2image)

        message2print = "{(Step 2B/33) performed" + "; trace2image4tpu-performed-by-trace2image-of-TPU2Image4Xi: " + str(trace2image4tpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function trace2image4tpu at 0x00000000039DF348>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; trace2xi-returning-by-trace2image-of-TPU2Image4Xi: " + str(trace2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4tpu-failed {}; time: 01:05:20 09/08/11 Pacific Daylight Time}',
        #  'CLASS': 'trace2image4tpu',
        #  'METHOD': 'trace2image4tpu'}; time: 01:05:20 09/08/11 Pacific Daylight Time}
                 
                
        return trace2xi
                                                           
    #</methodize4xi>       


    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "set2imagecallback-of-TPU2Image4Xi" usage = "
    #   print('(Step 1A/33) OF TPU2IMAGE4XI - TRYING TO SET TPU IMAGE')
    #   try:
    #       cc2xi = TPU2Image4Xi()                            
    #       image2tpu = cc2xi.set2imagecallback()                                  
    #       print('(Step 1B/33) OF TPU4XI5TEST - SET TPU IMAGE: ', image2tpu)                                                  
    #   except:
    #       print('(Step 1Z/33) OF TPU4XI5TEST *** FAILED TO SET TPU IMAGE ***')  
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def set2imagecallback(self, task4set2imagecallback = None): 
        '''
        set2imagecallback of TPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "TPU2Image4Xi"
        method2print = "set2imagecallback"
                     
        message2print = "{(Step 1A/33) started" + "; set2imagecallback-of-TPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time
            
        message2print = "{(Step 1B/33) passed" + "; task4set2imagecallback-passed-to-set2imagecallback-of-TPU2Image4Xi: " + str(task4set2imagecallback) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None    
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4set2imagecallback-passing-to-set2imagecallback -by-set2imagecallback-of-TPU2Image4Xi: " + str(task4set2imagecallback) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
        
        callback2tpu4image = {} 
        try:
            callback2tpu4image = set2imagecallback4tpu (task4set2imagecallback)

            message2print = "{(Step 2B/33) callback2tpu4image" + "; callback2tpu4image-set-by-set2imagecallback-of-TPU2Image4Xi to: " + str(callback2tpu4image) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            # <function trace2image4tpu at 0x00000000039DF348>
        except:
            message2print = "{(Step 4A/33) failed" + "; callback2tpu4image-failed-by-set2imagecallback-of-TPU2Image4Xi: " + str(callback2tpu4image) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
                                            
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; callback2tpu4image-returning-by-set2imagecallback-of-TPU2Image4Xi: " + str(callback2tpu4image) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # <function trace2image4tpu at 0x0000000003AB31C8> 
                                             
        return callback2tpu4image
                                                           
    #</methodize4xi>     
    
                                                              
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "sense-of-TPU2Image4Xi" usage = "
    #   try:
    #       print(xi2tpu.sense())
    #       print('(Step 24B/33) OF TPU2IMAGE4XI5TEST - SENSED ', time4xi()) 
    #   except:
    #       print('(Step 24Z/33) OF TPU2IMAGE4XI5TEST *** FAILED TO SENSE ', time4xi())  
    # ">
    
    # ---------------------------------------------------------------------------------------------
    def sense(self):
        '''
        method sense: senses TPU2Image4Xi class objects
        '''
        
        # -----------------------------------------------------------------------------------------
        #
        # ----------------------------------------------------------------------------------------- 
        class2print = 'TPU2Image4Xi'
        method2print = 'sense'
        
                     
        message2print = "{(Step 1A/33) started" + "; sense-of-TPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:02:22 09/07/11 Pacific Daylight Time 
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------  
        
        message2print = "{(Step 2A/33) sensing" + "; TPU2Image4Xi-sensing-by-sense-of-TPU2Image4Xi: " + str(TPU2Image4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        # <class 'tpu2image4xi.TPU2Image4Xi'>         
        
        # -----------------------------------------------------------------------------------------
        # get sensed knowledge for this object
        # -----------------------------------------------------------------------------------------        
        
        kb2xi = sense_object4xi(self)
        
        message2print = "{(Step 3A/33) got" + "; kb2xi-got-by-sense-of-TPU2Image4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'object': <tpu2image4xi.TPU2Image4Xi object at 0x0000000004188400>}   
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb2xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------         
        try:
            kb2xi['classvariable2tpu-of-TPU2Image4Xi'] = TPU2Image4Xi.classvariable2tpu
        except:
            message2print = "{(Step 4A/33) yielded" + "; kb2xi-yielded-by-sense-of-TPU2Image4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
      
        message2print = "{(Step 5A/33) sensed" + "; TPU2Image4Xi-sensed-by-sense-of-TPU2Image4Xi: " + str(TPU2Image4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        # <class 'tpu2image4xi.TPU2Image4Xi'>
        
        # -----------------------------------------------------------------------------------------
        # 
        # -----------------------------------------------------------------------------------------         
        
        message2print = "{(Step 6A/33) returning" + "; kb2xi-returning-by-sense-of-TPU2Image4Xi: " + str(kb2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # {'object': <tpu2image4xi.TPU2Image4Xi object at 0x0000000004188400>, 'classvariable2tpu-of-TPU2Image4Xi': 4}             
          
        return kb2xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
    #</methodize4xi>
    
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "reason-of-TPU2Image4Xi" usage = "
    #   print('(Step 25A/33) OF TPU2IMAGE4XI5TEST - REASONING ', time4xi())
    #   try:
    #       print(xi2tpu.reason())
    #       print('(Step 25B/33) OF TPU2IMAGE4XI5TEST - REASONED ', time4xi())
    #   except:
    #       print('(Step 25Z/33) OF TPU2IMAGE4XI5TEST *** FAILED TO REASON ', time4xi())   
    #">
    
    # ---------------------------------------------------------------------------------------------
    def reason(self):
        '''
        method reason: reasons TPU2Image4Xi class objects
        '''
        
        # -----------------------------------------------------------------------------------------
        #
        # ----------------------------------------------------------------------------------------- 
        class2print = 'TPU2Image4Xi'
        method2print = 'reason'

                     
        message2print = "{(Step 1A/33) started" + "; reason-of-TPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:02:22 09/07/11 Pacific Daylight Time 
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        
        message2print = "{(Step 2A/33) reasoning" + "; TPU2Image4Xi-reasoning-by-reason-of-TPU2Image4Xi: " + str(TPU2Image4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <class 'tpu2image4xi.TPU2Image4Xi'>  
        
        # -----------------------------------------------------------------------------------------
        # get reasoned knowledge for this object
        # -----------------------------------------------------------------------------------------         
        
        kb4xi = reason_object4xi(self)
        
        
        message2print = "{(Step 3A/33) got" + "; kb4xi-got-by-reason-of-TPU2Image4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'object-reason-object4xi': <tpu2image4xi.TPU2Image4Xi object at 0x0000000004188400>}      
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb4xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------          
        try:
            kb4xi['time-reason-of-TPU2Image4Xi'] = time4xi() #adding time
        except:
            message2print = "{(Step 4Z/33) yielded" + "; kb4xi-yielded-by-reason-of-TPU2Image4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
        
          
        
        message2print = "{(Step 5A/33) reasoned" + "; TPU2Image4Xi-reasoned-by-reason-of-TPU2Image4Xi: " + str(TPU2Image4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        # <class 'tpu2image4xi.TPU2Image4Xi'>
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        
        message2print = "{(Step 6A/33) returning" + "; kb4xi-returning-by-reason-of-TPU2Image4Xi: " + str(kb4xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # {'object-reason-object4xi': <tpu2image4xi.TPU2Image4Xi object at 0x0000000004188400>, 'time-reason-of-TPU2Image4Xi': '01:02:22 09/07/11 Pacific Daylight Time'}                
          
        return kb4xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "ace-of-TPU2Image4Xi" usage = "
    #   print('(Step 26A/33) OF TPU2IMAGE4XI5TEST - ACING ', time4xi())   
    #   try:
    #       print(xi2tpu.ace())
    #       print('(Step 26B/33) OF TPU2IMAGE4XI5TEST - ACED ', time4xi())                                                 
    #   except:
    #       print('(Step 26Z/33) OF TPU2IMAGE4XI5TEST *** FAILED TO ACE ', time4xi())                                       
    #    
    #">
    
    # ---------------------------------------------------------------------------------------------
    def ace(self):
        '''
        method ace: aces TPU2Image4Xi class objects 
        '''
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------       
        
        
        class2print = 'TPU2Image4Xi'
        method2print = 'ace'
        
                     
        message2print = "{(Step 1A/33) started" + "; ace-of-TPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:02:22 09/07/11 Pacific Daylight Time   
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        
        message2print = "{(Step 2A/33) acing" + "; TPU2Image4Xi-acing-by-ace-of-TPU2Image4Xi: " + str(TPU2Image4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        # <class 'tpu2image4xi.TPU2Image4Xi'>           
        
        # -----------------------------------------------------------------------------------------
        # get aced knowledge for this object
        # -----------------------------------------------------------------------------------------        
        
        kb8xi = ace_object4xi(self)  
        
        message2print = "{(Step 3A/33) got" + "; kb8xi-got-by-ace-of-TPU2Image4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'object8xi': <tpu2image4xi.TPU2Image4Xi object at 0x0000000004188400>}   


        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb8xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------              
        try:
             kb8xi['cores-of-TPU2Image4Xi'] = TPU2Image4Xi.classvariable2tpu
        except:
             message2print = "{(Step 4A/33) yielded" + "; kb8xi-yielded-by-ace-of-TPU2Image4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
             print4xi(class2print, method2print, message2print) 
  
        message2print = "{(Step 5A/33) aced" + "; TPU2Image4Xi-aced-by-ace-of-TPU2Image4Xi: " + str(TPU2Image4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        # <class 'tpu2image4xi.TPU2Image4Xi'>  
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{(Step 6A/33) returning" + "; kb8xi-returning-by-ace-of-TPU2Image4Xi: " + str(kb8xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # {'cores-of-TPU2Image4Xi': 4, 'object8xi': <tpu2image4xi.TPU2Image4Xi object at 0x0000000004188400>}
        
        return kb8xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
#</class4xi>
#</program4xi>

# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#   tpu2image4xi5test.txt
#   sense-tpu2image4xi5test.xml
#   reason-tpu2image4xi5test.xml
#   ace-tpu2image4xi5test.xml
#"
#>

#<acknowledge4xi acknowledgement = "33" tester = "Small & Smart"/>

# -------------------------------------------------------------------------------------------------
# to-be-done                              
# -------------------------------------------------------------------------------------------------    
#<do4xi do = "
# nothing
#"
#/>

#<note4xi note = "
# parse4xi.py - to parse this module
# "/>

# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>

#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of tpu2image4xi.py                              
# ------------------------------------------------------------------------------------------------- 
#"
#/>
